package com.tofly.dyrqGps.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.Date8Util;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrqGps.entity.EsCarGpsUpload;
import com.tofly.dyrqGps.entity.EsCarGpsUploadQuery;
import com.tofly.dyrqGps.entity.EsQuery;
import com.tofly.dyrqGps.repository.EsCarGpsUploadService;
import com.tofly.dyrqGps.service.CarGpsService;
import com.tofly.dyrqGps.util.EsQueryUtil;
import com.tofly.dyrqGps.util.EsSaveInfoUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.elasticsearch.client.Client;
import org.elasticsearch.index.query.QueryBuilder;
import org.springframework.data.elasticsearch.core.ElasticsearchTemplate;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author tuonbed
 * @date 2020/5/6  15:38
 */
@RestController
@AllArgsConstructor
@RequestMapping("/esCarGps")
@Api(tags = "ES存储汽车GPS信息")
public class EsCarGpsUploadController {

    private final EsCarGpsUploadService esCarGpsUploadService;
    private final ElasticsearchTemplate elasticsearchTemplate;

    private final EsSaveInfoUtil esSaveInfoUtil;

    private final CarGpsService carGpsService;


    @ToFlyAppLog(title="新增汽车GPS信息" )
    @ApiOperation(value = "新增汽车GPS信息")
    @PostMapping
    public ResultRespone save(@RequestBody EsCarGpsUpload esCarGpsUpload) {
        return ResultRespone.success(carGpsService.cargpsSave(esCarGpsUpload));
    }

    @GetMapping("/updateCarInfo")
    @ApiOperation(value = "更新汽车车载设备绑定的GPS信息")
    public ResultRespone updateCarInfo(){
        esSaveInfoUtil.updateCarGpsInfo();
        return ResultRespone.success(true);
    }


    @ToFlyAppLog(title = "修改汽车GPS信息")
    @ApiOperation(value = "修改汽车GPS信息")
    @PutMapping
    public ResultRespone update(@RequestBody EsCarGpsUpload esCarGpsUpload) {
        return ResultRespone.success(esCarGpsUploadService.update(esCarGpsUpload));
    }


    @ApiOperation(value = "根据ID获取汽车GPS信息")
    @GetMapping("/findone")
    public ResultRespone get(String id) {
        return ResultRespone.success(esCarGpsUploadService.findById(id).get());
    }


    /**
     * 通过id删除GPS信息表
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除汽车GPS信息")
    @ApiOperation(value = "通过id删除汽车GPS信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable String id) {
        esCarGpsUploadService.deleteById(id);
        return ResultRespone.success(true);
    }


    @ApiOperation(value = "分页获取汽车GPS信息")
    @GetMapping("/page")
    public ResultRespone get(Page page, @ModelAttribute EsQuery esQuery) {
        EsCarGpsUpload esCarGpsUpload = null;
        QueryBuilder[] queryBuilders = EsQueryUtil.putQUery(page, esQuery);
        return ResultRespone.success(esCarGpsUploadService.getPage(esCarGpsUpload, page, queryBuilders));
    }

    @ApiOperation(value = "获取所有index的汽车GPS信息")
    @GetMapping("/getAllPage")
    public ResultRespone getAllPage(Page page, @ModelAttribute EsQuery esQuery) {
        EsCarGpsUpload esCarGpsUpload = null;
        QueryBuilder[] queryBuilders = EsQueryUtil.putQUery(page, esQuery);
        return ResultRespone.success(esCarGpsUploadService.getAllPage(esCarGpsUpload, page,
                esSaveInfoUtil.DyMonth(esCarGpsUploadService.getIndex()), elasticsearchTemplate, queryBuilders));
    }

    @ApiOperation(value = "获取所有index的汽车GPS信息（非分页）")
    @GetMapping("/getAll")
    public ResultRespone getAllpage(@ModelAttribute EsQuery esQuery) {
        EsCarGpsUpload esCarGpsUpload = null;
        Page page = new Page();
        QueryBuilder[] queryBuilders = EsQueryUtil.putQUery(page, esQuery);
        return ResultRespone.success(esCarGpsUploadService.getAll(esCarGpsUpload, page,
                esSaveInfoUtil.DyMonth(esCarGpsUploadService.getIndex()), elasticsearchTemplate, queryBuilders));
    }

    @ApiOperation(value = "获取汽车GPS最新位置信息")
    @GetMapping("/getCarLocation")
    public ResultRespone getCarLocation(@ModelAttribute EsCarGpsUploadQuery esCarGpsUploadQuery) {
    	
    	esCarGpsUploadQuery.setGpsType(Long.valueOf("1"));
    	esCarGpsUploadQuery.setOrderAsc(true);
    	
    	
    	System.out.println("################################1"+esCarGpsUploadQuery.getCarId());
    	System.out.println("################################2"+esCarGpsUploadQuery.getGpsType());
    	System.out.println("################################3"+esCarGpsUploadQuery.getEndTime());
    	System.out.println("################################4"+esCarGpsUploadQuery.getOrderAsc());
    	System.out.println("################################5"+esCarGpsUploadQuery.getStartTime());
    	
    	
    	
    	if (esCarGpsUploadQuery.getCarId() == null){
            throw new ToflyDeniedException("carId不能为空");
        }
    	
        EsCarGpsUpload esCarGpsUpload = null;
        Page page = new Page();
        page.setSize(new Long(10000));
        QueryBuilder[] queryBuilders = EsQueryUtil.CarGpsUploadQuery(page, esCarGpsUploadQuery);
                             
        List<EsCarGpsUpload> result =new ArrayList<EsCarGpsUpload>();
        
        try {
        	   result = esCarGpsUploadService.getAll(esCarGpsUpload, page,
                      esSaveInfoUtil.DyMonth(esCarGpsUploadService.getIndex()), elasticsearchTemplate, queryBuilders);
        	   System.out.println("################################6"+result);
        
        }catch (Exception e) {
			e.printStackTrace();
		}
        
        
      
        
        
        
        
        Map resultMap = new HashMap<>();
        int end = result.size() - 1;
        resultMap.put("lat", result.size() == 0 ? null : result.get(0).getLocationLatitude());
        resultMap.put("lng", result.size() == 0 ? null : result.get(0).getLocationLongitude());
        resultMap.put("onTime",result.size() == 0 ? null : result.get(end).getGpstime());
        resultMap.put("offTime",result.size() == 0 ? null : result.get(0).getGpstime());
        return ResultRespone.success(resultMap);
    }

    @ApiOperation(value = "获取所有的汽车GPS信息（非分页orderby gpstime）")
    @GetMapping("/getList")
    public ResultRespone getList(@ModelAttribute EsCarGpsUploadQuery esCarGpsUploadQuery) {
        EsCarGpsUpload esCarGpsUpload = null;
        Page page = new Page();
        page.setSize(new Long(10000));
        QueryBuilder[] queryBuilders = EsQueryUtil.CarGpsUploadQuery(page, esCarGpsUploadQuery);
        return ResultRespone.success(esCarGpsUploadService.getAll(esCarGpsUpload, page,
                esSaveInfoUtil.DyMonth(esCarGpsUploadService.getIndex()), elasticsearchTemplate, queryBuilders));
    }

    @ApiOperation(value = "获取所有的汽车GPS信息（分页orderby gpstime）")
    @GetMapping("/getListPage")
    public ResultRespone getListPage(Page page, @ModelAttribute EsCarGpsUploadQuery esCarGpsUploadQuery) {
        EsCarGpsUpload esCarGpsUpload = null;
        
        esCarGpsUploadQuery.setGpsType(Long.valueOf("1"));
    	esCarGpsUploadQuery.setOrderAsc(true);
        
    	System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@1"+esCarGpsUploadQuery.getCarId());
    	System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@2"+esCarGpsUploadQuery.getGpsType());
    	System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@3"+esCarGpsUploadQuery.getEndTime());
    	System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@4"+esCarGpsUploadQuery.getOrderAsc());
    	System.out.println("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@5"+esCarGpsUploadQuery.getStartTime());
    	
     	
        
        QueryBuilder[] queryBuilders = EsQueryUtil.CarGpsUploadQuery(page, esCarGpsUploadQuery);
        return ResultRespone.success(esCarGpsUploadService.getAllPage(esCarGpsUpload, page,
                esSaveInfoUtil.DyMonth(esCarGpsUploadService.getIndex()), elasticsearchTemplate, queryBuilders));
    }
}
