package com.tofly.dyrqGps.controller;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.Date8Util;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrqGps.entity.EsCarGpsUpload;
import com.tofly.dyrqGps.entity.EsGpsUpload;
import com.tofly.dyrqGps.entity.EsQuery;
import com.tofly.dyrqGps.mapper.EsSaveInfoMapper;
import com.tofly.dyrqGps.repository.EsGpsUploadService;
import com.tofly.dyrqGps.util.DyrqCommonRedis;
import com.tofly.dyrqGps.util.EsQueryUtil;
import com.tofly.dyrqGps.util.EsSaveInfoUtil;
import com.tofly.dyrqGps.util.GpsDistanceUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.client.Client;
import org.elasticsearch.index.query.*;
import org.elasticsearch.search.sort.SortBuilder;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.elasticsearch.core.ElasticsearchTemplate;
import org.springframework.data.elasticsearch.core.aggregation.AggregatedPage;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.data.elasticsearch.core.query.SearchQuery;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author tuonbed
 * @date 2020/5/6  15:38
 */
@RestController
@AllArgsConstructor
@RequestMapping("/esGps")
@Api(tags = "ES存储GPS信息")
@Slf4j
public class EsGpsUploadController {

    private final EsGpsUploadService esGpsUploadService;
    private final ElasticsearchTemplate elasticsearchTemplate;
    private final RedisTemplate redisTemplate;

    private final ValueOperations<String, String> valueOperations;
    private final EsSaveInfoUtil esSaveInfoUtil;
    private final EsSaveInfoMapper esSaveInfoMapper;

    //    @ToFlyAppLog(title="新增GPS信息" )
    @ApiOperation(value = "新增GPS信息")
    @PostMapping
    public ResultRespone save(@RequestBody EsGpsUpload esGpsUpload) {
        esGpsUpload.setUserid(SecurityUtils.getUserId());
        esGpsUpload.setUserName(SecurityUtils.getUser().getRealName());
        if (esGpsUpload.getUploadtime() == null) {
            esGpsUpload.setUploadtime(Date8Util.getCurrentDate());
        }
        if (esGpsUpload.getGpstime() == null) {
            esGpsUpload.setGpstime(Date8Util.getCurrentDate());
        }
//        esGpsUploadService.save(esGpsUpload);
        esSaveInfoUtil.checkorinsertEsInfo(esGpsUploadService.getIndex());
        Boolean b = saveGps(esGpsUpload);
        if (b) {
            new Thread(new Runnable() {
                @Override
                public void run() {
                    saveToRedis(esGpsUpload);
                }
            }).start();
        }
        return ResultRespone.success(b);
    }

    @ToFlyAppLog(title = "批量新增GPS信息")
    @ApiOperation(value = "批量新增GPS信息")
    @PostMapping("/saveBatch")
    public ResultRespone saveBatch(@RequestBody List<EsGpsUpload> gpsList) {
        Long userId = SecurityUtils.getUserId();
        String userName = SecurityUtils.getUser().getRealName();
        Date now = Date8Util.getCurrentDate();
        for (EsGpsUpload gps : gpsList) {
            gps.setUserid(userId);
            gps.setUserName(userName);
            if (gps.getUploadtime() == null) {
                gps.setUploadtime(now);
            }
        }
        esGpsUploadService.saveAll(gpsList);
        esSaveInfoUtil.checkorinsertEsInfo(esGpsUploadService.getIndex());

        new Thread(new Runnable() {
            @Override
            public void run() {
                saveToRedis(gpsList.get(gpsList.size() - 1));
            }
        }).start();
        return ResultRespone.success(true);
    }


    @ToFlyAppLog(title = "修改GPS信息")
    @ApiOperation(value = "修改GPS信息")
    @PutMapping
    public ResultRespone update(@RequestBody EsGpsUpload esGpsUpload) {
        return ResultRespone.success(esGpsUploadService.update(esGpsUpload));
    }


    @ApiOperation(value = "根据ID获取GPS信息")
    @GetMapping("/findone")
    public ResultRespone get(String id) {
        return ResultRespone.success(esGpsUploadService.findById(id).get());
    }


    /**
     * 通过id删除GPS信息表
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除GPS信息")
    @ApiOperation(value = "通过id删除GPS信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable String id) {
        esGpsUploadService.deleteById(id);
        return ResultRespone.success(true);
    }


    @ApiOperation(value = "分页获取GPS信息")
    @GetMapping("/page")
    public ResultRespone get(Page page, @ModelAttribute EsQuery esQuery) {
        EsGpsUpload esGpsUpload = null;
        QueryBuilder[] queryBuilders = EsQueryUtil.putQUery(page, esQuery);

        return ResultRespone.success(esGpsUploadService.getPage(esGpsUpload, page, queryBuilders));
    }

    @ApiOperation(value = "获取所有index的GPS信息")
    @GetMapping("/getAllPage")
    public ResultRespone getAllPage(Page page, @ModelAttribute EsQuery esQuery) {
        EsGpsUpload esGpsUpload = null;
        QueryBuilder[] queryBuilders = EsQueryUtil.putQUery(page, esQuery);
        return ResultRespone.success(esGpsUploadService.getAllPage(esGpsUpload, page,
                esSaveInfoUtil.DyMonth(esGpsUploadService.getIndex()), elasticsearchTemplate, queryBuilders));
    }

    @ApiOperation(value = "获取GPS信息并统计数据")
    @GetMapping("/getAllPageStatistics")
    public ResultRespone getAllPageStatistics(Page page, @ModelAttribute EsQuery esQuery) {
        EsGpsUpload esGpsUpload = null;
        QueryBuilder[] queryBuilders = EsQueryUtil.putQUery(page, esQuery);
        return ResultRespone.success(esGpsUploadService.getAll(esGpsUpload, page,
                esSaveInfoUtil.DyMonth(esGpsUploadService.getIndex()), elasticsearchTemplate, queryBuilders));
    }

    @ApiOperation(value = "获取所有index的GPS信息（非分页）")
    @GetMapping("/getAll")
    public ResultRespone getAll(@ModelAttribute EsQuery esQuery) {
        EsGpsUpload esGpsUpload = null;
        Page page = new Page();
        page.setSize(10000);
        QueryBuilder[] queryBuilders = EsQueryUtil.putQUery(page, esQuery);
        return ResultRespone.success(esGpsUploadService.getAll(esGpsUpload, page,
                esSaveInfoUtil.DyMonth(esGpsUploadService.getIndex()), elasticsearchTemplate, queryBuilders));
    }


    private void saveToRedis(EsGpsUpload esGpsUpload) {
        log.info("异步方式gps信息存储到redis：{}", esGpsUpload);
        Map map = new HashMap();
        map.put("x", esGpsUpload.getLocationX());
        map.put("y", esGpsUpload.getLocationY());
        valueOperations.set(DyrqCommonRedis.REDIS_PRE_GPS + esGpsUpload.getUserid(), JSON.toJSONString(map));
        Boolean aBoolean = redisTemplate.expireAt(DyrqCommonRedis.REDIS_PRE_GPS + esGpsUpload.getUserid(), Date8Util.getEndTime());
    }

    @ApiOperation(value = "获取所有GPS信息分段信息")
    @GetMapping("/getsection")
    public ResultRespone getSection(@ModelAttribute EsQuery esQuery) {
        if (esQuery.getUserId() == null) {
            throw new ToflyDeniedException("用户id不能为空");
        }
        if (esQuery.getUploadTimes() == null && esQuery.getUploadTimee() != null) {
            throw new ToflyDeniedException("开始时间和结束时间需同时输入");
        }
        if (esQuery.getUploadTimes() != null && esQuery.getUploadTimee() == null) {
            throw new ToflyDeniedException("开始时间和结束时间需同时输入");
        }
        EsGpsUpload esGpsUpload = null;
        Page page = new Page();
        page.setSize(new Long(10000));
        esQuery.setIsAsc(true);
        QueryBuilder[] queryBuilders = EsQueryUtil.putQUery(page, esQuery);
        List<EsGpsUpload> list = esGpsUploadService.getAll(esGpsUpload, page,
                esSaveInfoUtil.DyMonth(esGpsUploadService.getIndex()), elasticsearchTemplate, queryBuilders);
        //截取分段信息
        int pointNum = 1000;
        int total = list.size();
        float section = (float) total / pointNum;
        List<String> result = new ArrayList<>();
        for (int i = 0; i < section; i++) {
            int start = i * pointNum;
            int end = start + pointNum - 1;
            if (section - i < 1) {
                end = start + total % pointNum - 1;
            }
            result.add("第" + (i + 1) + "段:" + Date8Util.format(list.get(start).getUploadtime(), "yyyy-MM-dd HH:mm:ss") + " - "
                    + Date8Util.format(list.get(end).getUploadtime(), "yyyy-MM-dd HH:mm:ss"));
        }
        return ResultRespone.success(result);
    }

    private Boolean saveGps(EsGpsUpload esGpsUpload) {
        //精度判断
        if (esGpsUpload.getGpsPrecision() != null && Integer.valueOf(esGpsUpload.getGpsPrecision()) > 100) {
            return false;
        }
//        //查询最新记录，比较时间，判断速度是否大于设置的最大容差
//        EsGpsUpload lastest = queryLastestOne(elasticsearchTemplate, esGpsUpload.getUserid());
//        if (lastest != null) {
//            double timeSpan = (esGpsUpload.getGpstime().getTime() - lastest.getGpstime().getTime()) / 1000;
//            //间隔时间小于5分钟的进行判断，超出5分钟直接录入
//            if (timeSpan < 300.0) {
//                BigDecimal lastLng = lastest.getLocationLongitude();
//                BigDecimal lastLat = lastest.getLocationLatitude();
//                double maxSpeed = esSaveInfoMapper.getMaxSpeed();
//                double distance = GpsDistanceUtil.getDistance(lastLng.doubleValue(), lastLat.doubleValue(), esGpsUpload.getLocationLongitude().doubleValue(), esGpsUpload.getLocationLatitude().doubleValue());
//                double speed = distance / timeSpan;
//                if (speed < maxSpeed) {
//                    esGpsUploadService.save(esGpsUpload);
//                    return true;
//                } else {
//                    return false;
//                }
//            } else {
//                esGpsUploadService.save(esGpsUpload);
//                return true;
//            }
//        } else {
//            esGpsUploadService.save(esGpsUpload);
//            return true;
//        }
        esGpsUploadService.save(esGpsUpload);
        return true;
    }

    public EsGpsUpload queryLastestOne(ElasticsearchTemplate elasticsearchTemplate, Long userId) {
        BoolQueryBuilder qb = QueryBuilders.boolQuery();
        MatchQueryBuilder qbIsvalid = QueryBuilders.matchQuery("userid", userId);
        qb.must(qbIsvalid);
        PageRequest pageRequest = PageRequest.of(0, 1);
        SortBuilder sortBuilder = SortBuilders.fieldSort("gpstime").order(SortOrder.fromString("DESC"));
        SearchQuery searchQuery = new NativeSearchQueryBuilder()
                .withQuery(qb)
                .withSort(sortBuilder)
                .withPageable(pageRequest)
                .build();

        try {
            AggregatedPage<EsGpsUpload> list = elasticsearchTemplate.queryForPage(searchQuery, esGpsUploadService.getEntityClass());
            return list.getContent().get(0);
        } catch (Exception e) {
            return null;
        }
    }

}
