package com.tofly.dyrqGps.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.Date8Util;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrqGps.entity.EsPdaDeviceUpload;
import com.tofly.dyrqGps.entity.EsQuery;
import com.tofly.dyrqGps.repository.EsPdaDeviceUploadService;
import com.tofly.dyrqGps.util.DyrqCommonRedis;
import com.tofly.dyrqGps.util.EsQueryUtil;
import com.tofly.dyrqGps.util.EsSaveInfoUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.elasticsearch.client.Client;
import org.elasticsearch.index.query.QueryBuilder;
import org.springframework.data.elasticsearch.core.ElasticsearchTemplate;
import org.springframework.data.redis.core.ListOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;

/**
 * @author tuonbed
 * @date 2020/5/6  15:38
 */
@RestController
@AllArgsConstructor
@RequestMapping("/esPdaDevice")
@Api(tags="ES存储移动设备信息")
@Slf4j
public class EsPdaDeviceUploadController {

    private final EsPdaDeviceUploadService esPdaDeviceUploadService;
    private final ElasticsearchTemplate elasticsearchTemplate;
    private final Client client;

    private final ListOperations<String, Object> listOperations;

    private final RedisTemplate redisTemplate;
    private final EsSaveInfoUtil esSaveInfoUtil;


//    @ToFlyAppLog(title="新增移动设备信息" )
    @ApiOperation(value = "新增移动设备信息")
    @PostMapping
    public ResultRespone save(@RequestBody EsPdaDeviceUpload esPdaDeviceUpload) {
        if (esPdaDeviceUpload.getWorkState() == null){
            esPdaDeviceUpload.setWorkState(new BigDecimal(2));
        }
        esPdaDeviceUpload.setUserid(SecurityUtils.getUserId());
        esPdaDeviceUpload.setUserName(SecurityUtils.getUser().getRealName());
        if(esPdaDeviceUpload.getUploadtime()==null){
            esPdaDeviceUpload.setUploadtime(Date8Util.getCurrentDate());
        }
        esPdaDeviceUploadService.save(esPdaDeviceUpload);
        esSaveInfoUtil.checkorinsertEsInfo(esPdaDeviceUploadService.getIndex());

        new Thread(new Runnable() {
            @Override
            public void run() {
                saveToRedis(esPdaDeviceUpload);
            }
        }).start();
        return ResultRespone.success(true);
    }


    @ToFlyAppLog(title="修改移动设备信息" )
    @ApiOperation(value = "修改移动设备信息")
    @PutMapping
    public ResultRespone update(@RequestBody EsPdaDeviceUpload esPdaDeviceUpload) {
         return ResultRespone.success(esPdaDeviceUploadService.update(esPdaDeviceUpload));
    }


    @ApiOperation(value = "根据ID获取移动设备信息")
    @GetMapping("/findone")
    public ResultRespone get(String id){
        return ResultRespone.success(esPdaDeviceUploadService.findById(id).get());
    }




    /**
     * 通过id删除GPS信息表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除移动设备信息" )
    @ApiOperation(value = "通过id删除移动设备信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable String id) {
        esPdaDeviceUploadService.deleteById(id);
        return ResultRespone.success(true);
    }


    @ApiOperation(value = "分页获取移动设备信息")
    @GetMapping("/page")
    public ResultRespone get(Page page, @ModelAttribute EsQuery esQuery){
        EsPdaDeviceUpload esPdaDeviceUpload=null;
        QueryBuilder[] queryBuilders = EsQueryUtil.deviceQuery(page, esQuery);
        return ResultRespone.success(esPdaDeviceUploadService.getPage(esPdaDeviceUpload,page,queryBuilders));
    }

    @ApiOperation(value = "获取所有index的移动设备信息")
    @GetMapping("/getAllPage")
    public ResultRespone getAllPage(Page page, @ModelAttribute EsQuery esQuery){
        EsPdaDeviceUpload esPdaDeviceUpload=null;
        QueryBuilder[] queryBuilders = EsQueryUtil.deviceQuery(page, esQuery);
        return ResultRespone.success(esPdaDeviceUploadService.getAllPage(esPdaDeviceUpload,page,
                esSaveInfoUtil.DyMonth(esPdaDeviceUploadService.getIndex()),elasticsearchTemplate,queryBuilders));
    }


    @ApiOperation(value = "获取所有index的移动设备信息（非分页）")
    @GetMapping("/getAll")
    public ResultRespone getAllpage(@ModelAttribute EsQuery esQuery){
        System.out.println("---------------"+esQuery);
        EsPdaDeviceUpload esPdaDeviceUpload=null;
        Page page=new Page();
        QueryBuilder[] queryBuilders = EsQueryUtil.deviceQuery(page, esQuery);
        return ResultRespone.success(esPdaDeviceUploadService.getAll(esPdaDeviceUpload,page,
                esSaveInfoUtil.DyMonth(esPdaDeviceUploadService.getIndex()),elasticsearchTemplate,queryBuilders));
    }


    private void saveToRedis(EsPdaDeviceUpload esPdaDeviceUpload){
        log.info("异步方式设备信息存储到redis：{}",esPdaDeviceUpload);
        listOperations.rightPush(DyrqCommonRedis.REDIS_PRE_PDA_DEVICE+esPdaDeviceUpload.getUserid(),esPdaDeviceUpload);
        redisTemplate.expireAt(DyrqCommonRedis.REDIS_PRE_PDA_DEVICE+esPdaDeviceUpload.getUserid(), Date8Util.getEndTime());

    }

}
