package com.tofly.dyrqGps.controller;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.Date8Util;
import com.tofly.common.core.util.StringUtil;
import com.tofly.dyrqGps.entity.EsGpsUpload;
import com.tofly.dyrqGps.entity.EsPdaDeviceUpload;
import com.tofly.dyrqGps.entity.RtUser;
import com.tofly.dyrqGps.entity.RtUserD;
import com.tofly.dyrqGps.repository.EsGpsUploadService;
import com.tofly.dyrqGps.repository.EsPdaDeviceUploadService;
import com.tofly.dyrqGps.service.UserRtInfoService;
import com.tofly.dyrqGps.util.DyrqCommonRedis;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.formula.functions.T;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.MatchQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.search.sort.FieldSortBuilder;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.elasticsearch.core.ElasticsearchTemplate;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.data.redis.core.ListOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ValueOperations;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author tuonbed
 * @date 2020/5/13  17:37
 */
@RestController
@AllArgsConstructor
@RequestMapping("/rtinfo")
@Api(tags = "实时信息获取")
@Slf4j
public class UserRTInfoController {

    private ListOperations<String, Object> listOperations;
    private RedisTemplate redisTemplate;
    private ElasticsearchTemplate elasticsearchTemplate;

    private final ValueOperations<String, String> valueOperations;

    private final UserRtInfoService userRtInfoService;

    private final EsGpsUploadService esGpsUploadService;

    private final EsPdaDeviceUploadService esPdaDeviceUploadService;

    /**
     * 查询用户设备绑定情况
     */
    @ApiOperation(value = "用户实时信息数据")
    @GetMapping("/userRTInfoPage")
    public ResultRespone userPdaPage(Page page, @ModelAttribute RtUser user) {
        RtUserD rtUser = new RtUserD();
        rtUser.setRealName(user.getRealName());
        rtUser.setDepartmentId(user.getDepartmentId());
        rtUser.setDate(Date8Util.getCurrentDay(Date8Util.DATE));

//        List<Object> user1 = listOperations.range("user", 0, -1);
        List<Map> usersRtInfo = userRtInfoService.getUsersRtInfo(page, rtUser);
        usersRtInfo.parallelStream().forEach(map -> {
            if (map.get("ontime") != null && map.get("offtime") != null) {
                SimpleDateFormat format = new SimpleDateFormat("yyyy-M-dd HH:mm:ss");
                try {
                    Date onTimeDate = format.parse(map.get("ontime").toString());
                    Date offTimeDate = format.parse(map.get("offtime").toString());
                    if (onTimeDate.getTime() > offTimeDate.getTime()) {
                        map.put("offtime", null);
                    }
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            map.put("x_y", JSONObject.parseObject(valueOperations.get(DyrqCommonRedis.REDIS_PRE_GPS + map.get("id"))));
            if (user.getIsLastest() != null && user.getIsLastest()) {
                map.put("equip_info", listOperations.range(DyrqCommonRedis.REDIS_PRE_PDA_DEVICE + map.get("id"), -1, -1));
            } else {
                map.put("equip_info", listOperations.range(DyrqCommonRedis.REDIS_PRE_PDA_DEVICE + map.get("id"), 0, -1));
            }
        });
        page.setRecords(usersRtInfo);
        //JSONObject user1 = JSONObject.parseObject(valueOperations.get("user")) ;
        return ResultRespone.success(page);
    }

    /**
     * 查询用户设备绑定情况
     */
    //@ApiOperation(value = "用户历史信息数据")
    @GetMapping("/userHtInfoPage")
    public ResultRespone userPdaPageD(Page page, @ModelAttribute RtUserD user) {
        if (StringUtil.isEmpty(user.getDate())) {
            return ResultRespone.failed("查询日期不能为空");
        }
        Long start = System.currentTimeMillis();

//        List<Object> user1 = listOperations.range("user", 0, -1);
        List<Map> usersRtInfo = userRtInfoService.getUsersRtInfo(page, user);
        usersRtInfo.parallelStream().forEach(map -> {
            Long userid = ((BigDecimal) map.get("id")).longValue();
            map.put("lat_lng", findesListByDate(user.getDate(), userid, esGpsUploadService.getIndex(), esGpsUploadService.getEntityClass()));
            map.put("equip_info", findesListByDate(user.getDate(), userid, esPdaDeviceUploadService.getIndex(), esPdaDeviceUploadService.getEntityClass()));
        });
        page.setRecords(usersRtInfo);
        //JSONObject user1 = JSONObject.parseObject(valueOperations.get("user")) ;
        log.info("方法1共耗时：{}", (System.currentTimeMillis() - start));
        return ResultRespone.success(page);
    }


    /**
     * 查询用户设备绑定情况
     */
    @ApiOperation(value = "用户历史信息数据")
    @GetMapping("/userHtInfoPage2")
    public ResultRespone userPdaPageD2(Page page, @ModelAttribute RtUserD user) {
        if (StringUtil.isEmpty(user.getDate())) {
            return ResultRespone.failed("查询日期不能为空");
        }
        if (user.getIsDesc() == null) {
            user.setIsDesc(false);
        }
        if (user.getIsLastest() == null) {
            user.setIsLastest(false);
        }
        Long start = System.currentTimeMillis();
//        List<Object> user1 = listOperations.range("user", 0, -1);
        List<Map> usersRtInfo = userRtInfoService.getUsersRtInfo(page, user);
        List<Long> userIdList = usersRtInfo.stream()
                .map(map -> ((BigDecimal) map.get("id")).longValue()).collect(Collectors.toList());
        Map[] maps = findesListByuser(user.getDate(), userIdList, user.getIsLastest(), user.getIsDesc());

        usersRtInfo.parallelStream().forEach(map -> {
            if (map.get("ontime") != null && map.get("offtime") != null) {
                SimpleDateFormat format = new SimpleDateFormat("yyyy-M-dd HH:mm:ss");
                try {
                    Date onTimeDate = format.parse(map.get("ontime").toString());
                    Date offTimeDate = format.parse(map.get("offtime").toString());
                    if (onTimeDate.getTime() > offTimeDate.getTime()) {
                        map.put("offtime", null);
                    }
                } catch (ParseException e) {
                    e.printStackTrace();
                }
            }
            Long userid = ((BigDecimal) map.get("id")).longValue();
            map.put("lat_lng", maps[0].get(userid));
            map.put("equip_info", maps[1].get(userid));
        });
        page.setRecords(usersRtInfo);
        log.info("方法2共耗时：{}", (System.currentTimeMillis() - start));
        return ResultRespone.success(page);
    }

    private List findesListByDate(String date, Long userid, String preindex, Class c) {

        Pageable pageRequest;
        pageRequest = PageRequest.of(0, 10000);
//        Sort sort=new Sort(Sort.Direction.ASC,"uploadtime");
        FieldSortBuilder sort = SortBuilders.fieldSort("uploadtime").order(SortOrder.ASC);
        String[] split = date.split("-");
        String index = preindex + "_" + split[0] + split[1];
        NativeSearchQueryBuilder nativeSearchQueryBuilder = new NativeSearchQueryBuilder();
        BoolQueryBuilder qb = QueryBuilders.boolQuery();
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("uploadtime");
        rangeQueryBuilder.from(Date8Util.parse(date + " 00:00:00", Date8Util.DATE_TIME).getTime());
        rangeQueryBuilder.to(Date8Util.parse(date + " 23:59:59", Date8Util.DATE_TIME).getTime());
        qb.filter(rangeQueryBuilder);
        qb.filter(QueryBuilders.matchQuery("userid", userid));
        nativeSearchQueryBuilder.withIndices(index).withQuery(qb).withSort(sort).withPageable(pageRequest);
        List<T> list = elasticsearchTemplate.queryForList(nativeSearchQueryBuilder.build(), c);
        return list;
    }

    private Map[] findesListByuser(String date, List<Long> userids, Boolean isLastest, Boolean isDesc) {
        int size = isLastest ? 1 : 10000;
        Pageable pageRequest = PageRequest.of(0, size);
        FieldSortBuilder sort = SortBuilders.fieldSort("uploadtime").order(SortOrder.ASC);
        if (isDesc || isLastest) {
            sort = SortBuilders.fieldSort("uploadtime").order(SortOrder.DESC);
        }
        Map[] dataMap = new Map[2];
        dataMap[0] = new HashMap();
        dataMap[1] = new HashMap();
        for (Long userid : userids) {
            BoolQueryBuilder qb = QueryBuilders.boolQuery();
            RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("uploadtime");
            rangeQueryBuilder.from(Date8Util.parse(date + " 00:00:00", Date8Util.DATE_TIME).getTime());
            rangeQueryBuilder.to(Date8Util.parse(date + " 23:59:59", Date8Util.DATE_TIME).getTime());
            qb.filter(rangeQueryBuilder);
//        qb.filter(QueryBuilders.termsQuery("userid", userids));
            NativeSearchQueryBuilder nativeSearchQueryBuilder1 = new NativeSearchQueryBuilder();
            NativeSearchQueryBuilder nativeSearchQueryBuilder2 = new NativeSearchQueryBuilder();
            String[] split = date.split("-");
            String index1 = esGpsUploadService.getIndex() + "_" + split[0] + split[1];
            String index2 = esPdaDeviceUploadService.getIndex() + "_" + split[0] + split[1];

            MatchQueryBuilder qbUserId = QueryBuilders.matchQuery("userid", userid);
            qb.must(qbUserId);

            nativeSearchQueryBuilder1.withIndices(index1).withQuery(qb).withSort(sort).withPageable(pageRequest);
            nativeSearchQueryBuilder2.withIndices(index2).withQuery(qb).withSort(sort).withPageable(pageRequest);
            List<EsGpsUpload> list1 = elasticsearchTemplate.queryForList(nativeSearchQueryBuilder1.build(), esGpsUploadService.getEntityClass());
            List<EsPdaDeviceUpload> list2 = elasticsearchTemplate.queryForList(nativeSearchQueryBuilder2.build(), esPdaDeviceUploadService.getEntityClass());
            dataMap[0].put(userid,list1);
            dataMap[1].put(userid,list2);
        }
        return dataMap;
    }


    //    @ApiOperation(value = "保存")
    @GetMapping("/saveuserRTInfoPage")
    public ResultRespone userPdaPage(String value) {
        Map m = new HashMap();
        m.put("date", Date8Util.getCurrentDate());
        m.put("value", value);
        valueOperations.set("user", JSON.toJSONString(m));

//        User user1=new User();
//        user1.setRealName("zzzzzz");
//        User user2=new User();
//        user2.setRealName("ttttttt");
//        List<User> list=new ArrayList<>();
//        list.add(user1);
//        list.add(user2);
//        listOperations.rightPush("user",list);

//        listOperations.rightPush();  //存储
        return ResultRespone.success(true);
    }


    //    @ApiOperation(value = "清除")
    @GetMapping("/clear")
    public ResultRespone clear() {

        redisTemplate.delete("user");

//        listOperations.rightPush();  //存储
        return ResultRespone.success(true);
    }

    //    @ApiOperation(value = "清除index")
    @GetMapping("/clearIndex")
    public ResultRespone clearIndex(String index) {
        return ResultRespone.success(elasticsearchTemplate.deleteIndex(index));
    }

    //    @ApiOperation(value = "查询")
    @GetMapping("/query")
    public ResultRespone query() {

//        List<Object> user1 = listOperations.range("user", 0, -1);
        JSONObject user1 = JSONObject.parseObject(valueOperations.get("user"));
        return ResultRespone.success(user1);
    }
}
