package com.tofly.dyrqGps.service.impl;

import com.baomidou.mybatisplus.core.metadata.OrderItem;
import com.tofly.common.core.util.Date8Util;
import com.tofly.dyrqGps.entity.EsCarGpsUpload;
import com.tofly.dyrqGps.mapper.EsSaveInfoMapper;
import com.tofly.dyrqGps.repository.EsCarGpsUploadService;
import com.tofly.dyrqGps.service.CarGpsService;
import com.tofly.dyrqGps.util.EsSaveInfoUtil;
import lombok.AllArgsConstructor;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.MatchQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.search.sort.SortBuilder;
import org.elasticsearch.search.sort.SortBuilders;
import org.elasticsearch.search.sort.SortOrder;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.elasticsearch.core.ElasticsearchTemplate;
import org.springframework.data.elasticsearch.core.aggregation.AggregatedPage;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.data.elasticsearch.core.query.SearchQuery;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;

/**
 * @author tuonbed
 * @date 2020/9/11  10:51
 */
@Service
@AllArgsConstructor
public class CarGpsServiceImpl implements CarGpsService {

    private final EsSaveInfoUtil esSaveInfoUtil;
    private final EsCarGpsUploadService esCarGpsUploadService;
    private final ElasticsearchTemplate elasticsearchTemplate;
    private final EsSaveInfoMapper esSaveInfoMapper;


    @Override
    public Boolean cargpsSave(EsCarGpsUpload esCarGpsUpload) {
        esSaveInfoUtil.checkorinsertEsInfo(esCarGpsUploadService.getIndex());
        if (esCarGpsUpload.getUploadtime() == null) {
            esCarGpsUpload.setUploadtime(Date8Util.getCurrentDate());
        }
        if (esCarGpsUpload.getGpstime() == null) {
            esCarGpsUpload.setGpstime(Date8Util.getCurrentDate());
        }
        Long carId = esSaveInfoUtil.getCarId(esCarGpsUpload.getGpsId());
        esCarGpsUpload.setCarid(carId);
//        //查询最新记录，比较时间，判断速度是否大于设置的最大容差
//        EsCarGpsUpload lastest = queryLastestOne(elasticsearchTemplate, carId);
//        if (lastest != null){
//            double timeSpan = (esCarGpsUpload.getGpstime().getTime() - lastest.getGpstime().getTime())/1000;
//            //间隔时间小于5分钟的进行判断，超出5分钟直接录入
//            if (timeSpan < 300.0){
//                BigDecimal lastLng = lastest.getLocationLongitude();
//                BigDecimal lastLat = lastest.getLocationLatitude();
//                double maxSpeed = esSaveInfoMapper.getCarMaxSpeed();
//                double distance = getDistance(lastLng.doubleValue(),lastLat.doubleValue(),esCarGpsUpload.getLocationLongitude().doubleValue(), esCarGpsUpload.getLocationLatitude().doubleValue());
//                double speed = distance/timeSpan;
//                if (speed < maxSpeed){
//                    esCarGpsUploadService.save(esCarGpsUpload);
//                    return true;
//                }
//                else{
//                    return false;
//                }
//            }else{
//                esCarGpsUploadService.save(esCarGpsUpload);
//                return true;
//            }
//        }
//        else{
//            esCarGpsUploadService.save(esCarGpsUpload);
//            return true;
//        }

        esCarGpsUploadService.save(esCarGpsUpload);
        return true;


    }


    public EsCarGpsUpload queryLastestOne(ElasticsearchTemplate elasticsearchTemplate, Long carId) {
        BoolQueryBuilder qb = QueryBuilders.boolQuery();
        MatchQueryBuilder qbIsvalid = QueryBuilders.matchQuery("carid", carId);
        qb.must(qbIsvalid);
//        OrderItem orderItem = new OrderItem();
//        orderItem.setColumn("gpstime");
//        orderItem.setAsc(false);
//        page.addOrder(orderItem);
        PageRequest pageRequest = PageRequest.of(0, 1);
        SortBuilder sortBuilder = SortBuilders.fieldSort("gpstime").order(SortOrder.fromString("DESC"));
        SearchQuery searchQuery = new NativeSearchQueryBuilder()
                .withQuery(qb)
                .withSort(sortBuilder)
                .withPageable(pageRequest)
                .build();

        try {
            AggregatedPage<EsCarGpsUpload> list = elasticsearchTemplate.queryForPage(searchQuery, esCarGpsUploadService.getEntityClass());
            return list.getContent().get(0);
        } catch (Exception e) {
            return null;
        }
    }

    /**
     * @param lng1 点1经度
     * @param lat1 点1纬度
     * @param lng2 点2经度
     * @param lat2 点2纬度
     * @return 两点距离，单位m
     */
    public static double getDistance(double lng1, double lat1, double lng2, double lat2) {
        Double EARTH_RADIUS = 6378.137; // 地球长半径  地球半径系数6370.996
        Double radLat1 = lat1 * Math.PI / 180.0;
        Double radLat2 = lat2 * Math.PI / 180.0;

        Double radLng1 = lng1 * Math.PI / 180.0;
        Double radLng2 = lng2 * Math.PI / 180.0;

        Double a = radLat1 - radLat2;
        Double b = radLng1 - radLng2;

        double s = 2 * Math.asin(Math.sqrt(Math.pow(Math.sin(a / 2), 2) + Math.cos(radLat1) * Math.cos(radLat2) * Math.pow(Math.sin(b / 2), 2)));
        s = s * EARTH_RADIUS * 1000;
        //s = Math.round(s * 10000) / 10000;
        return s;
    }

}
