/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrqSchedule.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrqSchedule.entity.PdaUser;
import com.tofly.dyrqSchedule.service.PdaUserService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 巡检系统用户
 *
 * @author admin
 * @date Tue Sep 01 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/pdauser")
@Api(tags="巡检系统用户接口")
public class PdaUserController {

    private final  PdaUserService pdaUserService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param pdaUser 巡检系统用户
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getPdaUserPage(Page page, @ModelAttribute PdaUser pdaUser) {
      return ResultRespone.success(pdaUserService.page(page, Wrappers.query(pdaUser)));
    }


    /**
     * 通过id查询巡检系统用户
     * @param userid id
     * @return ResultRespone
     */
    @GetMapping("/{userid}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userid", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("userid" ) Long userid) {
      return ResultRespone.success(pdaUserService.getById(userid));
    }

    /**
     * 新增巡检系统用户
     * @param pdaUser 巡检系统用户
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增巡检系统用户" )
    @ApiOperation(value = "新增巡检系统用户")
    @PostMapping
    public ResultRespone save(@RequestBody PdaUser pdaUser) {
      return ResultRespone.success(pdaUserService.save(pdaUser));
    }

    /**
     * 修改巡检系统用户
     * @param pdaUser 巡检系统用户
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改巡检系统用户" )
    @ApiOperation(value = "修改巡检系统用户")
    @PutMapping
    public ResultRespone updateById(@RequestBody PdaUser pdaUser) {
      return ResultRespone.success(pdaUserService.updateById(pdaUser));
    }

    /**
     * 通过id删除巡检系统用户
     * @param userid id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除巡检系统用户" )
    @ApiOperation(value = "通过ID删除巡检系统用户")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "userid", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{userid}" )
    public ResultRespone removeById(@PathVariable Long userid) {
      return ResultRespone.success(pdaUserService.removeById(userid));
    }

    /**
     * 通过id批量删除巡检系统用户
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除巡检系统用户" )
    @ApiOperation(value = "通过ID批量删除巡检系统用户")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(pdaUserService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
