/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrqSchedule.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrqSchedule.entity.User;
import com.tofly.dyrqSchedule.service.UserService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 用户表
 *
 * @author pra
 * @date Mon Nov 16 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/user")
@Api(tags="用户表接口")
public class UserController {

    private final  UserService userService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param user 用户表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getUserPage(Page page, @ModelAttribute User user) {
      return ResultRespone.success(userService.page(page, Wrappers.query(user)));
    }


    /**
     * 通过id查询用户表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(userService.getById(id));
    }

    /**
     * 新增用户表
     * @param user 用户表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增用户表" )
    @ApiOperation(value = "新增用户表")
    @PostMapping
    public ResultRespone save(@RequestBody User user) {
      return ResultRespone.success(userService.save(user));
    }

    /**
     * 修改用户表
     * @param user 用户表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改用户表" )
    @ApiOperation(value = "修改用户表")
    @PutMapping
    public ResultRespone updateById(@RequestBody User user) {
      return ResultRespone.success(userService.updateById(user));
    }

    /**
     * 通过id删除用户表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除用户表" )
    @ApiOperation(value = "通过ID删除用户表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(userService.removeById(id));
    }

    /**
     * 通过id批量删除用户表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除用户表" )
    @ApiOperation(value = "通过ID批量删除用户表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(userService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
