/*
 * Author: wangjian
 * 生成entity
 */
package com.tofly.dyrqSchedule.entity;

import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.math.BigDecimal;
import java.util.Date;

import com.fasterxml.jackson.annotation.JsonFormat;
import org.springframework.format.annotation.DateTimeFormat;

import javax.persistence.Entity;
import javax.persistence.Id;


/**
 * 巡检计划表
 *
 * @author pra
 * @date Fri Aug 21 00:00:00 CST 2020
 */
@Data
@EqualsAndHashCode(callSuper = true)
@TableName(value = "TF_PLAN", resultMap = "planMap")
@KeySequence("TF_PLAN")
@ApiModel
@Entity(name = "TF_PLAN")
public class Plan extends Model<Plan> {
    private static final long serialVersionUID = 1L;

    /**
     * $column.comments
     */
    @TableLogic
    @ApiModelProperty(value = "$column.comments")
    private String delFlag;
    /**
     * 计划关联的设备和管线ObjectIds
     */
    @ApiModelProperty(value = "计划关联的设备和管线ObjectIds")
    private String objectIds;
    /**
     * 巡检计划编码
     */
    @Id
    @TableId(value = "ID", type = IdType.INPUT)
    @ApiModelProperty(value = "巡检计划编码")
    private Long id;
    /**
     * 巡检片区编码
     */
    @ApiModelProperty(value = "巡检片区编码")
    private Long regionId;
    @ApiModelProperty(hidden = true)
    private String regionName;
    /**
     * 被安排用户编码
     */
    @ApiModelProperty(value = "被安排用户编码")
    @TableField(value = "USER_ID", fill = FieldFill.INSERT)
    private Long userId;
    @ApiModelProperty(hidden = true)
    private String userIdName;
    /**
     * 巡检周期编码
     */
    @ApiModelProperty(value = "巡检周期编码")
    private Long periodId;
    @ApiModelProperty(hidden = true)
    private String preiodName;
    /**
     * 设备类型唯一编码
     */
    @ApiModelProperty(value = "设备类型唯一编码")
    private Long deviceTypeId;
    @ApiModelProperty(hidden = true)
    private String planPointTypeName;
    /**
     * 是否发现隐患
     */
    @ApiModelProperty(value = "是否发现隐患")
    private String isFindTrouble;
    /**
     * 巡检状态 0：待审核，1：未开始，2：正在执行 3：已完成 4：超时未完成  5：申请转移  6：未提交 7：计划已转移
     */
    @ApiModelProperty(value = "巡检状态 0：待审核，1：未开始，2：正在执行 3：已完成 4：超时未完成  5：申请转移  6：未提交 7：计划已转移")
    private String planState;
    @ApiModelProperty(hidden = true)
    private String chilePlanStateName;
    /**
     * 巡检计划开始时间
     */
    @ApiModelProperty(value = "巡检计划开始时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date planBegindate;
    /**
     * 巡检计划结束时间
     */
    @ApiModelProperty(value = "巡检计划结束时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date planEnddate;
    /**
     * 自定义间隔天数
     */
    @ApiModelProperty(value = "自定义间隔天数")
    private BigDecimal intervalDays;
    /**
     * 描述
     */
    @ApiModelProperty(value = "描述")
    private String description;
    /**
     * 创建用户
     */
    @ApiModelProperty(value = "创建用户")
    @TableField(value = "CREATE_USER", fill = FieldFill.INSERT)
    private Long createUser;
    @ApiModelProperty(hidden = true)
    private String createUserName;
    /**
     * 创建日期
     */
    @ApiModelProperty(value = "创建日期")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    @TableField(value = "CREATE_TIME", fill = FieldFill.INSERT)
    private Date createTime;
    /**
     * 所属公司为一编码
     */
    @ApiModelProperty(value = "所属公司为一编码")
    private Long companyid;
    /**
     * 最后一次修改时间
     */
    @ApiModelProperty(value = "最后一次修改时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date lastTime;
    /**
     * 0:待审核 1:通过 2:未通过
     */
    @ApiModelProperty(value = "0:待审核 1:通过 2:未通过")
    private String isvalid;
    @ApiModelProperty(hidden = true)
    private String isvalidName;
    /**
     * 审核备注
     */
    @ApiModelProperty(value = "审核备注")
    private String checkNote;
    /**
     * 审核时间
     */
    @ApiModelProperty(value = "审核时间")
    @JsonFormat(pattern = "yyyy-MM-dd HH:mm:ss", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
    private Date checkTime;
    /**
     * 审核人
     */
    @ApiModelProperty(value = "审核人")
    private Long checkUser;
    /**
     * 计划详情
     */
    @ApiModelProperty(value = "计划详情")
    private String pointInfo;
    /**
     * 用户是否阅读 0：未读 1：已读
     */
    @ApiModelProperty(value = "用户是否阅读 0：未读 1：已读")
    private String isread;
    @ApiModelProperty(hidden = true)
    private String isreadName;
    /**
     * 0：人巡 1：车巡
     */
    @ApiModelProperty(value = "0：人巡 1：车巡 ")
    private String planMode;
    @ApiModelProperty(hidden = true)
    private String modeName;

}
