package com.tofly.dyrqSchedule.handler;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.tofly.common.core.util.Date8Util;
import com.tofly.dyrqSchedule.entity.*;
import com.tofly.dyrqSchedule.mapper.CustomCalculateMapper;
import com.tofly.dyrqSchedule.service.*;
import com.tofly.dyrqSchedule.util.EsQuery;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.AllArgsConstructor;
import org.springframework.data.elasticsearch.core.ElasticsearchTemplate;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;

/**
 * @author pra
 * @date 2020/9/15 14:06
 * @description 统计巡检里程、耗时、车巡里程耗时计算，巡检点巡检线计划数与巡检数，计划巡检设备，
 * 已巡检设备数，计划管线长度，实际长度，发现隐患数、发现工地数、抢维修上报数、签到时间段
 */
@Component
@AllArgsConstructor
public class CalculateHandler {
    private final SysSettingService sysSettingService;
    private final PdaUserDeviceService pdaUserDeviceService;
    private final TroubleService troubleService;
    private final PdaSignService pdaSignService;
    private final ConstructionSiteService constructionSiteService;
    private final GpsDayCalculateService gpsDayCalculateService;
    private final ElasticsearchTemplate elasticsearchTemplate;
    private final CustomCalculateMapper customCalculateMapper;
    private final PlanPointTempService planPointTempService;
    private final RushRepairService rushRepairService;
    private final UserService userService;

    @XxlJob("CalculateHandler")
    public ReturnT<String> CalculateHandler(String param) {
        //查询人巡最小到位容差
//        BigDecimal Tolorance = sysSettingService.getById(1).getMinDistanceInPlace();
        Date startDate = Date8Util.getStartTime();
        Date endDate = Date8Util.getEndTime();

        if(param != null){
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            try {
                startDate = sdf.parse( param + "0:00:00");
                endDate = sdf.parse(param +"23:00:00");
            } catch (ParseException e) {
                e.printStackTrace();
            }
        }

        //查询所有巡检用户（绑定设备的用户）
//        List<PdaUserDevice> usersList = pdaUserDeviceService.list(Wrappers.<PdaUserDevice>lambdaQuery()
//                .select(PdaUserDevice::getUserId));
        List<User> usersList = userService.list(Wrappers.<com.tofly.dyrqSchedule.entity.User>lambdaQuery()
                .select(User::getId));
        for (User user : usersList) {
            Long userId = user.getId();
            GpsDayCalculate gpsDayCalculate = new GpsDayCalculate();
            Long deptId = customCalculateMapper.getUserDept(userId);
            gpsDayCalculate.setCompanyid(deptId);

            //总里程耗时计算
            EsQueryParam esQueryParam = new EsQueryParam();
            esQueryParam.setStartDate(startDate);
            esQueryParam.setEndDate(endDate);
            esQueryParam.setUserId(userId);
            esQueryParam.setGpsType(Long.valueOf(0));
            esQueryParam.setIsvalid(new BigDecimal(1));
            List<Double> getTotal = EsQuery.getGpsTotal(elasticsearchTemplate, esQueryParam);
            gpsDayCalculate.setPlanTotalDistance(getTotal.get(0));
            gpsDayCalculate.setPlanTotalTime(getTotal.get(1));
            gpsDayCalculate.setPlanTotalAverage(gpsDayCalculate.getPlanTotalTime() == 0 ? 0 : getTotal.get(0) / getTotal.get(1));

            //车巡里程耗时计算
            esQueryParam.setGpsType(Long.valueOf(1));
            List<Double> getCarTotal = EsQuery.getGpsTotal(elasticsearchTemplate, esQueryParam);
            gpsDayCalculate.setCarTotalDistance(getCarTotal.get(0));
            gpsDayCalculate.setCarTotalTime(getCarTotal.get(1));
            gpsDayCalculate.setCarTotalAverage(gpsDayCalculate.getCarTotalTime() == 0 ? 0 : getCarTotal.get(0) / getCarTotal.get(1));

            //计划巡检点数和实际巡检点数
//            List<PlanChild> pointNumlist = planChildService.list(Wrappers.<PlanChild>lambdaQuery().eq(PlanChild::getUserId, userId));
//            BigDecimal pointTotal = BigDecimal.ZERO;
//            BigDecimal pointTotalIsover = BigDecimal.ZERO;
//            for(PlanChild p:pointNumlist) {
//                if (p.getTotal1() != null) {
//                    pointTotal = pointTotal.add(p.getTotal1());
//                }
//                if (p.getIsovertototal1() != null) {
//                    pointTotalIsover = pointTotalIsover.add(p.getIsovertototal1());
//                }
//            };
//            gpsDayCalculate.setPointinplan(pointTotal);
//            gpsDayCalculate.setPointpatroled(pointTotalIsover);

            String pointCountPlan = customCalculateMapper.getPointCountPlan(userId, startDate, endDate);
            String pointCountRun = customCalculateMapper.getPointCountRun(userId, startDate, endDate);
            gpsDayCalculate.setPointinplan(pointCountPlan == null ? BigDecimal.ZERO :new BigDecimal(pointCountPlan));
            gpsDayCalculate.setPointpatroled(pointCountRun == null ? BigDecimal.ZERO :new BigDecimal(pointCountRun));

            //计划管线长度，实际长度
            String pipeLengthPlan = customCalculateMapper.getPipeLengthPlan(userId, startDate, endDate);
            String pipeLengthRun = customCalculateMapper.getPipeLengthRun(userId, startDate, endDate);
            gpsDayCalculate.setPipeinplan(pipeLengthPlan == null ? BigDecimal.ZERO : new BigDecimal(pipeLengthPlan));
            gpsDayCalculate.setPipepatroled(pipeLengthRun == null ? BigDecimal.ZERO : new BigDecimal(pipeLengthRun));

            //计划巡检线长度，实际长度
            String pathLengthPlan = customCalculateMapper.getPathLengthPlan(userId, startDate, endDate);
            String pathLengthRun = customCalculateMapper.getPathLengthRun(userId, startDate, endDate);
            gpsDayCalculate.setPathinplan(pathLengthPlan == null ? BigDecimal.ZERO : new BigDecimal(pathLengthPlan));
            gpsDayCalculate.setPathpatroled(pathLengthRun == null ? BigDecimal.ZERO : new BigDecimal(pathLengthRun));

            //计划巡检设备，已巡检设备数
            int devicesPlan = planPointTempService.count(Wrappers.<PlanPointTemp>lambdaQuery().eq(PlanPointTemp::getUserId, userId)
                    .apply("child_plan_id in( select c.id from TF_PLAN_CHILD c, TF_SYS_DEVICE_TYPE b where" +
                            " c.device_type_id = b.point_type_id and b.point_type_id = device_type_id and b.check_content is not null " +
                            " and not(({0} > c.CHILD_END_DATE) OR ({1} < c.CHILD_BEGIN_DATE)))", startDate, endDate));
            int devicesRun = planPointTempService.count(Wrappers.<PlanPointTemp>lambdaQuery()
                    .eq(PlanPointTemp::getUserId, userId)
                    .eq(PlanPointTemp::getCheckState, '1')
                    .ge(PlanPointTemp::getCheckDate,startDate)
                    .le(PlanPointTemp::getCheckDate,endDate));

            gpsDayCalculate.setEquipmentsinplan(new BigDecimal(devicesPlan));
            gpsDayCalculate.setEquipmentspatroled(new BigDecimal(devicesRun));

            //有效里程和有效时间(无片区绑定)
//            List<EsGpsUpload> gpsList = EsQuery.queryList(elasticsearchTemplate, esQueryParam);

            //发现隐患数
            int troubleNums = troubleService.count(Wrappers.<Trouble>lambdaQuery()
                    .eq(Trouble::getFindUserId, userId)
                    .ge(Trouble::getFindTime, startDate)
                    .le(Trouble::getFindTime, endDate));
            gpsDayCalculate.setTroublefinded(new BigDecimal(troubleNums));

            // 抢维修上报数
            int rushRepairNums = rushRepairService.count(Wrappers.<RushRepair>lambdaQuery()
                    .eq(RushRepair::getCreatUser, userId)
                    .ge(RushRepair::getCreatTime, startDate)
                    .le(RushRepair::getCreatTime, endDate));
            gpsDayCalculate.setRushRepairNum(new BigDecimal(rushRepairNums));

            //工地发现数,工地监护数
            int siteNums = constructionSiteService.count(Wrappers.<ConstructionSite>lambdaQuery()
                    .eq(ConstructionSite::getCreatUser, userId)
                    .ge(ConstructionSite::getCreatTime, startDate)
                    .le(ConstructionSite::getCreatTime, endDate));
            gpsDayCalculate.setBuildingsitefinded(new BigDecimal(siteNums));

            int siteWatchNums = constructionSiteService.count(Wrappers.<ConstructionSite>lambdaQuery()
                    .eq(ConstructionSite::getConstructionPerson, userId)
                    .ge(ConstructionSite::getCreatTime, startDate)
                    .le(ConstructionSite::getCreatTime, endDate));
            gpsDayCalculate.setBuildingsitemonitor(new BigDecimal(siteWatchNums));

            //签到时间段
            List<PdaSign> signList = pdaSignService.list(Wrappers.<PdaSign>lambdaQuery()
                    .select(PdaSign::getSignTime, PdaSign::getSignType)
                    .eq(PdaSign::getUserid, userId)
                    .ge(PdaSign::getSignTime, startDate)
                    .le(PdaSign::getSignTime, endDate)
                    .orderByAsc(PdaSign::getSignTime));

            int k = 0;
            StringBuffer sb = new StringBuffer();
            for (PdaSign sign : signList) {
                if (k % 2 == 0) {
                    sb.append(Date8Util.format(sign.getSignTime(), "yyyy-MM-dd HH:mm:ss") + "~");
                    if(signList.size() == (k+1)){
                        sb.append(Date8Util.format(Date8Util.getCurrentDate(), "yyyy-MM-dd HH:mm:ss"));
                    }
                } else {
                    sb.append(Date8Util.format(sign.getSignTime(), "yyyy-MM-dd HH:mm:ss"));
                    if(signList.size() != (k+1)){
                        sb.append(", ");
                    }
                }
                k++;
            }
            gpsDayCalculate.setSignRange(sb.toString());

            //gpsDayCalculate更新，插入
            //判断当前里程表是否存在数据，若存在，则更新
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(endDate);
            calendar.add(Calendar.DATE, -1);
            Date lastStartDate = calendar.getTime();
//            List<EsGpsUpload> li = EsQuery.queryList(elasticsearchTemplate,esQueryParam,esGpsUploadRepository.getEntityClass());
            List<GpsDayCalculate> gpsDaylist = gpsDayCalculateService.list(Wrappers.<GpsDayCalculate>lambdaQuery()
                    .eq(GpsDayCalculate::getUserid, userId)
                    .ge(GpsDayCalculate::getPlanDate, lastStartDate)
                    .le(GpsDayCalculate::getPlanDate, endDate));
            if (gpsDaylist == null || gpsDaylist.size() == 0) {
                gpsDayCalculate.setUserid(userId);
                gpsDayCalculate.setPlanDate(startDate);
                gpsDayCalculateService.save(gpsDayCalculate);
            } else {
                gpsDayCalculate.setId(gpsDaylist.get(0).getId());
                gpsDayCalculateService.updateById(gpsDayCalculate);
            }

            //工地监护日志监控

        }
        return ReturnT.SUCCESS;
    }
}
