package com.tofly.dyrqSchedule.handler;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tofly.dyrqSchedule.entity.*;
import com.tofly.dyrqSchedule.repository.*;
import com.tofly.dyrqSchedule.util.DateTimeUtil;
import com.tofly.dyrqSchedule.util.GpsHelperUtil;
import lombok.Data;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.*;

/**
 * @program: tofly-dyrq
 * @description: 管线计划重新统计服务
 * @author: Sumer
 * @create: 2020-08-26 09:26
 */
@Component
public class ColPipeHandler {

    @Autowired
    private PlanChildRepository planChildRepository;

    @Autowired
    private PlanRepository planRepository;

    @Autowired
    private PlanPointRepository planPointRepository;

    @Autowired
    private PlanPathTempRepository planPathTempRepository;

    @Autowired
    private PlanPointTempRepository planPointTempRepository;

    public void colPipe(String[] args) {

        List<PlanChild> planChildList = new ArrayList<>();
        // 判断传入请求参数长度
        if (args.length > 0 ) {
            // 获取 child_plan_id 的值
            String[] child_plan_id = args[0].split(",");
            List<String> ids = Arrays.asList(child_plan_id);

            planChildList = planChildRepository.findByIdIn(ids);

        }

        if (null != planChildList && planChildList.size() > 0) {
            // 区分管线图层名
            //中压管线
            String[] pipeArrmp = new String[]{"10", "14"};
            //次高压管线
            String[] pipeArrip = new String[]{"9", "13"};
            //低压管线
            String[] pipeArrlp = new String[]{"11"};
            //高压管线
            String[] pipeArrhp = new String[]{"12", "18"};

            String nickName = "";
            for (PlanChild p : planChildList) {
                String beginDate = DateTimeUtil.getLocalDateTime(DateTimeUtil.fromLong2LocalDateTime(p.getChildBeginDate().getTime()), DateTimeUtil.DAY_SLASH_FORMAT);
                String endDate = DateTimeUtil.getLocalDateTime(DateTimeUtil.fromLong2LocalDateTime(p.getChildEndDate().getTime()), DateTimeUtil.DAY_SLASH_FORMAT);
                String userId = p.getUserId().toString();
                String childPlanId = p.getId().toString();
                long childPlanIdl = Long.parseLong(childPlanId);
                String pointInfo = "";
                List<Plan> planInfoList = planRepository.findAllById(Collections.singleton(childPlanIdl));
                // 一对多关系 只会查出一个 TF_PLAN
                if (planInfoList.size() > 0) {
                    pointInfo = planInfoList.get(0).getPointInfo();
                }
                List<GeoClassPath> geoClassPaths = JSONArray.parseArray(pointInfo, GeoClassPath.class);
                double total = 0;
                for (GeoClassPath path : geoClassPaths) {
                    String sid = path.getSid();
                    long sidl = Long.parseLong(sid);
                    double pipeLength = 0;
                    String device_type_id = path.getDevice_type_id();
                    if (Arrays.asList(pipeArrmp).contains(device_type_id)) {
                        nickName = "pipesectionmp-";
                    } else if (Arrays.asList(pipeArrip).contains(device_type_id)) {
                        nickName = "pipesectionip-";
                    } else if (Arrays.asList(pipeArrlp).contains(device_type_id)) {
                        nickName = "pipesectionlp-";
                    } else if (Arrays.asList(pipeArrhp).contains(device_type_id)) {
                        nickName = "pipesectionhp-";
                    } else {
                        // 获取根据巡检点编码查询管线长度
                        Optional<PlanPoint> planPoint = planPointRepository.findById(sidl);
                        String selPipeLength = "0";
                        if(planPoint.isPresent()){
                            selPipeLength =  planPoint.get().getPipeLength().toString();
                        }

                        if ("0".equals(selPipeLength)) {
                            List<PlanPathTemp> planPathTemps = planPathTempRepository.findByPointIdAndChildPlanId(sidl, childPlanIdl);
                            if (null != planPathTemps && planPathTemps.size() > 0) {
                                //如果当前管线只有一个点，则视为无效管线,计划视为作废
                                if (planPathTemps.size() == 1) {
                                    PlanPoint point = new PlanPoint();
                                    point.setValid("0");
                                    point.setId(sidl);
                                    planPointRepository.saveAndFlush(point);
                                    PlanChild planChild = new PlanChild();
                                    planChild.setId(childPlanIdl);
                                    planChild.setChildPlanState("8");
                                    planChildRepository.saveAndFlush(planChild);
                                    continue;
                                }
                                int j = 0;
                                double lat1 = 0;
                                double lng1 = 0;
                                double lat2 = 0;
                                double lng2 = 0;
                                for (PlanPathTemp line : planPathTemps) {
                                    if (j == 0) {
                                        lat2 = line.getLat().doubleValue();
                                        lng2 = line.getLng().doubleValue();
                                        j++;
                                        continue;
                                    }
                                    lat1 = lat2;
                                    lng1 = lng2;
                                    lat2 = line.getLat().doubleValue();
                                    lng2 = line.getLng().doubleValue();
                                    pipeLength += GpsHelperUtil.GetDistance(lat1, lng1, lat2, lng2);
                                    j++;
                                }
                                total += pipeLength;
                            }
                        } else {
                            pipeLength = BigDecimal.valueOf(Double.parseDouble(selPipeLength)).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                            total += pipeLength;
                        }

                        PlanPoint planPointLine = new PlanPoint();
                        planPointLine.setPipeLength(BigDecimal.valueOf(pipeLength));
                        planPointLine.setId(sidl);
                        // 更新线的长度
                        planPointRepository.saveAndFlush(planPointLine);
                        // 更新巡检点记录表中的管线长度
                        planPointTempRepository.updatePlanPointTempLengthLine(String.valueOf(pipeLength), Long.parseLong(nickName + sid), childPlanIdl);
                    }


                    ContentModel contentModel = JSONObject.parseObject(path.getContent(), ContentModel.class);
                    if (StringUtils.isNotEmpty(contentModel.getSid())) {
                        continue;
                    }
                    pipeLength = BigDecimal.valueOf(Double.parseDouble(contentModel.getPIPELENGTH())).setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
                    total += pipeLength;
                    // 更新巡检点记录表中的管线长度
                    planPointTempRepository.updatePlanPointTempLengthLine(String.valueOf(pipeLength), Long.parseLong(nickName + sid), childPlanIdl);
                }
                if (total > 0) {
                    planChildRepository.updatePlanChildTotal(total, childPlanIdl);
                }
                // TODO 调用 PlanCalculate 服务
            }


        }
    }


    @Data
    class GeoClassPath {

        private String sid;

        private GeometryPath geometry;

        private String name;

        private String address;

        private String device_type_id;

        private String code;

        private String regionname;

        private String pipe_length;

        private String content;

        private String rfid;
    }

    @Data
    class GeometryPath {

        private List<List<List<Double>>> paths;

        private SpatialReference spatialReference;
    }

    @Data
    class SpatialReference {
        private int wkid;
    }

    @Data
    class ContentModel {
        private String sid;
        private String PIPELENGTH;
        private String device_type_id;
    }
}