package com.tofly.dyrqSchedule.handler;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.tofly.common.core.util.Date8Util;
import com.tofly.dyrqSchedule.entity.*;
import com.tofly.dyrqSchedule.mapper.CustomPlanCalculateMapper;
import com.tofly.dyrqSchedule.service.*;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author pra
 * @date 2020/9/15 14:06
 * @description tf_point_day_calculate表内容统计，巡检到位情况统计
 */
@Component
@AllArgsConstructor
public class PlanCalculateHandler {
    private final PdaUserDeviceService pdaUserDeviceService;
    private final CustomPlanCalculateMapper customPlanCalculateMapper;
    private final PointDayCalculateService pointDayCalculateService;
    private final PlanPointTempService planPointTempService;
    private final PlanPathTempService planPathTempService;
    private final UserService userService;

    @XxlJob("PlanCalculateHandler")
    public ReturnT<String> PlanCalculateHandler(String param) {
        Long _childPlanId = null;
        Date startDate = Date8Util.getStartTime();
        Date endDate = Date8Util.getEndTime();
        Long _userId = null;
        Map arg = null;
        if (arg != null) {
            startDate = (Date) arg.get("startDate");
            endDate = (Date) arg.get("endDate");
            _userId = (Long) arg.get("userId");
        }

        //查询所有巡检用户（绑定设备的用户）
//        List<PdaUserDevice> usersList = pdaUserDeviceService.list(Wrappers.<PdaUserDevice>lambdaQuery()
//                .select(PdaUserDevice::getUserId)
//                .eq(_userId != null, PdaUserDevice::getUserId, _userId));
        List<User> usersList = userService.list(Wrappers.<com.tofly.dyrqSchedule.entity.User>lambdaQuery()
                .select(User::getId));

        //统计 计划点数，线长度
        if (usersList.size() == 0) {
           return ReturnT.FAIL;
        }
        for (User user : usersList) {
            Long userId = user.getId();
            Map param1 = new HashMap();
            param1.put("userId", userId);
            param1.put("startDate", startDate);
            param1.put("endDate", endDate);
            param1.put("childPlanId", _childPlanId);

            //查询每个用户的计划
            List<Map> childPlans = customPlanCalculateMapper.getUserPlanChild(param1);
//            childPlans.forEach(childPlan->{
            for (Map childPlan : childPlans) {
                Long childPlanId = Long.parseLong(childPlan.get("id").toString());
                Long deviceTypeId = Long.parseLong(childPlan.get("deviceTypeId").toString());

                //查询tf_point_day_calculate 是否存在计划
                List<PointDayCalculate> hasChildPlan = pointDayCalculateService.list(Wrappers.<PointDayCalculate>lambdaQuery()
                        .eq(PointDayCalculate::getUserid, userId)
                        .eq(PointDayCalculate::getChildPlanId, childPlanId));

                //获取计划的点到位情况
                PointDayCalculate pointDayCalculate = new PointDayCalculate();
                if ("0".equals(childPlan.get("type"))) {
                    int count = planPointTempService.count(Wrappers.<PlanPointTemp>lambdaQuery()
                            .eq(PlanPointTemp::getChildPlanId, childPlanId)
                            .eq(PlanPointTemp::getState, 1)
                            .ge(PlanPointTemp::getOverDate, startDate)
                            .le(PlanPointTemp::getOverDate, endDate));
                    pointDayCalculate.setIsovertototal(Double.valueOf(count));
                    pointDayCalculate.setIsovertototal1(new BigDecimal(count));
                } else {
                    //线的isover
                    int isover = planPathTempService.count(Wrappers.<PlanPathTemp>lambdaQuery()
                            .eq(PlanPathTemp::getChildPlanId, childPlanId)
                            .eq(PlanPathTemp::getIsover, 1)
                            .ge(PlanPathTemp::getOverDate, startDate)
                            .le(PlanPathTemp::getOverDate, endDate));
                    int total = planPathTempService.count(Wrappers.<PlanPathTemp>lambdaQuery()
                            .eq(PlanPathTemp::getChildPlanId, childPlanId));
                    int num = planPointTempService.count(Wrappers.<PlanPointTemp>lambdaQuery()
                            .eq(PlanPointTemp::getChildPlanId, childPlanId)
                            .eq(PlanPointTemp::getState, 1)
                            .ge(PlanPointTemp::getOverDate, startDate)
                            .le(PlanPointTemp::getOverDate, endDate));
                    pointDayCalculate.setIsovertototal(Double.valueOf(isover));
                    pointDayCalculate.setIsovertototal1(new BigDecimal(num));
                }
                pointDayCalculate.setTotal(childPlan.get("total") == null ? new BigDecimal(0) : (BigDecimal) childPlan.get("total"));
                pointDayCalculate.setTotal1(childPlan.get("total1") == null ? new BigDecimal(0) : (BigDecimal) childPlan.get("total1"));

                pointDayCalculate.setChildBeginDate((Date) childPlan.get("childBeginDate"));
                pointDayCalculate.setChildEndDate((Date) childPlan.get("childEndDate"));
                if (hasChildPlan == null || hasChildPlan.size() == 0) {
                    pointDayCalculate.setPlanDate(startDate);
                    pointDayCalculate.setUserid(userId);
                    pointDayCalculate.setChildPlanId(childPlanId);
                    pointDayCalculate.setDeviceTypeId(deviceTypeId);
                    pointDayCalculateService.save(pointDayCalculate);
                } else {
                    pointDayCalculate.setId(hasChildPlan.get(0).getId());
                    pointDayCalculateService.updateById(pointDayCalculate);
                }
            }
        }
        return ReturnT.SUCCESS;
    }
}
