package com.tofly.dyrqSchedule.handler;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.tofly.common.core.util.Date8Util;
import com.tofly.dyrqSchedule.entity.PlanChild;
import com.tofly.dyrqSchedule.entity.PlanPathTemp;
import com.tofly.dyrqSchedule.entity.PlanPointTemp;
import com.tofly.dyrqSchedule.entity.SysDeviceType;
import com.tofly.dyrqSchedule.mapper.CustomSyncMonitorMapper;
import com.tofly.dyrqSchedule.service.*;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author pra
 * @date 2020/9/15 14:18
 * @description 更新点到位情况，SyncMonitor的前500行部部分
 */

@Component
@AllArgsConstructor
public class PlanPointHandle {
    private final PdaUserDeviceService pdaUserDeviceService;
    private final CustomSyncMonitorMapper customSyncMonitorMapper;
    private final PlanChildService planChildService;
    private final PlanPointTempService planPointTempService;
    private final PlanPathTempService planPathTempService;
    private final SysDeviceTypeService sysDeviceTypeService;

    @XxlJob("PlanPointHandle")
    public ReturnT<String> PlanPointHandle(String param) {
        Date now = Date8Util.getCurrentDate();
        //查询当天的计划
        Map param1 = new HashMap();
        param1.put("now", now);
        List<Map> planList = customSyncMonitorMapper.getTodayPlan(param1);
        for (Map plan : planList) {
            int pointTotal = 0;
            int pointIsoverTotal = 0;
            int isOver = 0;  //---------巡检点，巡检线isover个数，其他设备check个数
            int total = 0; //point_temp个数
            double percent = 0;
            double percent1 = 0;
            Long childPlanId = Long.parseLong(plan.get("id").toString());

            //查询包含巡检检查内容的类型
            List<SysDeviceType> devices = sysDeviceTypeService.list(Wrappers.<SysDeviceType>lambdaQuery().isNotNull(SysDeviceType::getCheckContent));
            Set<Long> checkTypes = new HashSet<>();
            devices.stream().filter(n -> !checkTypes.add(n.getPointTypeId()))
                    .collect(Collectors.toSet());

            //更新巡检线录入状态
            if ("1".equals(plan.get("type"))) {
                List<Map> pathTotal = customSyncMonitorMapper.getPathTotalInfo(childPlanId);
                total = Integer.valueOf(pathTotal.get(0).get("total").toString());
                isOver = checkTypes.contains(plan.get("deviceTypeId")) ? Integer.valueOf(pathTotal.get(0).get("checkTotal").toString()) : Integer.valueOf(pathTotal.get(0).get("isovertotal").toString());
                pointTotal = Integer.valueOf(pathTotal.get(0).get("pointTotal").toString());
                pointIsoverTotal = Integer.valueOf(pathTotal.get(0).get("pointIsoverTotal").toString());
//                percent1 = pointTotal == 0 ? 0 : new BigDecimal((float) pointIsoverTotal / pointTotal).setScale(4, BigDecimal.ROUND_HALF_UP).doubleValue();
//                percent = pointTotal == 0 ? 0 : new BigDecimal((float) isOver / total).setScale(4, BigDecimal.ROUND_HALF_UP).doubleValue();
                //修改percent1 为point_temp到位情况
                percent = pointTotal == 0 ? 0 : new BigDecimal((float) pointIsoverTotal / pointTotal).setScale(4, BigDecimal.ROUND_HALF_UP).doubleValue();
                percent1 = pointTotal == 0 ? 0 : new BigDecimal((float) pointIsoverTotal / pointTotal).setScale(4, BigDecimal.ROUND_HALF_UP).doubleValue();

                //更新到位条数
                PlanChild planChild =new PlanChild();
                planChild.setId(childPlanId);
                planChild.setPercent(percent);
                planChild.setPercent1(percent1);
                planChild.setTotal1(new BigDecimal(pointTotal));
                planChild.setIsovertototal(percent * total);
                planChild.setIsovertototal1(new BigDecimal(pointIsoverTotal));
                planChild.setIsovertototal0(new BigDecimal(pointIsoverTotal));
                planChildService.updateById(planChild);

                //更新线到位比例
                List<PlanPointTemp> stateList = planPointTempService.list(Wrappers.<PlanPointTemp>lambdaQuery().eq(PlanPointTemp::getChildPlanId,childPlanId)
                        .eq(PlanPointTemp::getState,"0"));
                List<PlanPointTemp> pointTemps = new ArrayList<>();
//                List<PlanPathTemp> planPathTemps = new ArrayList<>();
//                for (PlanPointTemp pointTemp:stateList){
//                    Long pointTempId =  pointTemp.getId();
//                    int isOverNum = planPathTempService.count(Wrappers.<PlanPathTemp>lambdaQuery().eq(PlanPathTemp::getTempId,pointTempId).eq(PlanPathTemp::getIsover,"1"));
//                    int allNum = planPathTempService.count(Wrappers.<PlanPathTemp>lambdaQuery().eq(PlanPathTemp::getTempId,pointTempId));
//                    double pathOver = allNum == 0 ? 0 : new BigDecimal((float) isOverNum / allNum).setScale(4, BigDecimal.ROUND_HALF_UP).doubleValue();
//                    PlanPointTemp one = new PlanPointTemp();
//                    one.setId(pointTempId);
//                    one.setPathIsover(new BigDecimal(pathOver));
//                    pointTemps.add(one);
////                    planPointTempService.updateById(one);
//                }
                planPointTempService.updateBatchById(pointTemps);

                //更新录入状态
                List<Map> pathMax = customSyncMonitorMapper.getPathMaxSpeed(childPlanId);

                pointTemps.clear();
                for(Map i:pathMax){
                    PlanPointTemp planPointTemp = new PlanPointTemp();
                    planPointTemp.setIsover("1");
                    planPointTemp.setGpsSpeed((BigDecimal)i.get("gpsSpeed"));
                    planPointTemp.setState("1");
                    planPointTemp.setId(Long.parseLong(i.get("tempId").toString()));
                    planPointTemp.setOverDate((Date)i.get("overDate"));
                    pointTemps.add(planPointTemp);
//                    planPointTempService.updateById(planPointTemp);
                }
                planPointTempService.updateBatchById(pointTemps);
            }
            else {
                //更新巡检点录入状态
                total = planPointTempService.count(Wrappers.<PlanPointTemp>lambdaQuery().eq(PlanPointTemp::getChildPlanId, childPlanId));
                int pointIsOver = planPointTempService.count(Wrappers.<PlanPointTemp>lambdaQuery().eq(PlanPointTemp::getChildPlanId, childPlanId).eq(PlanPointTemp::getIsover,"1"));
                int pointIsCheck = planPointTempService.count(Wrappers.<PlanPointTemp>lambdaQuery().eq(PlanPointTemp::getChildPlanId, childPlanId).eq(PlanPointTemp::getCheckState,"1"));
                isOver = checkTypes.contains(Long.parseLong(plan.get("deviceTypeId").toString())) ? pointIsCheck : pointIsOver;
                percent = isOver == 0 ? 0 : new BigDecimal((float) isOver / total).setScale(4, BigDecimal.ROUND_HALF_UP).doubleValue();
                //更新到位点数
                PlanChild planChild = new PlanChild();
                planChild.setId(childPlanId);
                planChild.setPercent(percent);
                planChild.setPercent1(percent);
//                planChild.setTotal(Double.valueOf(total));
                planChild.setTotal1(new BigDecimal(total));
                planChild.setIsovertototal(Double.valueOf(isOver));
                planChild.setIsovertototal1(new BigDecimal(isOver));
                planChildService.updateById(planChild);
            }
            //更新计划状态 完成进度percent1为1 计划变成完成状态
            planChildService.update(Wrappers.<PlanChild>lambdaUpdate().set(PlanChild::getChildPlanState,"3")
                    .eq(PlanChild::getPercent1,1).eq(PlanChild::getId,childPlanId));
        }
        return ReturnT.SUCCESS;
    }
}
