package com.tofly.dyrqSchedule.handler;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.tofly.common.core.util.Date8Util;
import com.tofly.common.core.util.StringUtil;
import com.tofly.dyrqSchedule.entity.EsPdaDeviceUpload;
import com.tofly.dyrqSchedule.entity.PdaSign;
import com.tofly.dyrqSchedule.entity.Plan;
import com.tofly.dyrqSchedule.entity.PlanChild;
import com.tofly.dyrqSchedule.mapper.CustomPlanCalculateMapper;
import com.tofly.dyrqSchedule.repository.EsPdaDeviceUploadRepository;
import com.tofly.dyrqSchedule.service.PdaSignService;
import com.tofly.dyrqSchedule.service.PlanChildService;
import com.tofly.dyrqSchedule.util.DyrqCommonRedis;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.AllArgsConstructor;
import org.codehaus.groovy.runtime.dgmimpl.arrays.LongArrayGetAtMetaMethod;
import org.springframework.data.redis.core.ListOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;
import sun.util.resources.cldr.ha.LocaleNames_ha;

import java.math.BigDecimal;
import java.util.*;

/**
 * @author pra
 * @date 2020/9/22 14:41
 * @description 对昨天结束的巡检子计划状态改变,当天未开始计划变为正在执行，每天凌晨执行,自动签到下班
 */
@Component
@AllArgsConstructor
public class PlanStateHandler {
    private final PlanChildService planChildService;
    private final PdaSignService pdaSignService;
    private final CustomPlanCalculateMapper customPlanCalculateMapper;
    private final EsPdaDeviceUploadRepository esPdaDeviceUploadRepository;
    private ListOperations<String, Object> listOperations;
    private final RedisTemplate redisTemplate;


    @XxlJob("PlanStateHandler")
    public ReturnT<String> PlanStateHandler(String param) {
        //获取当前正在执行并超期的计划
        Date now = Date8Util.getCurrentDate();
        List<PlanChild> overDatePlans = planChildService.list(Wrappers.<PlanChild>lambdaQuery()
                .in(PlanChild::getChildPlanState,"2","6")
                .le(PlanChild::getChildEndDate,now));

        //对比完成率和预计完成率，如果完成率大于预计完成率则，修改为完成（3），否则超期未完成（4）
        for (PlanChild pc:overDatePlans){
            Double per = pc.getPercent() * 100;
            if (per.compareTo(Double.valueOf(StringUtil.isEmpty(pc.getPlanRate()) ? "1" : pc.getPlanRate())) >= 0){
                pc.setChildPlanState("3");
            }
            else{
                pc.setChildPlanState("4");
            }
            planChildService.updateById(pc);
        }

        //当天未开始计划变为正在执行
        List<PlanChild> notStartPlans = planChildService.list(Wrappers.<PlanChild>lambdaQuery()
                .in(PlanChild::getChildPlanState,"1","5")
                .le(PlanChild::getChildBeginDate,now));
        for (PlanChild pc:notStartPlans){
            if ("1".equals(pc.getChildPlanState())) {
                pc.setChildPlanState("2");
            } else {
                pc.setChildPlanState("6");
            }
            planChildService.updateById(pc);
        }


        //每天自动签退
        Calendar calendar = Calendar.getInstance();
        calendar.add(Calendar.DATE,-1);
        calendar.set(Calendar.HOUR_OF_DAY, 0);
        calendar.set(Calendar.MINUTE, 0);
        calendar.set(Calendar.SECOND, 0);
        Date yestodayStart = calendar.getTime();
        calendar.set(Calendar.HOUR_OF_DAY, 23);
        calendar.set(Calendar.MINUTE, 59);
        calendar.set(Calendar.SECOND, 59);
        Date yestodayEnd = calendar.getTime();
        calendar.set(Calendar.SECOND, 0);
        Date signAndUploadTime = calendar.getTime();
        //获取未签退用户,添加签退数控
        Map signParam = new HashMap();
        signParam.put("startDate",yestodayStart);
        signParam.put("endDate",yestodayEnd);
        List<PdaSign> signList = customPlanCalculateMapper.getSignLast(signParam);
        List<EsPdaDeviceUpload> pdaDeviceUploads = new ArrayList<>();
        Long sign_id = customPlanCalculateMapper.getSignMaxId();
        for (PdaSign i:signList){
            i.setId(sign_id);
            i.setSignTime(signAndUploadTime);
            i.setSignType("2");
            sign_id++;
            pdaSignService.save(i);
            EsPdaDeviceUpload esPdaDeviceUpload = new EsPdaDeviceUpload();
            esPdaDeviceUpload.setUserid(i.getUserid());
            esPdaDeviceUpload.setCheckTime(signAndUploadTime);
            esPdaDeviceUpload.setUploadtime(signAndUploadTime);
            esPdaDeviceUpload.setWorkState(new BigDecimal(3));
            pdaDeviceUploads.add(esPdaDeviceUpload);
        }
        //批量添加签到信息 设备信息
//        pdaSignService.saveBatch(signList);
        if (pdaDeviceUploads.size() > 0){
            esPdaDeviceUploadRepository.saveAll(pdaDeviceUploads);
            saveToRedis(pdaDeviceUploads,yestodayEnd);
        }
        return ReturnT.SUCCESS;
    }

    private void saveToRedis(List<EsPdaDeviceUpload> esPdaDeviceUploads,Date yestodayEnd){
        for (EsPdaDeviceUpload esPdaDeviceUpload:esPdaDeviceUploads){
            listOperations.rightPush(DyrqCommonRedis.REDIS_PRE_PDA_DEVICE+esPdaDeviceUpload.getUserid(),esPdaDeviceUpload);
            redisTemplate.expireAt(DyrqCommonRedis.REDIS_PRE_PDA_DEVICE+esPdaDeviceUpload.getUserid(), yestodayEnd);
        }
    }
}
