package com.tofly.dyrqSchedule.handler;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tofly.dyrqSchedule.entity.*;
import com.tofly.dyrqSchedule.repository.*;
import com.tofly.dyrqSchedule.service.EsPdaDeviceUploadService;
import com.tofly.dyrqSchedule.util.DateTimeUtil;
import com.tofly.dyrqSchedule.util.GetStartEndDateUtil;
import com.tofly.dyrqSchedule.util.StringHelperUtil;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.math.BigDecimal;
import java.time.Duration;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @program: tofly-dyrq
 * @description: 自动签到签退、自动复制、隐患系统同步、更新管线完成率服务
 * @author: Sumer
 * @create: 2020-08-31 16:04
 */
@Component
@Slf4j
public class SyncMonitorHandler {
    @Autowired
    private PlanPathTempRepository planPathTempRepository;

    @Autowired
    private ConjunctiveQueryRepository conjunctiveQueryRepository;

    @Autowired
    private PdaSignRepository pdaSignRepository;

    @Autowired
    private PlanPointTempRepository planPointTempRepository;

    @Autowired
    private PlanPointRepository planPointRepository;

    @Autowired
    private PlanChildRepository planChildRepository;

    @Autowired
    private PdaUserRepository pdaUserRepository;

    @Autowired
    private EsPdaDeviceUploadService esPdaDeviceUploadService;

    /**
     * 自动复制签退签到
     *
     * @param start_date
     * @param sql
     * @param end_date
     */
    private void AutoSign(String start_date, String sql, String end_date) {
        LocalDateTime _start_date = DateTimeUtil.fromString2LocalDateTime(start_date, DateTimeUtil.FULL_SLASH_FORMAT).minus(1, ChronoUnit.DAYS);

        BigDecimal on_num = BigDecimal.ZERO;
        BigDecimal off_num = BigDecimal.ZERO;

        LocalDateTime offDate = _start_date.plusDays(1).minusSeconds(1);
        LocalDateTime onDate = _start_date.plusSeconds(1);
        Date default_off = DateTimeUtil.convertLDTToDate(offDate);
        Date default_on = DateTimeUtil.convertLDTToDate(onDate);
        // TODO sql = string.Format("select * from tf_pda_monitor t left join tf_pda_user t1 on t.userid=t1.userid where t.monitor_date=to_date('{0}', 'yyyy/MM/dd') and t.isautosign is null", str_start_date)
        List<ConjunctiveQuery> dt = new ArrayList<>();
        for (ConjunctiveQuery row : dt) {
            /**
             * TODO
             * if (row["userid"].ToString() == "0B5E385DA3B04B32A1E160CDD2544129" && row["work_state"].ToString() == "5")
             * continue;
             * if (row["userid"].ToString() == "3EB3BA1F85EC43F88E74649EF88781E4" && row["work_state"].ToString() == "5")
             * continue;
             */
            // #region 自动签到签退
            List<ConjunctiveQuery> dt1 = JSONObject.parseArray(JSONObject.toJSONString(conjunctiveQueryRepository.getPdaSignOnOffNum(default_off, default_on, row.getUser_id().toString())),ConjunctiveQuery.class);
            if (null != dt1 && dt1.size() > 0) {
                on_num = new BigDecimal(dt1.get(0).getOn_num());
                off_num = new BigDecimal(dt1.get(0).getOff_num());

                if (on_num.compareTo(BigDecimal.ZERO) > 0 && on_num.compareTo(off_num) > 0) {
                    // # 自动下班
                    // 插入设备下班日志
                    esPdaDeviceUploadService.automaticPunchSave(row,default_off,null);
                    // TODO 缺少 section software_ver 属性
                    // 插入签退数据
                    PdaSign pdaSign = new PdaSign();
                    pdaSign.setUserid(row.getUser_id());
                    pdaSign.setSignTime(DateTimeUtil.convertLDTToDate(offDate));
                    pdaSign.setPadid(row.getPath_id());
                    pdaSign.setSignAddress("");
                    pdaSign.setSignType("2");
                    pdaSign.setLocationLongitude(row.getLng());
                    pdaSign.setLocationLatitude(row.getLat());
                    pdaSign.setCompanyid(Long.parseLong(row.getCompanyid()));
                    pdaSign.setFlag("2");
                    // TODO tf_pda_sign缺少 section software_ver 属性
                    pdaSignRepository.save(pdaSign);

                    // 用户上线未离线状态
                    PdaUser pdaUser = new PdaUser();
                    pdaUser.setUserid(row.getUser_id());
                    pdaUser.setOnlineState(BigDecimal.ONE);
                    pdaUserRepository.saveAndFlush(pdaUser);
                    // #endregion

                    // #region 自动上班
                    // 插入设备上班日志
                    esPdaDeviceUploadService.automaticPunchSave(row,null,default_on);
                    // TODO 缺少 section software_ver 属性
                    // 插入签到数据
                    pdaSign.setSignTime(DateTimeUtil.convertLDTToDate(onDate));
                    pdaSign.setSignType("1");
                    // TODO tf_pda_sign缺少 section software_ver 属性
                    pdaSignRepository.save(pdaSign);
                    // #endregion

                }
                // #region 自动签到
                // TODO 没实体 tf_pda_monitor
            }
        }
    }

    /**
     * 自动复制
     *
     * @param start_date
     * @param sql
     * @param end_date
     */
    private void AutoCopy(String start_date, String sql, String end_date) {
        LocalDateTime time = DateTimeUtil.fromString2LocalDateTime(start_date, DateTimeUtil.DAY_SLASH_FORMAT);
        LocalDateTime now = LocalDateTime.now();
        LocalDateTime _start_date = time.plusHours(now.getHour()).plusMinutes(now.getMinute()).plusSeconds(now.getSecond());
        String msg = "";
        List<ConjunctiveQuery> dt = JSONObject.parseArray(JSONObject.toJSONString(conjunctiveQueryRepository.getAutoCopyData()),ConjunctiveQuery.class);
        // 增加日志记录
        log.info("循环的日志列表：数量: {}", dt.size());
        String[] pipeArr = new String[]{"9", "10", "11", "12", "13", "14", "18"};//次高压管线、中压管线、低压管线、高压管线、次高压管线、中压管线、高压管线
        //调压箱、调压柜
        String[] boosterArr = new String[]{"2", "3"};
        //阀门
        String[] valveArr = new String[]{"4"};
        //阀室
        String[] valveCellArr = new String[]{"7"};
        //巡检点、巡检线
        String[] pointArr = new String[]{"1", "6"};
        String tablename = "";
        List<PlanPointTemp> dtCorrect = new ArrayList<>();
        List<CollectModel> listCorrect = new ArrayList<>();
        String plan_id = "";
        String companyid = "";
        String device_type_id = "";
        String type = "";
        // TODO ConfigurationManager.AppSettings["isautocopy"] ?? "0";
        String iscorrect = "";

        for (ConjunctiveQuery row : dt) {
            LocalDateTime dateTime = LocalDateTime.now();
            plan_id = String.valueOf(row.getChild_plan_id());
            companyid = row.getCompanyid();
            device_type_id = String.valueOf(row.getDevice_type_id());
            type = row.getType();
            Date date = DateTimeUtil.convertLDTToDate(dateTime);
            TF_PLAN_CHILD entity = new TF_PLAN_CHILD();
            entity.setCREATE_USER(row.getCreate_user());
            entity.setCREATE_TIME(date);
            entity.setCOMPANYID(companyid);
            entity.setPLAN_MODE(new BigDecimal(StringUtils.isNotEmpty(row.getPlan_mode()) ? "0" : row.getPlan_mode()));
            entity.setREGION_ID(row.getPlan_mode());
            entity.setUSERID(String.valueOf(row.getRegion_id()));
            entity.setDEVICE_TYPE_ID(String.valueOf(row.getDevice_type_id()));
            entity.setPERIOD_ID(String.valueOf(row.getPeriod_id()));
            entity.setINTERVAL_DAYS(row.getInterval_days() == null ? BigDecimal.ZERO : row.getInterval_days());
            entity.setN(row.getN() == null || row.getN().equals(BigDecimal.ZERO) ? BigDecimal.ONE : row.getN());
            int n = entity.getN().intValue();
            Date child_begin_date = row.getChild_begin_date();
            Date child_end_date = row.getChild_end_date();
            // 处理时间
            GetStartEndDateUtil.getMonth(row.getPeriod_name(),child_begin_date,child_end_date,n);
            entity.setCHILD_BEGIN_DATE(row.getChild_begin_date());
            entity.setCHILD_END_DATE(row.getChild_end_date());

            entity.setN(BigDecimal.ZERO);
            entity.setDESCRIPTION(row.getDescription());
            entity.setISJURISDICTION(row.getIsjurisdiction() == null ? BigDecimal.ZERO : row.getIsjurisdiction());
            entity.setMAP_RANGE(row.getMap_range());
            entity.setPLAN_NAME(row.getPlan_name());
            entity.setPLAN_TYPE(row.getPlan_type() == null ? BigDecimal.ZERO : row.getPlan_type());
            entity.setLAST_TIME(date);

            BigDecimal copy_num = BigDecimal.ZERO;
            copy_num = row.getCopy_num();
            entity.setCOPY_NUM(copy_num.subtract(BigDecimal.ONE));
            if (copy_num.compareTo(BigDecimal.ZERO) < 0) {
                entity.setCOPY_NUM(BigDecimal.ONE);
            }
            entity.setCOPY_STATE(BigDecimal.ONE);
            entity.setPERIOD_TYPE(row.getPeriod_type() == null ? BigDecimal.ZERO : row.getPeriod_type());

            if (entity.getCHILD_BEGIN_DATE().before(date)) {
                entity.setCHILD_PLAN_STATE(BigDecimal.ONE);
            } else {
                entity.setCHILD_PLAN_STATE(BigDecimal.valueOf(2));
            }

            // #矫正计划内容
            if (!"1".equals(entity.getREGION_ID())) {
                if (Arrays.asList(pipeArr).contains(device_type_id) && "1".equals(iscorrect)) {
                    tablename = "PIPESECTION";
                    // 矫正管线
                    if ("113".equals(companyid)) {
                        tablename = "PIPESECTION_GW";
                    }
                    // 区分管线图层名
                    //中压管线
                    String[] pipeArrmp = new String[]{"10", "14"};
                    //次高压管线
                    String[] pipeArrip = new String[]{"9", "13"};
                    //低压管线
                    String[] pipeArrlp = new String[]{"11"};
                    //高压管线
                    String[] pipeArrhp = new String[]{"12", "18"};
                    String nickname = "";

                    if (Arrays.asList(pipeArrmp).contains(device_type_id)) {
                        nickname = "pipesectionmp-";
                    } else if (Arrays.asList(pipeArrip).contains(device_type_id)) {
                        nickname = "pipesectionip-";
                    } else if (Arrays.asList(pipeArrlp).contains(device_type_id)) {
                        nickname = "pipesectionlp-";
                    } else if (Arrays.asList(pipeArrhp).contains(device_type_id)) {
                        nickname = "pipesectionhp-";
                    }

                    // TODO sql PIPESECTION@SDE_JXW_CS_DBLINK PIPESECTION_GW@SDE_JXW_CS_DBLINK
                    dtCorrect = null;
                    listCorrect = null;
                } else if (Arrays.asList(boosterArr).contains(device_type_id) && "1".equals(iscorrect)) {
                    tablename = "BOOSTER";
                    //校正调压箱
                    if ("113".equals(companyid)) {
                        tablename = "BOOSTER_GW";
                    }
                    // TODO BOOSTER@SDE_JXW_CS_DBLINK BOOSTER_GW@SDE_JXW_CS_DBLINK
                    dtCorrect = null;
                    listCorrect = null;
                } else if (Arrays.asList(valveArr).contains(device_type_id) && "1".equals(iscorrect)) {
                    tablename = "VALVE";
                    //校正阀门
                    if ("113".equals(companyid)) {
                        tablename = "VALVE_GW";
                    }
                    // TODO VALVE@SDE_JXW_CS_DBLINK VALVE_GW@SDE_JXW_CS_DBLINK
                    dtCorrect = null;
                    listCorrect = null;
                } else if (Arrays.asList(valveCellArr).contains(device_type_id) && "1".equals(iscorrect)) {
                    tablename = "VALVECELL";
                    //校正阀室
                    if ("113".equals(companyid)) {
                        tablename = "VALVE_GW";
                    }
                    // TODO VALVECELL@SDE_JXW_CS_DBLINK VALVE_GW@SDE_JXW_CS_DBLINK
                    dtCorrect = null;
                    listCorrect = null;
                } else if (Arrays.asList(pointArr).contains(device_type_id) && "1".equals(iscorrect)) {
                    tablename = "tf_plan_point";
                    dtCorrect = planPointTempRepository.getPlanPointTempIdAndPointId(Long.parseLong(plan_id));
                    listCorrect = JSONArray.parseArray(JSON.toJSONString(dtCorrect), CollectModel.class);

                }
            }
            List<String> ids = new ArrayList<>();
            listCorrect.stream().forEach(m -> ids.add(m.getPoint_id()));
            List<GeoClass> list = new ArrayList<>();
            List<GeoClassPath> listPath = new ArrayList<>();

            List<GeoClass> list1 = new ArrayList<>();
            List<GeoClassPath> listPath1 = new ArrayList<>();

            if ("0".equals(type)) {
                list = JSONArray.parseArray(JSON.toJSONString(row.getPoint_info()), GeoClass.class);
                if (0 < ids.size()) {
                    list1 = list.stream().filter(m -> ids.contains(m.sid)).collect(Collectors.toList());
                    list = list.stream().filter(m -> !ids.contains(m.sid)).collect(Collectors.toList());
                    entity.setPOINT_INFO(JSON.toJSONString(list));
                } else {
                    entity.setPOINT_INFO(row.getPoint_info());
                }
            } else {
                listPath = JSONArray.parseArray(JSON.toJSONString(row.getPoint_info()), GeoClassPath.class);
                if (0 < ids.size()) {
                    listPath1 = listPath.stream().filter(m -> ids.contains(m.sid)).collect(Collectors.toList());
                    listPath = listPath.stream().filter(m -> !ids.contains(m.sid)).collect(Collectors.toList());
                    entity.setPOINT_INFO(JSON.toJSONString(listPath));
                } else {
                    entity.setPOINT_INFO(row.getPoint_info());
                }
            }
            // #endregion

            if ("0".equals(type)) {
                list = JSONArray.parseArray(entity.getPOINT_INFO(), GeoClass.class);
            } else {
                listPath = JSONArray.parseArray(entity.getPOINT_INFO(), GeoClassPath.class);
            }

            // #region 参数配置
            // TODO 没实体 TF_SYS_DEVICE_SETTING
            List<Setting_Plan> _settingPlan = new ArrayList<>();
            // #endregion

            // 增加日志记录
            log.info("循环的日志列表：copy_from_plan: {}", plan_id);

            planChildRepository.updatePlanChildCopyState(Long.parseLong(plan_id));
            // 添加子计划、添加巡检点到临时表
            this.addPlanPointCommand(entity, list, listPath, type, msg, _settingPlan, true, list1, listPath1);
        }

    }

    /**
     * 添加子计划、添加巡检点到临时表
     *
     * @param entity
     * @param list
     * @param listPath
     * @param type
     * @param msg
     * @param settingplan
     * @param allow
     * @param list1
     * @param listPath1
     */
    private void addPlanPointCommand(TF_PLAN_CHILD entity, List<GeoClass> list, List<GeoClassPath> listPath, String type, String msg, List<Setting_Plan> settingplan, boolean allow, List<GeoClass> list1, List<GeoClassPath> listPath1) {
        //确定巡检计划循环次数
        int serial_number = 0;
        //每次加多少天
        int addition_days = 0;
        Duration ts = Duration.between(DateTimeUtil.fromLong2LocalDateTime(entity.getCHILD_END_DATE().getTime()), DateTimeUtil.fromLong2LocalDateTime(entity.getCHILD_BEGIN_DATE().getTime()));
        switch (entity.getPERIOD_ID()) {
            case "1":
                serial_number = Integer.parseInt(String.valueOf(ts.toDays())) + 1;
                addition_days = 1;
                break;
            case "2":
                serial_number = Integer.parseInt(String.valueOf(ts.toDays())) / 2;
                addition_days = 2;
                break;
        }

        if ("1".equals(entity.getPERIOD_ID()) || "2".equals(entity.getPERIOD_ID())) {

        }
    }


    private void addPlanCommand(TF_PLAN_CHILD entity, List<GeoClass> list, List<GeoClassPath> listPath, String type, String msg, List<Setting_Plan> settingplan, boolean allow, List<GeoClass> list1, List<GeoClassPath> listPath1) {
        LocalDateTime now = LocalDateTime.now();
        TF_PLAN_CHILD entity_child = new TF_PLAN_CHILD();
        String child_plan_id = StringHelperUtil.NewId();
        int point_num = 0;
        // 先初始化bean相同属性值
        BeanUtils.copyProperties(entity, entity_child);
        entity_child.setCHILD_PLAN_ID(child_plan_id);
        if (BigDecimal.ONE.compareTo(entity.getPLAN_TYPE()) == 0) {
            entity_child.setCHILD_BEGIN_DATE(entity.getCHILD_BEGIN_DATE());
            entity_child.setCHILD_END_DATE(entity.getCHILD_END_DATE());
        } else {
            entity_child.setCHILD_BEGIN_DATE(entity.getCHILD_BEGIN_DATE());
            entity_child.setCHILD_END_DATE(DateTimeUtil.convertLDTToDate(DateTimeUtil.fromLong2LocalDateTime(entity.getCHILD_END_DATE().getTime()).plusDays(1).minusSeconds(1)));
        }
        if (now.compareTo(DateTimeUtil.fromLong2LocalDateTime(entity_child.getCHILD_BEGIN_DATE().getTime())) > 0) {
            entity_child.setCHILD_PLAN_STATE(BigDecimal.valueOf(2));
        } else {
            entity_child.setCHILD_PLAN_STATE(BigDecimal.ONE);
        }
        entity_child.setCHILD_NUM(BigDecimal.ZERO);
        entity_child.setPERCENT(BigDecimal.ZERO);
        entity_child.setPERCENT1(BigDecimal.ZERO);
        entity_child.setISOVERTOTOTAL(BigDecimal.ZERO);
        entity_child.setISOVERTOTOTAL1(BigDecimal.ZERO);

        // 日志
        log.info("Add_Plan_Command：copy_from_plan: {},copy_new_plan: {}", entity.getCHILD_PLAN_ID(), child_plan_id);

        this.addPlanSend(entity, list, listPath, type, settingplan, list1, listPath1, point_num, entity_child, child_plan_id);

    }


    private void addDayPlanCommand(TF_PLAN_CHILD entity, int serial_number, int addition_days, List<GeoClass> list, List<GeoClassPath> listPath, String type, String msg, List<Setting_Plan> settingplan, boolean allow, List<GeoClass> list1, List<GeoClassPath> listPath1) {
        LocalDateTime now = LocalDateTime.now();
        for (int i = 1; i <= serial_number; i++) {
            int point_num = 0;
            TF_PLAN_CHILD entity_child = new TF_PLAN_CHILD();
            String child_plan_id = StringHelperUtil.NewId();
            BeanUtils.copyProperties(entity, entity_child);
            entity_child.setCHILD_PLAN_ID(child_plan_id);
            log.info("Add_Day_Plan_Command：copy_from_plan:" + entity.getCHILD_PLAN_ID() + "copy_new_plan:" + child_plan_id + "serial_number:" + serial_number + "i:" + i);
            entity_child.setCHILD_BEGIN_DATE(DateTimeUtil.convertLDTToDate(DateTimeUtil.fromLong2LocalDateTime(entity.getCHILD_BEGIN_DATE().getTime()).plusDays(i - 1)));
            entity_child.setCHILD_END_DATE(DateTimeUtil.convertLDTToDate(DateTimeUtil.fromLong2LocalDateTime(entity.getCHILD_BEGIN_DATE().getTime()).plusDays(i * addition_days).minusSeconds(1)));
            entity_child.setCHILD_NUM(BigDecimal.valueOf(i));
            entity_child.setPERCENT(BigDecimal.ZERO);
            entity_child.setPERCENT1(BigDecimal.ZERO);
            entity_child.setISOVERTOTOTAL(BigDecimal.ZERO);
            entity_child.setISOVERTOTOTAL1(BigDecimal.ZERO);
            if (now.compareTo(DateTimeUtil.fromLong2LocalDateTime(entity_child.getCHILD_BEGIN_DATE().getTime())) > 0) {
                entity_child.setCHILD_PLAN_STATE(BigDecimal.valueOf(2));
            } else {
                entity_child.setCHILD_PLAN_STATE(BigDecimal.ONE);
            }
            this.addPlanSend(entity, (List<GeoClass>) list, (List<GeoClassPath>) listPath, type, (List<Setting_Plan>) settingplan, (List<GeoClass>) list1, (List<GeoClassPath>) listPath1, point_num, entity_child, child_plan_id);
        }
    }

    private void addPlanSend(TF_PLAN_CHILD entity, List<GeoClass> list, List<GeoClassPath> listPath, String type, List<Setting_Plan> settingplan, List<GeoClass> list1, List<GeoClassPath> listPath1, int point_num, TF_PLAN_CHILD entity_child, String child_plan_id) {
        if ("1".equals(type)) {
            //线总长度
            entity_child.setTOTAL(BigDecimal.valueOf(listPath.stream().mapToDouble(e -> Double.parseDouble(e.getPipe_length())).reduce(0, Double::sum)));
            if (BigDecimal.ZERO.compareTo(entity_child.getTOTAL()) == 0 && "6".equals(entity_child.getDEVICE_TYPE_ID())) {
                List<String> pointid = listPath.stream().map(GeoClassPath::getSid).distinct().collect(Collectors.toList());
                StringBuilder arrStr = new StringBuilder();
                int k = 0;
                for (String a : pointid) {
                    if (k == pointid.size() - 1) {
                        arrStr.append(a);
                    } else {
                        arrStr.append(a).append(",");
                    }
                    k++;
                }
                Double planPointPipeLengthSum = planPointRepository.getPlanPointPipeLengthSum(arrStr.toString());
                entity_child.setTOTAL(BigDecimal.valueOf(planPointPipeLengthSum == null ? 0 : planPointPipeLengthSum));
            } else {
                if (BigDecimal.ZERO.compareTo(entity_child.getTOTAL()) == 0) {
                    List<String> contents = listPath.stream().map(GeoClassPath::getContent).distinct().collect(Collectors.toList());
                    double pipelength = 0;
                    for (String content : contents) {
                        ContentModel cont = JSONObject.parseObject(content, ContentModel.class);
                        pipelength += cont.getPIPELENGTH();
                    }
                    entity_child.setTOTAL(BigDecimal.valueOf(pipelength));

                }

            }
            //线条数
            entity_child.setTOTAL1(BigDecimal.valueOf(Double.valueOf(listPath.size())));

        } else {
            //线总点数
            entity_child.setTOTAL(BigDecimal.valueOf(Double.valueOf(list.size())));
            //线总点数
            entity_child.setTOTAL1(BigDecimal.valueOf(Double.valueOf(list.size())));
            point_num = list.size();
        }

        // 添加配置参数
        if (null != settingplan) {
            if (0 < settingplan.size()) {
                // TODO DbCommand.AddRange(ToflyExtendDbHelper.GetInstance().AddSysDeviceSetting(settingplan, child_plan_id));
            }
        }

        if ("0".equals(type)) {
            // #region 巡检点
            for (GeoClass o : list) {
                PlanPointTemp entity_temp = new PlanPointTemp();
                entity_temp.setId(Long.parseLong(StringHelperUtil.NewId()));
                entity_temp.setGeometry(JSON.toJSONString(o.getGeometry()));
                entity_temp.setCode(child_plan_id);
                entity_temp.setUserId(Long.parseLong(entity.getUSERID()));
                entity_temp.setCompanyid(Long.parseLong(entity.getCOMPANYID()));
                entity_temp.setIsover("0");
                entity_temp.setRegionname(o.getRegionname());
                entity_temp.setContent(o.getContent() == null ? "" : o.getContent());
                entity_temp.setRegionId(Long.parseLong(entity.getREGION_ID()));
                entity_temp.setDeviceTypeId(Long.parseLong(o.getDevice_type_id()));
                entity_temp.setName(o.getName());
                entity_temp.setAddress(o.getAddress());
                entity_temp.setPeriodId(Long.parseLong(entity.getPERIOD_ID()));
                entity_temp.setLat(o.getGeometry().getX() == null ? BigDecimal.ZERO : o.getGeometry().getX());
                entity_temp.setLng(o.getGeometry().getY() == null ? BigDecimal.ZERO : o.getGeometry().getY());
                entity_temp.setCode(o.getCode());
                entity_temp.setState("0");
                entity_temp.setPipeLength("0");
                planPointTempRepository.save(entity_temp);
            }
            // 是否有纠正内容
            if (null != list1) {
                if (0 < list1.size()) {
                    // TODO  没有该实体 tf_plan_correct DbCommand.Add(ToflyExtendDbHelper.GetInstance().AddPlanCorrectCommand(child_plan_id, entity.CHILD_PLAN_ID, JsonConvert.SerializeObject(list1)));
                }
            }
            // #endregion
        } else {
            // #region 巡检线
            for (GeoClassPath o : listPath) {
                PlanPointTemp entity_temp = new PlanPointTemp();
                entity_temp.setId(Long.parseLong(StringHelperUtil.NewId()));
                entity_temp.setGeometry(JSON.toJSONString(o.getGeometry()));
                entity_temp.setChildPlanId(Long.parseLong(child_plan_id));
                entity_temp.setUserId(Long.parseLong(entity.getUSERID()));
                entity_temp.setCompanyid(Long.parseLong(entity.getCOMPANYID()));
                entity_temp.setIsover("0");
                entity_temp.setRegionname(o.getRegionname());
                entity_temp.setContent(o.getContent() == null ? "" : o.getContent());
                entity_temp.setRegionId(Long.parseLong(entity.getREGION_ID()));
                entity_temp.setDeviceTypeId(Long.parseLong(o.getDevice_type_id()));
                entity_temp.setName(o.getName());
                entity_temp.setAddress(o.getAddress());
                entity_temp.setPeriodId(Long.parseLong(entity.getPERIOD_ID()));
                entity_temp.setCode(o.getCode());
//                entity_temp.set(BigDecimal.valueOf(Double.valueOf(1)));
                entity_temp.setState("0");
                entity_temp.setPipeLength(o.getPipe_length());
//                entity_temp.setr(o.getRfid());
                GeometryPath paths = JSONObject.parseObject(JSON.toJSONString(o.getGeometry()), GeometryPath.class);
                int k = 1;
                for (List<Double> p : paths.getPaths().get(0)) {
                    PlanPathTemp pathTemp = new PlanPathTemp();
                    pathTemp.setId(Long.parseLong(StringHelperUtil.NewId()));
                    pathTemp.setTempId(entity_temp.getId());
                    pathTemp.setPointId(entity_temp.getPointId());
                    pathTemp.setLat(BigDecimal.valueOf(p.get(1)));
                    pathTemp.setLng(BigDecimal.valueOf(p.get(0)));
                    pathTemp.setIsover("");
                    pathTemp.setNum(BigDecimal.valueOf(k));
//                    pathTemp.setC(entity_temp.getCOMPANYID());
                    pathTemp.setChildPlanId(Long.parseLong(child_plan_id));
                    planPathTempRepository.save(pathTemp);
                    k++;
                    point_num++;
                }
                planPointTempRepository.save(entity_temp);
            }
            // 是否有纠正内容
            if (null != listPath1) {
                if (0 < listPath1.size()) {
                    // TODO 缺少tf_plan_correct DbCommand.Add(ToflyExtendDbHelper.GetInstance().AddPlanCorrectCommand(child_plan_id, entity.CHILD_PLAN_ID, JsonConvert.SerializeObject(listPath1)));
                }
                // #endregion
            }
            entity_child.setPOINT_NUM(BigDecimal.valueOf(point_num));
            this.addChildCommand(entity_child);
        }
    }

    private void addChildCommand(TF_PLAN_CHILD entity) {
        PlanChild planChild = new PlanChild();
        planChild.setId(Long.parseLong(entity.getCHILD_PLAN_ID()));
        planChild.setChildNum(entity.getCHILD_NUM());
        planChild.setChildBeginDate(entity.getCHILD_BEGIN_DATE());
        planChild.setChildEndDate(entity.getCHILD_END_DATE());
        planChild.setPeriodId(Long.parseLong(entity.getPERIOD_ID()));
        planChild.setLastTime(entity.getLAST_TIME());
        planChild.setUserId(Long.parseLong(entity.getUSERID()));
        planChild.setCompanyid(Long.parseLong(entity.getCOMPANYID()));
        planChild.setRegionId(Long.parseLong(entity.getREGION_ID()));
        planChild.setChildPlanState(entity.getCHILD_PLAN_STATE().toString());
        planChild.setIsovertototal(entity.getISOVERTOTOTAL().doubleValue());
        planChild.setPercent1(entity.getPERCENT1().doubleValue());
        planChild.setPointNum(entity.getPOINT_NUM().longValue());
        planChildRepository.save(planChild);
    }

    /**
     * 当巡检人员的登录账号有修改时，要同步修改隐患中间表的巡检人员账号信息
     *
     * @param sql_list
     */
    private void Up_Third_Trouble_User(List<String> sql_list) {
        // #region
        //同步隐患数据
        // TODO 无实体  third_trouble@third_trouble_dblink
        // #endregion
    }


    @Data
    class TF_PLAN_PATH_TEMP {
        private String ID;
        private String TEMP_ID;
        private String POINT_ID;
        private BigDecimal LAT;
        private BigDecimal LNG;
        private BigDecimal ISOVER;
        private BigDecimal NUM;
        private Date OVER_DATE;
        private BigDecimal PERCENT;
        private BigDecimal GPS_SPEED;
        private String CHILD_PLAN_ID;
        private Date FIRST_LEFT_TIME;
        private BigDecimal SID;
        private BigDecimal STATE;
        private String COMPANYID;
    }


    @Data
    class TF_PLAN_POINT_TEMP {
        private String ID;
        private String POINT_ID;
        private BigDecimal ISOVER;
        private String CHILD_PLAN_ID;
        private String USERID;
        private String COMPANYID;
        private String REGION_ID;
        private String DEVICE_TYPE_ID;
        private String PERIOD_ID;
        private String GEOMETRY;
        private String NAME;
        private String ADDRESS;
        private String REGIONNAME;
        private BigDecimal LAT;
        private BigDecimal LNG;
        private BigDecimal STATE;
        private String TROUBLE_ID;
        private Date OVER_DATE;
        private BigDecimal PATH_ISOVER;
        private String NOTES;
        private String CODE;
        private String CONTENT;
        private BigDecimal GPS_SPEED;
        private String PIPE_LENGTH;
        private BigDecimal TYPE;
        private BigDecimal SID;
        private String RFID;
    }


    @Data
    class ContentModel {
        private double PIPELENGTH;
    }

    @Data
    class GeoClass {
        private String sid;
        private Geometry geometry;
        private String name;
        private String address;
        private String device_type_id;
        private String regionname;

        private String code;
        private String content;

        private String pipe_length;

        private String rfid;
    }

    // 巡检线
    @Data
    class GeoClassPath {
        private String sid;
        private GeometryPath geometry;
        private String name;
        private String address;
        private String device_type_id;
        private String code;
        private String regionname;
        private String pipe_length;
        private String content;

        private String rfid;
    }

    @Data
    class Geometry {
        private BigDecimal x;
        private BigDecimal y;

        private SpatialReference spatialReference;
    }

    @Data
    class GeometryPath {
        private List<List<List<Double>>> paths;
        private SpatialReference spatialReference;
    }

    @Data
    class SpatialReference {
        private int wkid;

    }

    @Data
    class Setting_Plan {

        /**
         * 参数中文名字[选填]
         */

        public String cnname;

        /**
         * 参数英文名字[选填]
         */
        public String enname;

        /**
         * 参数类型 0 选填 1 不用填 2 必填[选填]
         */
        public BigDecimal type;

        /**
         * 是否拍照 0：选拍 1：必拍[选填]
         */
        public BigDecimal isphoto;

    }

    @Data
    class CollectModel {
        private String id;
        private String point_id;
    }

    @Data
    class TF_PLAN_CHILD {
        private String CHILD_PLAN_ID;
        private BigDecimal CHILD_NUM;
        private Date CHILD_BEGIN_DATE;
        private Date CHILD_END_DATE;
        private String PERIOD_ID;
        private String DEVICE_TYPE_ID;
        private BigDecimal PERCENT;
        private Date LAST_TIME;
        private String USERID;
        private String COMPANYID;
        private String REGION_ID;
        private BigDecimal CHILD_PLAN_STATE;
        private BigDecimal PERCENT1;
        private BigDecimal TOTAL1;
        private BigDecimal ISOVERTOTOTAL1;
        private BigDecimal ISOVERTOTOTAL;
        private BigDecimal TOTAL;
        private BigDecimal COPY_STATE;
        private String POINT_INFO;
        private BigDecimal PLAN_MODE;
        private String PLAN_NAME;
        private String MAP_RANGE;
        private BigDecimal ISJURISDICTION;
        private BigDecimal INTERVAL_DAYS;
        private BigDecimal N;
        private String DESCRIPTION;
        private Date CREATE_TIME;
        private String CREATE_USER;
        private BigDecimal PERIOD_TYPE;
        private BigDecimal COPY_NUM;
        private BigDecimal POINT_NUM;
        private BigDecimal PLAN_TYPE;

        private BigDecimal TYPE;
    }

}