package com.tofly.dyrqSchedule.repository;


import com.tofly.dyrqSchedule.entity.ConjunctiveQuery;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

/**
 * 联合查询
 */
@Repository
public interface ConjunctiveQueryRepository extends JpaRepository<ConjunctiveQuery, Long> {
    /**
     * @param userId
     * @param startDate
     * @param endDate
     * @return
     */
    @Query(value = "SELECT " +
            "CASE WHEN d.TYPE = 0 THEN b.LAT ELSE c.LAT END LAT ," +
            "CASE WHEN d.TYPE = 0 THEN b.LNG ELSE c.LNG END LNG ," +
            "CASE WHEN d.TYPE = 0 THEN b.ISOVER ELSE  c.ISOVER END ISOVER ," +
            "CASE WHEN d.TYPE = 0 THEN b.ID ELSE c.ID END TEMP_ID ," +
            "CASE WHEN d.TYPE = 0 THEN NULL ELSE b.ID END PATH_ID ," +
            "d.TYPE,b.USER_ID,c.NUM,a.ID PLAN_ID,a.DEVICE_TYPE_ID,a.CHILD_BEGIN_DATE,a.CHILD_END_DATE " +
            "FROM " +
            "TF_PLAN_CHILD a LEFT JOIN TF_PLAN_POINT_TEMP b " +
            "ON a.ID = b.CHILD_PLAN_ID LEFT JOIN TF_PLAN_PATH_TEMP c " +
            "ON b.ID = c.TEMP_ID LEFT JOIN TF_PLAN_POINT_TYPE d " +
            "ON b.DEVICE_TYPE_ID = d.ID " +
            "WHERE a.USER_ID = ?1 " +
            "AND a.CHILD_BEGIN_DATE >= ?2 " +
            "AND a.CHILD_END_DATE <= ?3 " +
            "AND(b.ISOVER = 0 OR c.ISOVER = 0)"
            , nativeQuery = true)
    List<Map<String,Object>> getPointDt(Long userId, Date startDate, Date endDate);

    @Query(value = "SELECT b.TYPE, " +
            "e.PLAN_MODE, " +
            "a.DEVICE_TYPE_ID, " +
            "a.TOTAL, " +
            "a.TOTAL1, " +
            "a.CHILD_BEGIN_DATE, " +
            "a.CHILD_END_DATE, " +
            "c.GEOMETRY " +
            "FROM TF_PLAN_CHILD a " +
            "LEFT JOIN TF_PLAN_POINT_TYPE b " +
            "ON a.DEVICE_TYPE_ID = b.ID " +
            "LEFT JOIN TF_GPS_REGION c " +
            "ON a.REGION_ID = c.ID " +
            "LEFT JOIN TF_PLAN e " +
            "on e.id = a.MAIN_PLAN_ID " +
            "WHERE a.ID = ?1 ", nativeQuery = true)
    List<Map<String,Object>> getSettingDt(Long planId);


    @Query(value = "SELECT  (SELECT NVL(COUNT(*),0) " +
            "FROM TF_PLAN_POINT_TEMP t " +
            "WHERE t.CHILD_PLAN_ID = ?1) TOTAL, " +
            "  (SELECT NVL(COUNT(*),0) " +
            "   FROM TF_PLAN_POINT_TEMP T " +
            "   WHERE t.CHILD_PLAN_ID = ?1 " +
            "   AND t.STATE = 1) ISOVERTOTAL " +
            "FROM DUAL", nativeQuery = true)
    List<Map<String,Object>> getPlanPointNvl(Long childPlanId);

    /**
     * 有效时常单独计算
     * @param userId
     * @param startDate
     * @param endDate
     * @return
     */
    @Query(value = "SELECT new NVL(SUM(t.DISTANCE), 0) VALIDDISTANCE, NVL(SUM(t.GPS_TIMESPAN), 0) VALIDTIME " +
            "FROM TF_GPS_UPLOAD t " +
            "WHERE t.USERID = ?1 " +
            "  AND t.CHECK_TIME >=  ?2 " +
            "  AND t.CHECK_TIME <= ?3 " +
            "  AND t.ISVALID=1 " +
            "  AND t.GPS_TYPE=0 " +
            "  AND t.TEMP = 1 " +
            "ORDER BY t.CHECK_TIME", nativeQuery = true)
    List<Map<String,Object>> getPlanPointValid(Long userId ,Date startDate ,Date endDate);

    /**
     *  服务器更新线到位比例
     * @param childPlanId
     * @return
     */
    @Query(value = "SELECT " +
            "(SELECT COUNT(*) " +
            "      FROM TF_PLAN_PATH_TEMP t " +
            "     WHERE t.CHILD_PLAN_ID = ?1 " +
            "       AND t.ISOVER = 1) ISOVERTOTAL, " +
            "   (SELECT COUNT(*) " +
            "      FROM TF_PLAN_PATH_TEMP t " +
            "     WHERE t.CHILD_PLAN_ID = ?1) TOTAL, " +
            "   (SELECT COUNT(*) " +
            "      FROM TF_PLAN_POINT_TEMP t " +
            "     WHERE t.CHILD_PLAN_ID = ?1 " +
            "       AND t.STATE = 1) TIAOSHU " +
            "FROM DUAL",nativeQuery = true)
    List<Map<String,Object>> getPlanPointStaticPlan(Long childPlanId);

    @Query(value = "SELECT (SELECT NVL(COUNT(*), 0) " +
            "FROM TF_PLAN_PATH_TEMP a " +
            "WHERE a.CHILD_PLAN_ID = ?1 " +
            "   AND a.ISOVER = 1 " +
            "   AND a.OVER_DATE >= " +
            "        ?2  " +
            "   AND a.OVER_DATE <= " +
            "        ?3 ) ISOVER, " +
            "(SELECT NVL(COUNT(*), 0) " +
            "  FROM TF_PLAN_PATH_TEMP a " +
            " WHERE a.CHILD_PLAN_ID = ?1) TOTAL, " +
            "(SELECT NVL(COUNT(*), 0) " +
            "FROM TF_PLAN_POINT_TEMP a " +
            "WHERE a.CHILD_PLAN_ID = ?1 " +
            "AND a.STATE =1 " +
            "AND a.OVER_DATE >= " +
            "     ?2  " +
            "AND a.OVER_DATE <= " +
            "     ?3 )NUM  FROM DUAL",nativeQuery = true)
    List<Map<String,Object>> getPlanPathTempTotal(Long childPlanId ,Date startDate ,Date endDate);

    /**
     * 取得当天需要计算有效时间地到位点
     * @param startDate
     * @param endDate
     * @param userId
     * @return
     */
    @Query(value = "SELECT a.LAT, a.LNG, a.OVER_DATE, a.ID, b.USER_ID,a.FIRST_LEFT_TIME " +
            "FROM TF_PLAN_PATH_TEMP a " +
            "LEFT JOIN TF_PLAN_CHILD b " +
            "ON a.CHILD_PLAN_ID = b.CHILD_PLAN_ID " +
            "WHERE a.OVER_DATE >= ?1 " +
            "   AND a.OVER_DATE <= ?2 " +
            "   AND a.ISOVER=1 " +
            "   AND b.USER_ID=?3 " +
            "   AND a.FIRST_LEFT_TIME IS NOT NULL " +
            "ORDER BY a.OVER_DATE ",nativeQuery = true)
    List<Map<String,Object>> getLocationTimeAay(Date startDate ,Date endDate ,String userId);

    /**
     * 自动签到签退
     * @param startDate
     * @param endDate
     * @param userId
     * @return
     */
    @Query(value = "SELECT (SELECT NVL(COUNT(*),0)  FROM TF_PDA_SIGN t " +
            "WHERE t.SIGN_TIME >=  ?1  " +
            " AND t.SIGN_TIME <=  ?2  " +
            " AND t.USERID = ?3 AND t.SIGN_TYPE='1')ON_NUM,  " +
            "(SELECT NVL(COUNT(*),0)  FROM TF_PDA_SIGN t " +
            "WHERE t.SIGN_TIME >=  ?1  " +
            " AND t.SIGN_TIME <=  ?2  " +
            " AND t.USERID = ?3 AND t.SIGN_TYPE='2')OFF_NUM FROM DUAL",nativeQuery = true)
    List<Map<String,Object>> getPdaSignOnOffNum(Date startDate , Date endDate , String userId);

    @Query(value = "SELECT a.*, " +
            "b.TYPE, " +
            "c.PERIOD_NAME, " +
            "c.PERIOD_TYPE " +
            "FROM TF_PLAN_CHILD a " +
            "LEFT JOIN TF_PLAN_POINT_TYPE b   " +
            "ON a.DEVICE_TYPE_ID=b.ID " +
            "LEFT JOIN TF_PLAN_PERIOD_TYPE c  " +
            "ON c.ID=a.PERIOD_ID " +
            "WHERE  a.COPY_STATE=1 AND a.CHILD_PLAN_STATE=2  AND a.COPY_NUM>0 ",nativeQuery = true)
    List<Map<String,Object>> getAutoCopyData();
}
