package com.tofly.dyrqSchedule.repository;


import com.tofly.dyrqSchedule.entity.PlanPointTemp;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Modifying;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import java.math.BigDecimal;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Repository
public interface PlanPointTempRepository extends JpaRepository<PlanPointTemp, Long> {
    @Query(value = " UPDATE TF_PLAN_POINT_TEMP t " +
            "SET t.ISOVER = 1, t.OVER_DATE = ?3, " +
            "t.GPS_SPEED = ?4 WHERE t.ID = ?1 AND t.child_plan_id = ?2", nativeQuery = true)
    @Modifying
    Integer updatePlanPointTemp(Long tempId, Long planId, Date checkTime, BigDecimal gpsSpeed );

    /**
     * 更新巡检点记录表中的管线长度
     * @param pipeLength
     * @param nickNameSid
     * @param childPlanId
     * @return
     */
    @Query(value = "UPDATE TF_PLAN_POINT_TEMP t SET T.PIPE_LENGTH=?1 WHERE t.POINT_ID=?2 AND t.CHILD_PLAN_ID=?3", nativeQuery = true)
    @Modifying
    Integer updatePlanPointTempLengthLine (String pipeLength, long nickNameSid,long childPlanId);

    @Query(value = "SELECT * FROM TF_PLAN_POINT_TEMP t  WHERE t.CHILD_PLAN_ID=?1 AND t.ISOVER=?2", nativeQuery = true)
    List<Map<String,Object>> getByChildPlanIdAndIsover(Long childPlanId , String isover);

    @Query(value = "UPDATE TF_PLAN_POINT_TEMP t SET " +
                    "t.ISOVER=1, " +
                    "t.STATE=1, " +
                    "t.OVER_DATE=?1, " +
                    "t.GPS_SPEED=?2 WHERE t.ID  IN (?3)",nativeQuery = true)
    @Modifying
    Integer updatePlanPointTempInPlace(Date checkTime ,double gpsSpeed ,String arrStr );

    @Query(value = "SELECT (SELECT NVL(COUNT(*), 0) " +
            "FROM TF_PLAN_POINT_TEMP a " +
            "WHERE a.CHILD_PLAN_ID = ?1 " +
            "AND a.STATE = 1 " +
            "AND a.OVER_DATE >= ?2 " +
            "AND a.OVER_DATE <= ?3 )ISOVER FROM DUAL ",nativeQuery = true)
    BigDecimal getPlanPointTempIsover(Long childPlanId , Date startDate , Date endDate);

    @Query(value = "SELECT t.ID,t.POINT_ID " +
            "FROM TF_PLAN_POINT_TEMP t " +
            "WHERE t.CHILD_PLAN_ID = ?1 " +
            "AND t.POINT_ID <>'NULL' " +
            "AND t.POINT_ID <> 'UNDEFINED' " +
            "AND NOT EXISTS (SELECT t1.POINT_ID FROM {1} t1 " +
            "WHERE t1.POINT_ID = t.POINT_ID  AND t1.VALID=1)",nativeQuery = true)
    List<PlanPointTemp> getPlanPointTempIdAndPointId(long planId);
}
