package com.tofly.dyrqSchedule.service.impl;

import com.tofly.dyrqSchedule.service.CalLostPlanHandlerService;
import org.springframework.stereotype.Service;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tofly.dyrqSchedule.entity.*;
import com.tofly.dyrqSchedule.repository.*;
import com.tofly.dyrqSchedule.service.EsGpsUploadService;
import com.tofly.dyrqSchedule.util.*;
import com.tofly.dyrqSchedule.util.c_code.PointF;
import com.tofly.dyrqSchedule.util.c_code.Range;

import lombok.Data;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;
/**
 * @program: tofly-dyrq
 * @description:
 * @author: Sumer
 * @create: 2020-09-18 09:15
 */
@Service
public class CalLostPlanHandlerServiceImpl implements CalLostPlanHandlerService {
    @Autowired
    private SysSettingRepository sysSettingRepository;

    @Autowired
    private ConjunctiveQueryRepository conjunctiveQueryRepository;

    @Autowired
    private PlanPointTempRepository planPointTempRepository;

    @Autowired
    private PlanChildRepository planChildRepository;

    @Autowired
    private PlanPathTempRepository planPathTempRepository;

    @Autowired
    private GpsDayCalculateRepository gpsDayCalculateRepository;

    @Autowired
    private PointDayCalculateRepository pointDayCalculateRepository;

    @Autowired
    private EsGpsUploadService esGpsUploadService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void calLostPlan() {
        // 获取 到位最小距离 单位米(人巡最大容差)  车辆最大速度(最快参考速度)
        List<SysSetting> elements = sysSettingRepository.findFirstByOrderByIdAsc();
        double _distance = 0;
        double _car_max_speed = 0;
        if (elements.size() > 0) {
            SysSetting sysSetting = elements.get(0);
            _distance = sysSetting.getMinDistanceInPlace().doubleValue();
            _car_max_speed = sysSetting.getCarMaxSpeed().doubleValue();
        }
        LocalDateTime now = LocalDateTime.now();
        Date startDate = DateTimeUtil.convertLDTToDate(now.minusMinutes(20));
        // 向后推1天 减一秒
        Date endDate = DateTimeUtil.convertLDTToDate(DateTimeUtil.getAssignTodayEndTime(now));

        // 获取所有计划相关信息
        List<PlanChild> all = JSONObject.parseArray(JSONObject.toJSONString(planChildRepository.findPlanChildList(startDate, endDate)), PlanChild.class);
        for (PlanChild planChild : all) {
            Long planId = planChild.getId();
            Long userid = planChild.getUserId();

            // region 计算
            List<ConjunctiveQuery> settingDt = JSONObject.parseArray(JSONObject.toJSONString(conjunctiveQueryRepository.getSettingDt(planId)), ConjunctiveQuery.class);
            if (null != settingDt && settingDt.size() > 0) {
                ConjunctiveQuery cont = settingDt.get(0);

                // 获取 轨迹
                List<EsGpsUpload> gpsDt = esGpsUploadService.getCarTrack(userid, startDate, endDate, _car_max_speed, cont.getPlan_mode());
                if (null != gpsDt && gpsDt.size() > 0) {
                    List<Gps_Model> gpsModels = JSONArray.parseArray(JSONObject.toJSONString(gpsDt), Gps_Model.class);

                    // region 服务器计算点到位 type 0为计算更新点到位  1为计算更新线到位
                    if ("0".equals(cont.getType())) {
                        this.calculateUpdatePointsPlace(planId, gpsModels, _distance);
                    } else {
                        this.calculateUpdateLinesPlace(planId, userid, gpsModels, _distance, cont, startDate, endDate);
                    }
                    this.updatePlanStatistics(planId, userid, startDate, endDate, cont);

                }

            }
        }

    }

    /**
     * 计算更新点到位 点到位比例
     *
     * @param planId
     * @param gpsModels
     * @param _distance
     */
    private void calculateUpdatePointsPlace(Long planId, List<Gps_Model> gpsModels, double _distance) {
        List<PlanPointTemp> pointDt = JSONObject.parseArray(JSONObject.toJSONString(planPointTempRepository.getByChildPlanIdAndIsover(planId, "0")), PlanPointTemp.class);
        if (null != pointDt && pointDt.size() > 0) {
            List<Point_Model> point = JSONArray.parseArray(JSONObject.toJSONString(pointDt), Point_Model.class);
            String arrIds = "";
            double gpsSpeed = 0L;
            Date chekDate = new Date();
            for (Gps_Model que : gpsModels) {
                double lng1 = que.getLocationY().doubleValue();
                double lat1 = que.getLocationX().doubleValue();
                Range range = GpsHelperUtil.GpsRange(_distance, lng1, lat1);
                // 条件过滤
                List<Point_Model> hasPoint = point.stream().filter(m ->
                        m.getLng().compareTo(range.getXMin()) > 0
                                && range.getXMax().compareTo(m.getLng()) > 0
                                && m.getLat().compareTo(range.getYMin()) > 0
                                && range.getYMax().compareTo(m.getLat()) > 0).collect(Collectors.toList());
                if (hasPoint.size() > 0) {
                    List<String> ids = new ArrayList<>();
                    hasPoint.forEach(m -> ids.add(m.getId()));
                    StringBuilder arrStr = new StringBuilder();
                    for (String a : ids) {
                        if (0 == ids.size() - 1) {
                            arrStr.append(a);
                        } else {
                            arrStr.append(a).append(",");
                        }
                    }
                    arrIds = arrStr.toString();
                    gpsSpeed = que.getGpsSpeed().doubleValue();
                    chekDate = que.getCheckTime();
                }
            }

            planPointTempRepository.updatePlanPointTempInPlace(chekDate, gpsSpeed, arrIds);

            // endregion
            // region 更新点到位比例
            List<ConjunctiveQuery> staticPlan = JSONObject.parseArray(JSONObject.toJSONString(conjunctiveQueryRepository.getPlanPointNvl(planId)), ConjunctiveQuery.class);
            if (null != staticPlan && staticPlan.size() > 0) {
                ConjunctiveQuery con = staticPlan.get(0);
                BigDecimal isovertotal = BigDecimal.valueOf(Double.parseDouble(con.getIsovertotal()));
                BigDecimal total = con.getTotal();
                if (total.intValue() > 0) {
                    BigDecimal percent = (isovertotal.divide(total, BigDecimal.ROUND_HALF_UP).setScale(4, BigDecimal.ROUND_HALF_UP));
                    planChildRepository.updatePlanChildPercent(percent.doubleValue(), planId);
                }
            }
            // endregion
        }
    }

    /**
     * 计算更新线到位 线到位比例
     *
     * @param planId
     * @param userid
     * @param gpsModels
     * @param _distance
     * @param con
     * @param startDate
     * @param endDate
     */
    private void calculateUpdateLinesPlace(Long planId, Long userid, List<Gps_Model> gpsModels, double _distance, ConjunctiveQuery con, Date startDate, Date endDate) {
        // region 服务器更新线到位
        List<PlanPathTemp> pointDt = JSONObject.parseArray(JSONObject.toJSONString(planPathTempRepository.getByChildPlanIdAndIsover(planId, "0")), PlanPathTemp.class);
        if (null != pointDt && pointDt.size() > 0) {
            List<Point_Model> point = JSONArray.parseArray(JSONObject.toJSONString(pointDt), Point_Model.class);
            List<String> idsOverList = new ArrayList<>();
            for (Gps_Model que : gpsModels) {

                double lng1 = que.getLocationY().doubleValue();
                double lat1 = que.getLocationX().doubleValue();
                Range range = GpsHelperUtil.GpsRange(_distance, lng1, lat1);
                // 条件过滤
                List<Point_Model> hasOverPoint = point.stream().filter(m ->
                        m.getLng().compareTo(range.getXMin()) > 0
                                && range.getXMax().compareTo(m.getLng()) > 0
                                && m.getLat().compareTo(range.getYMin()) > 0
                                && range.getYMax().compareTo(m.getLat()) > 0
                                && !idsOverList.contains(m.getId())).collect(Collectors.toList());
                if (hasOverPoint.size() > 0) {
                    // 标记为轨迹为到位点
                    if (que.getState().intValue() != 1) {
                        esGpsUploadService.updateStateTemp(que.getId());
                    }
                    List<String> ids = new ArrayList<>();
                    hasOverPoint.forEach(m -> ids.add(m.getId()));
                    for (String id : ids) {
                        if (idsOverList.contains(id)) {
                            continue;
                        }
                        // 更新到位时间
                        planPathTempRepository.updatePlanPointInPlaceTime(que.getCheckTime(), que.getGpsSpeed().doubleValue(), id);
                        idsOverList.add(id);
                    }
                    continue;
                }
                // 判断当前点是否在片区范围内
                GeomeryModel region = JSON.toJavaObject(JSON.parseObject(con.getGeometry()), GeomeryModel.class);
                List<PointF> listPointF = new ArrayList<>();
                for (List<Float> p : region.getRings().get(0)) {
                    PointF pointf = new PointF();
                    pointf.setX(p.get(0));
                    pointf.setY(p.get(1));
                    listPointF.add(pointf);
                }
                PointF inputGps = new PointF();
                inputGps.setX(new BigDecimal(String.valueOf(lng1)).floatValue());
                inputGps.setY(new BigDecimal(String.valueOf(lat1)).floatValue());
                if (GpsHelperUtil.IsInside(inputGps, listPointF)) {
                    esGpsUploadService.updateGpsUploadScope(que.getId());
                }
            }
        } else {
            return;
        }
        // endregion
        // region 有效时常单独计算
        List<ConjunctiveQuery> validDt = JSONObject.parseArray(JSONObject.toJSONString(conjunctiveQueryRepository.getPlanPointValid(userid, startDate, endDate)), ConjunctiveQuery.class);
        BigDecimal valid_time = BigDecimal.ZERO;
        BigDecimal valid_distance = BigDecimal.ZERO;
        if (null != validDt && validDt.size() > 0) {
            ConjunctiveQuery query = validDt.get(0);
            valid_distance = new BigDecimal(query.getValiddistance());
            valid_time = new BigDecimal(query.getValidtime());
        }
        // endregion
        // region 里程统计更新
        List<GpsDayCalculate> isHas = gpsDayCalculateRepository.getGpsDayCalculateMileageStatistics(startDate, endDate, userid);
        // 判断当前里程表是否存在数据，若存在，则更新
        GpsDayCalculate calculate = new GpsDayCalculate();
        if (null != isHas && isHas.size() > 0) {
            calculate.setValidTime(valid_time.doubleValue());
            calculate.setValidDistance(valid_distance.doubleValue());
            calculate.setId(isHas.get(0).getId());
            gpsDayCalculateRepository.saveAndFlush(calculate);
        } else {
            calculate.setUserid(userid);
            calculate.setPlanDate(startDate);
            calculate.setValidTime(valid_time.doubleValue());
            calculate.setValidDistance(valid_distance.doubleValue());
            gpsDayCalculateRepository.save(calculate);
        }
        // endregion
        // region 服务器更新线到位比例

        List<ConjunctiveQuery> staticPlan = JSONObject.parseArray(JSONObject.toJSONString(conjunctiveQueryRepository.getPlanPointStaticPlan(planId)), ConjunctiveQuery.class);
        if (null != staticPlan && staticPlan.size() > 0) {
            ConjunctiveQuery query = staticPlan.get(0);
            BigDecimal isovertotal = new BigDecimal(query.getIsovertotal());
            BigDecimal total = query.getTotal();
            BigDecimal tiaoshu = new BigDecimal(query.getTiaoshu());
            if (total.intValue() > 0) {
                BigDecimal percent = isovertotal.divide(total).setScale(4, BigDecimal.ROUND_HALF_UP);
                planChildRepository.updatePlanChildLine(percent.doubleValue(), tiaoshu.doubleValue(), planId);
            }
        }

        // endregion
    }

    /**
     * 更新计划统计
     *
     * @param planId
     * @param userid
     * @param startDate
     * @param endDate
     * @param cont
     */
    private void updatePlanStatistics(Long planId, Long userid, Date startDate, Date endDate, ConjunctiveQuery cont) {
        // region 更新计划统计
        List<PointDayCalculate> hasChildPlan = pointDayCalculateRepository.getStatisticsPatrolPlans(startDate, endDate, userid, planId);
        BigDecimal isovertototal = BigDecimal.ZERO;
        BigDecimal isovertototal1 = BigDecimal.ZERO;
        BigDecimal total = BigDecimal.ZERO;
        BigDecimal total1 = BigDecimal.ZERO;
        if (null != hasChildPlan && hasChildPlan.size() > 0) {
            PointDayCalculate calculate = hasChildPlan.get(0);
            total = calculate.getTotal();
            total1 = calculate.getTotal1();
            if ("0".equals(cont.getType())) {
                BigDecimal totalDt = planPointTempRepository.getPlanPointTempIsover(planId, startDate, endDate);
                if (null != totalDt) {
                    isovertototal = totalDt;
                    isovertototal1 = totalDt;
                }
            } else {
                List<ConjunctiveQuery> totalDt = JSONObject.parseArray(JSONObject.toJSONString(conjunctiveQueryRepository.getPlanPathTempTotal(planId, startDate, endDate)), ConjunctiveQuery.class);
                ConjunctiveQuery con = totalDt.get(0);
                isovertototal = Integer.parseInt(con.getIsover()) == 0 ? BigDecimal.ZERO : total.multiply(new BigDecimal(con.getIsover()).divide(con.getTotal()));
                isovertototal1 = new BigDecimal(con.getNum());
            }
            PointDayCalculate up = new PointDayCalculate();
            up.setTotal(total);
            up.setTotal1(total1);
            up.setIsovertototal(isovertototal.doubleValue());
            up.setIsovertototal1(isovertototal1);
            up.setChildBeginDate(cont.getChild_begin_date());
            up.setChildEndDate(cont.getChild_end_date());
            up.setId(calculate.getId());
            up.setChildPlanId(calculate.getChildPlanId());
            up.setPlanDate(startDate);
            up.setUserid(userid);
            pointDayCalculateRepository.saveAndFlush(up);

        } else {
            PointDayCalculate p = new PointDayCalculate();
            p.setPlanDate(startDate);
            p.setUserid(userid);
            p.setChildPlanId(planId);
            p.setDeviceTypeId(cont.getDevice_type_id());
            p.setTotal(total);
            p.setTotal1(total1);
            p.setIsovertototal(isovertototal.doubleValue());
            p.setIsovertototal1(isovertototal1);
            p.setChildBeginDate(cont.getChild_begin_date());
            p.setChildEndDate(cont.getChild_end_date());
            pointDayCalculateRepository.save(p);
        }
        // endregion
    }



    @Data
    static class apply {
        private String id;
        private Date start_date;
        private BigDecimal state;
    }

    @Data
    static class Gps_Model {
        private String id;
        private BigDecimal locationX;
        private BigDecimal locationY;
        private BigDecimal gpsSpeed;
        private BigDecimal state;
        private Date checkTime;
    }

    @Data
    static class Point_Model {
        private String id;
        private String temp_id;
        private BigDecimal lat;
        private BigDecimal lng;
        private BigDecimal isover;
    }

    @Data
    static class SpatialReference {
        private int wkid;
        private int latestWkid;
    }

    @Data
    static class GeomeryModel {
        private List<List<List<Float>>> rings;
        private SpatialReference spatialReference;
    }
}