package com.tofly.dyrqSchedule.service.impl;

import com.tofly.dyrqSchedule.entity.EsGpsUpload;
import com.tofly.dyrqSchedule.service.EsGpsUploadService;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.elasticsearch.core.ElasticsearchTemplate;
import org.springframework.data.elasticsearch.core.query.IndexQuery;
import org.springframework.data.elasticsearch.core.query.IndexQueryBuilder;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.data.elasticsearch.core.query.SearchQuery;
import org.springframework.stereotype.Service;


import java.math.BigDecimal;
import java.util.Date;
import java.util.List;

import static org.elasticsearch.index.query.QueryBuilders.*;
import static org.elasticsearch.index.query.QueryBuilders.matchQuery;

/**
 * @program: tofly-dyrq
 * @description:
 * @author: Sumer
 * @create: 2020-08-28 10:14
 */
@Service
public class EsGpsUploadServiceImpl implements EsGpsUploadService {

    @Autowired
    private ElasticsearchTemplate template;


    @Override
    public List<EsGpsUpload> getGpsDt(Long userId, Date startDate, Date endDate) {
        // 封装查询条件
        BoolQueryBuilder sql = boolQuery();
        sql.must(matchQuery("userid", userId));
        sql.must(matchQuery("isvalid", 1));
        sql.must(matchQuery("gpsType", 0));
        return send(startDate, endDate, sql);

    }


    @Override
    public List<EsGpsUpload> getCarTrack(Long userId, Date startDate, Date endDate, double carMaxSpeed, String gpsType) {
        BoolQueryBuilder sql = boolQuery();
        sql.must(matchQuery("userid", userId));
        sql.must(matchQuery("isvalid", 1));
        sql.must(rangeQuery("gpsTimespan").lt(300));
        sql.mustNot(matchQuery("network", 5)).should(matchQuery("network", 5)).must(rangeQuery("gpsPrecision").lt(30));
        sql.must(rangeQuery("speed").lte(carMaxSpeed));
        sql.must(matchQuery("gpsType", gpsType));
        return send(startDate, endDate, sql);
    }

    @Override
    public List<EsGpsUpload> getCarTrackOther(Long userId, Date startDate, Date endDate, double carMaxSpeed, String gpsType) {
        BoolQueryBuilder sql = boolQuery();
        sql.must(matchQuery("userid", userId));
        sql.must(matchQuery("isvalid", 1));
        sql.must(rangeQuery("speed").lte(carMaxSpeed));
        sql.must(matchQuery("gpsType", gpsType));
        return send(startDate, endDate, sql);
    }

    private List<EsGpsUpload> send(Date startDate, Date endDate, BoolQueryBuilder sql) {
        sql.filter((QueryBuilders.rangeQuery("checkTime").from(startDate.getTime()).to(endDate.getTime())));

        SearchQuery searchQuery = new NativeSearchQueryBuilder()
                .withQuery(sql)
                .build();

        return template.queryForList(searchQuery, EsGpsUpload.class);
    }


    @Override
    public void updateStateTemp(String id) {
        EsGpsUpload esGpsUpload = new EsGpsUpload();
        esGpsUpload.setId(id);
        esGpsUpload.setState(BigDecimal.ONE);
//        esGpsUpload.setTemp(BigDecimal.ONE)
        IndexQuery indexQuery = new IndexQueryBuilder().withObject(esGpsUpload).build();
        template.index(indexQuery);
    }

    @Override
    public void updateStateTempOther(String id) {
        EsGpsUpload esGpsUpload = new EsGpsUpload();
        esGpsUpload.setId(id);
        esGpsUpload.setState(BigDecimal.ONE);
        IndexQuery indexQuery = new IndexQueryBuilder().withObject(esGpsUpload).build();
        template.index(indexQuery);
    }

    @Override
    public void updateGpsUploadScope(String id) {
        EsGpsUpload esGpsUpload = new EsGpsUpload();
        esGpsUpload.setId(id);
//        esGpsUpload.setTemp(BigDecimal.ONE)
        IndexQuery indexQuery = new IndexQueryBuilder().withObject(esGpsUpload).build();
        template.index(indexQuery);
    }

}