package com.tofly.dyrqSchedule.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.tofly.dyrqSchedule.entity.*;
import com.tofly.dyrqSchedule.handler.GpsCalculateHandler;
import com.tofly.dyrqSchedule.repository.*;
import com.tofly.dyrqSchedule.service.EsGpsUploadService;
import com.tofly.dyrqSchedule.service.GpsCalculateHandlerService;
import com.tofly.dyrqSchedule.util.DateTimeUtil;
import com.tofly.dyrqSchedule.util.GpsHelperUtil;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.time.temporal.ChronoUnit;
import java.util.Date;
import java.util.List;
import java.util.Objects;

/**
 * @program: tofly-dyrq
 * @description: GPS统计服务
 * @author: Sumer
 * @create: 2020-09-11 15:52
 */
@Service
public class GpsCalculateHandlerServiceImpl implements GpsCalculateHandlerService {
    private static Logger logger = LoggerFactory.getLogger(GpsCalculateHandler.class);
    @Autowired
    private SysSettingRepository sysSettingRepository;

    @Autowired
    private PdaUserDeviceRepository pdaUserDeviceRepository;

    @Autowired
    private ConjunctiveQueryRepository conjunctiveQueryRepository;

    @Autowired
    private EsGpsUploadService esGpsUploadService;

    @Autowired
    private PlanPathTempRepository planPathTempRepository;

    @Autowired
    private PlanChildRepository planChildRepository;

    @Autowired
    private PlanPointTempRepository planPointTempRepository;

    @Transactional(rollbackFor = Exception.class)
    public void gpsCalculate() {
        long l = System.currentTimeMillis();
        // 获取相差时间
        int days = 1;
        // 获取到位最小距离 单位米(人巡最大容差)
        List<SysSetting> all = sysSettingRepository.findFirstByOrderByIdAsc();
        double minDistanceInPlace = 0;
        if (all.size() > 0) {
            minDistanceInPlace = all.get(0).getMinDistanceInPlace().doubleValue();
        }
        LocalDateTime dateTime = LocalDateTime.now();
        for (int q = 0; q < days; q++) {
            // 起始日期向后推q天
            LocalDateTime calStartLocalDateTime = dateTime.plusDays(q).minusMinutes(20);
            // 起始日期向后1+q天减1秒
            LocalDateTime calEndLocalDateTime = DateTimeUtil.getAssignTodayEndTime(dateTime).plusDays(q+1).minusSeconds(1);

            // 查询 巡检人员与移动设备关联对象集合
            List<PdaUserDevice> userDevices = JSONObject.parseArray(JSONObject.toJSONString(pdaUserDeviceRepository.selAll()), PdaUserDevice.class);
            if (userDevices.size() > 0) {
                for (PdaUserDevice p : userDevices) {
                    Long uId = p.getUserId();
                    Date sDate = DateTimeUtil.convertLDTToDate(calStartLocalDateTime);
                    Date eDate = DateTimeUtil.convertLDTToDate(calEndLocalDateTime);
                    List<ConjunctiveQuery> conjunctiveQuery = JSONObject.parseArray(JSONObject.toJSONString(conjunctiveQueryRepository.getPointDt(p.getUserId(), sDate, eDate)), ConjunctiveQuery.class);
                    if (null != conjunctiveQuery && conjunctiveQuery.size() > 0) {
                        // es 调用
                        List<EsGpsUpload> gpsDt = esGpsUploadService.getGpsDt(uId,sDate,eDate);
                        if (null != gpsDt && gpsDt.size() > 0) {
                            for (ConjunctiveQuery point : conjunctiveQuery) {
                                double lat1 = point.getLat().doubleValue();
                                double lng1 = point.getLng().doubleValue();
                                String type = point.getType();
                                Long tempId = point.getTemp_id();
                                Long planId = point.getPlan_id();
                                BigDecimal gpsSpeed = BigDecimal.ZERO;
                                Date checkTime = null;
                                for (EsGpsUpload esGpsUpload : gpsDt) {
                                    double lat2 = esGpsUpload.getLocationX().doubleValue();
                                    double lng2 = esGpsUpload.getLocationY().doubleValue();
                                    // 计算与人巡最大容差的距离
                                    double v = GpsHelperUtil.GetDistance(lng1, lat1, lat2, lng2);
                                    if (v > minDistanceInPlace) {
                                        continue;
                                    }
                                    gpsSpeed = esGpsUpload.getGpsSpeed();
                                    String check_time = DateTimeUtil.getLocalDateTime(DateTimeUtil.fromLong2LocalDateTime(esGpsUpload.getCheckTime().getTime()), DateTimeUtil.FULL_FORMAT);
                                    checkTime = DateTimeUtil.convertLDTToDate(Objects.requireNonNull(DateTimeUtil.fromString2LocalDateTime(check_time, DateTimeUtil.FULL_FORMAT)));

                                }
                                if ("1".equals(type)) {
                                    planPathTempRepository.updatePlanPointInPlaceLinePlace(tempId, planId,gpsSpeed, checkTime);
                                    planChildRepository.updateChildPlan(planId);
                                } else {
                                    planPointTempRepository.updatePlanPointTemp(tempId, planId,checkTime, gpsSpeed);
                                    planChildRepository.updateChildPlanOther(planId);
                                }
                            }
                        }
                    }

                }
            }

        }
        logger.info("GPS统计服务耗时: {}ms", System.currentTimeMillis() - l);
    }


}