package com.tofly.dyrqSchedule.util;

import com.alibaba.druid.util.DaemonThreadFactory;
import com.tofly.common.core.util.Date8Util;
import com.tofly.dyrqSchedule.entity.EsGpsUpload;
import com.tofly.dyrqSchedule.entity.EsQueryParam;
import com.tofly.dyrqSchedule.entity.GpsDayCalculate;
import com.tofly.dyrqSchedule.service.EsGpsUploadService;
import lombok.AllArgsConstructor;
import org.apache.poi.ss.formula.functions.T;
import org.elasticsearch.index.query.BoolQueryBuilder;
import org.elasticsearch.index.query.MatchQueryBuilder;
import org.elasticsearch.index.query.QueryBuilders;
import org.elasticsearch.index.query.RangeQueryBuilder;
import org.elasticsearch.search.aggregations.AggregationBuilders;
import org.elasticsearch.search.aggregations.metrics.sum.InternalSum;
import org.elasticsearch.search.aggregations.metrics.sum.SumAggregationBuilder;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.elasticsearch.core.ElasticsearchTemplate;
import org.springframework.data.elasticsearch.core.query.NativeSearchQueryBuilder;
import org.springframework.data.elasticsearch.core.query.SearchQuery;

import java.lang.reflect.MalformedParameterizedTypeException;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@AllArgsConstructor
public class EsQuery {

    private final EsGpsUploadService esGpsUploadService;

    /**
     * 总里程总耗时计算
     * @param elasticsearchTemplate
     * @param esQueryParam
     * @return
     */
    public static List  getGpsTotal(ElasticsearchTemplate elasticsearchTemplate, EsQueryParam esQueryParam) {
        BoolQueryBuilder qb = QueryBuilders.boolQuery();
        if (esQueryParam.getUserId() != null) {
            MatchQueryBuilder qbUserId = QueryBuilders.matchQuery("userid", esQueryParam.getUserId());
            qb.must(qbUserId);
        }
        if (esQueryParam.getGpsType() != null) {
            MatchQueryBuilder qbGpsType = QueryBuilders.matchQuery("gpsType", esQueryParam.getGpsType());
            qb.must(qbGpsType);
        }
        if (esQueryParam.getIsvalid() != null) {
            MatchQueryBuilder qbIsvalid = QueryBuilders.matchQuery("isvalid", esQueryParam.getIsvalid().toString());
            qb.must(qbIsvalid);
        }
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("checkTime");
        rangeQueryBuilder.from(esQueryParam.getStartDate().getTime());
        rangeQueryBuilder.to(esQueryParam.getEndDate().getTime());
        qb.must(rangeQueryBuilder);

        SearchQuery searchQuery = new NativeSearchQueryBuilder()
                .withQuery(qb)
                .addAggregation(AggregationBuilders.sum("gpsTimespan").field("gpsTimespan"))
                .addAggregation(AggregationBuilders.sum("distance").field("distance"))
                .build();
        List<Double> sumList = elasticsearchTemplate.query(searchQuery, response-> {
            InternalSum sum =  response.getAggregations().get("distance");
            InternalSum sum2 =  response.getAggregations().get("gpsTimespan");
            List<Double> list = new ArrayList<>();
            list.add(fix2(sum.getValue()));
            list.add(fix2(sum2.getValue()));
            return list;
        });
        return sumList;
    }


    public static List<EsGpsUpload> queryList(ElasticsearchTemplate elasticsearchTemplate, EsQueryParam esQueryParam, Class c) {
        BoolQueryBuilder qb = QueryBuilders.boolQuery();
//        if (esQueryParam.getUserId() != null) {
//            MatchQueryBuilder qbUserId = QueryBuilders.matchQuery("userid", esQueryParam.getUserId());
//            qb.must(qbUserId);
//        }
//        if (esQueryParam.getGpsType() != null) {
//            MatchQueryBuilder qbGpsType = QueryBuilders.matchQuery("gpsType", esQueryParam.getGpsType());
//            qb.must(qbGpsType);
//        }
//        if (esQueryParam.getIsvalid() != null) {
//            MatchQueryBuilder qbIsvalid = QueryBuilders.matchQuery("isvalid", esQueryParam.getIsvalid().toString());
//            qb.must(qbIsvalid);
//        }
        RangeQueryBuilder rangeQueryBuilder = QueryBuilders.rangeQuery("checkTime");
        rangeQueryBuilder.from(esQueryParam.getStartDate().getTime());
        rangeQueryBuilder.to(esQueryParam.getEndDate().getTime());
        qb.must(rangeQueryBuilder);

        SearchQuery searchQuery = new NativeSearchQueryBuilder()
                .withQuery(qb)
                .build();

        List<EsGpsUpload> list= elasticsearchTemplate.queryForList(searchQuery, c);
        return list;
    }

    /**
     * double 保留两位小数（四舍五入）
     * @param d
     * @return
     */
    private static Double fix2(Double d){
        BigDecimal b = new BigDecimal(d);
        return b.setScale(2, BigDecimal.ROUND_HALF_UP).doubleValue();
    }
}
