package com.tofly.dyrqSchedule.util;

import org.apache.commons.lang.ArrayUtils;

import java.time.Duration;
import java.time.LocalDateTime;
import java.util.*;

/**
 * @program: tofly-dyrq
 * @description: 获取计划开始时间与结束时间
 * @author: Sumer
 * @create: 2020-09-03 17:40
 */
public class GetStartEndDateUtil {
    /**
     * 获取计划开始时间与结束时间
     * @param period_name
     * @param start_date
     * @param end_date
     * @param n
     */
    public static void getMonth(String period_name, Date start_date, Date end_date, int n) {
        switch (period_name) {
            case "两天一巡":
                TwoDayXunjian(start_date, end_date, n);
                break;
            case "一周一巡":
                SevenDayXunjian(start_date, end_date, n);
                break;
            case "半月一巡":
                HalfMonthXunjian(start_date, end_date, n);
                break;
            case "一月一巡":
                OneMonthXunjian(start_date, end_date, n);
                break;
            case "两月一巡":
                TwoMonthXunjian(start_date, end_date, n);
                break;
            case "半年一巡":
                HalfYearXunjian(start_date, end_date, n);
                break;
            case "自定义计划":
                CustomXunjian(start_date, end_date, n);
                break;
            default:
                OneDayXunjian(start_date, end_date, n);
                break;
        }
    }

    /**
     * 两天一巡 跨周  跨月 跨年
     * @param start_date
     * @param end_date
     * @param n
     */
    private static void TwoDayXunjian(Date start_date, Date end_date, int n) {
        LocalDateTime sDate = DateTimeUtil.fromLong2LocalDateTime(end_date.getTime()).plusDays(n);
        start_date =  DateTimeUtil.convertLDTToDate(sDate);
        int month = sDate.getMonth().getValue();
        int year = sDate.getYear();
        LocalDateTime eDate = DateTimeUtil.fromLong2LocalDateTime(end_date.getTime());
        int eDateYear = eDate.getYear();
        int eDateMonth = eDate.getMonth().getValue();
        int eDateDayOfMonth = eDate.getDayOfMonth();
        //跨年
        if (year != eDateYear) {
            start_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(eDateYear, 1, 1, 0, 0));
        }else if (month != eDateMonth) {//跨月
            start_date = DateTimeUtil.convertLDTToDate(eDate.minusDays(eDateDayOfMonth - 1));
        }
        //跨周
        String week = "";
        LocalDateTime endDate = DateTimeUtil.fromLong2LocalDateTime(start_date.getTime());
        week = endDate.getDayOfWeek().toString();
        if ("sunday".equals(week)) {
            start_date = DateTimeUtil.convertLDTToDate(endDate.plusDays(1));
        }
        end_date = DateTimeUtil.convertLDTToDate(endDate.plusDays(1));
    }

    /**
     * 一周一巡  1    8    15   22
     * @param start_date
     * @param end_date
     * @param n
     */
    private static void SevenDayXunjian(Date start_date, Date end_date, int n) {
        LocalDateTime sDate = DateTimeUtil.fromLong2LocalDateTime(end_date.getTime()).plusDays(n);
        start_date = DateTimeUtil.convertLDTToDate(sDate);

        int[] arr = { 1, 8, 15, 22 };
        List<Integer> integers = Arrays.asList(ArrayUtils.toObject(arr));
        int day = sDate.getDayOfMonth();
        int month = sDate.getMonth().getValue();
        int year = sDate.getYear();
        if (integers.contains(day))
        {
            end_date = DateTimeUtil.convertLDTToDate(sDate.plusDays(6));
            return;
        }

        if (day > 1 && day < 8)
            start_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(year,month,8,0,0));

        else if (day > 8 && day < 15)
            start_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(year,month,15,0,0));

        else if (day > 15 && day < 22)
            start_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(year,month,22,0,0));

        else
        {
            //跨年
            if (month == 12)
                start_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(year+1,1,1,0,0));
            else
                start_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(year,month+1,1,0,0));
        }
        LocalDateTime eDate = DateTimeUtil.fromLong2LocalDateTime(start_date.getTime());
        end_date = DateTimeUtil.convertLDTToDate(eDate.plusDays(6));
    }

    /**
     * 半月一巡  1  15
     * @param start_date
     * @param end_date
     * @param n
     */
    private static void HalfMonthXunjian(Date start_date, Date end_date, int n) {
        LocalDateTime sDate = DateTimeUtil.fromLong2LocalDateTime(end_date.getTime()).plusDays(n);
        start_date = DateTimeUtil.convertLDTToDate(sDate);
        int day = sDate.getDayOfMonth();
        int month = sDate.getMonth().getValue();
        if (day == 1) {
            end_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear(), month, 15, 0, 0));
            return;
        } else if (day == 16) {
            if (month == 12) {
                end_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear() + 1, 1, 1, 0, 0).minusDays(1));
            } else {
                end_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear(), month + 1, 1, 0, 0).minusDays(1));
            }
            return;
        }


        if (day < 16) {
            start_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear(), month, 16, 0, 0));
            //跨年
            if (month == 12) {
                end_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear() + 1, 1, 1, 0, 0).minusDays(1));
            } else {
                end_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear(), month + 1, 1, 0, 0).minusDays(1));
            }
        } else {
            //跨年
            if (month == 12) {
                start_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear() + 1, 1, 1, 0, 0));
                end_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear(), 1, 15, 0, 0));
            } else {
                start_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear(), month + 1, 1, 0, 0));
                end_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear(), month, 15, 0, 0));
            }
        }
    }

    /**
     * 一月一巡 月初 月底
     *
     * @param start_date
     * @param end_date
     * @param n
     */
    private static void OneMonthXunjian(Date start_date, Date end_date, int n) {
        LocalDateTime sDate = DateTimeUtil.fromLong2LocalDateTime(end_date.getTime()).plusDays(n);
        start_date = DateTimeUtil.convertLDTToDate(sDate);
        int day = sDate.getDayOfMonth();
        int month = sDate.getMonth().getValue();
        if (day == 1) {
            end_date = DateTimeUtil.convertLDTToDate(sDate.plusMonths(1).minusDays(1));
            return;
        }

        //跨年
        if (month == 12) {
            start_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear() + 1, 1, 1, 0, 0));
        }else {
            start_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear(), month + 1, 1, 0, 0));
        }
        LocalDateTime eDate = DateTimeUtil.fromLong2LocalDateTime(start_date.getTime());
        end_date = DateTimeUtil.convertLDTToDate(eDate.plusMonths(1).minusDays(1));
    }

    /**
     * 两月一巡 月初 月底（不能跨年）
     *
     * @param start_date
     * @param end_date
     * @param n
     */
    private static void TwoMonthXunjian(Date start_date, Date end_date, int n) {
        LocalDateTime sDate = DateTimeUtil.fromLong2LocalDateTime(end_date.getTime()).plusDays(n);
        start_date = DateTimeUtil.convertLDTToDate(sDate);
        int day = sDate.getDayOfMonth();
        int month = sDate.getMonth().getValue();
        if (day == 1 && month < 12) {
            end_date = DateTimeUtil.convertLDTToDate(sDate.plusMonths(2).minusDays(1));
            return;
        }

        //跨年
        if (month == 12) {
            start_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear() + 1, 1, 1, 0, 0));
        }else {
            start_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear(), month + 1, 1, 0, 0));
        }
        LocalDateTime eDate = DateTimeUtil.fromLong2LocalDateTime(start_date.getTime());
        end_date = DateTimeUtil.convertLDTToDate(eDate.plusMonths(2).minusDays(1));
    }

    /**
     * 半年一巡  1  7  （不能跨年）
     *
     * @param start_date
     * @param end_date
     * @param n
     */
    private static void HalfYearXunjian(Date start_date, Date end_date, int n) {
        LocalDateTime sDate = DateTimeUtil.fromLong2LocalDateTime(end_date.getTime()).plusDays(n);
        start_date = DateTimeUtil.convertLDTToDate(sDate);
        int[] arr = new int[]{1, 7};//月份
        int day = sDate.getDayOfMonth();
        int month = sDate.getMonth().getValue();
        if (day == 1 && month == 1) {
            end_date = DateTimeUtil.convertLDTToDate(sDate.plusMonths(6).minusDays(1));
            return;
        } else if (day == 1 && month == 7) {

            end_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear(), 12, 31, 0, 0));
            return;
        }
        //跨年
        if (month < 7) {
            start_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear(), 7, 1, 0, 0));
            end_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear(), 12, 31, 0, 0));
        } else {
            start_date = DateTimeUtil.convertLDTToDate(LocalDateTime.of(sDate.getYear() + 1, 1, 1, 0, 0));
            end_date = DateTimeUtil.convertLDTToDate(sDate.plusMonths(6).minusDays(1));
        }
    }

    /**
     * 自定义计划
     *
     * @param start_date
     * @param end_date
     * @param n
     */
    private static void CustomXunjian(Date start_date, Date end_date, int n) {
        LocalDateTime sDate = DateTimeUtil.fromLong2LocalDateTime(start_date.getTime());
        LocalDateTime eDate = DateTimeUtil.fromLong2LocalDateTime(end_date.getTime());
        Duration ts = Duration.between(sDate, eDate);
        int interdays = (int) ts.toDays();
        start_date = DateTimeUtil.convertLDTToDate(sDate.plusDays(interdays + n));
        end_date = DateTimeUtil.convertLDTToDate(eDate.plusDays(interdays + n));
    }

    /**
     * 一天一巡
     *
     * @param start_date
     * @param end_date
     * @param n
     */
    private static void OneDayXunjian(Date start_date, Date end_date, int n) {
        start_date = DateTimeUtil.convertLDTToDate(DateTimeUtil.fromLong2LocalDateTime(start_date.getTime()).plusDays(n));
        end_date = start_date;
    }
}