package com.tofly.dyrqSchedule.util;

import com.tofly.dyrqSchedule.util.c_code.PointF;
import com.tofly.dyrqSchedule.util.c_code.Range;

import java.math.BigDecimal;
import java.util.List;

/**
 * @program: tofly-dyrq
 * @description: 经纬度处理
 * @author: Sumer
 * @create: 2020-08-25 14:54
 */
public class GpsHelperUtil {

    /**
     * 赤道半径
     */
    private static final double EARTH_RADIUS = 6378137;

    private static double rad(double d) {
        return d * Math.PI / 180.0;
    }

    private static double ConvertDegreesToRadians(double degrees) {
        double radians = (Math.PI / 180) * degrees;
        return radians;
    }

    /**
     * 通过GPS坐标计算两点间的距离
     *
     * @param lon1 纬度1
     * @param lat1 经度1
     * @param lon2 纬度2
     * @param lat2 经度2
     * @return
     */
    public static double GetDistance(double lon1, double lat1, double lon2, double lat2) {
        double radLat1 = rad(lat1);
        double radLat2 = rad(lat2);
        double a = radLat1 - radLat2;
        double b = rad(lon1) - rad(lon2);
        double s = 2 * Math.asin(Math.sqrt(Math.pow(Math.sin(a / 2), 2) + Math.cos(radLat1) * Math.cos(radLat2) * Math.pow(Math.sin(b / 2), 2)));
        s = s * EARTH_RADIUS;
        //单位米
        return s;
    }

    /**
     *  获取到位容差范围
     * @param check_radius
     * @param lng 经度
     * @param lat 纬度
     * @return
     */
    public static Range GpsRange(double check_radius, double lng, double lat) {
        // 检测单位，一千米
        int check_units = 1000;
        double check_scale = (double) check_units / check_radius;
        double d_lng_radius_unit = Math.cos(ConvertDegreesToRadians(23.14));
        // 1km->经度
        double d_lng_scale = 1 / (6369 * d_lng_radius_unit * 2 * Math.PI / 360);
        // 1km->纬度
        double d_lat_scale = 1 / (6369 * 2 * Math.PI / 360);
        double xscale = d_lng_scale / check_scale;
        double yscale = d_lat_scale / check_scale;
        Range range = new Range();
        range.setXMin(BigDecimal.valueOf(lng - xscale));
        range.setXMax(BigDecimal.valueOf(lng + xscale));
        range.setYMin(BigDecimal.valueOf(lat - yscale));
        range.setYMax(BigDecimal.valueOf(lat + yscale));
        return range;
    }

    public static boolean IsInside(PointF p, List<PointF> Points)
    {
        int count = Points.size();

        if (count < 3)
        {
            return false;
        }

        boolean result = false;

        for (int i = 0, j = count - 1; i < count; i++)
        {
            PointF p1 = Points.get(i);
            PointF p2 = Points.get(j);

            if (p1.getY() < p.getY() && p2.getY() >= p.getY() || p2.getY() < p.getY() && p1.getY() >= p.getY())
            {
                if (p1.getX() + (p.getY() - p1.getY()) / (p2.getY() - p1.getY()) * (p2.getX() - p1.getX()) < p.getX())
                {
                    result = !result;
                }
            }
            j = i;
        }
        return result;
    }

}