﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CalLostPlan
{
    public class GpsHelper
    {
        private const double EARTH_RADIUS = 6378137;
        private static double rad(double d)
        {
            return d * Math.PI / 180.0;
        }
        /// <summary>
        /// 通过GPS坐标计算两点间的距离
        /// </summary>
        /// <param name="lat1">纬度1</param>
        /// <param name="lng1">经度1</param>
        /// <param name="lat2">纬度2</param>
        /// <param name="lng2">经度2</param>
        /// <returns></returns>
        public static double Distance(double lat1, double lng1, double lat2, double lng2)
        {
            double radLat1 = rad(lat1);
            double radLat2 = rad(lat2);
            double a = radLat1 - radLat2;
            double b = rad(lng1) - rad(lng2);
            double s = 2 * Math.Asin(Math.Sqrt(Math.Pow(Math.Sin(a / 2), 2) +
            Math.Cos(radLat1) * Math.Cos(radLat2) * Math.Pow(Math.Sin(b / 2), 2)));
            s = s * EARTH_RADIUS;
            s = Math.Round(s * 10000) / 10000;
            return s;
        }

        /// <summary>
        /// 获取到位容差范围
        /// </summary>
        /// <param name="check_radius"></param>
        /// <param name="lng">经度</param>
        /// <param name="lat">纬度</param>
        /// <returns></returns>
        public static Range GpsRange( double check_radius, double lng, double lat)
        {
            /*检测单位，一千米*/
            int check_units = 1000;
            double check_scale = (double)check_units / check_radius;
            double d_lng_radius_unit = Math.Cos(ConvertDegreesToRadians(23.14));
            /*1km->经度*/
            double d_lng_scale = 1 / (6369 * d_lng_radius_unit * 2 * Math.PI / 360);
            /*1km->纬度*/
            double d_lat_scale = 1 / (6369 * 2 * Math.PI / 360);
            double xscale = d_lng_scale / check_scale;
            double yscale = d_lat_scale / check_scale;
            Range range = new CalLostPlan.Range();
            range.xMin = decimal.Parse((lng - xscale).ToString());
            range.xMax = decimal.Parse((lng + xscale).ToString());
            range.yMin = decimal.Parse((lat - yscale).ToString());
            range.yMax = decimal.Parse((lat + yscale).ToString());
            return range;
        }

        public static bool IsInside(PointF p, List<PointF> Points)
        {
            int count = Points.Count;

            if (count < 3)
            {
                return false;
            }

            bool result = false;

            for (int i = 0, j = count - 1; i < count; i++)
            {
                var p1 = Points[i];
                var p2 = Points[j];

                if (p1.Y < p.Y && p2.Y >= p.Y || p2.Y < p.Y && p1.Y >= p.Y)
                {
                    if (p1.X + (p.Y - p1.Y) / (p2.Y - p1.Y) * (p2.X - p1.X) < p.X)
                    {
                        result = !result;
                    }
                }
                j = i;
            }
            return result;
        }
        private static double ConvertDegreesToRadians(double degrees)
        {
            double radians = (Math.PI / 180) * degrees;
            return radians;
        }
    }

    public class Range
    {
        public decimal xMin { get; set; }
        public decimal xMax { get; set; }
        public decimal yMin { get; set; }
        public decimal yMax { get; set; }
    }
}
