﻿using Oracle.ManagedDataAccess.Client;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Configuration;
using System.Data;
using System.Data.Common;

namespace SyncMonitor
{
    /// <summary>
    /// 扩展数据操作类，执行存储过程
    /// </summary>
    public sealed class ToflyExtendDbHelper
    {
        private static OracleConnection conn;
        private static ToflyExtendDbHelper _instance;
        public static ToflyExtendDbHelper GetInstance()
        {
            if (_instance == null)
            {
                if (conn==null)
                {
                    conn = new OracleConnection(ConfigurationManager.ConnectionStrings["cqgas"].ConnectionString);
                }

                if (conn.State == ConnectionState.Closed)
                {
                    conn.Open();
                }
                else if (conn.State == ConnectionState.Broken)
                {
                    conn.Close();
                    conn.Open();
                }
                _instance = new ToflyExtendDbHelper();
            }
            return _instance;
        }

        /// <summary>
        /// 执行sql语句返回DataTable
        /// </summary>
        /// <param name="sql"></param>
        /// <returns></returns>
        public  DataTable ExecuteSql(string sql)
        {
            try
            {
                OracleDataAdapter command = new OracleDataAdapter(sql, conn);
                DataSet ds = new DataSet();
                command.Fill(ds, "ds");
                return ds.Tables[0];
            }
            catch (Exception e)
            {
                Logger.WriteErrorLog("ExecuteSql执行失败" + "\r\n" + e.Message + "\r\n" + e.StackTrace);
                return null;
            }
        }

       /// <summary>
       /// 
       /// </summary>
       /// <param name="SQLString"></param>
       /// <returns></returns>
        public  int ExecuteOneSql(string SQLString)
        {
            using (OracleCommand cmd = new OracleCommand(SQLString, conn))
            {
                try
                {

                    int rows = cmd.ExecuteNonQuery();
                    return rows;
                }
                catch (System.Data.SqlClient.SqlException e)
                {
                    Logger.WriteErrorLog("ExecuteOneSql执行失败" + "\r\n" + e.Message + "\r\n" + e.StackTrace);
                    return -1;
                }
            }
        }

        /// <summary>
        /// 提交一组（多条）SQL语句操作数据库
        /// </summary>
        /// <param name="commandStringList">SQL列表</param>
        /// <returns>执行结果</returns>
        public  int ExcuteBatchCommand(List<string> commandStringList)
        {
            OracleTransaction m_OraTrans = conn.BeginTransaction();//创建事务对象
            OracleCommand com;
            com = new OracleCommand();
            com.Connection = conn;
            com.Transaction = m_OraTrans;
            string tmpStr = "";
            int influenceRowCount = 0;
            try
            {
                foreach (string commandString in commandStringList)
                {
                    tmpStr = commandString;
                    com.CommandText = tmpStr;
                    influenceRowCount += com.ExecuteNonQuery();
                }
                m_OraTrans.Commit();
                return influenceRowCount;
            }
            catch (OracleException e)
            {
                m_OraTrans.Rollback();
                Logger.WriteErrorLog("ExcuteBatchCommand" + "\r\n" + e.Message + "\r\n" + e.StackTrace);
                return -1;
            }
        }

        public DbCommand UpdatePlanCommand(string child_plan_id)
        {
            DbCommand DbCommand = conn.CreateCommand();
            DbCommand.CommandText = string.Format("update tf_plan_child  set copy_state=2 where child_plan_id='{0}'", child_plan_id);
            return DbCommand;
        }

        public  bool ExecDB(List<DbCommand> DbCommands)
        {
            bool Result = false;
            DbCommand Command = null;
            DbTransaction Trans = null;

            if (DbCommands == null)
            {
                return true;
            }
            if (DbCommands.Count == 0)
            {
                return true;
            }
            Command = new OracleCommand();
            try
            {
                Trans = conn.BeginTransaction();
                int i = 0;
                for (i = 0; i < DbCommands.Count; i++)
                {
                    Command = DbCommands[i];
                    Command.Connection = conn;
                    Command.Transaction = Trans;
                    Command.ExecuteNonQuery();
                }
                Trans.Commit();
                Result = true;
            }
            catch (Exception e)
            {
                if (Trans != null)
                {
                    Trans.Rollback();
                }
                Result = false;
                Logger.WriteErrorLog("ExcuteBatchCommand" + "\r\n" + e.Message + "\r\n" + e.StackTrace);
            }
            finally
            {
                Trans = null;
            }
            return Result;
        }
        public  DbCommand AddChildCommand(TF_PLAN_CHILD entity)
        {
            DbCommand DbCommand = conn.CreateCommand();

            DbParameter child_plan_id = DbCommand.CreateParameter();
            child_plan_id.ParameterName = ":child_plan_id";
            child_plan_id.Value = entity.CHILD_PLAN_ID;
            DbCommand.Parameters.Add(child_plan_id);


            DbParameter child_num = DbCommand.CreateParameter();
            child_num.ParameterName = ":child_num";
            child_num.Value = entity.CHILD_NUM;
            DbCommand.Parameters.Add(child_num);

            DbParameter child_begin_date = DbCommand.CreateParameter();
            child_begin_date.ParameterName = ":child_begin_date";
            child_begin_date.DbType = DbType.Date;
            child_begin_date.Value = entity.CHILD_BEGIN_DATE;
            DbCommand.Parameters.Add(child_begin_date);

            DbParameter child_end_date = DbCommand.CreateParameter();
            child_end_date.ParameterName = ":child_end_date";
            child_end_date.DbType = DbType.Date;
            child_end_date.Value = entity.CHILD_END_DATE;
            DbCommand.Parameters.Add(child_end_date);

            DbParameter period_id = DbCommand.CreateParameter();
            period_id.ParameterName = ":period_id";
            period_id.Value = entity.PERIOD_ID;
            DbCommand.Parameters.Add(period_id);

            DbParameter device_type_id = DbCommand.CreateParameter();
            device_type_id.ParameterName = ":device_type_id";
            device_type_id.Value = entity.DEVICE_TYPE_ID;
            DbCommand.Parameters.Add(device_type_id);

            DbParameter percent = DbCommand.CreateParameter();
            percent.ParameterName = ":percent";
            percent.Value = entity.PERCENT;
            DbCommand.Parameters.Add(percent);

            DbParameter last_time = DbCommand.CreateParameter();
            last_time.ParameterName = ":last_time";
            last_time.DbType = DbType.Date;
            last_time.Value = entity.LAST_TIME;
            DbCommand.Parameters.Add(last_time);

            DbParameter userid = DbCommand.CreateParameter();
            userid.ParameterName = ":userid";
            userid.Value = entity.USERID;
            DbCommand.Parameters.Add(userid);

            DbParameter companyid = DbCommand.CreateParameter();
            companyid.ParameterName = ":companyid";
            companyid.Value = entity.COMPANYID;
            DbCommand.Parameters.Add(companyid);

            DbParameter region_id = DbCommand.CreateParameter();
            region_id.ParameterName = ":region_id";
            region_id.Value = entity.REGION_ID;
            DbCommand.Parameters.Add(region_id);

            DbParameter child_plan_state = DbCommand.CreateParameter();
            child_plan_state.ParameterName = ":child_plan_state";
            child_plan_state.Value = entity.CHILD_PLAN_STATE;
            DbCommand.Parameters.Add(child_plan_state);

            DbParameter isovertototal = DbCommand.CreateParameter();
            isovertototal.ParameterName = ":isovertototal";
            isovertototal.Value = entity.ISOVERTOTOTAL;
            DbCommand.Parameters.Add(isovertototal);

            DbParameter total = DbCommand.CreateParameter();
            total.ParameterName = ":total";
            total.Value = entity.TOTAL;
            DbCommand.Parameters.Add(total);


            DbParameter isovertototal1 = DbCommand.CreateParameter();
            isovertototal1.ParameterName = ":isovertototal1";
            isovertototal1.Value = entity.ISOVERTOTOTAL1;
            DbCommand.Parameters.Add(isovertototal1);


            DbParameter total1 = DbCommand.CreateParameter();
            total1.ParameterName = ":total1";
            total1.Value = entity.TOTAL1;
            DbCommand.Parameters.Add(total1);

            DbParameter percent1 = DbCommand.CreateParameter();
            percent1.ParameterName = ":percent1";
            percent1.Value = entity.PERCENT1;
            DbCommand.Parameters.Add(percent1);


            DbParameter copy_state = DbCommand.CreateParameter();
            copy_state.ParameterName = ":copy_state";
            copy_state.Value = entity.COPY_STATE;
            DbCommand.Parameters.Add(copy_state);


            DbParameter point_info = DbCommand.CreateParameter();
            point_info.ParameterName = ":point_info";
            point_info.Value = entity.POINT_INFO;
            DbCommand.Parameters.Add(point_info);

            DbParameter plan_mode = DbCommand.CreateParameter();
            plan_mode.ParameterName = ":plan_mode";
            plan_mode.Value = entity.PLAN_MODE;
            DbCommand.Parameters.Add(plan_mode);

            DbParameter plan_name = DbCommand.CreateParameter();
            plan_name.ParameterName = ":plan_name";
            plan_name.Value = entity.PLAN_NAME;
            DbCommand.Parameters.Add(plan_name);

            DbParameter map_range = DbCommand.CreateParameter();
            map_range.ParameterName = ":map_range";
            map_range.Value = entity.MAP_RANGE;
            DbCommand.Parameters.Add(map_range);

            DbParameter isjurisdiction = DbCommand.CreateParameter();
            isjurisdiction.ParameterName = ":isjurisdiction";
            isjurisdiction.Value = entity.ISJURISDICTION;
            DbCommand.Parameters.Add(isjurisdiction);

            DbParameter interval_days = DbCommand.CreateParameter();
            interval_days.ParameterName = ":interval_days";
            interval_days.Value = entity.INTERVAL_DAYS;
            DbCommand.Parameters.Add(interval_days);

            DbParameter description = DbCommand.CreateParameter();
            description.ParameterName = ":description";
            description.Value = entity.DESCRIPTION;
            DbCommand.Parameters.Add(description);

            DbParameter create_time = DbCommand.CreateParameter();
            create_time.ParameterName = ":create_time";
            create_time.DbType = DbType.Date;
            create_time.Value = entity.CREATE_TIME;
            DbCommand.Parameters.Add(create_time);

            DbParameter create_user = DbCommand.CreateParameter();
            create_user.ParameterName = ":create_user";
            create_user.Value = entity.CREATE_USER;
            DbCommand.Parameters.Add(create_user);

            DbParameter n = DbCommand.CreateParameter();
            n.ParameterName = ":n";
            n.Value = entity.N;
            DbCommand.Parameters.Add(n);

            DbParameter copy_num = DbCommand.CreateParameter();
            copy_num.ParameterName = ":copy_num";
            copy_num.Value = entity.COPY_NUM;
            DbCommand.Parameters.Add(copy_num);

            DbParameter point_num = DbCommand.CreateParameter();
            point_num.ParameterName = ":POINT_NUM";
            point_num.Value = entity.POINT_NUM;
            DbCommand.Parameters.Add(point_num);

            DbParameter plan_type = DbCommand.CreateParameter();
            plan_type.ParameterName = ":PLAN_TYPE";
            plan_type.Value = entity.PLAN_TYPE;
            DbCommand.Parameters.Add(plan_type);

            DbParameter type = DbCommand.CreateParameter();
            type.ParameterName = ":TYPE";
            type.Value = entity.TYPE;
            DbCommand.Parameters.Add(type);

            string sql = string.Empty;
            sql = @"insert into  tf_plan_child(  child_plan_id,
                                    child_num,
                                    child_begin_date,
                                    child_end_date,
                                    period_id,
                                    device_type_id,
                                    percent,
                                    last_time,
                                    userid,
                                    companyid,
                                    region_id,
                                    child_plan_state,
                                    isovertototal,
                                    total,
                                    isovertototal1,
                                    total1,
                                    percent1,
                                    copy_state ,
                                    point_info,
                                    plan_mode,
                                    plan_name,
                                    map_range,
                                    isjurisdiction,
                                    interval_days,
                                    description,
                                    create_time,
                                    create_user,
                                    n,
                                    copy_num,
                                    point_num,
                                    plan_type,
                                    type) values (
                                    :child_plan_id,
                                    :child_num,
                                    :child_begin_date,
                                    :child_end_date,
                                    :period_id,
                                    :device_type_id,
                                    :percent,
                                    :last_time,
                                    :userid,
                                    :companyid,
                                    :region_id,
                                    :child_plan_state,
                                    :isovertototal,
                                    :total,
                                    :isovertototal1,
                                    :total1,
                                    :percent1,
                                    :copy_state,
                                    :point_info,
                                    :plan_mode,
                                    :plan_name,
                                    :map_range,
                                    :isjurisdiction,
                                    :interval_days,
                                    :description,
                                    :create_time,
                                    :create_user,
                                    :n,
                                    :copy_num,
                                    :point_num,
                                    :plan_type,
                                    :type) ";
            DbCommand.CommandText = sql;
            return DbCommand;
        }

        public  DbCommand AddPointTempCommand(TF_PLAN_POINT_TEMP entity)
        {
            DbCommand DbCommand = conn.CreateCommand();

            DbParameter geometry = DbCommand.CreateParameter();
            geometry.ParameterName = ":geometry";
            geometry.Value = entity.GEOMETRY;
            DbCommand.Parameters.Add(geometry);

            DbParameter content = DbCommand.CreateParameter();
            content.ParameterName = ":content";
            content.Value = entity.CONTENT;
            DbCommand.Parameters.Add(content);

            string sql = string.Empty;
            sql = string.Format(@"insert into  tf_plan_point_temp( id,
                                   point_id,
                                   geometry,
                                   child_plan_id,
                                   userid,
                                   companyid,
                                   isover,
                                   regionname,
                                   content,
                                   region_id,
                                   device_type_id,
                                   name,
                                   address,
                                   period_id,
                                   lat,
                                   lng,
                                   code,
                                   state,
                                   trouble_id,
                                   pipe_length,
                                   type,
                                   rfid
                                   ) values (
                                   '{0}',
                                   '{1}',
                                   :geometry,
                                   '{2}',
                                   '{3}',
                                   '{4}',
                                   '{5}',
                                   '{6}',
                                   :content,
                                   '{7}',
                                   '{8}',
                                   '{9}',
                                   '{10}',
                                   '{11}',
                                   {12},
                                   {13},
                                   '{14}',
                                   {15},
                                   '{16}',
                                   '{17}',
                                    '{18}',
                                    '{19}')", entity.ID, entity.POINT_ID, entity.CHILD_PLAN_ID, entity.USERID, entity.COMPANYID,
                                    entity.ISOVER, entity.REGIONNAME, entity.REGION_ID, entity.DEVICE_TYPE_ID, entity.NAME, entity.ADDRESS, entity.PERIOD_ID,
                                    entity.LAT == null ? 0 : entity.LAT, entity.LNG == null ? 0 : entity.LNG, entity.CODE, entity.STATE == null ? 0 : entity.STATE,
                                    entity.TROUBLE_ID, entity.PIPE_LENGTH, entity.TYPE,entity.RFID);
            DbCommand.CommandText = sql;
            return DbCommand;
        }
        public  List<DbCommand> AddSysDeviceSetting(List<Setting_Plan> settingplan, string child_plan_id)
        {
            List<DbCommand> list = new List<DbCommand>();
            foreach (var setting in settingplan)
            {
                DbCommand DbCommand = conn.CreateCommand();

                DbCommand.CommandText = string.Format(@"insert into tf_sys_device_setting (id,child_plan_id,cnname,enname,para_type,isphoto) 
                        values(sys_guid(),'{0}','{1}','{2}','{3}','{4}')", child_plan_id, setting.cnname, setting.enname, setting.type, setting.isphoto);
                list.Add(DbCommand);
            }
            return list;
        }

        public  DbCommand AddPathTempCommand(TF_PLAN_PATH_TEMP entity)
        {
            DbCommand DbCommand = conn.CreateCommand();

            DbParameter ID = DbCommand.CreateParameter();
            ID.ParameterName = ":ID";
            ID.Value = entity.ID;
            DbCommand.Parameters.Add(ID);

            DbParameter TEMP_ID = DbCommand.CreateParameter();
            TEMP_ID.ParameterName = ":TEMP_ID";
            TEMP_ID.Value = entity.TEMP_ID;
            DbCommand.Parameters.Add(TEMP_ID);

            DbParameter POINT_ID = DbCommand.CreateParameter();
            POINT_ID.ParameterName = ":POINT_ID";
            POINT_ID.Value = entity.POINT_ID;
            DbCommand.Parameters.Add(POINT_ID);

            DbParameter CHILD_PLAN_ID = DbCommand.CreateParameter();
            CHILD_PLAN_ID.ParameterName = ":CHILD_PLAN_ID";
            CHILD_PLAN_ID.Value = entity.CHILD_PLAN_ID;
            DbCommand.Parameters.Add(CHILD_PLAN_ID);

            DbParameter COMPANYID = DbCommand.CreateParameter();
            COMPANYID.ParameterName = ":COMPANYID";
            COMPANYID.Value = entity.COMPANYID;
            DbCommand.Parameters.Add(COMPANYID);

            string sql = string.Empty;
            sql = string.Format(@"insert into   TF_PLAN_PATH_TEMP (
                                    ID,
                                    TEMP_ID,
                                    POINT_ID,
                                    CHILD_PLAN_ID,
                                    COMPANYID,
                                    ISOVER,
                                    LAT,
                                    LNG,
                                    NUM
                                    )VALUES (
                                    :ID,
                                    :TEMP_ID,
                                    :POINT_ID,
                                    :CHILD_PLAN_ID,
                                    :COMPANYID,{0},{1},{2},{3})", entity.ISOVER, entity.LAT, entity.LNG, entity.NUM);
            DbCommand.CommandText = sql;

            DbCommand.CommandText = sql;
            return DbCommand;
        }
        public  DbCommand AddPlanCorrectCommand(string plan_id, string parent_plan_id, string point_info)
        {
            DbCommand DbCommand = conn.CreateCommand();

            DbParameter _plan_id = DbCommand.CreateParameter();
            _plan_id.ParameterName = ":plan_id";
            _plan_id.Value = plan_id;
            DbCommand.Parameters.Add(_plan_id);

            DbParameter _parent_plan_id = DbCommand.CreateParameter();
            _parent_plan_id.ParameterName = ":parent_plan_id";
            _parent_plan_id.Value = parent_plan_id;
            DbCommand.Parameters.Add(_parent_plan_id);

            DbParameter _point_info = DbCommand.CreateParameter();
            _point_info.ParameterName = ":point_info";
            _point_info.Value = point_info;
            DbCommand.Parameters.Add(_point_info);

            string sql = string.Empty;
            sql = @"insert into  tf_plan_correct (id,
                                    plan_id,
                                    parent_plan_id,
                                    point_info) values (
                                    sys_guid(),
                                    :plan_id,
                                    :parent_plan_id,
                                    :point_info) ";
            DbCommand.CommandText = sql;
            return DbCommand;
        }
    }
}