/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrqcar.controller;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrqcar.entity.CarOil;
import com.tofly.dyrqcar.service.CarOilService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;

import java.util.Arrays;
/**
 * 车辆油耗信息管理
 *
 * @author Zhangyu
 * @date Thu Apr 23 00:00:00 GMT+08:00 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/caroil")
@Api(tags="车辆油耗信息管理接口")
public class CarOilController {

    private final  CarOilService carOilService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param carOil 车辆油耗信息管理
    * @return
    */
//    @GetMapping("/page" )
//    @ApiOperation(value = "分页查询")
//    public ResultRespone getCarOilPage(Page page, @ModelAttribute CarOil carOil) {
//      return ResultRespone.success(carOilService.page(page, Wrappers.query(carOil)));
//    }


    /**
     * 通过id查询车辆油耗信息管理
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(carOilService.getById(id));
    }

    /**
     * 新增车辆油耗信息管理
     * @param carOil 车辆油耗信息管理
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增车辆油耗信息管理" )
    @ApiOperation(value = "新增车辆油耗信息管理")
    @PostMapping
    public ResultRespone save(@RequestBody CarOil carOil) {
      return ResultRespone.success(carOilService.save(carOil));
    }

    /**
     * 修改车辆油耗信息管理
     * @param carOil 车辆油耗信息管理
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改车辆油耗信息管理" )
    @ApiOperation(value = "修改车辆油耗信息管理")
    @PutMapping
    public ResultRespone updateById(@RequestBody CarOil carOil) {
      return ResultRespone.success(carOilService.updateById(carOil));
    }

    /**
     * 通过id删除车辆油耗信息管理
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除车辆油耗信息管理" )
    @ApiOperation(value = "通过ID删除车辆油耗信息管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(carOilService.removeById(id));
    }

    /**
     * 通过id批量删除车辆油耗信息管理
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除车辆油耗信息管理" )
    @ApiOperation(value = "通过ID批量删除车辆油耗信息管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(carOilService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
