package com.tofly.dyrqcar.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.FtpUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrqcar.config.CarFilePathBean;
import com.tofly.dyrqcar.custom.service.CustomCarBaseinfoService;
import com.tofly.dyrqcar.entity.CarBaseinfo;
import com.tofly.dyrqcar.service.CarBaseinfoService;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;

@RestController
@AllArgsConstructor
@RequestMapping("/carbaseinfo")
public class CustomCarBaseinfoController {

    private final CarBaseinfoService carBaseinfoService;

    private final CarFilePathBean carFilePathBean;

    private final CustomCarBaseinfoService customCarBaseinfoService;

    @ToFlyAppLog(title = "新增车辆基本信息管理")
    @ApiOperation(tags = "车辆基本信息管理接口", value = "新增车辆基本信息管理")
    @PostMapping
    public ResultRespone save(HttpServletRequest servletRequest, CarBaseinfo carBaseinfo) {
        MultiValueMap<String, MultipartFile> multiFileMap = ((MultipartHttpServletRequest) servletRequest).getMultiFileMap();
        if (multiFileMap != null) {
            multiFileMap.forEach((k, v) -> {
                String filename = carFilePathBean.getCarpic() + IdWorker.getIdStr() + v.get(0).getOriginalFilename().substring(v.get(0).getOriginalFilename().lastIndexOf("."));
                boolean b = FtpUtil.uploadFile2Ftp(v.get(0), filename);
                carBaseinfo.setCarPic(filename);
            });
        }
        return ResultRespone.success(carBaseinfoService.save(carBaseinfo));
    }

    @ToFlyAppLog(title = "修改车辆基本信息管理")
    @ApiOperation(tags = "车辆基本信息管理接口", value = "修改车辆基本信息管理")
    @PostMapping("/update")
    public ResultRespone updateById(HttpServletRequest servletRequest, CarBaseinfo carBaseinfo) {
        //查询本地是否有照片,如果有就根据路径删除照片文件,再进行修改
        String localPic = carBaseinfoService.getById(carBaseinfo.getId()).getCarPic();
        MultiValueMap<String, MultipartFile> multiFileMap = ((MultipartHttpServletRequest) servletRequest).getMultiFileMap();
        if (multiFileMap.size() > 0) {
            multiFileMap.forEach((k, v) -> {
                if (k.contains("file")) {
                    if (localPic != null) {
                        boolean d = FtpUtil.deleteFile(localPic);
                    }
                    v.forEach(f -> {
                        String filename = carFilePathBean.getCarpic() + IdWorker.getIdStr() + f.getOriginalFilename().substring(f.getOriginalFilename().lastIndexOf("."));
                        boolean b = FtpUtil.uploadFile2Ftp(f, filename);
                        carBaseinfo.setCarPic(filename);
                    });
                }
            });
        }
        return ResultRespone.success(carBaseinfoService.updateById(carBaseinfo));
    }

    @ApiOperation(tags = "车辆基本信息管理接口", value = "根据车牌号模糊分页查询")
    @GetMapping("/numberpage")
    public ResultRespone pageByCarNumber(Page page, String number) {
        return ResultRespone.success(customCarBaseinfoService.pageByCarNumber(page, number));
    }


    /**
     * 通过id批量删除车辆基本信息管理
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除车辆基本信息管理")
    @ApiOperation(tags = "车辆基本信息管理接口", value = "通过ID批量删除车辆基本信息管理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(customCarBaseinfoService.removeById(ids));
    }

}
