/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.AppVersion;
import com.tofly.dyrq.service.AppVersionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

/**
 * app版本发布
 *
 * @author wangjian
 * @date Tue Apr 07 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/appversion")
@Api(tags="app版本发布接口")
public class AppVersionController {

    private final  AppVersionService appVersionService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param appVersion app版本发布
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getAppVersionPage(Page page, @ModelAttribute AppVersion appVersion) {
      return ResultRespone.success(appVersionService.page(page, Wrappers.query(appVersion)));
    }


    /**
     * 通过id查询app版本发布
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(appVersionService.getById(id));
    }

//    /**
//     * 新增app版本发布
//     * @param appVersion app版本发布
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="新增app版本发布" )
//    @ApiOperation(value = "新增app版本发布")
//    @PostMapping
//    public ResultRespone save(@RequestBody AppVersion appVersion) {
//      return ResultRespone.success(appVersionService.save(appVersion));
//    }

    /**
     * 修改app版本发布
     * @param appVersion app版本发布
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改app版本发布" )
    @ApiOperation(value = "修改app版本发布")
    @PutMapping
    public ResultRespone updateById(@RequestBody AppVersion appVersion) {
      return ResultRespone.success(appVersionService.updateById(appVersion));
    }

//    /**
//     * 通过id删除app版本发布
//     * @param id id
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="通过id删除app版本发布" )
//    @ApiOperation(value = "通过ID删除app版本发布")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
//    })
//    @DeleteMapping("/{id}" )
//    public ResultRespone removeById(@PathVariable Long id) {
//      return ResultRespone.success(appVersionService.removeById(id));
//    }
}
