/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.ConstructionType;
import com.tofly.dyrq.service.ConstructionTypeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 工地监护类型表
 *
 * @author pra
 * @date Mon Aug 31 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/constructiontype")
@Api(tags="工地监护类型表接口")
public class ConstructionTypeController {

    private final  ConstructionTypeService constructionTypeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param constructionType 工地监护类型表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getConstructionTypePage(Page page, @ModelAttribute ConstructionType constructionType) {
      return ResultRespone.success(constructionTypeService.page(page, Wrappers.query(constructionType)));
    }


    /**
     * 通过id查询工地监护类型表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(constructionTypeService.getById(id));
    }

    /**
     * 新增工地监护类型表
     * @param constructionType 工地监护类型表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增工地监护类型表" )
    @ApiOperation(value = "新增工地监护类型表")
    @PostMapping
    public ResultRespone save(@RequestBody ConstructionType constructionType) {
      return ResultRespone.success(constructionTypeService.save(constructionType));
    }

    /**
     * 修改工地监护类型表
     * @param constructionType 工地监护类型表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改工地监护类型表" )
    @ApiOperation(value = "修改工地监护类型表")
    @PutMapping
    public ResultRespone updateById(@RequestBody ConstructionType constructionType) {
      return ResultRespone.success(constructionTypeService.updateById(constructionType));
    }

    /**
     * 通过id删除工地监护类型表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除工地监护类型表" )
    @ApiOperation(value = "通过ID删除工地监护类型表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(constructionTypeService.removeById(id));
    }

    /**
     * 通过id批量删除工地监护类型表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除工地监护类型表" )
    @ApiOperation(value = "通过ID批量删除工地监护类型表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(constructionTypeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
