/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.DevopsEquipment;
import com.tofly.dyrq.service.DevopsEquipmentService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 管网运维部工具类设备清单录入表
 *
 * @author Zhangyu
 * @date Fri May 22 00:00:00 GMT+08:00 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/devopsequipment")
@Api(tags="管网运维部工具类设备清单录入表接口")
public class DevopsEquipmentController {

    private final  DevopsEquipmentService devopsEquipmentService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param devopsEquipment 管网运维部工具类设备清单录入表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getDevopsEquipmentPage(Page page, @ModelAttribute DevopsEquipment devopsEquipment) {
      return ResultRespone.success(devopsEquipmentService.page(page, Wrappers.query(devopsEquipment)));
    }


    /**
     * 通过id查询管网运维部工具类设备清单录入表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(devopsEquipmentService.getById(id));
    }

    /**
     * 新增管网运维部工具类设备清单录入表
     * @param devopsEquipment 管网运维部工具类设备清单录入表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增管网运维部工具类设备清单录入表" )
    @ApiOperation(value = "新增管网运维部工具类设备清单录入表")
    @PostMapping
    public ResultRespone save(@RequestBody DevopsEquipment devopsEquipment) {
      return ResultRespone.success(devopsEquipmentService.save(devopsEquipment));
    }

    /**
     * 修改管网运维部工具类设备清单录入表
     * @param devopsEquipment 管网运维部工具类设备清单录入表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改管网运维部工具类设备清单录入表" )
    @ApiOperation(value = "修改管网运维部工具类设备清单录入表")
    @PutMapping
    public ResultRespone updateById(@RequestBody DevopsEquipment devopsEquipment) {
      return ResultRespone.success(devopsEquipmentService.updateById(devopsEquipment));
    }

    /**
     * 通过id删除管网运维部工具类设备清单录入表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除管网运维部工具类设备清单录入表" )
    @ApiOperation(value = "通过ID删除管网运维部工具类设备清单录入表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(devopsEquipmentService.removeById(id));
    }

    /**
     * 通过id批量删除管网运维部工具类设备清单录入表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除管网运维部工具类设备清单录入表" )
    @ApiOperation(value = "通过ID批量删除管网运维部工具类设备清单录入表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(devopsEquipmentService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
