/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.Geobookmark;
import com.tofly.dyrq.service.GeobookmarkService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

/**
 * 地理书签信息表
 *
 * @author wangjian
 * @date Wed Mar 25 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/geobookmark")
@Api(tags="地理书签信息表接口")
public class GeobookmarkController {

    private final  GeobookmarkService geobookmarkService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param geobookmark 地理书签信息表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getGeobookmarkPage(Page page, @ModelAttribute Geobookmark geobookmark) {
      return ResultRespone.success(geobookmarkService.page(page, Wrappers.query(geobookmark)));
    }


    /**
     * 通过id查询地理书签信息表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(geobookmarkService.getById(id));
    }

    /**
     * 新增地理书签信息表
     * @param geobookmark 地理书签信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增地理书签信息表" )
    @ApiOperation(value = "新增地理书签信息表")
    @PostMapping
    public ResultRespone save(@RequestBody Geobookmark geobookmark) {
      return ResultRespone.success(geobookmarkService.save(geobookmark));
    }

    /**
     * 修改地理书签信息表
     * @param geobookmark 地理书签信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改地理书签信息表" )
    @ApiOperation(value = "修改地理书签信息表")
    @PutMapping
    public ResultRespone updateById(@RequestBody Geobookmark geobookmark) {
      return ResultRespone.success(geobookmarkService.updateById(geobookmark));
    }

    /**
     * 通过id删除地理书签信息表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除地理书签信息表" )
    @ApiOperation(value = "通过ID删除地理书签信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(geobookmarkService.removeById(id));
    }
}
