/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.GpsRegion;
import com.tofly.dyrq.service.GpsRegionService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

/**
 * 巡检片区表
 *
 * @author wangjian
 * @date Tue Apr 07 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/gpsregion")
@Api(tags="巡检片区表接口")
public class GpsRegionController {

    private final  GpsRegionService gpsRegionService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param gpsRegion 巡检片区表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getGpsRegionPage(Page page, @ModelAttribute GpsRegion gpsRegion) {
//      return ResultRespone.success(gpsRegionService.page(page, Wrappers.query(gpsRegion)));
        return ResultRespone.success(gpsRegionService.page(page, Wrappers.<GpsRegion>lambdaQuery()
                .like(StringUtil.isNotEmpty(gpsRegion.getRegionname()),GpsRegion::getRegionname,gpsRegion.getRegionname()) ));

    }


    /**
     * 通过id查询巡检片区表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(gpsRegionService.getById(id));
    }

    /**
     * 新增巡检片区表
     * @param gpsRegion 巡检片区表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增巡检片区表" )
    @ApiOperation(value = "新增巡检片区表")
    @PostMapping
    public ResultRespone save(@RequestBody GpsRegion gpsRegion) {
      return ResultRespone.success(gpsRegionService.save(gpsRegion));
    }

//    /**
//     * 修改巡检片区表
//     * @param gpsRegion 巡检片区表
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="修改巡检片区表" )
//    @ApiOperation(value = "修改巡检片区表")
//    @PutMapping
//    public ResultRespone updateById(@RequestBody GpsRegion gpsRegion) {
//        String isPlan=gpsRegionService.isPlan(gpsRegion.getId());
//        if("1".equals(isPlan)){
//            return ResultRespone.failed("存在着计划，不允许修改");
//        }
//      return ResultRespone.success(gpsRegionService.updateById(gpsRegion));
//    }

    /**
     * 通过id删除巡检片区表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除巡检片区表" )
    @ApiOperation(value = "通过ID删除巡检片区表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(gpsRegionService.deleteById(id));
    }


    /**
     * 通过id删除巡检片区表
     * @param ids ids
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除巡检片区表" )
    @ApiOperation(value = "通过ID批量删除巡检片区表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "id,多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone deleteByIds(String ids) {
        return ResultRespone.success(gpsRegionService.deleteByIds(ids));
    }
}
