/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.GpsStatus;
import com.tofly.dyrq.service.GpsStatusService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * GPS状态表
 *
 * @author pra
 * @date Mon Apr 13 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/gpsstatus")
@Api(tags="GPS状态表接口")
public class GpsStatusController {

    private final  GpsStatusService gpsStatusService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param gpsStatus GPS状态表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getGpsStatusPage(Page page, @ModelAttribute GpsStatus gpsStatus) {
      return ResultRespone.success(gpsStatusService.page(page, Wrappers.query(gpsStatus)));
    }


    /**
     * 通过id查询GPS状态表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(gpsStatusService.getById(id));
    }

//    /**
//     * 新增GPS状态表
//     * @param gpsStatus GPS状态表
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="新增GPS状态表" )
//    @ApiOperation(value = "新增GPS状态表")
//    @PostMapping
//    public ResultRespone save(@RequestBody GpsStatus gpsStatus) {
//      return ResultRespone.success(gpsStatusService.save(gpsStatus));
//    }
//
//    /**
//     * 修改GPS状态表
//     * @param gpsStatus GPS状态表
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="修改GPS状态表" )
//    @ApiOperation(value = "修改GPS状态表")
//    @PutMapping
//    public ResultRespone updateById(@RequestBody GpsStatus gpsStatus) {
//      return ResultRespone.success(gpsStatusService.updateById(gpsStatus));
//    }

    /**
     * 通过id删除GPS状态表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除GPS状态表" )
    @ApiOperation(value = "通过ID删除GPS状态表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(gpsStatusService.removeById(id));
    }

    /**
     * 通过id批量删除GPS状态表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除GPS状态表" )
    @ApiOperation(value = "通过ID批量删除GPS状态表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(gpsStatusService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
