package com.tofly.dyrq.controller;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;

import com.tofly.dyrq.service.InspectionReportService;
import com.tofly.dyrq.util.DateTimeUtil;
import com.tofly.dyrq.vo.DailyPlanStaticsVO;
import com.tofly.dyrq.vo.StaticModelPlanResultVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @program: tofly-dyrq
 * @description: 巡检报表管理控制器
 * @author: Sumer
 * @create: 2020-09-09 17:14
 */
@RestController
@RequestMapping("/inspectionReport")
@Slf4j
@Api(tags = "巡检报表管理接口")
public class InspectionReportController {

    @Autowired
    private InspectionReportService inspectionReportService;

    @ResponseBody
    @ToFlyAppLog(title = "获取巡检(日/月)报")
    @ApiOperation(value = "获取巡检(日/月)报")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "departmentId", value = "部门id", required = false, dataType = "String"),
            @ApiImplicitParam(name = "userId", value = "用户id", required = false, dataType = "String"),
            @ApiImplicitParam(name = "planDate", value = "巡检日期(如: 2020-09-01) 格式统一为 yyyy-MM-dd", required = true, dataType = "String"),
            @ApiImplicitParam(name = "timeType", value = "日期类型 1 为日报, 2 为月报", required = true, dataType = "String")
    })
    @GetMapping("/dailyPlanStatics")
    public ResultRespone<List<DailyPlanStaticsVO>> disposeDailyPlanStatics(@RequestParam(name = "departmentId",required = false) String departmentId, @RequestParam(name = "userId",required = false)String userId, @RequestParam("planDate")String planDate,@RequestParam("timeType") String timeType) {
        log.info("获取巡检(日/月)报 departmentId : {},userId : {},planDate : {},timeType : {}", departmentId, userId, planDate, timeType);
        if (StringUtils.isEmpty(planDate) || StringUtils.isEmpty(timeType)) {
            return ResultRespone.failed("请检查参数,巡检日期/日期类型必填");
        }
        return ResultRespone.success(inspectionReportService.disposeDailyPlanStatics(departmentId, userId, planDate, timeType));

    }

    @ResponseBody
    @ToFlyAppLog(title = "获取巡检(日/月)报 统计数据")
    @ApiOperation(value = "获取巡检(日/月)报 统计数据")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "departmentId", value = "部门id", required = false, dataType = "String"),
            @ApiImplicitParam(name = "beginPlanDate", value = "巡检统计起始日期(如: 2020-09-01) 格式统一为 yyyy-MM-dd", required = true, dataType = "String"),
            @ApiImplicitParam(name = "endPlanDate", value = "巡检统计终止日期(如: 2020-10-01) 格式统一为 yyyy-MM-dd", required = true, dataType = "String")
    })
    @GetMapping("/staticPlan")
    public ResultRespone<List<StaticModelPlanResultVO>> disposeStaticPlan(@RequestParam(name = "departmentId",required = false)String departmentId, @RequestParam(name = "beginPlanDate")String beginPlanDate, @RequestParam(name = "endPlanDate")String endPlanDate) {
        log.info("获取巡检(日/月)报 统计数据 departmentId : {},beginPlanDate : {},endPlanDate : {}", departmentId, beginPlanDate, endPlanDate);
        if (StringUtils.isEmpty(beginPlanDate) || StringUtils.isEmpty(endPlanDate)) {
            return ResultRespone.failed("请检查参数,巡检统计起始/终止日期为空");
        }

        return ResultRespone.success(inspectionReportService.disposeStaticPlan(departmentId, beginPlanDate, endPlanDate));

    }



}