/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.custom.service.CustomKeyEmptTypeService;
import com.tofly.dyrq.entity.KeyEmptType;
import com.tofly.dyrq.service.KeyEmptTypeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 重点设备类型表
 *
 * @author Zhangyu
 * @date Tue Dec 15 00:00:00 GMT+08:00 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/keyempttype")
@Api(tags="重点设备类型表接口")
public class KeyEmptTypeController {

    private final  KeyEmptTypeService keyEmptTypeService;
    private final CustomKeyEmptTypeService customKeyEmptTypeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param keyEmptType 重点设备类型表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getKeyEmptTypePage(Page page, @ModelAttribute KeyEmptType keyEmptType) {
      return ResultRespone.success(keyEmptTypeService.page(page, Wrappers.query(keyEmptType)));
    }


    /**
     * 通过id查询重点设备类型表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(keyEmptTypeService.getById(id));
    }

    /**
     * 新增重点设备类型表
     * @param keyEmptType 重点设备类型表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增重点设备类型表" )
    @ApiOperation(value = "新增重点设备类型表")
    @PostMapping
    public ResultRespone save(@RequestBody KeyEmptType keyEmptType) {
      return ResultRespone.success(customKeyEmptTypeService.save(keyEmptType));
    }

    /**
     * 修改重点设备类型表
     * @param keyEmptType 重点设备类型表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改重点设备类型表" )
    @ApiOperation(value = "修改重点设备类型表")
    @PutMapping
    public ResultRespone updateById(@RequestBody KeyEmptType keyEmptType) {
      return ResultRespone.success(customKeyEmptTypeService.updateById(keyEmptType));
    }

    /**
     * 通过id删除重点设备类型表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除重点设备类型表" )
    @ApiOperation(value = "通过ID删除重点设备类型表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(keyEmptTypeService.removeById(id));
    }

    /**
     * 通过id批量删除重点设备类型表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除重点设备类型表" )
    @ApiOperation(value = "通过ID批量删除重点设备类型表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(keyEmptTypeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
