/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.MaintenanceDeviceType;
import com.tofly.dyrq.service.MaintenanceDeviceTypeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 设备设施保养类型表
 *
 * @author admin
 * @date Fri Jun 05 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/maintenancedevicetype")
@Api(tags="设备设施保养类型表接口")
public class MaintenanceDeviceTypeController {

    private final  MaintenanceDeviceTypeService maintenanceDeviceTypeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param maintenanceDeviceType 设备设施保养类型表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getMaintenanceDeviceTypePage(Page page, @ModelAttribute MaintenanceDeviceType maintenanceDeviceType) {
      return ResultRespone.success(maintenanceDeviceTypeService.page(page, Wrappers.query(maintenanceDeviceType)));
    }


    /**
     * 通过id查询设备设施保养类型表
     * @param delFlag id
     * @return ResultRespone
     */
    @GetMapping("/{delFlag}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "delFlag", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("delFlag" ) String delFlag) {
      return ResultRespone.success(maintenanceDeviceTypeService.getById(delFlag));
    }

    /**
     * 新增设备设施保养类型表
     * @param maintenanceDeviceType 设备设施保养类型表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增设备设施保养类型表" )
    @ApiOperation(value = "新增设备设施保养类型表")
    @PostMapping
    public ResultRespone save(@RequestBody MaintenanceDeviceType maintenanceDeviceType) {
      return ResultRespone.success(maintenanceDeviceTypeService.save(maintenanceDeviceType));
    }

    /**
     * 修改设备设施保养类型表
     * @param maintenanceDeviceType 设备设施保养类型表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改设备设施保养类型表" )
    @ApiOperation(value = "修改设备设施保养类型表")
    @PutMapping
    public ResultRespone updateById(@RequestBody MaintenanceDeviceType maintenanceDeviceType) {
      return ResultRespone.success(maintenanceDeviceTypeService.updateById(maintenanceDeviceType));
    }

    /**
     * 通过id删除设备设施保养类型表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除设备设施保养类型表" )
    @ApiOperation(value = "通过ID删除设备设施保养类型表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(maintenanceDeviceTypeService.removeById(id));
    }

    /**
     * 通过id批量删除设备设施保养类型表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除设备设施保养类型表" )
    @ApiOperation(value = "通过ID批量删除设备设施保养类型表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(maintenanceDeviceTypeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
