/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.MsLog;
import com.tofly.dyrq.service.MsLogService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
/**
 * 移动端错误日志表
 *
 * @author pra
 * @date Fri Sep 04 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/mslog")
@Api(tags="移动端错误日志表接口")
public class MsLogController {

    private final  MsLogService msLogService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param msLog 移动端错误日志表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getMsLogPage(Page page, @ModelAttribute MsLog msLog) {
      return ResultRespone.success(msLogService.page(page, Wrappers.query(msLog)));
    }


    /**
     * 通过id查询移动端错误日志表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(msLogService.getById(id));
    }

    /**
     * 新增移动端错误日志表
     * @param msLog 移动端错误日志表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增移动端错误日志表" )
    @ApiOperation(value = "新增移动端错误日志表",notes = "只需上传文件files")
    @PostMapping(headers = "content-type=multipart/form-data")
    public ResultRespone save(HttpServletRequest request, MsLog msLog) {
      return ResultRespone.success(msLogService.saveMsLog(request,msLog));
    }

    /**
     * 修改移动端错误日志表
     * @param msLog 移动端错误日志表
     * @return ResultRespone
     */
//    @ToFlyAppLog(title="修改移动端错误日志表" )
//    @ApiOperation(value = "修改移动端错误日志表")
//    @PutMapping
//    public ResultRespone updateById(@RequestBody MsLog msLog) {
//      return ResultRespone.success(msLogService.updateById(msLog));
//    }
//
//    /**
//     * 通过id删除移动端错误日志表
//     * @param id id
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="通过id删除移动端错误日志表" )
//    @ApiOperation(value = "通过ID删除移动端错误日志表")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
//    })
//    @DeleteMapping("/{id}" )
//    public ResultRespone removeById(@PathVariable Long id) {
//      return ResultRespone.success(msLogService.removeById(id));
//    }
//
//    /**
//     * 通过id批量删除移动端错误日志表
//     * @param ids id
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="通过id批量删除移动端错误日志表" )
//    @ApiOperation(value = "通过ID批量删除移动端错误日志表")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
//    })
//    @DeleteMapping("/deleteByIds" )
//    public ResultRespone removeById(String ids) {
//        return ResultRespone.success(msLogService.removeByIds(Arrays.asList(ids.split(","))));
//    }
}
