/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.custom.service.CustomNormalEmptTypeService;
import com.tofly.dyrq.entity.NormalEmptType;
import com.tofly.dyrq.service.NormalEmptTypeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 一般设备类型表
 *
 * @author Zhangyu
 * @date Tue Dec 15 00:00:00 GMT+08:00 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/normalempttype")
@Api(tags="一般设备类型表接口")
public class NormalEmptTypeController {

    private final  NormalEmptTypeService normalEmptTypeService;
    private final CustomNormalEmptTypeService customNormalEmptTypeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param normalEmptType 一般设备类型表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getNormalEmptTypePage(Page page, @ModelAttribute NormalEmptType normalEmptType) {
      return ResultRespone.success(normalEmptTypeService.page(page, Wrappers.query(normalEmptType)));
    }


    /**
     * 通过id查询一般设备类型表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(normalEmptTypeService.getById(id));
    }

    /**
     * 新增一般设备类型表
     * @param normalEmptType 一般设备类型表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增一般设备类型表" )
    @ApiOperation(value = "新增一般设备类型表")
    @PostMapping
    public ResultRespone save(@RequestBody NormalEmptType normalEmptType) {
      return ResultRespone.success(customNormalEmptTypeService.save(normalEmptType));
    }

    /**
     * 修改一般设备类型表
     * @param normalEmptType 一般设备类型表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改一般设备类型表" )
    @ApiOperation(value = "修改一般设备类型表")
    @PutMapping
    public ResultRespone updateById(@RequestBody NormalEmptType normalEmptType) {
      return ResultRespone.success(customNormalEmptTypeService.updateById(normalEmptType));
    }

    /**
     * 通过id删除一般设备类型表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除一般设备类型表" )
    @ApiOperation(value = "通过ID删除一般设备类型表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(normalEmptTypeService.removeById(id));
    }

    /**
     * 通过id批量删除一般设备类型表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除一般设备类型表" )
    @ApiOperation(value = "通过ID批量删除一般设备类型表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(normalEmptTypeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
