/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.NormalEquipment;
import com.tofly.dyrq.service.NormalEquipmentService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 管网运维部一般设备表
 *
 * @author Zhangyu
 * @date Tue Dec 15 00:00:00 GMT+08:00 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/normalequipment")
@Api(tags="管网运维部一般设备表接口")
public class NormalEquipmentController {

    private final  NormalEquipmentService normalEquipmentService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param normalEquipment 管网运维部一般设备表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getNormalEquipmentPage(Page page, @ModelAttribute NormalEquipment normalEquipment) {
      return ResultRespone.success(normalEquipmentService.page(page, Wrappers.query(normalEquipment)));
    }


    /**
     * 通过id查询管网运维部一般设备表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(normalEquipmentService.getById(id));
    }

    /**
     * 新增管网运维部一般设备表
     * @param normalEquipment 管网运维部一般设备表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增管网运维部一般设备表" )
    @ApiOperation(value = "新增管网运维部一般设备表")
    @PostMapping
    public ResultRespone save(@RequestBody NormalEquipment normalEquipment) {
      return ResultRespone.success(normalEquipmentService.save(normalEquipment));
    }

    /**
     * 修改管网运维部一般设备表
     * @param normalEquipment 管网运维部一般设备表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改管网运维部一般设备表" )
    @ApiOperation(value = "修改管网运维部一般设备表")
    @PutMapping
    public ResultRespone updateById(@RequestBody NormalEquipment normalEquipment) {
      return ResultRespone.success(normalEquipmentService.updateById(normalEquipment));
    }

    /**
     * 通过id删除管网运维部一般设备表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除管网运维部一般设备表" )
    @ApiOperation(value = "通过ID删除管网运维部一般设备表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(normalEquipmentService.removeById(id));
    }

    /**
     * 通过id批量删除管网运维部一般设备表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除管网运维部一般设备表" )
    @ApiOperation(value = "通过ID批量删除管网运维部一般设备表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(normalEquipmentService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
