/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrq.entity.PdaDevice;
import com.tofly.dyrq.service.PdaDeviceService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

/**
 * 巡检移动设备
 *
 * @author wangjian
 * @date Wed Apr 01 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/pdadevice")
@Api(tags="巡检移动设备接口")
public class PdaDeviceController {

    private final  PdaDeviceService pdaDeviceService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param pdaDevice 巡检移动设备
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询",notes = "部门查询，名称模糊查询")
    public ResultRespone getPdaDevicePage(Page page, @ModelAttribute PdaDevice pdaDevice) {
      return ResultRespone.success(pdaDeviceService.getPdaDevicePage(page,pdaDevice));
    }


    /**
     * 通过id查询巡检移动设备
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(pdaDeviceService.getById(id));
    }

//    /**
//     * 新增巡检移动设备
//     * @param pdaDevice 巡检移动设备
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="新增巡检移动设备" )
//    @ApiOperation(value = "新增巡检移动设备")
//    @PostMapping
//    public ResultRespone save(@RequestBody PdaDevice pdaDevice) {
//      return ResultRespone.success(pdaDeviceService.save(pdaDevice));
//    }

    /**
     * 修改巡检移动设备
     * @param pdaDevice 巡检移动设备
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改巡检移动设备" )
    @ApiOperation(value = "修改巡检移动设备")
    @PutMapping
    public ResultRespone updateById(@RequestBody PdaDevice pdaDevice) {
      return ResultRespone.success(pdaDeviceService.updateById(pdaDevice));
    }
//
//    /**
//     * 通过id删除巡检移动设备
//     * @param id id
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="通过id删除巡检移动设备" )
//    @ApiOperation(value = "通过ID删除巡检移动设备")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
//    })
//    @DeleteMapping("/{id}" )
//    public ResultRespone removeById(@PathVariable Long id) {
//      return ResultRespone.success(pdaDeviceService.removeById(id));
//    }
}
