/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.PdaUserDevice;
import com.tofly.dyrq.service.PdaUserDeviceService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

/**
 * 巡检人员与移动设备关联表
 *
 * @author wangjian
 * @date Tue Mar 31 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/pdauserdevice")
@Api(tags="巡检人员与移动设备关联表接口")
public class PdaUserDeviceController {

    private final  PdaUserDeviceService pdaUserDeviceService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param pdaUserDevice 巡检人员与移动设备关联表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getPdaUserDevicePage(Page page, @ModelAttribute PdaUserDevice pdaUserDevice) {
      return ResultRespone.success(pdaUserDeviceService.page(page, Wrappers.query(pdaUserDevice)));
    }


    /**
     * 通过id查询巡检人员与移动设备关联表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(pdaUserDeviceService.getById(id));
    }

    /**
     * 新增巡检人员与移动设备关联表
     * @param pdaUserDevice 巡检人员与移动设备关联表
     * @return ResultRespone
     */
//    @ToFlyAppLog(title="新增巡检人员与移动设备关联表" )
//    @ApiOperation(value = "新增巡检人员与移动设备关联表")
//    @PostMapping
//    public ResultRespone save(@RequestBody PdaUserDevice pdaUserDevice) {
//      return ResultRespone.success(pdaUserDeviceService.save(pdaUserDevice));
//    }

    /**
     * 修改巡检人员与移动设备关联表
     * @param pdaUserDevice 巡检人员与移动设备关联表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改巡检人员与移动设备关联表" )
    @ApiOperation(value = "修改巡检人员与移动设备关联表")
    @PutMapping
    public ResultRespone updateById(@RequestBody PdaUserDevice pdaUserDevice) {
      return ResultRespone.success(pdaUserDeviceService.updateById(pdaUserDevice));
    }

    /**
     * 通过id删除巡检人员与移动设备关联表
     * @param id id
     * @return ResultRespone
     */
//    @ToFlyAppLog(title="通过id删除巡检人员与移动设备关联表" )
//    @ApiOperation(value = "通过ID删除巡检人员与移动设备关联表")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
//    })
//    @DeleteMapping("/{id}" )
//    public ResultRespone removeById(@PathVariable Long id) {
//      return ResultRespone.success(pdaUserDeviceService.removeById(id));
//    }
}
