/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.PlanPeriodType;
import com.tofly.dyrq.service.PlanPeriodTypeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 巡检周期表
 *
 * @author wangjian
 * @date Mon Apr 20 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/planperiodtype")
@Api(tags="巡检周期表接口")
public class PlanPeriodTypeController {

    private final  PlanPeriodTypeService planPeriodTypeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param planPeriodType 巡检周期表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getPlanPeriodTypePage(Page page, @ModelAttribute PlanPeriodType planPeriodType) {
      return ResultRespone.success(planPeriodTypeService.page(page, Wrappers.query(planPeriodType)));
    }


    /**
     * 通过id查询巡检周期表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(planPeriodTypeService.getById(id));
    }

    /**
     * 新增巡检周期表
     * @param planPeriodType 巡检周期表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增巡检周期表" )
    @ApiOperation(value = "新增巡检周期表")
    @PostMapping
    public ResultRespone save(@RequestBody PlanPeriodType planPeriodType) {
      return ResultRespone.success(planPeriodTypeService.save(planPeriodType));
    }

    /**
     * 修改巡检周期表
     * @param planPeriodType 巡检周期表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改巡检周期表" )
    @ApiOperation(value = "修改巡检周期表")
    @PutMapping
    public ResultRespone updateById(@RequestBody PlanPeriodType planPeriodType) {
      return ResultRespone.success(planPeriodTypeService.updateById(planPeriodType));
    }

    /**
     * 通过id删除巡检周期表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除巡检周期表" )
    @ApiOperation(value = "通过ID删除巡检周期表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(planPeriodTypeService.removeById(id));
    }

    /**
     * 通过id批量删除巡检周期表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除巡检周期表" )
    @ApiOperation(value = "通过ID批量删除巡检周期表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(planPeriodTypeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
