/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.PlanPoint;
import com.tofly.dyrq.service.PlanPointService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

/**
 * 巡检点表
 *
 * @author wangjian
 * @date Fri Apr 10 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/planpoint")
@Api(tags="巡检点表接口")
public class PlanPointController {

    private final  PlanPointService planPointService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param planPoint 巡检点表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getPlanPointPage(Page page, @ModelAttribute PlanPoint planPoint) {
      return ResultRespone.success(planPointService.page(page, Wrappers.query(planPoint)));
    }


    /**
     * 通过id查询巡检点表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(planPointService.getById(id));
    }

    /**
     * 新增巡检点表
     * @param planPoint 巡检点表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增巡检点表" )
    @ApiOperation(value = "新增巡检点表")
    @PostMapping
    public ResultRespone save(@RequestBody PlanPoint planPoint) {
      return ResultRespone.success(planPointService.save(planPoint));
    }

    /**
     * 修改巡检点表
     * @param planPoint 巡检点表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改巡检点表" )
    @ApiOperation(value = "修改巡检点表")
    @PutMapping
    public ResultRespone updateById(@RequestBody PlanPoint planPoint) {
      return ResultRespone.success(planPointService.updateById(planPoint));
    }

    /**
     * 通过id删除巡检点表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除巡检点表" )
    @ApiOperation(value = "通过ID删除巡检点表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
        String isPlan=planPointService.isPlan(id);
        if("1".equals(isPlan)){
            return ResultRespone.failed("存在着计划，不允许删除");
        }
      return ResultRespone.success(planPointService.removeById(id));
    }

    /**
     * 通过id批量删除巡检点表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除巡检点表" )
    @ApiOperation(value = "通过ID批量删除巡检点表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {

//        List idList=Arrays.asList(ids.split(","));
//        Collection<PlanPoint> collection = planPointService.listByIds(idList);
//        String msg= collection.stream().filter(gp -> "1".equals(gp.getIsPlan()))
//                .map(PlanPoint::getPointName).collect(Collectors.joining(","));
//        if(StringUtil.isNotEmpty(msg)){
//            return ResultRespone.failed("存在着计划不允许删除的数据"+msg);
//        }
        return ResultRespone.success(planPointService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
