/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.PlanTransfer;
import com.tofly.dyrq.service.PlanTransferService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 计划转移记录表
 *
 * @author pangrongan
 * @date Wed May 13 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/plantransfer")
@Api(tags="计划转移记录表接口")
public class PlanTransferController {

    private final  PlanTransferService planTransferService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param planTransfer 计划转移记录表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getPlanTransferPage(Page page, @ModelAttribute PlanTransfer planTransfer) {
      return ResultRespone.success(planTransferService.page(page, Wrappers.query(planTransfer)));
    }


    /**
     * 通过id查询计划转移记录表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(planTransferService.getById(id));
    }

    /**
     * 新增计划转移记录表
     * @param planTransfer 计划转移记录表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增计划转移记录表" )
    @ApiOperation(value = "新增计划转移记录表")
    @PostMapping
    public ResultRespone save(@RequestBody PlanTransfer planTransfer) {
      return ResultRespone.success(planTransferService.save(planTransfer));
    }

    /**
     * 修改计划转移记录表
     * @param planTransfer 计划转移记录表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改计划转移记录表" )
    @ApiOperation(value = "修改计划转移记录表")
    @PutMapping
    public ResultRespone updateById(@RequestBody PlanTransfer planTransfer) {
      return ResultRespone.success(planTransferService.updateById(planTransfer));
    }

    /**
     * 通过id删除计划转移记录表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除计划转移记录表" )
    @ApiOperation(value = "通过ID删除计划转移记录表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(planTransferService.removeById(id));
    }

    /**
     * 通过id批量删除计划转移记录表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除计划转移记录表" )
    @ApiOperation(value = "通过ID批量删除计划转移记录表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(planTransferService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
