/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.Questionmark;
import com.tofly.dyrq.service.QuestionmarkService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

/**
 * 疑问标识信息表
 *
 * @author wangjian
 * @date Wed Mar 25 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/questionmark")
@Api(tags="疑问标识信息表接口")
public class QuestionmarkController {

    private final  QuestionmarkService questionmarkService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param questionmark 疑问标识信息表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getQuestionmarkPage(Page page, @ModelAttribute Questionmark questionmark) {
      return ResultRespone.success(questionmarkService.page(page, Wrappers.query(questionmark)));
    }


    /**
     * 通过id查询疑问标识信息表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(questionmarkService.getById(id));
    }

//    /**
//     * 新增疑问标识信息表
//     * @param questionmark 疑问标识信息表
//     * @return ResultRespone
//     */
//    @ToFlyAppLog(title="新增疑问标识信息表" )
//    @ApiOperation(value = "新增疑问标识信息表")
//    @PostMapping
//    public ResultRespone save(@RequestBody Questionmark questionmark) {
//      return ResultRespone.success(questionmarkService.save(questionmark));
//    }

    /**
     * 修改疑问标识信息表
     * @param questionmark 疑问标识信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改疑问标识信息表" )
    @ApiOperation(value = "修改疑问标识信息表")
    @PutMapping
    public ResultRespone updateById(@RequestBody Questionmark questionmark) {
      return ResultRespone.success(questionmarkService.updateById(questionmark));
    }

    /**
     * 通过id删除疑问标识信息表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除疑问标识信息表" )
    @ApiOperation(value = "通过ID删除疑问标识信息表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(questionmarkService.removeById(id));
    }
}
