/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.ScadapntsStations;
import com.tofly.dyrq.service.ScadapntsStationsService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * scada站点
 *
 * @author admin
 * @date Tue Apr 14 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/scadapntsstations")
@Api(tags="scada站点接口")
public class ScadapntsStationsController {

    private final  ScadapntsStationsService scadapntsStationsService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param scadapntsStations scada站点
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getScadapntsStationsPage(Page page, @ModelAttribute ScadapntsStations scadapntsStations) {
      return ResultRespone.success(scadapntsStationsService.page(page, Wrappers.query(scadapntsStations)));
    }


    /**
     * 通过id查询scada站点
     * @param delFlag id
     * @return ResultRespone
     */
    @GetMapping("/{delFlag}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "delFlag", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("delFlag" ) String delFlag) {
      return ResultRespone.success(scadapntsStationsService.getById(delFlag));
    }

    /**
     * 新增scada站点
     * @param scadapntsStations scada站点
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增scada站点" )
    @ApiOperation(value = "新增scada站点")
    @PostMapping
    public ResultRespone save(@RequestBody ScadapntsStations scadapntsStations) {
      return ResultRespone.success(scadapntsStationsService.save(scadapntsStations));
    }

    /**
     * 修改scada站点
     * @param scadapntsStations scada站点
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改scada站点" )
    @ApiOperation(value = "修改scada站点")
    @PutMapping
    public ResultRespone updateById(@RequestBody ScadapntsStations scadapntsStations) {
      return ResultRespone.success(scadapntsStationsService.updateById(scadapntsStations));
    }

    /**
     * 通过id删除scada站点
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除}" )
    @ApiOperation(value = "通过ID删除")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(scadapntsStationsService.removeById(id));
    }

    /**
     * 通过id批量删除scada站点
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除" )
    @ApiOperation(value = "通过ID批量删除")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(scadapntsStationsService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
