/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.custom.service.CustomSpecialEmptTypeService;
import com.tofly.dyrq.entity.SpecialEmptType;
import com.tofly.dyrq.service.SpecialEmptTypeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 特种设备类型表
 *
 * @author Zhangyu
 * @date Tue Dec 15 00:00:00 GMT+08:00 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/specialempttype")
@Api(tags="特种设备类型表接口")
public class SpecialEmptTypeController {

    private final  SpecialEmptTypeService specialEmptTypeService;
    private final CustomSpecialEmptTypeService customSpecialEmptTypeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param specialEmptType 特种设备类型表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getSpecialEmptTypePage(Page page, @ModelAttribute SpecialEmptType specialEmptType) {
      return ResultRespone.success(specialEmptTypeService.page(page, Wrappers.query(specialEmptType)));
    }


    /**
     * 通过id查询特种设备类型表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(specialEmptTypeService.getById(id));
    }

    /**
     * 新增特种设备类型表
     * @param specialEmptType 特种设备类型表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增特种设备类型表" )
    @ApiOperation(value = "新增特种设备类型表")
    @PostMapping
    public ResultRespone save(@RequestBody SpecialEmptType specialEmptType) {
      return ResultRespone.success(customSpecialEmptTypeService.save(specialEmptType));
    }

    /**
     * 修改特种设备类型表
     * @param specialEmptType 特种设备类型表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改特种设备类型表" )
    @ApiOperation(value = "修改特种设备类型表")
    @PutMapping
    public ResultRespone updateById(@RequestBody SpecialEmptType specialEmptType) {
      return ResultRespone.success(customSpecialEmptTypeService.updateById(specialEmptType));
    }

    /**
     * 通过id删除特种设备类型表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除特种设备类型表" )
    @ApiOperation(value = "通过ID删除特种设备类型表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(specialEmptTypeService.removeById(id));
    }

    /**
     * 通过id批量删除特种设备类型表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除特种设备类型表" )
    @ApiOperation(value = "通过ID批量删除特种设备类型表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(specialEmptTypeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
