/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.SysConfig;
import com.tofly.dyrq.service.SysConfigService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

/**
 * 系统配置表
 *
 * @author wangjian
 * @date Tue Apr 07 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/sysconfig")
@Api(tags = "系统配置表接口")
public class SysConfigController {

    private final SysConfigService sysConfigService;

    /**
     * 分页查询
     *
     * @param page      分页对象
     * @param sysConfig 系统配置表
     * @return
     */
    @GetMapping("/page")
    @ApiOperation(value = "分页查询")
    public ResultRespone getSysConfigPage(Page page, @ModelAttribute SysConfig sysConfig) {
        return ResultRespone.success(sysConfigService.page(page, Wrappers.<SysConfig>lambdaQuery()
                .eq(sysConfig.getId() != null, SysConfig::getId, sysConfig.getId())
                .eq(StringUtil.isNotEmpty(sysConfig.getKey()), SysConfig::getKey, sysConfig.getKey())
                .eq(StringUtil.isNotEmpty(sysConfig.getForbidDelete()), SysConfig::getForbidDelete, sysConfig.getForbidDelete())
                .like(StringUtil.isNotEmpty(sysConfig.getValue()), SysConfig::getValue, sysConfig.getValue())));
    }


    /**
     * 通过id查询系统配置表
     *
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return ResultRespone.success(sysConfigService.getById(id));
    }

    /**
     * 新增系统配置表
     *
     * @param sysConfig 系统配置表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增系统配置表")
    @ApiOperation(value = "新增系统配置表")
    @PostMapping
    public ResultRespone save(@RequestBody SysConfig sysConfig) {
        return ResultRespone.success(sysConfigService.save(sysConfig));
    }

    /**
     * 修改系统配置表
     *
     * @param sysConfig 系统配置表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改系统配置表")
    @ApiOperation(value = "修改系统配置表")
    @PutMapping
    public ResultRespone updateById(@RequestBody SysConfig sysConfig) {
        return ResultRespone.success(sysConfigService.updateById(sysConfig));
    }

    /**
     * 通过id删除系统配置表
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除系统配置表")
    @ApiOperation(value = "通过ID删除系统配置表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        return ResultRespone.success(sysConfigService.removeById(id));
    }
}
