/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.TfUserStatus;
import com.tofly.dyrq.service.TfUserStatusService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

/**
 * 用户状态记录表
 *
 * @author admin
 * @date Fri Apr 03 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfuserstatus")
@Api(tags="用户状态记录表接口")
public class TfUserStatusController {

    private final  TfUserStatusService tfUserStatusService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param tfUserStatus 用户状态记录表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getTfUserStatusPage(Page page, @ModelAttribute TfUserStatus tfUserStatus) {
      return ResultRespone.success(tfUserStatusService.page(page, Wrappers.query(tfUserStatus)));
    }


    /**
     * 通过id查询用户状态记录表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(tfUserStatusService.getById(id));
    }

    /**
     * 新增用户状态记录表
     * @param tfUserStatus 用户状态记录表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增用户状态记录表" )
    @ApiOperation(value = "新增用户状态记录表")
    @PostMapping
    public ResultRespone save(@RequestBody TfUserStatus tfUserStatus) {
      return ResultRespone.success(tfUserStatusService.save(tfUserStatus));
    }

    /**
     * 修改用户状态记录表
     * @param tfUserStatus 用户状态记录表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改用户状态记录表" )
    @ApiOperation(value = "修改用户状态记录表")
    @PutMapping
    public ResultRespone updateById(@RequestBody TfUserStatus tfUserStatus) {
      return ResultRespone.success(tfUserStatusService.updateById(tfUserStatus));
    }

    /**
     * 通过id删除用户状态记录表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除用户状态记录表" )
    @ApiOperation(value = "通过ID删除用户状态记录表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(tfUserStatusService.removeById(id));
    }
}
