/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.TroubleCategory;
import com.tofly.dyrq.service.TroubleCategoryService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

/**
 * 隐患类别
 *
 * @author wangjian
 * @date Fri Apr 03 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/troublecategory")
@Api(tags="隐患类别接口")
public class TroubleCategoryController {

    private final  TroubleCategoryService troubleCategoryService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param troubleCategory 隐患类别
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getTroubleCategoryPage(Page page, @ModelAttribute TroubleCategory troubleCategory) {
      return ResultRespone.success(troubleCategoryService.page(page, Wrappers.query(troubleCategory)));
    }


    /**
     * 通过id查询隐患类别
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(troubleCategoryService.getById(id));
    }

    /**
     * 新增隐患类别
     * @param troubleCategory 隐患类别
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增隐患类别" )
    @ApiOperation(value = "新增隐患类别")
    @PostMapping
    public ResultRespone save(@RequestBody TroubleCategory troubleCategory) {
      return ResultRespone.success(troubleCategoryService.save(troubleCategory));
    }

    /**
     * 修改隐患类别
     * @param troubleCategory 隐患类别
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改隐患类别" )
    @ApiOperation(value = "修改隐患类别")
    @PutMapping
    public ResultRespone updateById(@RequestBody TroubleCategory troubleCategory) {
      return ResultRespone.success(troubleCategoryService.updateById(troubleCategory));
    }

    /**
     * 通过id删除隐患类别
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除隐患类别" )
    @ApiOperation(value = "通过ID删除隐患类别")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(troubleCategoryService.removeById(id));
    }
}
