/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.TroubleCheck;
import com.tofly.dyrq.service.TroubleCheckService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 隐患流程记录表
 *
 * @author admin
 * @date Thu Mar 11 00:00:00 GMT+08:00 2021
 */
@RestController
@AllArgsConstructor
@RequestMapping("/troublecheck")
@Api(tags="隐患流程记录表接口")
public class TroubleCheckController {

    private final  TroubleCheckService troubleCheckService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param troubleCheck 隐患流程记录表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getTroubleCheckPage(Page page, @ModelAttribute TroubleCheck troubleCheck) {
      return ResultRespone.success(troubleCheckService.page(page, Wrappers.query(troubleCheck)));
    }


    /**
     * 通过id查询隐患流程记录表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(troubleCheckService.getById(id));
    }

    /**
     * 新增隐患流程记录表
     * @param troubleCheck 隐患流程记录表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增隐患流程记录表" )
    @ApiOperation(value = "新增隐患流程记录表")
    @PostMapping
    public ResultRespone save(@RequestBody TroubleCheck troubleCheck) {
      return ResultRespone.success(troubleCheckService.save(troubleCheck));
    }

    /**
     * 修改隐患流程记录表
     * @param troubleCheck 隐患流程记录表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改隐患流程记录表" )
    @ApiOperation(value = "修改隐患流程记录表")
    @PutMapping
    public ResultRespone updateById(@RequestBody TroubleCheck troubleCheck) {
      return ResultRespone.success(troubleCheckService.updateById(troubleCheck));
    }

    /**
     * 通过id删除隐患流程记录表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除隐患流程记录表" )
    @ApiOperation(value = "通过ID删除隐患流程记录表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(troubleCheckService.removeById(id));
    }

    /**
     * 通过id批量删除隐患流程记录表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除隐患流程记录表" )
    @ApiOperation(value = "通过ID批量删除隐患流程记录表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(troubleCheckService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
