/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.TroubleEnentType;
import com.tofly.dyrq.service.TroubleEnentTypeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;
/**
 * 隐患事件级别表
 *
 * @author admin
 * @date Sat Jul 17 00:00:00 CST 2202
 */
@RestController
@AllArgsConstructor
@RequestMapping("/troubleenenttype")
@Api(tags="隐患事件级别表接口")
public class TroubleEnentTypeController {

    private final  TroubleEnentTypeService troubleEnentTypeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param troubleEnentType 隐患事件级别表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getTroubleEnentTypePage(Page page, @ModelAttribute TroubleEnentType troubleEnentType) {
      return ResultRespone.success(troubleEnentTypeService.page(page, Wrappers.query(troubleEnentType)));
    }


    /**
     * 通过id查询隐患事件级别表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(troubleEnentTypeService.getById(id));
    }

    /**
     * 新增隐患事件级别表
     * @param troubleEnentType 隐患事件级别表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增隐患事件级别表" )
    @ApiOperation(value = "新增隐患事件级别表")
    @PostMapping
    public ResultRespone save(@RequestBody TroubleEnentType troubleEnentType) {
      return ResultRespone.success(troubleEnentTypeService.save(troubleEnentType));
    }

    /**
     * 修改隐患事件级别表
     * @param troubleEnentType 隐患事件级别表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改隐患事件级别表" )
    @ApiOperation(value = "修改隐患事件级别表")
    @PutMapping
    public ResultRespone updateById(@RequestBody TroubleEnentType troubleEnentType) {
      return ResultRespone.success(troubleEnentTypeService.updateById(troubleEnentType));
    }

    /**
     * 通过id删除隐患事件级别表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除隐患事件级别表" )
    @ApiOperation(value = "通过ID删除隐患事件级别表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(troubleEnentTypeService.removeById(id));
    }

    /**
     * 通过id批量删除隐患事件级别表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除隐患事件级别表" )
    @ApiOperation(value = "通过ID批量删除隐患事件级别表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(troubleEnentTypeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
