/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.TroubleLevel;
import com.tofly.dyrq.service.TroubleLevelService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;

/**
 * 隐患级别
 *
 * @author wangjian
 * @date Fri Apr 03 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/troublelevel")
@Api(tags="隐患级别接口")
public class TroubleLevelController {

    private final  TroubleLevelService troubleLevelService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param troubleLevel 隐患级别
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getTroubleLevelPage(Page page, @ModelAttribute TroubleLevel troubleLevel) {
      return ResultRespone.success(troubleLevelService.page(page, Wrappers.query(troubleLevel)));
    }


    /**
     * 通过id查询隐患级别
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(troubleLevelService.getById(id));
    }

    /**
     * 新增隐患级别
     * @param troubleLevel 隐患级别
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增隐患级别" )
    @ApiOperation(value = "新增隐患级别")
    @PostMapping
    public ResultRespone save(@RequestBody TroubleLevel troubleLevel) {
      return ResultRespone.success(troubleLevelService.save(troubleLevel));
    }

    /**
     * 修改隐患级别
     * @param troubleLevel 隐患级别
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改隐患级别" )
    @ApiOperation(value = "修改隐患级别")
    @PutMapping
    public ResultRespone updateById(@RequestBody TroubleLevel troubleLevel) {
      return ResultRespone.success(troubleLevelService.updateById(troubleLevel));
    }

    /**
     * 通过id删除隐患级别
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除隐患级别" )
    @ApiOperation(value = "通过ID删除隐患级别")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(troubleLevelService.removeById(id));
    }

    /**
     * 通过id批量删除隐患级别
     * @param ids ids
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除隐患级别" )
    @ApiOperation(value = "通过ID批量删除隐患级别")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(troubleLevelService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
