/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.TroubleType;
import com.tofly.dyrq.service.TroubleTypeService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

import java.util.Arrays;

/**
 * 隐患分类
 *
 * @author wangjian
 * @date Fri Apr 03 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/troubletype")
@Api(tags="隐患分类接口")
public class TroubleTypeController {

    private final  TroubleTypeService troubleTypeService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param troubleType 隐患分类
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getTroubleTypePage(Page page, @ModelAttribute TroubleType troubleType) {
      return ResultRespone.success(troubleTypeService.page(page, Wrappers.query(troubleType)));
    }


    /**
     * 通过id查询隐患分类
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(troubleTypeService.getById(id));
    }

    /**
     * 新增隐患分类
     * @param troubleType 隐患分类
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增隐患分类" )
    @ApiOperation(value = "新增隐患分类")
    @PostMapping
    public ResultRespone save(@RequestBody TroubleType troubleType) {
      return ResultRespone.success(troubleTypeService.save(troubleType));
    }

    /**
     * 修改隐患分类
     * @param troubleType 隐患分类
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改隐患分类" )
    @ApiOperation(value = "修改隐患分类")
    @PutMapping
    public ResultRespone updateById(@RequestBody TroubleType troubleType) {
      return ResultRespone.success(troubleTypeService.updateById(troubleType));
    }

    /**
     * 通过id删除隐患分类
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除隐患分类" )
    @ApiOperation(value = "通过ID删除隐患分类")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(troubleTypeService.removeById(id));
    }

    /**
     * 通过id批量删除隐患分类
     * @param ids ids
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除隐患分类" )
    @ApiOperation(value = "通过ID批量删除隐患分类")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(troubleTypeService.removeByIds(Arrays.asList(ids.split(","))));
    }
}
