/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.entity.WebWarningconfig;
import com.tofly.dyrq.service.WebWarningconfigService;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import org.springframework.web.bind.annotation.ModelAttribute;

/**
 * 预警分析参数配置表
 *
 * @author wangjian
 * @date Wed Apr 08 00:00:00 CST 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/webwarningconfig")
@Api(tags="预警分析参数配置表接口")
public class WebWarningconfigController {

    private final  WebWarningconfigService webWarningconfigService;

    /**
    * 分页查询
    * @param page 分页对象
    * @param webWarningconfig 预警分析参数配置表
    * @return
    */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getWebWarningconfigPage(Page page, @ModelAttribute WebWarningconfig webWarningconfig) {
      return ResultRespone.success(webWarningconfigService.page(page, Wrappers.query(webWarningconfig)));
    }


    /**
     * 通过id查询预警分析参数配置表
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
      return ResultRespone.success(webWarningconfigService.getById(id));
    }

    /**
     * 新增预警分析参数配置表
     * @param webWarningconfig 预警分析参数配置表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增预警分析参数配置表" )
    @ApiOperation(value = "新增预警分析参数配置表")
    @PostMapping
    public ResultRespone save(@RequestBody WebWarningconfig webWarningconfig) {
      return ResultRespone.success(webWarningconfigService.save(webWarningconfig));
    }

    /**
     * 修改预警分析参数配置表
     * @param webWarningconfig 预警分析参数配置表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改预警分析参数配置表" )
    @ApiOperation(value = "修改预警分析参数配置表")
    @PutMapping
    public ResultRespone updateById(@RequestBody WebWarningconfig webWarningconfig) {
      return ResultRespone.success(webWarningconfigService.updateById(webWarningconfig));
    }

    /**
     * 通过id删除预警分析参数配置表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除预警分析参数配置表" )
    @ApiOperation(value = "通过ID删除预警分析参数配置表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/{id}" )
    public ResultRespone removeById(@PathVariable Long id) {
      return ResultRespone.success(webWarningconfigService.removeById(id));
    }
}
