/*
 * Author: wangjian
 * 生成controller
 */
package com.tofly.dyrq.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrq.custom.entity.ConstructionSiteSearch;
import com.tofly.dyrq.custom.entity.ConstructionSiteSearchUser;
import com.tofly.dyrq.custom.entity.RushRepairSearch;
import com.tofly.dyrq.custom.service.CustomConstructionSiteService;
import com.tofly.dyrq.custom.service.CustomMessageService;
import com.tofly.dyrq.entity.*;
import com.tofly.dyrq.service.ConstructionSiteService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
//上报
/**
 * 工地监护工单表自定义接口
 *
 * @author Zhangyu
 * @date Sun Jun 28 00:00:00 GMT+08:00 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/constructionsite")
@Api(tags="工地监护工单表接口")
public class CustomConstructionSiteController {
    private final CustomConstructionSiteService customConstructionSiteService;
    private final ConstructionSiteService constructionSiteService;
    private final CustomMessageService customMessageService;

    /**
     * 工地监护工单新增
     *
     * @param constructionSite 工地监护工单表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "工地监护工单新增")
    @ApiOperation(value = "工地监护工单新增")
    @PostMapping
    public ResultRespone saveConstructionSite(HttpServletRequest servletRequest, ConstructionSite constructionSite) {
        return ResultRespone.success(customConstructionSiteService.saveConstructionSite(servletRequest, constructionSite));
    }

    /**
     * 按条件分页查询
     *
     * @param page
     * @param constructionSiteSearch
     * @return
     */
    @ApiOperation(value = "按条件分页查询")
    @GetMapping("/page")
    public ResultRespone getConstructionPage(Page page, @ModelAttribute ConstructionSiteSearch constructionSiteSearch) {
        return ResultRespone.success(customConstructionSiteService.pageByCondition2(page, constructionSiteSearch));
    }

    /**
     * 按条件分页查询(新)
     *
     * @param page
     * @param constructionSiteSearch
     * @return
     */
    @ApiOperation(value = "按条件分页查询(新)")
    @GetMapping("/pagenew")
    public ResultRespone getConstructionPage2(Page page, @ModelAttribute ConstructionSiteSearch constructionSiteSearch) {
        return ResultRespone.success(customConstructionSiteService.pageByCondition(page, constructionSiteSearch));
    }

    /**
     * 根据ID查询工地监护工单详情
     *
     * @param id 根据ID查询工地监护工单详情
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "根据ID查询工地监护工单详情")
    @ApiOperation(value = "根据ID查询工地监护工单详情")
    @GetMapping("/{id}")
    public ResultRespone getRushRepairById(@PathVariable("id") Long id) {
        return ResultRespone.success(customConstructionSiteService.getConstructionSiteById(id));
    }

    @ApiOperation(value = "工地监护工单个人查询")
    @GetMapping("/pageByUser")
    public ResultRespone getConstructionPageByUser(Page page, @ModelAttribute ConstructionSiteSearchUser constructionSiteSearchUser) {
//        Long userid= SecurityUtils.getUserId();
//        return ResultRespone.success(constructionSiteService.page(page, Wrappers.<ConstructionSite>lambdaQuery()
//            .eq(StringUtil.isNotEmpty(constructionSiteSearchUser.getConstructionState()), ConstructionSite::getConstructionState,constructionSiteSearchUser.getConstructionState())
//            .eq(StringUtil.isNotEmpty(constructionSiteSearchUser.getSiteName()), ConstructionSite::getSiteName,constructionSiteSearchUser.getSiteName())
//            .eq(StringUtil.isNotEmpty(constructionSiteSearchUser.getSiteType()), ConstructionSite::getSiteType,constructionSiteSearchUser.getSiteType())
//            .and(wrapper1->wrapper1.eq(ConstructionSite::getConstructionPerson,userid)
//                    .or(wrapper->wrapper.eq(ConstructionSite::getDispatchUserId,userid))
//                    .or(wrapper2->wrapper2.eq(ConstructionSite::getCreatUser,userid))
//                )
//            ));
        return ResultRespone.success(customConstructionSiteService.getConstructionPageByUser(page,constructionSiteSearchUser));
    }

    /**
     * 根据ID删除抢维修工单
     *
     * @param id 根据ID删除抢维修工单
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "根据ID删除抢维修工单")
    @ApiOperation(value = "根据ID删除抢维修工单")
    @DeleteMapping("/{id}")
    public ResultRespone removeConstructionSiteById(@PathVariable("id") Long id) {
        return ResultRespone.success(customConstructionSiteService.removeById(id));
    }

    /**
     * 工地监护过程管理
     *
     * @param constructionProcess
     * @param servletRequest
     * @return
     */
    @ToFlyAppLog(title = "工地监护过程管理")
    @ApiOperation(value = "工地监护过程管理")
    @PostMapping("/processmanage")
    public ResultRespone saveConstructionSiteProcess(HttpServletRequest servletRequest, ConstructionProcess constructionProcess) {
        return ResultRespone.success(customConstructionSiteService.saveConstructionProcess(servletRequest, constructionProcess));
    }

    /**
     * 工地监护工单修改
     *
     * @param constructionSite
     * @return
     */
    @ToFlyAppLog(title = "工地监护工单修改")
    @ApiOperation(value = "工地监护工单修改")
    @PutMapping
    public ResultRespone updateConstructionSite(@RequestBody ConstructionSite constructionSite) {
        Boolean res = constructionSiteService.updateById(constructionSite);
        //派工
        if (res && constructionSite.getConstructionPerson() != null) {
            if (StringUtil.isNotEmpty(constructionSite.getConstructionState()) && "2".equals(constructionSite.getConstructionState())) {
                customMessageService.sendMessage(constructionSite.getConstructionPerson().toString() , "您有一条待处理抢维修工单", "1","");
            }
        }
        //审核
        else if (StringUtil.isNotEmpty(constructionSite.getConstructionState()) && "3".equals(constructionSite.getConstructionState())) {
            ConstructionSite one = constructionSiteService.getById(constructionSite.getId());
            customMessageService.sendMessage(one.getDispatchUserId().toString(), "您有一条待审核抢维修工单", "1","");
        }
        return ResultRespone.success(res);
    }

    /**
     * 工地监护工单修改(移动端)
     *
     * @param constructionSite
     * @return
     */
    @ToFlyAppLog(title = "工地监护工单修改(移动端)")
    @ApiOperation(value = "工地监护工单修改(移动端)")
    @PostMapping("/updateapp")
    public ResultRespone updateConstructionSiteForApp(@ModelAttribute ConstructionSite constructionSite) {
        return ResultRespone.success(constructionSiteService.updateById(constructionSite));
    }

    /**
     * 工地监护审核施工结束
     *
     * @param constructionAuditEnd
     * @param servletRequest
     * @return
     */
    @ToFlyAppLog(title = "工地监护审核施工结束")
    @ApiOperation(value = "工地监护审核施工结束")
    @PostMapping("/auditend")
    public ResultRespone saveConstructionSiteAuditEnd(HttpServletRequest servletRequest, ConstructionAuditEnd
            constructionAuditEnd) {
        return ResultRespone.success(customConstructionSiteService.saveConstructionAuditEnd(servletRequest, constructionAuditEnd));
    }
}
