package com.tofly.dyrq.custom.controller;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.custom.service.CustomEpmtArchivesService;
import com.tofly.dyrq.custom.service.CustomEpmtBudgetService;
import com.tofly.dyrq.entity.EpmtArchives;
import com.tofly.dyrq.entity.EpmtBudget;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 设备相关档案表
 *
 * @author Zhangyu
 * @date Tue Dec 15 00:00:00 GMT+08:00 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/epmtarchives")
@Api(tags="设备管理档案表接口")
public class CustomEmptArchivesController {

    private final CustomEpmtArchivesService customEpmtArchivesService;

    /**
            * 新增设备管理档案表
     * @param epmtArchives 设备管理档案表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增设备管理档案表" )
    @ApiOperation(value = "新增设备管理档案表")
    @PostMapping(consumes = "multipart/form-data")
    public ResultRespone save(EpmtArchives epmtArchives, HttpServletRequest request) {
      return ResultRespone.success(customEpmtArchivesService.save(epmtArchives,request));
    }

    /**
     * 修改设备管理档案表
     * @param epmtArchives 设备管理档案表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改设备管理档案表" )
    @ApiOperation(value = "修改设备管理档案表")
    @PostMapping(value = "/update",consumes = "multipart/form-data")
    public ResultRespone updateById(EpmtArchives epmtArchives,HttpServletRequest request) {
      return ResultRespone.success(customEpmtArchivesService.updateById(epmtArchives,request));
    }

    /**
     * 通过id批量删除设备管理档案表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除设备管理档案表" )
    @ApiOperation(value = "通过ID批量设备管理档案表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(customEpmtArchivesService.removeById(Arrays.asList(ids.split(","))));
    }
}
