package com.tofly.dyrq.custom.controller;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.custom.service.CustomEpmtBudgetService;
import com.tofly.dyrq.custom.service.CustomEpmtSystemService;
import com.tofly.dyrq.entity.EpmtBudget;
import com.tofly.dyrq.entity.EpmtSystem;
import io.swagger.annotations.*;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 设备相关预算表
 *
 * @author Zhangyu
 * @date Tue Dec 15 00:00:00 GMT+08:00 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/epmtbudget")
@Api(tags="设备管理预算表接口")
public class CustomEmptBudgetController {

    private final CustomEpmtBudgetService customEpmtBudgetService;

    /**
            * 新增设备相关预算表
     * @param epmtBudget 设备相关预算表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增设备管理预算表" )
    @ApiOperation(value = "新增设备管理预算表")
    @PostMapping(consumes = "multipart/form-data")
    public ResultRespone save(EpmtBudget epmtBudget, HttpServletRequest request) {
      return ResultRespone.success(customEpmtBudgetService.save(epmtBudget,request));
    }

    /**
     * 修改设备相关预算表
     * @param epmtBudget 设备相关预算表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改设备管理预算表" )
    @ApiOperation(value = "修改设备管理预算表")
    @PostMapping(value = "/update",consumes = "multipart/form-data")
    public ResultRespone updateById(EpmtBudget epmtBudget,HttpServletRequest request) {
      return ResultRespone.success(customEpmtBudgetService.updateById(epmtBudget,request));
    }

    /**
     * 通过id批量删除设备相关预算表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除设备管理预算表" )
    @ApiOperation(value = "通过ID批量设备管理预算表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(customEpmtBudgetService.removeById(Arrays.asList(ids.split(","))));
    }
}
