package com.tofly.dyrq.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.custom.service.CustomEpmtSystemService;
import com.tofly.dyrq.entity.EpmtSystem;
import com.tofly.dyrq.service.EpmtSystemService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.bouncycastle.math.ec.ECMultiplier;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 设备相关制度表
 *
 * @author Zhangyu
 * @date Tue Dec 15 00:00:00 GMT+08:00 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/epmtsystem")
@Api(tags="设备管理制度表接口")
public class CustomEmptSystemController {

    private final CustomEpmtSystemService customEpmtSystemService;
    private final EpmtSystemService epmtSystemService;

    /**
            * 新增设备管理制度表
     * @param epmtSystem 设备管理制度表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增设备管理制度表" )
    @ApiOperation(value = "新增设备管理制度表")
    @PostMapping(consumes = "multipart/form-data")
    public ResultRespone save(EpmtSystem epmtSystem, HttpServletRequest request) {
      return ResultRespone.success(customEpmtSystemService.save(epmtSystem,request));
    }

    /**
     * 修改设备管理制度表
     * @param epmtSystem 设备管理制度表
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改设备管理制度表" )
    @ApiOperation(value = "修改设备管理制度表")
    @PostMapping(value = "/update",consumes = "multipart/form-data")
    public ResultRespone updateById(EpmtSystem epmtSystem,HttpServletRequest request) {
      return ResultRespone.success(customEpmtSystemService.updateById(epmtSystem,request));
    }

    /**
     * 通过id批量删除设备管理制度表
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除设备管理制度表" )
    @ApiOperation(value = "通过ID批量设备管理制度表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(customEpmtSystemService.removeById(Arrays.asList(ids.split(","))));
    }

    /**
     * 分页查询
     * @param page 分页对象
     * @param epmtSystem 设备管理制度表
     * @return
     */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getEpmtSystemPage(Page page, @ModelAttribute EpmtSystem epmtSystem) {
        return ResultRespone.success(epmtSystemService.page(page, Wrappers.<EpmtSystem>lambdaQuery()
                .like(StringUtil.isNotEmpty(epmtSystem.getName()),EpmtSystem::getName,epmtSystem.getName())));
    }
}
