package com.tofly.dyrq.custom.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.dyrq.custom.entity.*;
import com.tofly.dyrq.custom.service.CustomMaintenanceService;
import com.tofly.common.mapper.ToflyCodeCommonMapper;
import com.tofly.dyrq.entity.*;
import com.tofly.dyrq.service.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.elasticsearch.search.aggregations.support.ValuesSourceAggregationBuilder;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


@RestController
@AllArgsConstructor
public class CustomMaintenanceController {
    private final CustomMaintenanceService customMaintenanceService;
    private final MaintenanceService maintenanceService;
    private final MaintenanceDeviceService maintenanceDeviceService;
    private final PlanPointTempService planPointTempService;

    @ToFlyAppLog(title = "新增巡护记录")
    @PostMapping(value = "/maintenance",headers = "content-type=multipart/form-data")
    @ApiOperation(tags = "调压站及楼栋调压箱巡护记录表接口", value = "新增巡护记录",notes = "photos:照片文件[多个/选填];需传参数：用气片区，地址，检查起始时间，检查结束时间，巡护员,巡检计划id, 巡检计划临时点(pointId)")
    public ResultRespone addMaintenance(HttpServletRequest request,AddMaintenance addMaintenance) {
        if (addMaintenance.getChildPlanId() == null) {
            throw new ToflyDeniedException("巡检计划id不能为空");
        }
        if (addMaintenance.getPlanTempPointId() == null) {
            throw new ToflyDeniedException("巡检计划临时点ID");
        }
        //判断该计划中该点是否已上传
        Maintenance one = maintenanceService.getOne(Wrappers.<Maintenance>lambdaQuery().eq(Maintenance::getChildPlanId, addMaintenance.getChildPlanId())
                .eq(Maintenance::getPlanTempPointId, addMaintenance.getPlanTempPointId()));
        if (one != null) {
            return ResultRespone.failed(2,"当前计划该设备记录已上传");
        }
        return ResultRespone.success(customMaintenanceService.addMaintenance(request,addMaintenance));
    }

    @ToFlyAppLog(title = "修改巡护记录")
    @PutMapping(value = "/maintenance",headers = "content-type=multipart/form-data")
    @ApiOperation(tags = "调压站及楼栋调压箱巡护记录表接口", value = "修改巡护记录",notes = "photos:照片文件[多个/选填];需传参数：用气片区，地址，检查起始时间，检查结束时间，巡护员,巡检计划id, 巡检计划临时点(pointId)")
    public ResultRespone putMaintenance(HttpServletRequest request,AddMaintenance addMaintenance) {
        if (addMaintenance.getId() == null){
            throw new ToflyDeniedException("巡护记录ID不能为空");
        }
        return ResultRespone.success(customMaintenanceService.putMaintenance(request,addMaintenance));
    }

    @GetMapping("/maintenance/page")
    @ApiOperation(tags = "调压站及楼栋调压箱巡护记录表接口", value = "巡护记录分页查询")
    public ResultRespone getMaintenancePage(Page page, @ModelAttribute MaintenanceSearch maintenanceSearch) {
        return ResultRespone.success(customMaintenanceService.getMaintenancePage(page,maintenanceSearch));
    }


    @ToFlyAppLog(title = "新增登记表")
    @PostMapping(value = "/maintenancedevice",headers = "content-type=multipart/form-data")
    @ApiOperation(tags = "设备设施运行维修保养登记表接口", value = "新增登记表",notes = "photos:照片文件[多个/选填];需传参数：设备操作人,保养内容id数组,巡检计划id，巡检计划临时点(pointId)")
    public ResultRespone addMaintenanceDevice(HttpServletRequest request,AddMaintenanceDevice addMaintenanceDevice) {
        if (addMaintenanceDevice.getChildPlanId() == null) {
            throw new ToflyDeniedException("巡检计划id不能为空");
        }
        if (addMaintenanceDevice.getPlanTempPointId() == null) {
            throw new ToflyDeniedException("巡检计划临时点ID");
        }
        Long childPlanId = addMaintenanceDevice.getChildPlanId();
        Long pointId = addMaintenanceDevice.getPlanTempPointId();
        //判断是否该计划该点存在
        PlanPointTemp pointTemp = planPointTempService.getOne(Wrappers.<PlanPointTemp>lambdaQuery()
                .eq(PlanPointTemp::getChildPlanId, childPlanId)
                .eq(PlanPointTemp::getPointId, pointId));
        if (pointTemp == null) {
            throw new ToflyDeniedException("该计划不存在该点信息");
        }
        //判断该计划中该点是否已上传
        MaintenanceDevice one = maintenanceDeviceService.getOne(Wrappers.<MaintenanceDevice>lambdaQuery()
                .eq(MaintenanceDevice::getChildPlanId, childPlanId)
                .eq(MaintenanceDevice::getPlanTempPointId, pointId));
        if (one != null) {
            return ResultRespone.failed(2,"当前计划该设备记录已上传");
        }
        return ResultRespone.success(customMaintenanceService.addMaintenanceDevice(request,addMaintenanceDevice));
    }

    @ToFlyAppLog(title = "修改登记表")
    @PutMapping(value = "/maintenancedevice",headers = "content-type=multipart/form-data")
    @ApiOperation(tags = "设备设施运行维修保养登记表接口", value = "修改登记表",notes = "photos:照片文件[多个/选填];需传参数：设备操作人,保养内容id数组,巡检计划id，巡检计划临时点(pointId)")
    public ResultRespone putMaintenanceDevice(HttpServletRequest request,AddMaintenanceDevice addMaintenanceDevice) {
        if (addMaintenanceDevice.getId() == null){
            throw new ToflyDeniedException("登记表记录ID不能为空");
        }
        return ResultRespone.success(customMaintenanceService.putMaintenanceDevice(request,addMaintenanceDevice));
    }

    @GetMapping("/maintenancedevice/page")
    @ApiOperation(tags = "设备设施运行维修保养登记表接口", value = "登记表分页查询")
    public ResultRespone getMaintenanceDevicePage(Page page, @ModelAttribute MaintenanceDeviceSearch maintenanceDeviceSearch) {
        return ResultRespone.success(customMaintenanceService.getMaintenanceDevicePage(page, maintenanceDeviceSearch));
    }
}
