package com.tofly.dyrq.custom.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.dyrq.custom.entity.MessageSave;
import com.tofly.dyrq.custom.service.CustomMessageService;

import com.tofly.dyrq.service.MessageRecipientService;
import com.tofly.dyrq.service.MessageService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

@RestController
@AllArgsConstructor
@RequestMapping("/message")
@Api(tags = "消息推送表接口")
public class CustomMessageController {
    private final CustomMessageService customMessageService;

    @GetMapping("/reveipt")
    @ApiOperation(value = "当前用户消息列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "searchText", value = "信息内容模糊查询", required = false, dataType = "String")
    })
    public ResultRespone receiptMessage(Page page,String searchText) {
        return ResultRespone.success(customMessageService.receiptMessage(page,searchText));
    }

    @ToFlyAppLog(title = "新增消息推送表")
    @ApiOperation(value = "新增消息推送表",notes = "photos:照片文件[多个/选填],audios:语音文件[多个/选填]")
    @PostMapping(headers = "content-type=multipart/form-data")
    public ResultRespone save(HttpServletRequest request,MessageSave messageSave) {
        if (StringUtil.isEmpty(String.valueOf(messageSave.getMessage()))) {
            throw new ToflyDeniedException("消息内容不能为空");
        }
        if (StringUtil.isEmpty(String.valueOf(messageSave.getReceipt()))) {
            throw new ToflyDeniedException("接收人不能为空");
        }
        return ResultRespone.success(customMessageService.save(request, messageSave));
    }

    @GetMapping("/{id}")
    @ApiOperation(value = "通过消息ID查询详情，当前用户接收消息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long"),
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return ResultRespone.success(customMessageService.getById(id));
    }

    @GetMapping("/getbyid/{id}")
    @ApiOperation(value = "通过消息ID查询详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long"),
    })
    public ResultRespone getInfoById(@PathVariable("id") Long id) {
        return ResultRespone.success(customMessageService.getInfoById(id));
    }

    @GetMapping("/sent")
    @ApiOperation(value = "当前用户已发消息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "searchText", value = "信息内容模糊查询", required = false, dataType = "String")
    })
    public ResultRespone sent(Page page,String searchText) {
        return ResultRespone.success(customMessageService.sent(page,searchText));
    }


    @ToFlyAppLog(title="通过id批量删除当前用户消息" )
    @ApiOperation(value = "通过id批量删除当前用户消息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "messageId字符串，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone deleteByIds(String ids) {
        if (StringUtil.isEmpty(String.valueOf(ids))) {
            throw new ToflyDeniedException("messageId不能为空");
        }
        return ResultRespone.success(customMessageService.deleteByIds(ids));
    }

    @ToFlyAppLog(title="当前用户批量已读消息" )
    @ApiOperation(value = "当前用户批量已读消息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "messageId字符串，多个id以逗号隔开", required = true, dataType = "String")
    })
    @PutMapping("/readByIds")
    public ResultRespone readByIds(String ids) {
        if (StringUtil.isEmpty(String.valueOf(ids))) {
            throw new ToflyDeniedException("messageId不能为空");
        }
      return ResultRespone.success(customMessageService.readByIds(ids));
    }

}
