package com.tofly.dyrq.custom.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.custom.service.CustomPhotoService;
import com.tofly.dyrq.entity.UploadPhoto;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * 图片上传相关自定义接口
 * @author tuonbed
 * @date 2020/4/28  17:16
 */
@RestController
@AllArgsConstructor
@RequestMapping("/uploadphoto")
@Api(tags="上传照片接口")
public class CustomPhotoController {


    private final CustomPhotoService customPhotoService;
    /**
     * 新增上传照片
     * @return ResultRespone
     */
    @ToFlyAppLog(title="新增上传照片" )
    @ApiOperation(value = "新增上传照片")
    @PostMapping
    public ResultRespone save(HttpServletRequest request, UploadPhoto uploadPhoto) {
        return ResultRespone.success(customPhotoService.savePhoto(request,uploadPhoto));
    }

    /**
     * 分页查询
     * @param page 分页对象
     * @param uploadPhoto 上传照片
     * @return
     */
    @GetMapping("/page" )
    @ApiOperation(value = "分页查询")
    public ResultRespone getUploadPhotoPage(Page page, @ModelAttribute UploadPhoto uploadPhoto) {
        return ResultRespone.success(customPhotoService.getUploadPhotoPage(page,uploadPhoto));
    }


    /**
     * 通过id查询上传照片
     * @param id id
     * @return ResultRespone
     */
    @GetMapping("/{id}" )
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id" ) Long id) {
        return ResultRespone.success(customPhotoService.getById(id));
    }

    /**
     * 修改上传照片
     * @param uploadPhoto 上传照片
     * @return ResultRespone
     */
    @ToFlyAppLog(title="修改上传照片" )
    @ApiOperation(value = "修改上传照片")
    @PutMapping
    public ResultRespone updateById(HttpServletRequest request, UploadPhoto uploadPhoto) {
        return ResultRespone.success(customPhotoService.updatePhoto(request,uploadPhoto));
    }

    /**
     * 通过id删除上传照片
     * @param id id
     * @return ResultRespone
     */
//    @ToFlyAppLog(title="通过id删除上传照片" )
//    @ApiOperation(value = "通过ID删除上传照片")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
//    })
//    @DeleteMapping("/{id}" )
//    public ResultRespone removeById(@PathVariable Long id) {
//        return ResultRespone.success(customPhotoService.removeById(id));
//    }

    /**
     * 通过id批量删除上传照片
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id批量删除上传照片" )
    @ApiOperation(value = "通过ID批量删除上传照片")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds" )
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(customPhotoService.removeById(ids));
    }

}
