package com.tofly.dyrq.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.Date8Util;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.dyrq.custom.entity.*;
import com.tofly.dyrq.custom.service.CustomPlanService;
import com.tofly.dyrq.custom.service.PlanTaskService;
import com.tofly.dyrq.entity.Plan;
import com.tofly.dyrq.entity.PlanPointTemp;
import com.tofly.dyrq.entity.PlanPointType;
import com.tofly.dyrq.entity.SysDeviceType;
import com.tofly.dyrq.service.*;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.text.ParseException;

/**
 * @author tuonbed
 * @date 2020/4/13  13:21
 * 自定义的一些计划相关的接口
 */
@RestController
@AllArgsConstructor
public class CustomPlanController {
    private final CustomPlanService customPlanService;
    private final PlanPointTypeService planPointTypeService;
    private final SysDeviceTypeService sysDeviceTypeService;
    private final PlanChildService planChildService;
    private final PlanService planService;


    @ToFlyAppLog(title = "修改巡检片区表")
    @ApiOperation(tags = "巡检片区表接口", value = "修改巡检片区表")
    @PutMapping("/gpsregion")
    public ResultRespone updateGpsRegion(@RequestBody RegionValidEntity regionValidEntity) {
        return ResultRespone.success(customPlanService.updateGpsRegion(regionValidEntity));
    }

    @ApiOperation(tags = "设备类型表接口", value = "查询可选巡检类型列表",notes = "巡检设备type(0:点，1：线)")
    @GetMapping("/sysdevicetype/getPlanType")
    public ResultRespone getPlanType(Page page,@RequestParam(required = false) String type) {
        return ResultRespone.success(planPointTypeService.page(page, Wrappers.<PlanPointType>lambdaQuery()
                .eq(StringUtil.isNotEmpty(type),PlanPointType::getType,type)
                .notExists("select 1 from tf_sys_device_type where del_flag='1' and point_type_id=tf_plan_point_type.id")));
    }

    /**
     * 通过id删除设备类型表
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除设备类型表")
    @ApiOperation(tags = "设备类型表接口", value = "通过ID删除设备类型表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/sysdevicetype/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        SysDeviceType sysDeviceType = sysDeviceTypeService.getOne(Wrappers.<SysDeviceType>lambdaQuery().eq(SysDeviceType::getId, id)
                .apply("exists( select 1 from tf_plan_point_type a, tf_plan b,TF_SYS_DEVICE_TYPE c where\n" +
                        "        b.device_type_id = a.id and c.point_type_id = a.id and b.del_flag = '1' and a.del_flag = '1' and c.del_flag = '1' and c.id = {0}) ", id));
        if (sysDeviceType != null) {
            return ResultRespone.failed("当前设备类型存在着计划，不允许删除");
        }
        return ResultRespone.success(sysDeviceTypeService.removeById(id));
    }

    @ToFlyAppLog(title = "新增巡检计划表")
    @PostMapping("/plan")
    @ApiOperation(tags = "巡检计划表接口", value = "新增巡检计划表")
    public ResultRespone savePlan(@RequestBody CustomPlan customPlan) {
        if (customPlan.getPlanMode() == null) {
            throw new ToflyDeniedException("请添加PlanMode");
        }
        if (customPlan.getUserId() == null) {
            throw new ToflyDeniedException("请添加UserId");
        }
        if (customPlan.getDeviceTypeId() == null) {
            throw new ToflyDeniedException("请添加DeviceTypeId");
        }
        if (customPlan.getPeriodId() == null) {
            throw new ToflyDeniedException("请添加getPeriodId");
        }
        if (customPlan.getPlanBegindate() == null) {
            throw new ToflyDeniedException("请添加PlanBegindate");
        }
        if (customPlan.getPlanEnddate() == null) {
            throw new ToflyDeniedException("请添加PlanEnddate");
        }
        return ResultRespone.success(customPlanService.savePlan(customPlan));
    }

    @ToFlyAppLog(title = "通过id批量删除巡检主计划表")
    @ApiOperation(tags = "巡检计划表接口", value = "通过ID批量删除巡检主计划表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/plan/deleteByIds")
    public ResultRespone removeByIds(String ids) {
        return ResultRespone.success(customPlanService.removeByIds(ids));
    }

    @GetMapping("/plan/page")
    @ApiOperation(tags = "巡检计划表接口", value = "主计划分页查询")
    public ResultRespone getPlanPage(Page page, @ModelAttribute Plan plan) {
        return ResultRespone.success(customPlanService.getPlanPage(page, plan));
    }

    @GetMapping("/planchild/page")
    @ApiOperation(tags = "巡检计划表接口", value = "子计划分页查询")
    public ResultRespone getPlanChildPage(Page page, @ModelAttribute ChildPlanSearch childPlanSearch) {
        return ResultRespone.success(customPlanService.getPlanChildPage(page, childPlanSearch));
    }

    @GetMapping("/planchild/geoInfo/{id}")
    @ApiOperation(tags = "巡检计划表接口", value = "计划geometry信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "计划ID", required = true, dataType = "Long")
    })
    public ResultRespone getPlanChildGeo(@PathVariable Long id) {
        return ResultRespone.success(planService.getOne(Wrappers.<Plan>lambdaQuery()
                .apply("id = (select t.main_plan_id from tf_plan_child t where id = {0})",id)));
    }

    @ToFlyAppLog(title = "通过id批量删除巡检子计划表")
    @ApiOperation(tags = "巡检计划表接口", value = "通过ID批量删除巡检子计划表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/planchild/deleteByIds")
    public ResultRespone removeChildPlaByIds(String ids) {
        return ResultRespone.success(customPlanService.removeChildPlanByIds(ids));
    }

    @ToFlyAppLog(title = "计划转移申请")
    @PostMapping("/plan/transfer")
    @ApiOperation(tags = "巡检计划表接口", value = "计划转移申请")
    public ResultRespone transferPlan(@RequestBody TransferApply transferApply) {
        if (StringUtil.isEmpty(transferApply.getIds())) {
            throw new ToflyDeniedException("请添加申请计划ids");
        }
        if (StringUtil.isEmpty(transferApply.getApplyNote())) {
            throw new ToflyDeniedException("请添加申请计划备注");
        }
        return ResultRespone.success(customPlanService.transferPlan(transferApply.getIds(), transferApply.getApplyNote()));
    }

    @ToFlyAppLog(title = "审核计划转移申请")
    @PostMapping("/plan/checkTransfer")
    @ApiOperation(tags = "巡检计划表接口", value = "审核计划转移申请")
    public ResultRespone checkTransferPlan(@RequestBody CheckTransferPlan checkTransferPlan) {
        if (StringUtil.isEmpty(checkTransferPlan.getIds())) {
            throw new ToflyDeniedException("申请记录ids不能为空");
        }
        return ResultRespone.success(customPlanService.checkTransferPlan(checkTransferPlan));
    }

    @GetMapping("/planchild/transferPage")
    @ApiOperation(tags = "巡检计划表接口", value = "子计划转移分页查询")
    public ResultRespone getTransferPage(Page page, @ModelAttribute TransferSearch transferSearch) {
        return ResultRespone.success(customPlanService.getTransferPage(page, transferSearch));
    }

    @GetMapping("/planchild/pageByMoblie")
    @ApiOperation(tags = "巡检计划表接口", value = "子计划分页查询(移动端)")
    public ResultRespone getPlanChildPageByMoblie(Page page, @ModelAttribute ChildPlanSearch childPlanSearch) {
        return ResultRespone.success(customPlanService.getPlanChildPageByMoblie(page, childPlanSearch));
    }

    @GetMapping("/planchild/getPlanPoint")
    @ApiOperation(tags = "巡检计划表接口", value = "获取巡检计划所有的点")
    public ResultRespone getPlanPoint(@ModelAttribute PlanPointSearch planPointSearch) {
        return ResultRespone.success(customPlanService.getPlanPoint(planPointSearch));
    }

    @GetMapping("/planchild/getPlanPointPage")
    @ApiOperation(tags = "巡检计划表接口", value = "分页获取巡检计划所有的点")
    public ResultRespone getPlanPointPage(Page page,@ModelAttribute PlanPointSearch planPointSearch) {
        return ResultRespone.success(customPlanService.getPlanPointPage(page,planPointSearch));
    }

    @GetMapping("/planchild/getPointNum")
    @ApiOperation(tags = "巡检计划表接口", value = "获取用户计划点数量")
    public ResultRespone getPointNum(PointNumSearch pointNumSearch) {
        if (pointNumSearch.getSearchTime() == null) {
            pointNumSearch.setSearchTime(Date8Util.getCurrentDate());
        }
//        planTaskService.planTask();
        return ResultRespone.success(customPlanService.getPointNum(pointNumSearch));
    }

    @GetMapping("/pointInfo" )
    @ApiOperation(tags = "巡检计划表接口", value = "通过ID查询子计划巡检详情")
    public ResultRespone getChildPlanInfo(Page page, @RequestParam Long childPlanId) {
        if (childPlanId == null){
            throw new ToflyDeniedException("childPlanId不能为空");
        }
        return ResultRespone.success(customPlanService.getChildPlanInfo(childPlanId));
    }
}
