package com.tofly.dyrq.custom.controller;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.dyrq.custom.entity.PlanReportResult;
import com.tofly.dyrq.custom.entity.PlanReportStatistics;
import com.tofly.dyrq.custom.service.CustomPlanReportService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * @author pra
 * @date 2020/10/29 11:33
 * @description 巡检报表管理控制器 - 新
 */
@RestController
@RequestMapping("/inspectionReport")
@AllArgsConstructor
@Api(tags = "巡检报表管理接口")
public class CustomPlanReportController {
    private final CustomPlanReportService customPlanReportService;

    @ResponseBody
    @ApiOperation(value = "获取巡检(日/月)报 - 新")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "departmentId", value = "部门id", required = false, dataType = "String"),
            @ApiImplicitParam(name = "userId", value = "用户id", required = false, dataType = "String"),
            @ApiImplicitParam(name = "planDate", value = "巡检日期(如: 2020-09-01) 格式统一为 yyyy-MM-dd", required = true, dataType = "String"),
            @ApiImplicitParam(name = "timeType", value = "日期类型 1 为日报, 2 为月报", required = true, dataType = "String")
    })
    @GetMapping("/planReport")
    public ResultRespone<List<PlanReportResult>> getPlanReport(@RequestParam(name = "departmentId",required = false) String departmentId, @RequestParam(name = "userId",required = false)String userId, @RequestParam("planDate")String planDate, @RequestParam("timeType") String timeType) {
        if (StringUtils.isEmpty(planDate) || StringUtils.isEmpty(timeType)) {
            return ResultRespone.failed("请检查参数,巡检日期/日期类型必填");
        }
        return ResultRespone.success(customPlanReportService.getPlanReport(departmentId, userId, planDate, timeType));
    }

    @ResponseBody
    @ApiOperation(value = "获取巡检时间段内报表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "departmentId", value = "部门id", required = false, dataType = "String"),
            @ApiImplicitParam(name = "userId", value = "用户id", required = false, dataType = "String"),
            @ApiImplicitParam(name = "beginPlanDate", value = "巡检统计起始日期(如: 2020-09-01) 格式统一为 yyyy-MM-dd", required = true, dataType = "String"),
            @ApiImplicitParam(name = "endPlanDate", value = "巡检统计终止日期(如: 2020-10-01) 格式统一为 yyyy-MM-dd", required = true, dataType = "String")
    })
    @GetMapping("/planReportSpan")
    public ResultRespone<List<PlanReportResult>> getPlanReportSpan(@RequestParam(name = "departmentId",required = false) String departmentId, @RequestParam(name = "userId",required = false)String userId, @RequestParam(name = "beginPlanDate")String beginPlanDate, @RequestParam(name = "endPlanDate")String endPlanDate) {
        if (StringUtils.isEmpty(beginPlanDate) || StringUtils.isEmpty(endPlanDate)) {
            return ResultRespone.failed("请检查参数,起始/终止日期为空");
        }
        return ResultRespone.success(customPlanReportService.getPlanReportSpan(departmentId, userId, beginPlanDate,endPlanDate));
    }

    @ResponseBody
    @ApiOperation(value = "获取巡检(日/月)报 统计数据 - 新")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "departmentId", value = "部门id", required = false, dataType = "String"),
            @ApiImplicitParam(name = "beginPlanDate", value = "巡检统计起始日期(如: 2020-09-01) 格式统一为 yyyy-MM-dd", required = true, dataType = "String"),
            @ApiImplicitParam(name = "endPlanDate", value = "巡检统计终止日期(如: 2020-10-01) 格式统一为 yyyy-MM-dd", required = true, dataType = "String")
    })
    @GetMapping("/planReportStatistics")
    public ResultRespone<List<PlanReportStatistics>> getPlanReportStatic(@RequestParam(name = "departmentId",required = false)String departmentId, @RequestParam(name = "beginPlanDate")String beginPlanDate, @RequestParam(name = "endPlanDate")String endPlanDate) {
        if (StringUtils.isEmpty(beginPlanDate) || StringUtils.isEmpty(endPlanDate)) {
            return ResultRespone.failed("请检查参数,巡检统计起始/终止日期为空");
        }
        return ResultRespone.success(customPlanReportService.getPlanReportStatistics(departmentId,beginPlanDate,endPlanDate));

    }
}
