package com.tofly.dyrq.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.google.zxing.WriterException;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.FtpUtil;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.core.util.io.IOUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrq.config.FilePathBean;
import com.tofly.dyrq.custom.entity.CustomQrLayer;
import com.tofly.dyrq.custom.service.OtherService;
import com.tofly.dyrq.entity.Qrcode;
import com.tofly.dyrq.qrqd.PdfUtil;
import com.tofly.dyrq.qrqd.QRCodeMaker;
import com.tofly.dyrq.service.QrcodeService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.http.MediaType;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.annotation.PostConstruct;
import javax.imageio.stream.FileImageOutputStream;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.URLEncoder;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 二维码
 *
 * @author Zhangyu
 * @date 2020/5/10
 */
@RestController
@RequestMapping("/qrcode")
@Api(tags = "二维码信息表接口")
@Slf4j
public class CustomQrcodeController {
    @Autowired
    private  QrcodeService qrcodeService;
    @Autowired
    private  FilePathBean filePathBean;
    @Autowired
    private  OtherService otherService;

    private final String qrcodeLogoPath="logo.png";

    private volatile boolean logoexists=false;

    /**
     * 新增和修改二维码配置信息
     * @param servletRequest
     * @param qrcode         二维码实体类
     * @return
     */
    @ToFlyAppLog(title = "新增和修改二维码信息表")
    @ApiOperation(value = "新增和修改二维码信息表")
    @PostMapping
    public ResultRespone saveOrUpdate(HttpServletRequest servletRequest, Qrcode qrcode) {
        //boolean isEmpty = true;
        qrcode.setId((long) 1);
        //Qrcode sqlQr = qrcodeService.getOne(Wrappers.<Qrcode>lambdaQuery().eq(Qrcode::getId, 1));
        /*if (sqlQr != null) {
            String localPic = sqlQr.getLogo();
            if (StringUtil.isNotEmpty(localPic) && localPic != null) {
                FtpUtil.deleteFile(localPic);
            }
            //qrcode.setId(sqlQr.getId());
            //isEmpty = false;
        }*/

        MultiValueMap<String, MultipartFile> multiFileMap = ((MultipartHttpServletRequest) servletRequest).getMultiFileMap();
        if (multiFileMap != null) {
            multiFileMap.forEach((k, v) -> {
                String filename = filePathBean.getQrcode() + IdWorker.getIdStr() + v.get(0).getOriginalFilename().substring(v.get(0).getOriginalFilename().lastIndexOf("."));
                boolean b = FtpUtil.uploadFile2Ftp(v.get(0), filename);
                qrcode.setLogo(filename);
            });
        } /*else if (multiFileMap.size() == 0) {
            qrcode.setLogo("");
        }*/
        /*if(isEmpty==true){
            return ResultRespone.success(qrcodeService.save(qrcode));
        }   else {
            return ResultRespone.success(qrcodeService.updateById(qrcode));
        }*/
        boolean b = qrcodeService.saveOrUpdate(qrcode);
        if(b){
            flushLogo(qrcode.getLogo());
        }
        return ResultRespone.success(b);
    }

    /**
     * 查询二维码配置信息
     * @return ResultRespone
     */
    @GetMapping
    @ApiOperation(value = "通过ID查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById() {
        return ResultRespone.success(qrcodeService.getById(1));
    }


    /**
     * 查询二维码配置信息
     * @return ResultRespone
     */
//    @GetMapping("/getLayer")
//    @ApiOperation(value = "通过设备ID查询图层信息")
//    @ApiImplicitParams({
//            @ApiImplicitParam(name = "sbid", value = "二维码设备ID", required = true, dataType = "String")
//    })
//    public ResultRespone getLayer(String sbid) {
//        CustomQrLayer customQrLayer=new CustomQrLayer();
//        customQrLayer.setSbid(sbid);
//        return ResultRespone.success(otherService.getQrLayer(customQrLayer).get(0));
//    }



    /**
     * 二维码生成下载
     *
     * @param request
     * @param response
     * @return
     */
    @GetMapping("/download")
    @ToFlyAppLog(title = "二维码生成下载")
    @ApiOperation(value = "二维码生成下载")
    public void downloadQRCode(HttpServletRequest request, HttpServletResponse response, @ModelAttribute CustomQrLayer customQrLayer) throws IOException {
        //获取用户ID
        //Long userId = SecurityUtils.getUserId();
        response.reset();
        response.setCharacterEncoding("UTF-8");
        String userAgent = request.getHeader("User-Agent");
        String QRName;
        if (userAgent.contains("MSIE") || userAgent.contains("Trident")) {
            // 针对IE或者以IE为内核的浏览器：
            QRName = URLEncoder.encode(SecurityUtils.getUserName(), "UTF-8") + ".jpg";
        } else {
            // 非IE浏览器的处理：
            QRName = new String(SecurityUtils.getUserName().getBytes("UTF-8"), "ISO-8859-1") + ".jpg";
        }
        response.setHeader("Content-Disposition", String.format("attachment; filename=%s", QRName));
        //response.addHeader("Content-Length", "" + data.length);
        response.setContentType("application/octet-stream; charset=UTF-8");
        response.setHeader("content-type", "application/octet-stream");
        //查出当前用户二维码配置信息
        Qrcode qrcode = qrcodeService.getOne(Wrappers.<Qrcode>lambdaQuery().eq(Qrcode::getId, 1));
        //拼接二维码信息
        String info = customQrLayer.getSbid();
        //otherService.handlerData(customQrLayer);
        try {
            //判断是否需要插入logo
//            if (StringUtil.isNotEmpty(qrcode.getLogo()) && qrcode.getLogo() != null) {
            if(logoexists){
                //根据FTP文件地址获取图片
                String fileType = qrcode.getLogo().substring(qrcode.getLogo().lastIndexOf("."));
                if (StringUtil.isNotEmpty(fileType)) {
                    fileType = fileType.toLowerCase();
                }
                switch (fileType) {
                    case ".jpg":
                    case ".jpeg":
                        response.setContentType(MediaType.IMAGE_JPEG_VALUE);
                        break;
                    case ".png":
                        response.setContentType(MediaType.IMAGE_PNG_VALUE);
                        break;
                    case ".gif":
                        response.setContentType(MediaType.IMAGE_GIF_VALUE);
                        break;
                    default:
                        response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
                        break;
                }
//                byte[] b = FtpUtil.getByteDataforFile(qrcode.getLogo());
//                //临时文件目录 将查出的byte数组转为图片
//                String pathName = IOUtil.TEMP_FILEPATH + qrcode.getLogo();
//                if (b.length >= 3) {
//                    File file = new File(pathName);
//                    if (!file.getParentFile().exists()) {
//                        file.getParentFile().mkdirs();
//                    }
//                    FileImageOutputStream imageOutputStream = new FileImageOutputStream(file);
//                    try {
//                        imageOutputStream.write(b, 0, b.length);
//                    } catch (Exception e) {
//                        log.error(e.getMessage());
//                        e.getMessage();
//                    } finally {
//                        imageOutputStream.flush();
//                        imageOutputStream.close();
//                    }
//                }
                QRCodeMaker.MakeQrCodeWithLogo(info, response.getOutputStream(), qrcodeLogoPath, qrcode, info);
                //生成二维码后删除临时图片
//                try {
//                    pathName = pathName.toString();
//                    java.io.File tempImage = new java.io.File(pathName);
//                    tempImage.delete();
//                } catch (Exception e) {
//                    log.error(e.getMessage());
//                    e.getMessage();
//                }
            } else if (StringUtil.isEmpty(qrcode.getLogo()) || qrcode.getLogo() == null) {
                QRCodeMaker.MakeQRCode(info, response.getOutputStream(), qrcode);
            }
        } catch (IOException | WriterException e) {
            log.error(e.getMessage());
            e.getMessage();
        } finally {
            response.getOutputStream().flush();
            response.getOutputStream().close();
        }
    }

    @PostMapping("/downloadpdf")
    @ToFlyAppLog(title = "多张二维码生成pdf下载")
    @ApiOperation(value = "多张二维码生成pdf下载")
    public ResultRespone downloadPdf( @RequestBody CustomQrLayer customQrLayer) throws WriterException, IOException {
        //获取用户ID
        //Long userId = SecurityUtils.getUserId();
        String pdfName = SecurityUtils.getUserName()+".pdf";
        //查出当前用户二维码配置信息
        Qrcode qrcode = qrcodeService.getOne(Wrappers.<Qrcode>lambdaQuery().eq(Qrcode::getId, 1));
        //分割字符串numbers以“,”隔开
        String numberArray[] = customQrLayer.getSbid().split(",");
        //多张二维码临时存放的位置
        String filePath= IOUtil.TEMP_FILEPATH;
        String imagesPath = filePath+"images";

        //otherService.handlerData(customQrLayer);

        File file=new File(filePath+pdfName);
        if(file.exists()){
            file.delete();
        }
        for (int i = 0; i < numberArray.length; i++) {
            String info = numberArray[i];
            try {
                //判断是否需要插入logo
//                if (StringUtil.isNotEmpty(qrcode.getLogo()) && qrcode.getLogo() != null) {
                //根据FTP文件地址获取图片
//                    String fileType = qrcode.getLogo().substring(qrcode.getLogo().lastIndexOf("."));
//                    if (StringUtil.isNotEmpty(fileType)) {
//                        fileType = fileType.toLowerCase();
//                    }
//                    switch (fileType) {
//                        case ".jpg":
//                        case ".jpeg":
//                            response.setContentType(MediaType.IMAGE_JPEG_VALUE);
//                            break;
//                        case ".png":
//                            response.setContentType(MediaType.IMAGE_PNG_VALUE);
//                            break;
//                        case ".gif":
//                            response.setContentType(MediaType.IMAGE_GIF_VALUE);
//                            break;
//                        default:
//                            response.setContentType(MediaType.APPLICATION_OCTET_STREAM_VALUE);
//                            break;
//                    }
//                    byte[] b = FtpUtil.getByteDataforFile(qrcode.getLogo());
//                    //临时文件目录 将查出的byte数组转为图片
//                    String pathName = filePath + qrcode.getLogo();
//                    if (b.length >= 3) {
//                        File file = new File(pathName);
//                        if (!file.getParentFile().exists()) {
//                            file.getParentFile().mkdirs();
//                        }
//                        FileImageOutputStream imageOutputStream = new FileImageOutputStream(file);
//                        try {
//                            imageOutputStream.write(b, 0, b.length);
//                            imageOutputStream.flush();
//                        } catch (Exception e) {
//                            log.error(e.getMessage());
//                            e.getMessage();
//                        } finally {
//                            imageOutputStream.close();
//                        }
//                    }

                if(logoexists){
                    QRCodeMaker.MakeQrCodeWithLogo(info, imagesPath + File.separator + numberArray[i] + qrcode.getLogo().substring(qrcode.getLogo().lastIndexOf(".")), qrcodeLogoPath, qrcode, numberArray[i]);
                    //生成二维码后删除临时图片
//                    try {
//                        pathName = pathName.toString();
//                        java.io.File tempImage = new java.io.File(pathName);
//                        tempImage.delete();
//                    } catch (Exception e) {
//                        log.error(e.getMessage());
//                        e.getMessage();
//                    }
                } else if (StringUtil.isEmpty(qrcode.getLogo()) || qrcode.getLogo() == null) {
                    QRCodeMaker.MakeQRCode(info, imagesPath + File.separator + numberArray[i] + ".jpg", qrcode);
                }
            } catch (IOException | WriterException e) {
                log.error(e.getMessage());
                e.getMessage();
            }
        }
        //生成PDF文件
        PdfUtil.imagesToPdf(filePath+pdfName, imagesPath, qrcode);
        PdfUtil.deleteFiles(imagesPath);
        return ResultRespone.success(pdfName);
//        response.reset();
//        response.setCharacterEncoding("UTF-8");
//        String userAgent = request.getHeader("User-Agent");
//        if (userAgent.contains("MSIE") || userAgent.contains("Trident")) {
//            // 针对IE或者以IE为内核的浏览器：
//            pdfName = URLEncoder.encode(pdfName, "UTF-8");
//        } else {
//            // 非IE浏览器的处理：
//            pdfName = new String(pdfName.getBytes("UTF-8"), "ISO-8859-1");
//        }
//        response.setHeader("Content-Disposition", String.format("attachment; filename=%s.pdf", pdfName));
//        //response.addHeader("Content-Length", "" + content.length);
//        response.setContentType("application/pdf; charset=UTF-8");
//        response.setHeader("content-type", "application/pdf");
//        //创建一个PDF文件对象
//        File filePdf = new File(imagesPath + File.separator + pdfName + ".pdf");
//        BufferedInputStream bis = new BufferedInputStream(new FileInputStream(filePdf));
//        try {
//            byte[] content = new byte[bis.available() + 1000];
//            int len = 0;
//            while ((len = bis.read(content)) > 0) {
//                response.getOutputStream().write(content, 0, len);
//                response.getOutputStream().flush();
//            }
//        } catch (Exception e) {
//            log.error(e.getMessage());
//            e.getMessage();
//        } finally {
//            if (bis != null) {
//                try {
//                    bis.close();
//                } catch (IOException e) {
//                    e.printStackTrace();
//                }
//            }
//            if (response.getOutputStream() != null) {
//                try {
//                    response.getOutputStream().close();
//                } catch (IOException e2) {
//                    e2.printStackTrace();
//                }
//            }
//            PdfUtil.deleteFiles(imagesPath);
//        }
    }

    private void flushLogo(String logoPath) {
        logoexists=false;
        if (StringUtil.isNotEmpty(logoPath)) {

            byte[] b = FtpUtil.getByteDataforFile(logoPath);
            //临时文件目录 将查出的byte数组转为图片
            //String pathName = qrcode.getLogo();
            if (b.length >= 3) {
                File file = new File(qrcodeLogoPath);
                FileImageOutputStream imageOutputStream=null;
                try {
                    imageOutputStream = new FileImageOutputStream(file);
                    imageOutputStream.write(b, 0, b.length);
                    imageOutputStream.flush();
                    logoexists=true;
                } catch (Exception e) {
                    log.error(e.getMessage());
                    e.getMessage();
                } finally {
                    try {
                        imageOutputStream.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    }
                }
            }
        }
    }


    public void myInit(){
        Qrcode qrcode = qrcodeService.getById(1);
        if(qrcode != null){
            flushLogo(qrcode.getLogo());
        }
    }
}
