package com.tofly.dyrq.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrq.custom.entity.*;
import com.tofly.dyrq.custom.mapper.CustomMessageMapper;
import com.tofly.dyrq.custom.service.CustomMessageService;
import com.tofly.dyrq.custom.service.CustomRushRepairService;
import com.tofly.dyrq.entity.*;
import com.tofly.dyrq.service.RushRepairService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;

/**
 * 抢维修工单表自定义接口
 *
 * @author Zhangyu
 * @date Tue Jun 16 00:00:00 GMT+08:00 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/rushrepair")
@Api(tags = "抢维修工单表接口")
@Slf4j
public class CustomRushRepairController {

    private final RushRepairService rushRepairService;
    private final CustomRushRepairService customRushRepairService;
    private final CustomMessageService customMessageService;

    /**
     * 抢维修工单导入
     *
     * @param rushRepair 抢维修工单表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "抢维修工单导入")
    @ApiOperation(value = "抢维修工单导入")
    @PostMapping
    public ResultRespone saveRushRepair(HttpServletRequest servletRequest, RushRepair rushRepair) {
        return ResultRespone.success(customRushRepairService.saveRushRepair(servletRequest, rushRepair));
    }

    /**
     * 按条件分页查询
     *
     * @param page
     * @param rushRepairSearch
     * @return
     */
    @ApiOperation(value = "按条件分页查询")
    @GetMapping("/page")
    public ResultRespone getRushRepairPage(Page page, @ModelAttribute RushRepairSearch rushRepairSearch) {
        return ResultRespone.success(customRushRepairService.pageByCondition2(page, rushRepairSearch));
    }

    /**
     * 按条件分页查询(新)
     *
     * @param page
     * @param rushRepairSearch
     * @return
     */
    @ApiOperation(value = "按条件分页查询(新)")
    @GetMapping("/pagenew")
    public ResultRespone getRushRepairPage2(Page page, @ModelAttribute RushRepairSearch rushRepairSearch) {
        return ResultRespone.success(customRushRepairService.pageByCondition(page, rushRepairSearch));
    }

    /**
     * 按条件分页查询(App新)
     *
     * @param page
     * @param
     * @return
     *//*
    @ApiOperation(value = "按条件分页查询(App新)")
    @GetMapping("/pageapp")
    public ResultRespone getRushRepairPage3(Page page, @ModelAttribute RushRepairSearch rushRepairSearch) {
        return ResultRespone.success(customRushRepairService.pageByCondition3(page, rushRepairSearch));
    }*/

    @ApiOperation(value = "抢维修工单个人查询")
    @GetMapping("/pageByUser")
    public ResultRespone getConstructionPageByUser(Page page, @ModelAttribute RushRepairSearchUser rushRepairSearchUser) {
        Long userid = SecurityUtils.getUserId();
        return ResultRespone.success(customRushRepairService.getConstructionPageByUser(page, rushRepairSearchUser));
    }


    /**
     * 根据ID查询抢维修工单详情
     *
     * @param id 根据ID查询抢维修工单详情
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "根据ID查询抢维修工单详情")
    @ApiOperation(value = "根据ID查询抢维修工单详情")
    @GetMapping("/{id}")
    public ResultRespone getRushRepairById(@PathVariable("id") Long id) {
        return ResultRespone.success(customRushRepairService.getRushRepairById(id));
    }

    /**
     * 根据ID删除抢维修工单
     *
     * @param id 根据ID删除抢维修工单
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "根据ID删除抢维修工单")
    @ApiOperation(value = "根据ID删除抢维修工单")
    @DeleteMapping("/{id}")
    public ResultRespone removeRushRepairById(@PathVariable("id") Long id) {
        return ResultRespone.success(customRushRepairService.removeById(id));
    }

    /**
     * 抢维修派工审核
     *
     * @param rushRepairAuditDispatch
     * @return
     */
    @ToFlyAppLog(title = "抢维修派工审核")
    @ApiOperation(value = "抢维修派工审核")
    @PostMapping("/dispatch")
    public ResultRespone saveRushRepairDispatch(@RequestBody RushRepairAuditDispatch rushRepairAuditDispatch) {
        return ResultRespone.success(customRushRepairService.saveRushRepairAuditDispatch(rushRepairAuditDispatch));
    }

    /**
     * 抢维修过程管理
     *
     * @param rushRepairProcessManage
     * @param servletRequest
     * @return
     */
    @ToFlyAppLog(title = "抢维修过程管理")
    @ApiOperation(value = "抢维修过程管理")
    @PostMapping("/processmanage")
    public ResultRespone saveRushRepairProcess(HttpServletRequest servletRequest, RushRepairProcessManage
            rushRepairProcessManage) {
        return ResultRespone.success(customRushRepairService.saveRushRepairProcessManage(servletRequest, rushRepairProcessManage));
    }

    /**
     * 抢维修工单修改(申请施工结束)
     *
     * @param rushRepair
     * @return
     */
    @ToFlyAppLog(title = "抢维修工单修改(申请施工结束)")
    @ApiOperation(value = "抢维修工单修改(申请施工结束)")
    @PutMapping
    public ResultRespone updateRushRepair(@RequestBody RushRepair rushRepair) {
        if (StringUtil.isNotEmpty(rushRepair.getRushRepairPersons())) {
            if ("\"nul\"".equals(rushRepair.getRushRepairPersons().trim().toLowerCase()) || "\"null\"".equals(rushRepair.getRushRepairPersons().trim().toLowerCase())
                    || "nul".equals(rushRepair.getRushRepairPersons().trim().toLowerCase()) || "null".equals(rushRepair.getRushRepairPersons().trim().toLowerCase())) {
                rushRepair.setRushRepairPersons("");
            }
        }
        Boolean res = rushRepairService.updateById(rushRepair);
        //派工
        if (res && rushRepair.getRushRepairResponsible() != null) {
            if (StringUtil.isNotEmpty(rushRepair.getWorkOrderState()) && "2".equals(rushRepair.getWorkOrderState())) {
                if (StringUtil.isNotEmpty(rushRepair.getRushRepairPersons())){
                    customMessageService.sendMessage(rushRepair.getRushRepairResponsible().toString() + "," + rushRepair.getRushRepairPersons(), "您有一条待处理抢维修工单", "1","");
                }
                else{
                    customMessageService.sendMessage(rushRepair.getRushRepairResponsible().toString(), "您有一条待处理抢维修工单", "1","");
                }
            }
        }
        //审核
        else if (StringUtil.isNotEmpty(rushRepair.getWorkOrderState()) && "3".equals(rushRepair.getWorkOrderState())) {
            RushRepair one = rushRepairService.getById(rushRepair.getId());
            customMessageService.sendMessage(one.getDispatchUserId().toString(), "您有一条待审核抢维修工单", "1","");
        }
        return ResultRespone.success(res);
    }

    /**
     * 抢维修工单修改(移动端)
     *
     * @param rushRepair
     * @return
     */
    @ToFlyAppLog(title = "抢维修工单修改(移动端)")
    @ApiOperation(value = "抢维修工单修改(移动端)")
    @PostMapping("/updateapp")
    public ResultRespone updateRushRepairForApp(@ModelAttribute RushRepair rushRepair) {
        return ResultRespone.success(rushRepairService.updateById(rushRepair));
    }

    /**
     * 抢维修审核施工结束
     *
     * @param rushRepairAuditEnd
     * @param servletRequest
     * @return
     */
    @ToFlyAppLog(title = "抢维修审核施工结束")
    @ApiOperation(value = "抢维修审核施工结束")
    @PostMapping("/auditend")
    public ResultRespone saveRushRepairProcess(HttpServletRequest servletRequest, RushRepairAuditEnd
            rushRepairAuditEnd) {
        return ResultRespone.success(customRushRepairService.saveRushRepairAuditEnd(servletRequest, rushRepairAuditEnd));
    }


    @ApiOperation(value = "报表-根据工单类型统计")
    @GetMapping("/getReportByTYpe")
    public ResultRespone getReportByTYpe(@ModelAttribute CustomRushRepairReportQuery customRushRepairReportQuery) {
        return ResultRespone.success(customRushRepairService.getReportByTYpe(customRushRepairReportQuery));
    }

    @ApiOperation(value = "报表-根据时间类型统计")
    @GetMapping("/getReportByTime")
    public ResultRespone getReportByTime(@ModelAttribute CustomRushRepairReportQueryType customRushRepairReportQueryType) {
        return ResultRespone.success(customRushRepairService.getReportByTime(customRushRepairReportQueryType));
    }
}
