package com.tofly.dyrq.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.custom.entity.ScadaConfigList;
import com.tofly.dyrq.custom.entity.ScadaRealdataList;
import com.tofly.dyrq.custom.entity.ScadaStationResult;
import com.tofly.dyrq.custom.service.CustomScadaService;
import com.tofly.dyrq.entity.*;
import com.tofly.dyrq.service.ScadaRealdataService;
import com.tofly.dyrq.service.ScadapntsDesService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;

@RestController
@AllArgsConstructor
@RequestMapping("/scada")
@Api(tags = "scada接口")
public class CustomScadaController {

    private final CustomScadaService customScadaService;
    private final ScadapntsDesService scadapntsDesService;


    @GetMapping("/station")
    @ApiOperation(value = "站点简略信息分页查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "filter", value = "站场名(可选参数,模糊查询指定站点名)", required = false, dataType = "String")
    })
    public ResultRespone getStation(Page page, @RequestParam(required = false) String filter) {
        return ResultRespone.success(customScadaService.getStation(page, filter));
    }

    @GetMapping("/station/{ptid}")
    @ApiOperation(value = "某站点详细信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ptid", value = "站点ptid", required = true, dataType = "String")
    })
    public ResultRespone getStationById(@PathVariable("ptid") String ptid) {
        return ResultRespone.success(customScadaService.getStationById(ptid));
    }

    @GetMapping("/station/List")
    @ApiOperation(value = "查询指定站点集合的简略信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ptids", value = "批量查询，多个ptid以逗号隔开", required = true, dataType = "String")
    })
    public ResultRespone getStationByList(@RequestParam("ptids") String ptids) {
        return ResultRespone.success(customScadaService.getStationByList(Arrays.asList(ptids.split(","))));
    }

    @GetMapping("/alarmConfig")
    @ApiOperation(value = "查询指定站点集合的报警配置信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ptids", value = "批量查询，多个ptid以逗号隔开", required = true, dataType = "String")
    })
    public ResultRespone getAlarmConfigByList(Page page, @RequestParam("ptids") String ptids) {
        return ResultRespone.success(customScadaService.getAlarmConfigByList(page, Arrays.asList(ptids.split(","))));
    }

    @ToFlyAppLog(title = "根据参数名称批量修改报警参数配置表")
    @ApiOperation(value = "根据参数名称批量修改报警参数配置表")
    @PutMapping("/alarmConfig/set")
    public ResultRespone updateByName(@RequestBody List<ScadapntsDes> list) {
        return ResultRespone.success(customScadaService.updateByName(list));
    }

    @ToFlyAppLog(title = "根据参数描述修改报警参数配置表")
    @ApiOperation(value = "根据参数描述修改报警参数配置表")
    @PutMapping("/alarmConfig/batchset")
    public ResultRespone updateByDes(@RequestBody ScadapntsDes scadapntsDes) {
        return ResultRespone.success(scadapntsDesService.update(Wrappers.<ScadapntsDes>lambdaUpdate()
                .eq(ScadapntsDes::getDescription, scadapntsDes.getDescription())
                .set(ScadapntsDes::getMaxval, scadapntsDes.getMaxval())
                .set(ScadapntsDes::getMinval, scadapntsDes.getMinval())
        ));
    }

    @GetMapping("/alarm")
    @ApiOperation(value = "查询实时监控值的报警信息")
    public ResultRespone getAlarmInfo() {
        return ResultRespone.success(customScadaService.getAlarmInfo());
    }

    @GetMapping("/paramList")
    @ApiOperation(value = "查询配置表参数简略信息")
    public ResultRespone getParamList() {
        return ResultRespone.success(scadapntsDesService.listMaps(Wrappers.<ScadapntsDes>lambdaQuery().select(ScadapntsDes::getDescription,ScadapntsDes::getUnit)));
    }

    /////////
    @ToFlyAppLog(title="新增scada实时数据同步表" )
    @ApiOperation(value = "新增scada实时数据同步表")
    @PostMapping("/saveBatch")
//    @ResponseBody
    public ResultRespone<Boolean> save(@RequestBody ScadaRealdataList dataList) {
        return ResultRespone.success(customScadaService.updateRealData(dataList.getScadaRealdataList()));
    }

    @GetMapping("/realData")
    @ApiOperation(value = "scada实时数据列表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "searchText", value = "参数编号和名称分页查询", required = false, dataType = "String"),
            @ApiImplicitParam(name = "ptIds", value = "查询站点id字符串", required = false, dataType = "String")
    })
    @ResponseBody
    public ResultRespone<List<ScadaStationResult>> getRealData(String searchText,String ptIds) {
        return ResultRespone.success(customScadaService.getRealData(searchText,ptIds));
    }

    @ToFlyAppLog(title="批量报警配置" )
    @ApiOperation(value = "批量报警配置")
    @PostMapping("/updateConfig")
    @ResponseBody
    public ResultRespone updateConfig(@RequestBody ScadaConfigList scadaConfigList) {
        return ResultRespone.success(customScadaService.updateConfig(scadaConfigList.getScadaConfigs()));
    }
}
