package com.tofly.dyrq.custom.controller;

import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.dyrq.custom.service.CustomSpecialEpmtFileService;
import com.tofly.dyrq.entity.SpecialEpmtFile;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;

/**
 * 特种设备管理表
 *
 * @author Zhangyu
 * @date Tue Dec 15 00:00:00 GMT+08:00 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/specialepmtfile")
@Api(tags = "特种设备管理表接口")
public class CustomSpecialEpmtFileController {

    private final CustomSpecialEpmtFileService customSpecialEpmtFileService;

    /**
     * 新增特种设备管理表
     *
     * @param specialEpmtFile 特种设备管理表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增特种设备管理表")
    @ApiOperation(value = "新增特种设备管理表")
    @PostMapping(consumes = "multipart/form-data")
    public ResultRespone save(SpecialEpmtFile specialEpmtFile, HttpServletRequest request) {
        return ResultRespone.success(customSpecialEpmtFileService.save(specialEpmtFile, request));
    }

    /**
     * 修改特种设备管理表
     *
     * @param specialEpmtFile 特种设备管理表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改特种设备管理表")
    @ApiOperation(value = "修改特种设备管理表")
    @PostMapping(value = "/update",consumes = "multipart/form-data")
    public ResultRespone updateById(SpecialEpmtFile specialEpmtFile, HttpServletRequest request) {
        return ResultRespone.success(customSpecialEpmtFileService.updateById(specialEpmtFile, request));
    }

    /**
     * 通过id批量删除特种设备管理表
     *
     * @param ids id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id批量删除特种设备管理表")
    @ApiOperation(value = "通过ID批量特种设备管理表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "批量删除，多个id以逗号隔开", required = true, dataType = "String")
    })
    @DeleteMapping("/deleteByIds")
    public ResultRespone removeById(String ids) {
        return ResultRespone.success(customSpecialEpmtFileService.removeById(Arrays.asList(ids.split(","))));
    }
}
