package com.tofly.dyrq.custom.controller;


import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.FtpUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrq.config.FilePathBean;
import com.tofly.dyrq.custom.entity.CustomTfPda;
import com.tofly.dyrq.custom.service.CustomTfPdaService;
import com.tofly.dyrq.entity.TfPdaResponseFile;
import com.tofly.dyrq.entity.TfPdaTask;
import com.tofly.dyrq.entity.TfPdaTaskResponse;
import com.tofly.dyrq.service.TfPdaResponseFileService;
import com.tofly.dyrq.service.TfPdaTaskResponseService;
import com.tofly.dyrq.service.TfPdaTaskService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.util.MultiValueMap;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.text.ParseException;

import static org.springframework.util.ObjectUtils.isEmpty;

@RestController
@AllArgsConstructor
@RequestMapping("/TfPda")
@Api(tags = "工单信息接口")
public class CustomTfPdaControlle {

    private final CustomTfPdaService customTfPdaService;
    private final TfPdaTaskService tfPdaTaskService;

    @GetMapping("/userPdaPage")
    @ApiOperation(value = "当前用户工单信息分页查询")
    public ResultRespone getTfPdaTaskPage(Page page) {
        Long userId = SecurityUtils.getUserId();
        return ResultRespone.success(tfPdaTaskService.pageMaps(page, Wrappers.<TfPdaTask>lambdaQuery()
                .select(TfPdaTask::getId, TfPdaTask::getCaption, TfPdaTask::getContent, TfPdaTask::getX, TfPdaTask::getY, TfPdaTask::getLongitude, TfPdaTask::getLatitude, TfPdaTask::getTaskStatus, TfPdaTask::getDispatchDate, TfPdaTask::getDispatchRealname, TfPdaTask::getTaskbh)
                .eq(TfPdaTask::getUserid, userId)));
    }

    @GetMapping("/{id}")
    @ApiOperation(value = "通过ID查询工单详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "String")
    })
    public ResultRespone getById(@PathVariable("id") String id) {
        return ResultRespone.success(customTfPdaService.gettfPdaTaskDetail(Long.parseLong(id)));
    }


    @GetMapping("/pdaPage")
    @ApiOperation(value = "工单记录分页查询")
    public ResultRespone getTfPdaTaskPageNew(Page page, @ModelAttribute CustomTfPda customTfPda) {
        return ResultRespone.success(customTfPdaService.getTaskPage(page, customTfPda));
    }

    @ToFlyAppLog(title = "提交工单反馈")
    @PostMapping("/handle")
    @ApiOperation(value = "提交工单反馈")
    public ResultRespone saveHandle(HttpServletRequest request, TfPdaTaskResponse tfPdaTaskResponse){
        if (!customTfPdaService.saveHandle(request, tfPdaTaskResponse)){
            return ResultRespone.failed(2,"工单已处理，不能提交反馈");
        }
        else{
            return ResultRespone.success(true);
        }
    }

    @ToFlyAppLog(title = "新增工单记录表")
    @PostMapping
    @ApiOperation(value = "新增工单记录表")
    public ResultRespone save(@RequestBody TfPdaTask tfPdaTask) {
        Long userId = SecurityUtils.getUserId();
        String userName = SecurityUtils.getUserName();
        tfPdaTask.setDispatchUserid(userId);
        tfPdaTask.setDispatchRealname(userName);
        tfPdaTask.setTaskStatus(new Integer(0));
        //添加时间
        LocalDateTime localDateTime = LocalDateTime.now();
        DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyyMMddHH");
        //生成工单流水号
        boolean b = true;
        String bh = "";
        while (b) {
            bh = localDateTime.format(dtf) + String.format("%04d", new Random().nextInt(9999));
            List l = tfPdaTaskService.list(Wrappers.<TfPdaTask>lambdaQuery().eq(TfPdaTask::getTaskbh, bh));
            if (l.size() == 0) {
                b = false;
            }
        }
        tfPdaTask.setTaskbh(bh);
        Date date = Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant());
        tfPdaTask.setDispatchDate(date);
        return ResultRespone.success(tfPdaTaskService.save(tfPdaTask));
    }

    @GetMapping("/unfinished")
    @ApiOperation(value = "未完成工单查询")
    public ResultRespone getUnfinished() {
        int count = tfPdaTaskService.count(Wrappers.<TfPdaTask>lambdaQuery().eq(TfPdaTask::getTaskStatus, "0"));
        List<TfPdaTask> taskList = tfPdaTaskService.list(Wrappers.<TfPdaTask>lambdaQuery().select(TfPdaTask::getId).eq(TfPdaTask::getTaskStatus, "0"));
        List<Long> ids = new ArrayList<>();
        for (TfPdaTask task : taskList) {
            ids.add(task.getId());
        }
        Map map = new HashMap();
        map.put("count", count);
        map.put("taskids", ids);
        return ResultRespone.success(map);
    }

}
