package com.tofly.dyrq.custom.controller;

import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.mysql.cj.protocol.x.Notice;
import com.sun.org.apache.bcel.internal.generic.ATHROW;
import com.tofly.common.core.configs.AutoConfigConfiguration;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.Date8Util;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrq.custom.entity.*;
import com.tofly.dyrq.custom.service.CustomTfPdaSignService;
import com.tofly.dyrq.custom.service.OtherService;
import com.tofly.dyrq.entity.PlanChild;
import com.tofly.dyrq.entity.TfPdaSign;
import com.tofly.dyrq.service.PlanChildService;
import com.tofly.dyrq.service.TfPdaSignService;
import com.tofly.dyrq.util.WorkDayJson;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.cache.CacheManager;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.client.RestTemplate;

import java.io.IOException;
import java.security.Security;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.DateTimeException;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;

/**
 * 考勤签到表
 *
 * @author admin
 * @date Wed May 06 00:00:00 GMT+08:00 2020
 */
@RestController
@AllArgsConstructor
@RequestMapping("/tfpdasign")
@Api(tags = "考勤签到表接口")
public class CustomTfPdaSignController {
    private final TfPdaSignService tfPdaSignService;
    private final CustomTfPdaSignService customTfPdaSignService;
    private final CacheManager cacheManager;

    /**
     * 新增考勤签到表
     *
     * @param addSign 考勤签到表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "签到登记")
    @ApiOperation(value = "签到登记")
    @PostMapping
    public ResultRespone signSave(@RequestBody AddSign addSign) {
        if (addSign.getDevice() == null) {
            throw new ToflyDeniedException("巡检设备上传信息device不能为空");
        }
        if (StringUtil.isEmpty(addSign.getSignType())) {
            throw new ToflyDeniedException("签到类型signType不能为空");
        }
        return ResultRespone.success(customTfPdaSignService.signSave(addSign));
    }

    /**
     * 查询当前考勤状态
     *
     * @return ResultRespone
     */
    @ApiOperation(value = "获取签到最新地状态,0:开机 1:上班 2:下班")
    @GetMapping("/getLastestState")
    public ResultRespone getLastestState() {
        Date startDate = Date8Util.getStartTime();
        Date endDate = Date8Util.getEndTime();
        Long userId = SecurityUtils.getUserId();
        List<TfPdaSign> list = tfPdaSignService.list(Wrappers.<TfPdaSign>lambdaQuery()
                .eq(TfPdaSign::getUserid, userId)
                .between(TfPdaSign::getSignTime, startDate, endDate)
                .orderByDesc(TfPdaSign::getSignTime));
        Map result = new HashMap();
        if (list.size() == 0) {
            result.put("state", 0);
        } else {
            if ("1".equals(list.get(0).getSignType())) {
                result.put("state", 1);
            } else {
                result.put("state", 2);
            }
        }
        //获取签到分段
        int section = tfPdaSignService.count(Wrappers.<TfPdaSign>lambdaQuery()
                .eq(TfPdaSign::getUserid, userId)
                .eq(TfPdaSign::getSignType, "1")
                .between(TfPdaSign::getSignTime, startDate, endDate));
        result.put("section", section);
        return ResultRespone.success(result);
    }

    @GetMapping("/page")
    @ApiOperation(value = "签到记录分页查询")
    public ResultRespone getTfPdaSignPage(Page page, @ModelAttribute SignSearch signSearch) {
        if (signSearch.getIsToday() != null && signSearch.getIsToday()) {
            signSearch.setStartDate(Date8Util.getStartTime());
            signSearch.setEndDate(Date8Util.getEndTime());
        }
        return ResultRespone.success(tfPdaSignService.page(page, Wrappers.<TfPdaSign>lambdaQuery()
                .eq(signSearch.getUserId() != null, TfPdaSign::getUserid, signSearch.getUserId())
                .ge(signSearch.getStartDate() != null, TfPdaSign::getSignTime, signSearch.getStartDate())
                .le(signSearch.getEndDate() != null, TfPdaSign::getSignTime, signSearch.getEndDate())));
    }

    /**
     * 查询计划与签到分段
     *
     * @return ResultRespone
     */
    @ApiOperation(value = "查询计划与签到分段")
    @GetMapping("/getplanandsign")
    public ResultRespone getSign(@ModelAttribute SignPlanSearch signPlanSearch) {
        if (signPlanSearch.getUserId() == null) {
            throw new ToflyDeniedException("用户id userId不能为空");
        }
        if (signPlanSearch.getBeginDate() == null) {
            throw new ToflyDeniedException("查询的开始日期时间beginDate不能为空");
        }
        if (signPlanSearch.getEndDate() == null) {
            throw new ToflyDeniedException("查询的结束日期时间endDate不能为空");
        }
        if (signPlanSearch.getBeginDate().getTime() > signPlanSearch.getEndDate().getTime()) {
            throw new DateTimeException("起始时间大于结束时间");
        }
        return ResultRespone.success(customTfPdaSignService.getSignVO(signPlanSearch));
    }

    /**
     * 移动端查询月签到信息
     *
     * @return
     */
    @ApiOperation(value = "移动端查询月签到信息")
    @GetMapping("/getMonthSign")
    public ResultRespone getMonthSign(@ModelAttribute MonthSignSearch monthSignSearch) {
        if (monthSignSearch.getUserId() == null) {
            monthSignSearch.setUserId(SecurityUtils.getUserId());
        }
        if (monthSignSearch.getMonth() == null) {
            throw new ToflyDeniedException("查询的月份不能为空");
        }
        List<Map> signList = customTfPdaSignService.getMonthSign(monthSignSearch);

        //api获取放假信息
//        String data = "";
//        String monthNow = monthSignSearch.getMonth().replace("-","");
//        if (WorkDayJson.MonthWorkDay.containsKey(monthNow)){
//            data = WorkDayJson.MonthWorkDay.get(monthNow);
//        }
//        else{
//            try{
//                RestTemplate restTemplate = new RestTemplate();
//                Map<String,String> map = new HashMap();
//                map.put("d",monthNow);
//                map.put("info","1");
//            data = restTemplate.getForObject("http://tool.bitefu.net/jiari/?info=1&d="+monthNow
//                    , String.class,map);
////                ResponseEntity responseEntity = restTemplate.getForEntity("http://tool.bitefu.net/jiari/?info=1&d="+monthNow
////                        , String.class,map);
//                WorkDayJson.MonthWorkDay.put(monthNow,data);
//            }
//            catch (Exception exception){
//                ;
//            }
//        }
//        String today = Date8Util.getCurrentDay("yyyyMMdd");
//        JSONObject json = JSONObject.parseObject(data).getJSONObject(monthNow);
//        Set<String> keys = json.keySet();
//        List<Map> result = new ArrayList<>();
//        for (String key : keys) {
//            JSONObject date = json.getJSONObject(key);
//            String type = date.getString("type");
//            String day = date.getString("day");
//            //工作日
//            if ("0".equals(type) && (Integer.parseInt(day) <= Integer.parseInt(today))){
//                StringBuffer workDaysb = new StringBuffer(day).insert(4,"-").insert(7,"-");
//                String workDay = workDaysb.toString();
//                //判断工作日是否存在签到数据
//                Boolean isExist = false;
//                for (Map sign:signList){
//                    if (sign.containsValue(workDay)){
//                        isExist = true;
//                        break;
//                    }
//                }
//                if (!isExist){
//                    Map workMap = new HashMap();
//                    workMap.put("theTime",workDay);
//                    workMap.put("onTime","0");
//                    workMap.put("offTime","0");
//                    result.add(workMap);
//                }
//            }
//        }
//        signList.addAll(result);
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy-MM");
        Date monthDate = null;
        try {
            monthDate = formatter.parse(monthSignSearch.getMonth());
        } catch (ParseException e) {
            e.printStackTrace();
        }
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(monthDate);
        int days = calendar.getActualMaximum(Calendar.DAY_OF_MONTH);
        List<Map> result = new ArrayList<>();
        String monthNow = monthSignSearch.getMonth().replace("-", "");
        String today = Date8Util.getCurrentDay("yyyyMMdd");
        for (int i = 1; i <= days; i++) {
            String sDay = monthNow +String.format("%02d",i);
            if (Integer.parseInt(sDay) <= Integer.parseInt(today)) {
                StringBuffer workDaysb = new StringBuffer(sDay).insert(4, "-").insert(7, "-");
                String workDay = workDaysb.toString();
                //判断是否存在签到数据
                Boolean isExist = false;
                for (Map sign : signList) {
                    if (sign.containsValue(workDay)) {
                        isExist = true;
                        break;
                    }
                }
                if (!isExist) {
                    Map workMap = new HashMap();
                    workMap.put("theTime", workDay);
                    workMap.put("onTime", "0");
                    workMap.put("offTime", "0");
                    result.add(workMap);
                }
            }
        }
        signList.addAll(result);

        return ResultRespone.success(signList);
    }

}
