package com.tofly.dyrq.custom.controller;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.dyrq.custom.entity.*;
import com.tofly.dyrq.custom.service.CustomTroubleService;
import com.tofly.dyrq.entity.Trouble;
import com.tofly.dyrq.entity.TroubleDeviceinfo;
import com.tofly.dyrq.entity.TroubleEnentType;
import com.tofly.dyrq.entity.TroublePipeinfo;
import com.tofly.dyrq.service.TroubleDeviceinfoService;
import com.tofly.dyrq.service.TroubleEnentTypeService;
import com.tofly.dyrq.service.TroublePipeinfoService;
import com.tofly.dyrq.service.TroubleService;
import io.swagger.annotations.*;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.List;

@RestController
@AllArgsConstructor
@RequestMapping("/trouble")
@Api(tags = "隐患管理接口")
public class CustomTroubleController {
    private final CustomTroubleService customTroubleService;
    private final TroublePipeinfoService troublePipeinfoService;
    private final TroubleDeviceinfoService troubleDeviceinfoService;
    private final TroubleService troubleService;
    private final TroubleEnentTypeService troubleEnentTypeService;

    @GetMapping("/list")
    @ApiOperation(value = "根据条件查询隐患列表数据")
    public ResultRespone getList(Page page, @ModelAttribute TroubleSearch troubleSearch) {
        return ResultRespone.success(customTroubleService.getList(page, troubleSearch));
    }

    @ToFlyAppLog(title = "隐患信息上报")
    @ApiOperation(value = "隐患信息上报", notes = "web端:file:所有文件,rectifyFinalFile:整改决算书,rectifyNoticeFile:隐患整改通知书,protectNoticeFile:管道保护通知书,rectifyFile:整改方案 移动端:photo:照片文件[多个/选填],audio:语音文件[多个/选填],other:其他文件")
    @PostMapping(headers = "content-type=multipart/form-data")
    public ResultRespone save(HttpServletRequest request, TroubleSave troubleSave) {
        /*if (troubleSave.getDeptid() == null) {
            throw new ToflyDeniedException("隐患所属单位不能为空");
        }*/
//        if (troubeSave.getLevelid() == null) {
//            throw new ToflyDeniedException("隐患级别不能为空");
//        }
        /*if (troubeSave.getUndersecretaryUserid() == null) {
            throw new ToflyDeniedException("副部长不能为空");
        }
        if (troubeSave.getSecretaryUserid() == null) {
            throw new ToflyDeniedException("部长不能为空");
        }*/
//        if (troubeSave.getFindUserid() == null) {
//            throw new ToflyDeniedException("隐患发现人员不能为空");
//        }
//        if (troubeSave.getFindTime() == null) {
//            throw new ToflyDeniedException("隐患发现时间不能为空");
//        }
        if (StringUtil.isEmpty(troubleSave.getTroubleName())) {
            throw new ToflyDeniedException("隐患名称不能为空");
        }
//        if (StringUtil.isEmpty(troubeSave.getTroubleType())) {
//            throw new ToflyDeniedException("隐患类别不能为空");
//        }
        if (StringUtil.isEmpty(troubleSave.getAddress())) {
            throw new ToflyDeniedException("地址不能为空");
        }
        if (StringUtil.isEmpty(troubleSave.getTroubleInfo())) {
            throw new ToflyDeniedException("隐患详情不能为空");
        }
        /*if (StringUtil.isEmpty(troubleSave.getUploadType())) {
            throw new ToflyDeniedException("隐患上报途径不能为空");
        }*/
        /*if ("0".equals(troubleSave.getUploadType()) && troubleSave.getOndutyUserid() == null) {
            throw new ToflyDeniedException("非工作日值班人员不能为空");
        }*/
       /* if ("2".equals(troubleSave.getUploadType()) && troubleSave.getGisUserid() == null) {
            throw new ToflyDeniedException("GIS班班长不能为空");
        }*/
        return ResultRespone.success(customTroubleService.save(request, troubleSave));
    }

    /*@ToFlyAppLog(title = "(非工作时间)值班人员现场勘查")
    @ApiOperation(value = "(非工作时间)值班人员现场勘查", notes = "photos:照片文件[多个/选填],audios:语音文件[多个/选填]")
    @PostMapping(value = "/ondutyUserCheck", headers = "content-type=multipart/form-data")
    public ResultRespone ondutyUserCheck(HttpServletRequest request, TroubleOndutyUpload troubleOndutyUpload) {
        if (troubleOndutyUpload.getId() == null) {
            throw new ToflyDeniedException("隐患ID不能为空");
        }
        if (troubleOndutyUpload.getEventType() == null) {
            throw new ToflyDeniedException("隐患事件类型不能为空");
        }

        *//*if (troubleOndutyUpload.getLayerId() == null) {
            throw new ToflyDeniedException("隐患所属图层ID");
        }
        if (StringUtil.isEmpty(troubleOndutyUpload.getLayerName())) {
            throw new ToflyDeniedException("隐患所属图层名称");
        }*//*
//        if ("0".equals(troubleOndutyUpload.getEventType())) {
//            if (StringUtil.isEmpty(troubleOndutyUpload.getTempHandleProject()) || (StringUtil.isEmpty(troubleOndutyUpload.getTempHandleResult()))) {
//                throw new ToflyDeniedException("临时处理方案和临时处理结果不能为空");
//            }
//        }else
//        {
        if(troubleOndutyUpload.getEventType()!=null){
            TroubleEnentType byId = troubleEnentTypeService.getById(troubleOndutyUpload.getEventType());
//        if(("1".equals(troubleOndutyUpload.getEventType())) || "2".equals(troubleOndutyUpload.getEventType())){
            if("1".equals(byId.getIsTrouble())){
                if (troubleOndutyUpload.getTroubleTypeId() == null) {
                    throw new ToflyDeniedException("是隐患的时候，隐患类型ID为不能空");
                }
                if (troubleOndutyUpload.getLevelId() == null) {
                    throw new ToflyDeniedException("是隐患的时候，隐患级别不能空");
                }
            }
        }

        return ResultRespone.success(customTroubleService.ondutyUserCheck(request, troubleOndutyUpload));
    }

    @ToFlyAppLog(title = "(工作时间)承办班班长审核派工")
    @ApiOperation(value = "(工作时间)承办班班长审核派工")
    @PostMapping(value = "/undertakerCheck")
    public ResultRespone undertakerCheck(@RequestBody UndertakerCheck undertakerCheck) {
        if (undertakerCheck.getId() == null) {
            throw new ToflyDeniedException("隐患ID不能为空");
        }
        if (undertakerCheck.getUndertakerHandleUserid() == null) {
            throw new ToflyDeniedException("巡检人员不能为空");
        }
        if (StringUtil.isEmpty(undertakerCheck.getUndertakerCheckNotes())) {
            throw new ToflyDeniedException("承办班班长审核意见");
        }
        return ResultRespone.success(customTroubleService.undertakerCheck(undertakerCheck));
    }

    @ToFlyAppLog(title = "(工作时间)巡检人员现场核实")
    @ApiOperation(value = "(工作时间)巡检人员现场核实", notes = "photos:照片文件[多个/选填],audios:语音文件[多个/选填]", consumes = "multipart/form-data")
    @PostMapping(value = "/undertakerHandle", headers = "content-type=multipart/form-data")
    public ResultRespone undertakerHandle(HttpServletRequest request, UndertakerHandle undertakerHandle) {
        if (undertakerHandle.getId() == null) {
            throw new ToflyDeniedException("隐患ID不能为空");
        }
        if (undertakerHandle.getState() == null) {
            throw new ToflyDeniedException("是否是隐患不能为空");
        }
//        if (StringUtil.isEmpty(undertakerHandle.getUndertakerHandleNotes())) {
//            throw new ToflyDeniedException("巡检处理情况不能为空");
//        }
        if("1".equals(undertakerHandle.getState())){
            if (undertakerHandle.getTroubleTypeid() == null) {
                throw new ToflyDeniedException("隐患类型不能为空");
            }
            if (undertakerHandle.getLevelid() == null) {
                throw new ToflyDeniedException("隐患级别不能为空");
            }
        }

       *//* if (undertakerHandle.getLayerId() == null) {
            throw new ToflyDeniedException("隐患所属图层ID");
        }*//*
     *//* if (StringUtil.isEmpty(undertakerHandle.getLayerName())) {
            throw new ToflyDeniedException("隐患所属图层名称");
        }*//*
        return ResultRespone.success(customTroubleService.undertakerHandle(request, undertakerHandle));
    }*/

    @ToFlyAppLog(title = "班组长审核派工")
    @ApiOperation(value = "班组长审核派工")
    @PostMapping("/undersecretaryCheck")
    public ResultRespone undersecretaryCheck(HttpServletRequest request, UndersecretaryCheck undersecretaryCheck) {
        if (undersecretaryCheck.getId() == null) {
            throw new ToflyDeniedException("隐患ID不能为空");
        }
        if (undersecretaryCheck.getUndersecretaryCheckNotes() == null) {
            throw new ToflyDeniedException("班组长审核备注不能为空");
        }
        if (StringUtil.isEmpty(undersecretaryCheck.getState())) {
            throw new ToflyDeniedException("隐患审核状态不能为空");
        }
        if ("0".equals(undersecretaryCheck.getState()) && undersecretaryCheck.getHandleUserid() == null) {
            throw new ToflyDeniedException("继续处理情况下工单处理责任人不能为空");
        }
        if ("1".equals(undersecretaryCheck.getState()) && undersecretaryCheck.getOtherDutyUserId() == null) {
            throw new ToflyDeniedException("转入其他班组情况下其他班组长不能为空");
        }
//        if ("2".equals(undersecretaryCheck.getState()) && undersecretaryCheck.getUndersecretaryId() == null) {
//            throw new ToflyDeniedException("转入副部长处理情况下副部长不能为空");
//        }
        return ResultRespone.success(customTroubleService.undersecretaryCheck(request, undersecretaryCheck));
    }

    @ToFlyAppLog(title = "班组长(/副部长/其他班组长)编制方案")
    @ApiOperation(value = "班组长(/副部长/其他班组长)编制方案", notes = "rectifyFinalFile:整改决算书,rectifyNoticeFile:隐患整改通知书,protectNoticeFile:管道保护通知书,rectifyFile:整改方案")
    @PostMapping(value = "/designSchema", headers = "content-type=multipart/form-data")
    public ResultRespone designSchema(HttpServletRequest request, DesignSchema designSchema) {
        if (designSchema.getId() == null) {
            throw new ToflyDeniedException("隐患ID不能为空");
        }
        if (designSchema.getHandleUserid() == null) {
            throw new ToflyDeniedException("工单处理责任人不能为空");
        }
        /*if (StringUtil.isEmpty(designSchema.getCoprocessor())) {
            throw new ToflyDeniedException("协同处理人员不能为空");
        }*/
        return ResultRespone.success(customTroubleService.designSchema(request, designSchema));
    }


    @ToFlyAppLog(title = "副部长审批隐患处理方案")
    @ApiOperation(value = "副部长审批隐患处理方案")
    @PostMapping(value = "/secretaryCheck", headers = "content-type=multipart/form-data")
    public ResultRespone secretaryCheck(HttpServletRequest request, SecretaryCheck secretaryCheck) {
        if (secretaryCheck.getTroubleId() == null) {
            throw new ToflyDeniedException("隐患ID不能为空");
        }
        if (StringUtil.isEmpty(secretaryCheck.getSecretaryCheckState())) {
            throw new ToflyDeniedException("副部长审核结果不能为空");
        }
        if (StringUtil.isEmpty(secretaryCheck.getSecretaryCheckNotes())) {
            throw new ToflyDeniedException("副部长审核意见不能为空");
        }
        return ResultRespone.success(customTroubleService.secretaryCheck(request, secretaryCheck));
    }

    @ToFlyAppLog(title = "工单负责人处理回报")
    @ApiOperation(value = "工单负责人处理回报", notes = "photos:照片文件[多个/选填],audios:语音文件[多个/选填]", consumes = "multipart/form-data")
    @PostMapping(value = "/handleSchema", headers = "content-type=multipart/form-data")
    public ResultRespone handleSchema(HttpServletRequest request, HandleSchema handleSchema) {
        if (handleSchema.getId() == null) {
            throw new ToflyDeniedException("隐患ID不能为空");
        }
        if (StringUtil.isEmpty(handleSchema.getHandleNote())) {
            throw new ToflyDeniedException("处理结果不能为空");
        }
        return ResultRespone.success(customTroubleService.handleSchema(request, handleSchema));
    }

    @ToFlyAppLog(title = "副部长/班组长审核处理回报")
    @ApiOperation(value = "副部长审核处理回报")
    @PostMapping(value = "/handleCheck", headers = "content-type=multipart/form-data")
    public ResultRespone handleCheck(HttpServletRequest request, HandleCheck handleCheck) {
        if (handleCheck.getTroubleId() == null) {
            throw new ToflyDeniedException("隐患ID不能为空");
        }
        if (StringUtil.isEmpty(handleCheck.getHandleCheck())) {
            throw new ToflyDeniedException("处理结果副部长/班组长审核不能为空");
        }
        return ResultRespone.success(customTroubleService.handleCheck(request, handleCheck));
    }

    /**
     * 新增隐患上报管线基本信息表
     *
     * @param troublePipeinfo 隐患上报管线基本信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "管线基本信息表上传")
    @ApiOperation(value = "管线基本信息表上传")
    @PostMapping("/pipeinfo")
    public ResultRespone save(@RequestBody TroublePipeinfo troublePipeinfo) {
        return ResultRespone.success(troublePipeinfoService.save(troublePipeinfo));
    }

    /**
     * 新增隐患上报设备基本信息表
     *
     * @param troubleDeviceinfo 隐患上报设备基本信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "设备基本信息表上传")
    @ApiOperation(value = "设备基本信息表上传")
    @PostMapping("/deviceinfo")
    public ResultRespone save(@RequestBody TroubleDeviceinfo troubleDeviceinfo) {
        return ResultRespone.success(troubleDeviceinfoService.save(troubleDeviceinfo));
    }


    @ToFlyAppLog(title = "处理存量隐患")
    @ApiOperation(value = "通过ID使存量隐患能正常处理")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @PutMapping("/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        Trouble trouble = new Trouble();
        trouble.setId(id);
        trouble.setProcessId((long) 1);
        return ResultRespone.success(troubleService.updateById(trouble));
    }


    @ApiOperation(value = "报表-类型统计")
    @GetMapping("/getLxtj")
    public ResultRespone getLxtj(@ModelAttribute CustomRushRepairReportQuery customRushRepairReportQuery) {
        return ResultRespone.success(customTroubleService.lxtj(customRushRepairReportQuery));
    }

    @ApiOperation(value = "报表-趋势分析")
    @GetMapping("/getQsfx")
    public ResultRespone getQsfx(@ModelAttribute CustomRushRepairReportQuery customRushRepairReportQuery) {
        return ResultRespone.success(customTroubleService.qsfx(customRushRepairReportQuery));
    }


    /**
     * 修改设备管线基本信息
     *
     * @param customTroubleUpdatePipe 修改设备管线基本信息
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "修改设备管线基本信息")
    @ApiOperation(value = "修改设备管线基本信息")
    @PutMapping("/updatePipeDevice")
    public ResultRespone updatePipeDevice(@RequestBody CustomTroubleUpdatePipe customTroubleUpdatePipe) {
        return ResultRespone.success(customTroubleService.updatePipeDevice(customTroubleUpdatePipe));
    }

    @GetMapping("/listNew")
    @ApiOperation(value = "根据条件查询隐患列表-新")
    public ResultRespone getListNew(Page page, @ModelAttribute TroubleSearch troubleSearch) {
        return ResultRespone.success(customTroubleService.getListNew(page, troubleSearch));
    }

    @GetMapping("/info/{id}")
    @ApiOperation(value = "根据id查询隐患详情")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    public ResultRespone getById(@PathVariable("id") Long id) {
        return ResultRespone.success(customTroubleService.getById(id));
    }
}
