package com.tofly.dyrq.custom.controller;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.core.util.Date8Util;
import com.tofly.common.core.util.FtpUtil;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.log.annotation.ToFlyAppLog;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrq.config.FilePathBean;
import com.tofly.dyrq.custom.entity.CustomSysDatabase;
import com.tofly.dyrq.custom.entity.UserPda;
import com.tofly.dyrq.custom.service.OtherService;
import com.tofly.dyrq.entity.*;
import com.tofly.dyrq.service.*;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.Arrays;
import java.util.List;
import java.util.Optional;

/**
 * @author tuonbed
 * @date 2020/3/27  10:14
 * 其他非自动生成的接口。1.新增疑问标识
 */
@RestController
@AllArgsConstructor
public class OtherController {

    private final QuestionmarkService questionmarkService;

    private final FilePathBean filePathBean;

    private final OtherService otherService;

    private final PdaDeviceService pdaDeviceService;

    private final PdaUserDeviceService pdaUserDeviceService;

    private final SysDeviceTypeService sysDeviceTypeService;

    private final AppVersionService appVersionService;

    private final TfSysSettingService tfSysSettingService;

    private final WebWarningconfigService webWarningconfigService;

    private final GpsStatusService gpsStatusService;

    private final PlanPointTypeService planPointTypeService;

    /**
     * 新增疑问标识信息表
     *
     * @param questionmark 疑问标识信息表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增疑问标识信息表")
    @ApiOperation(tags = "疑问标识信息表接口", value = "新增疑问标识信息表")
    @PostMapping("/questionmark")
    public ResultRespone save(@RequestBody Questionmark questionmark) {
        //需要先将fileid取出
        String fileid = questionmark.getFileid();
        if (StringUtil.isEmpty(fileid)) {
            return ResultRespone.failed("fileid不能为空");
        }
        String file = fileid.substring(fileid.lastIndexOf("/") + 1);
        String filename = filePathBean.getQuestionmark() + IdWorker.getIdStr() + file.substring(file.lastIndexOf("."));
        try {
            boolean b = FtpUtil.uploadFile2Ftp(fileid, filename);
            if (b) {
                questionmark.setFileid(filename);
                questionmarkService.save(questionmark);
                return ResultRespone.success(true);
            } else {
                return ResultRespone.failed("文件获取失败");
            }
        } catch (Exception e) {
            return ResultRespone.failed(e.getMessage());
        }
    }

    @ToFlyAppLog(title = "移动端新增疑问标识信息表")
    @ApiOperation(tags = "疑问标识信息表接口", value = "移动端新增疑问标识信息表")
    @PostMapping("/questionmark/mobile")
    public ResultRespone save(MultipartFile file, Questionmark questionmark) {
        if (file != null) {
            String filename = filePathBean.getQuestionmark() + IdWorker.getIdStr() + file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf("."));
            boolean b = FtpUtil.uploadFile2Ftp(file, filename);
            if (b) {
                questionmark.setFileid(filename);
            } else {
                return ResultRespone.failed("上传到FTP失败");
            }
        }
        return ResultRespone.success(questionmarkService.save(questionmark));
    }

    /**
     * 查询用户设备绑定情况
     */
    @ApiOperation(tags = "巡检移动设备接口", value = "人员绑定设备列表")
    @GetMapping("/pdadevice/userPdaPage")
    public ResultRespone userPdaPage(Page page, @ModelAttribute UserPda user) {
        return ResultRespone.success(otherService.getUsersPda(page, user));
    }

    @ApiOperation(tags = "巡检移动设备接口", value = "人员绑定设备列表(移动端)")
    @GetMapping("/pdadevice/userPdaPage/mobile")
    public ResultRespone userPdaPageByMobile(Page page, @ModelAttribute UserPda user) {
        return ResultRespone.success(otherService.getUsersPdaByMobile(page, user));
    }


    /**
     * 查询用户设备绑定情况
     */
    @ApiOperation(tags = "巡检移动设备接口", value = "设备绑定校验", notes = "设备绑定校验接口。返回result:0[未绑定],1[已绑定],2[已绑定且当前登录用户与当前设备绑定的用户不一致],3[当前用户已绑定其他设备]")
    @GetMapping("/pdadevice/checkPda")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "imei", value = "IMEI码，校验人员绑定设备信息", required = true, dataType = "String")
    })
    public ResultRespone checkPda(String imei) {
        /**PdaUserDevice one1 = pdaUserDeviceService.getOne(Wrappers.<PdaUserDevice>lambdaQuery()
         .eq(PdaUserDevice::getUserId, SecurityUtils.getUserId()));
         String restr="0";
         if(one1!=null){
         PdaDevice one = pdaDeviceService.getById(one1.getDeviceId());
         if(one.getImei().equals(imei)){
         restr="1";
         }else{
         restr="2";
         }
         }**/
        return ResultRespone.success(otherService.checkPda(imei));
    }

    /**
     * 设备新增同时绑定人员
     */
    @ToFlyAppLog(title = "设备新增同时绑定人员")
    @ApiOperation(tags = "巡检移动设备接口", value = "设备新增同时绑定人员")
    @PostMapping("/pdadevice/add")
    public ResultRespone userPdaAdd(@RequestBody PdaDevice pdaDevice) {
        return ResultRespone.success(otherService.addPda(pdaDevice));
    }


    /**
     * 通过id删除巡检移动设备
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除巡检移动设备")
    @ApiOperation(tags = "巡检移动设备接口", value = "通过ID删除巡检移动设备")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/pdadevice/{id}")
    public ResultRespone removeById(@PathVariable Long id) {
        List<PdaUserDevice> list = pdaUserDeviceService.list(Wrappers.<PdaUserDevice>lambdaQuery()
                .eq(PdaUserDevice::getDeviceId, id));
        if (list.size() > 0) {
            return ResultRespone.failed("该设备还存在绑定信息，无法删除");
        }
        return ResultRespone.success(pdaDeviceService.removeById(id));
    }

    /**
     * 通过id删除巡检人员与移动设备关联表
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除巡检人员与移动设备关联表")
    @ApiOperation(tags = "巡检人员与移动设备关联表接口", value = "通过ID删除巡检人员与移动设备关联表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/pdauserdevice/{id}")
    public ResultRespone userDeviceRemoveById(@PathVariable Long id) {
        return ResultRespone.success(otherService.userDeviceRemoveById(id));
    }

    /**
     * 新增巡检人员与移动设备关联表
     *
     * @param pdaUserDevice 巡检人员与移动设备关联表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增巡检人员与移动设备关联表")
    @ApiOperation(tags = "巡检人员与移动设备关联表接口", value = "新增巡检人员与移动设备关联表")
    @PostMapping
    public ResultRespone userDeviceSave(@RequestBody PdaUserDevice pdaUserDevice) {
        return ResultRespone.success(otherService.userDeviceSave(pdaUserDevice));
    }


    /**
     * 分页查询
     *
     * @param page          分页对象
     * @param sysDeviceType 设备类型表
     * @return
     */
    @GetMapping("/sysdevicetype/page")
    @ApiOperation(tags = "设备类型表接口", value = "分页查询")
    public ResultRespone getSysDeviceTypePage(Page page, @ModelAttribute SysDeviceType sysDeviceType) {
        return ResultRespone.success(sysDeviceTypeService.page(page, Wrappers.query(sysDeviceType)
                .exists(StringUtil.isNotEmpty(sysDeviceType.getPointTypeName()),
                        "select 1 from tf_plan_point_type where id=TF_SYS_DEVICE_TYPE.POINT_TYPE_ID" +
                                " and name like '%" + sysDeviceType.getPointTypeName() + "%'"
                ))
        );
    }

    /**
     * 新增app版本发布
     *
     * @param appVersion app版本发布
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增app版本发布")
    @ApiOperation(tags = "app版本发布接口", value = "新增app版本发布")
    @PostMapping("/appversion")
    public ResultRespone save(MultipartFile file, AppVersion appVersion) {
        if (file != null) {
            String filename = filePathBean.getApp() + IdWorker.getIdStr() + file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf("."));
            boolean b = FtpUtil.uploadFile2Ftp(file, filename);
            appVersion.setDownurl(filename);
        }
        return ResultRespone.success(appVersionService.save(appVersion));
    }

    /**
     * 获取app信息
     *
     * @return ResultRespone
     */
    @ApiOperation(tags = "app版本发布接口", value = "获取app信息")
    @GetMapping("/appversion/getAppInfo")
    public ResultRespone getAppInfo() {
        AppVersion one = appVersionService.getOne(Wrappers.<AppVersion>lambdaQuery()
                .apply("id = (select max(id) from tf_app_version where del_flag='1')"));
        return ResultRespone.success(one);
    }

    /**
     * 获取app最新版本信息
     *
     * @return ResultRespone
     */
    @ApiOperation(tags = "app版本发布接口", value = "移动端获取app最新版本")
    @GetMapping("/appversion/new")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "version", value = "app版本号[1.0.4]", required = true, dataType = "String")
    })
    public ResultRespone getAppNew(String version) {
        if (StringUtil.isEmpty(version)) {
            throw new ToflyDeniedException("app版本号不能为空");
        }
        AppVersion one = appVersionService.getOne(Wrappers.<AppVersion>lambdaQuery()
                .apply("id = (select max(id) from tf_app_version where del_flag='1')"));
        if (one == null){
            throw new ToflyDeniedException("服务器无app资源");
        }
        //对比版本号
        List list = Arrays.asList(version.split("\\."));
        List lastList = Arrays.asList(one.getVersion().split("\\."));
        for (int i = 0; i < 3; i++) {
            if (!lastList.get(i).equals(list.get(i))){
                return ResultRespone.success(one);
            }
        }
        return ResultRespone.success(null);
    }

    /**
     * 查询系统设置表
     *
     * @return ResultRespone
     */
    @GetMapping("/tfsyssetting/getSysSetting")
    @ApiOperation(tags = "系统设置表接口", value = "通过ID查询系统设置表")
    public ResultRespone getSysSettingById() {
        return ResultRespone.success(tfSysSettingService.getById(1));
    }

    /**
     * 新增和修改系统设置表
     *
     * @param tfSysSetting 系统设置表
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "新增和修改系统设置表")
    @ApiOperation(tags = "系统设置表接口", value = "新增和修改系统设置表")
    @PostMapping("/tfsyssetting")
    public ResultRespone saveOrUpdate(@RequestBody TfSysSetting tfSysSetting) {
        tfSysSetting.setId((long) 1);
        return ResultRespone.success(tfSysSettingService.saveOrUpdate(tfSysSetting));
    }


    /**
     * 查询系统设置表
     *
     * @return ResultRespone
     */
    @GetMapping("/webwarningconfig/layers")
    @ApiOperation(tags = "预警分析参数配置表接口", value = "通过图层名查询")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "layername", value = "图层名，多个以逗号隔开", required = true, dataType = "String")
    })
    public ResultRespone getWebWarningBylayers(String layername) {
        return ResultRespone.success(webWarningconfigService.list(Wrappers.<WebWarningconfig>lambdaQuery()
                .in(WebWarningconfig::getLayername, Arrays.asList(layername.split(",")))));
    }


    /**
     * 通过id删除app版本发布
     *
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title = "通过id删除app版本发布")
    @ApiOperation(tags = "app版本发布接口", value = "通过ID删除app版本发布")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/appversion/{id}")
    public ResultRespone appversionRemoveById(@PathVariable Long id) {
        Optional.ofNullable(appVersionService.getById(id)).ifPresent(t -> {
            FtpUtil.deleteFile(t.getDownurl());
        });
        return ResultRespone.success(appVersionService.removeById(id));
    }

    /**
     * 新增GPS状态表
     *
     * @param gpsStatus GPS状态表
     * @return ResultRespone
     */
//    @ToFlyAppLog(title = "更新当前用户GPS状态")
    @ApiOperation(tags = "GPS状态表接口", value = "更新当前用户GPS状态")
    @PostMapping("tfgpsstatus")
    public ResultRespone save(@RequestBody GpsStatus gpsStatus) {
        Long userId = SecurityUtils.getUserId();
        String realName = SecurityUtils.getUserName();
        gpsStatus.setSystemtime(Date8Util.getCurrentDate());
        GpsStatus query = gpsStatusService.getOne(Wrappers.<GpsStatus>lambdaQuery().eq(GpsStatus::getUserId, userId));
        if (query != null) {
            gpsStatus.setId(query.getId());
            gpsStatus.setRealname(realName);
            return ResultRespone.success(gpsStatusService.updateById(gpsStatus));
        } else {
            return ResultRespone.success(gpsStatusService.save(gpsStatus));
        }
    }

    /**
     * 分页查询
     *
     * @param page          分页对象
     * @param planPointType 巡检类型配置表
     * @return
     */
    @GetMapping("/planpointtype/page")
    @ApiOperation(tags = "巡检类型配置表接口", value = "分页查询", notes = "巡检类型名称模糊")
    public ResultRespone getPlanPointTypePage(Page page, @ModelAttribute PlanPointType planPointType) {
        return ResultRespone.success(planPointTypeService.page(page, Wrappers.<PlanPointType>lambdaQuery()
                .like(StringUtil.isNotEmpty(planPointType.getName()), PlanPointType::getName, planPointType.getName())));
    }

    /**
     * 通过id删除巡检类型配置表
     * @param id id
     * @return ResultRespone
     */
    @ToFlyAppLog(title="通过id删除巡检类型配置表" )
    @ApiOperation(tags="巡检类型配置表接口",value = "通过ID删除巡检类型配置表")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "ID", required = true, dataType = "Long")
    })
    @DeleteMapping("/planpointtype/{id}" )
    public ResultRespone pointTypeRemoveById(@PathVariable Long id) {
        PlanPointType delete = planPointTypeService.getOne(Wrappers.<PlanPointType>lambdaQuery()
                .eq(PlanPointType::getId,id)
                .eq(PlanPointType::getForbidDelete,"1"));
        if (delete != null){
            throw new ToflyDeniedException("该类型禁止删除");
        }
        PlanPointType planOne = planPointTypeService.getOne(Wrappers.<PlanPointType>lambdaQuery()
                .eq(PlanPointType::getId,id)
                .apply("exists( select 1 from tf_plan_point_type a, tf_plan b where\n" +
                        "        b.device_type_id = a.id and a.del_flag = '1' and b.del_flag = '1' and a.id = {0}) ", id));
        if (planOne != null){
            throw new ToflyDeniedException("有计划存在该巡检类型，不能删除");
        }
        return ResultRespone.success(planPointTypeService.removeById(id));
    }


    @GetMapping("/sysDataBase/getDataBaseUseInfo")
    @ApiOperation(tags = "及时信息接口", value = "及时提醒查询")
    public ResultRespone getDataBaseUseInfo(Page page) {
        return ResultRespone.success(otherService.getSysDataSpace(page));
    }

    @PutMapping("/sysDataBase")
    @ToFlyAppLog(title="修改阈值")
    @ApiOperation(tags = "及时信息接口", value = "修改阈值")
    public ResultRespone updateDataBaseUseInfo(@RequestBody CustomSysDatabase customSysDatabase) {
        return ResultRespone.success(otherService.updateSysDataBase(customSysDatabase));
    }

    /**
     * 调压设备分页查询
     * @param page 分页对象
     * @param name 调压设备名称
     * @return
     */
    @GetMapping("/maintenancemenu/page")
    @ApiOperation(tags = "调压设备检查内容表接口",  value = "调压设备分页查询")
    public ResultRespone getMaintenanceMenuPage(Page page, String name) {
        return ResultRespone.success(otherService.getMaintenanceMenuPage(page, name));
    }
}
