package com.tofly.dyrq.custom.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.core.util.io.ExcelUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.dyrq.custom.entity.DevopHistory;
import com.tofly.dyrq.custom.service.CustomDevopsEquipmentService;
import com.tofly.dyrq.entity.DevopsEquipment;
import com.tofly.dyrq.mapper.DevopsEquipmentMapper;
import com.tofly.dyrq.service.DevopsEquipmentService;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.hssf.usermodel.*;
import org.apache.poi.ss.usermodel.*;
import org.apache.poi.ss.util.CellRangeAddress;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.math.BigDecimal;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.stream.Collectors;

@Service
@Slf4j
@AllArgsConstructor
public class CustomDevopsEquipmentServiceImpl implements CustomDevopsEquipmentService {
    private DevopsEquipmentService devopsEquipmentService;
    private DevopsEquipmentMapper devopsEquipmentMapper;

    @Override
    @SneakyThrows
    @Transactional
    public boolean uploadExcel(MultipartFile file) {
        if (file == null) {
            throw new ToflyDeniedException("上传文件为空");
        }

        List<DevopsEquipment> list = devopsEquipmentMapper.getAll();
        if (list.size() > 0) {
            devopsEquipmentMapper.updateAll();
        }

        InputStream inputStream = file.getInputStream();
        ExcelUtil opsExcel = new ExcelUtil(DevopsEquipment.class);
        List<DevopsEquipment> listOps = opsExcel.importExcel("录入", inputStream);
        List<DevopsEquipment> listAllOps = devopsEquipmentService.list();
        /*Map<String, DevopsEquipment> collect = listAllOps.stream().collect(Collectors.toMap(DevopsEquipment::getOwnNumber, m -> m, (k1, k2) -> k1));
        //StringBuffer sb=new StringBuffer();
        List<String> collect1 = listOps.stream().filter(k -> collect.get(k.getOwnNumber()) != null)
                .map(DevopsEquipment::getOwnNumber).collect(Collectors.toList());
        if (collect1.size() > 0) {
            throw new ToflyDeniedException("自编号:" + JSONArray.toJSONString(collect1) + "为重复数据,文件导入失败");
        }*/
        //listOps.sort((m1,m2)->m1.getCardNumber().compareTo(m2.getCardNumber()));
        //查询当前表中是否有数据 查出最大的台卡编号
        BigDecimal maxCardNum = listAllOps.size() > 0 ? devopsEquipmentService.list(Wrappers.<DevopsEquipment>lambdaQuery().
                orderByDesc(DevopsEquipment::getCardNumber)).get(0).getCardNumber() : null;
        //过滤空对象排序 用新的集合接收
        List<DevopsEquipment> listDev = listOps.stream().filter(k -> StringUtil.isNotEmpty(k.getOwnNumber()))
                .sorted(Comparator.comparing(DevopsEquipment::getCardNumber)).collect(Collectors.toList());
            /*if(menuBatch.getParentId()==null && !StringUtil.isEmpty(menuBatch.getParentName())){
                menuBatch.setParentId(collect.get(menuBatch.getParentName()).getId());
            }*/
        for (int i = 0; i < listDev.size(); i++) {
            if (maxCardNum != null) {//如果最大台卡编号不为空,则每次循环从最大台卡编号开始累加
                listDev.get(i).setCardNumber(maxCardNum.add(new BigDecimal(i + 1)));
            } else {
                listDev.get(i).setCardNumber(new BigDecimal(1));
                maxCardNum = new BigDecimal(0);
            }
            devopsEquipmentService.save(listDev.get(i));
            //collect.put(listDev.get(i).getOwnNumber(), listDev.get(i));
        }
        if (!listAllOps.isEmpty() && listAllOps.size() > 0 ) {
            List<Long> ids = new ArrayList<>();
            listAllOps.forEach(k -> {
                ids.add(k.getId());
            });
            devopsEquipmentMapper.updateByIds(ids);
            devopsEquipmentService.removeByIds(ids);
        }
        return true;
    }

    @Override
    public void exportExcel(String ids, OutputStream out, String fileName) {
        List<Long> idList = new ArrayList<>();
        Arrays.asList(ids.split(",")).forEach(k -> {
            idList.add(Long.parseLong(k));
        });
        List<DevopsEquipment> list = devopsEquipmentService.list(Wrappers.<DevopsEquipment>lambdaQuery().in(!idList.isEmpty(), DevopsEquipment::getId, idList));
        HSSFWorkbook wb = new HSSFWorkbook();
        HSSFCellStyle cs = wb.createCellStyle();
        cs.setAlignment(HorizontalAlignment.CENTER);
        cs.setVerticalAlignment(VerticalAlignment.CENTER);
        cs.setBorderBottom(BorderStyle.THIN);
        cs.setBorderLeft(BorderStyle.THIN);
        cs.setBorderRight(BorderStyle.THIN);
        cs.setBorderTop(BorderStyle.THIN);
        cs.setWrapText(true);
        HSSFCellStyle cs3 = wb.createCellStyle();
        cs3.setBorderBottom(BorderStyle.THIN);
        cs3.setBorderRight(BorderStyle.THIN);
        cs3.setBorderTop(BorderStyle.THIN);
        HSSFFont font = wb.createFont();
        font.setFontHeightInPoints((short) 11);
        cs.setFont(font);
        HSSFFont f = wb.createFont();
        f.setUnderline(Font.U_SINGLE);//下划线
        f.setFontHeightInPoints((short) 18);
        try {
            for (int i = 0; i < list.size(); i++) {
                HSSFSheet sheet = wb.createSheet("sheet" + (i + 1));
                HSSFRow row = sheet.createRow(0);
                row.setHeight((short) 800);
                Cell cell1 = row.createCell(0);
                cell1.setCellValue("四川德阳天然气有限责任公司");
                HSSFCellStyle cs1 = wb.createCellStyle();
                cs1.setAlignment(HorizontalAlignment.CENTER);
                cs1.setVerticalAlignment(VerticalAlignment.CENTER);
                cs1.setFont(f);
                CellRangeAddress region = new CellRangeAddress(0, 0, 0, 6);
                sheet.addMergedRegion(region);
                cell1.setCellStyle(cs1);
                HSSFRow row1 = sheet.createRow(1);
                row1.setHeight((short) 400);
                Cell cell3 = row1.createCell(0);
                cell3.setCellValue("台卡编号");
                cell3.setCellStyle(cs);
                Cell cell4 = row1.createCell(1);
                Cell cell68 = row1.createCell(2);
                cell68.setCellStyle(cs3);
                cell4.setCellStyle(cs);
                if (list.get(i).getCardNumber() != null) {
                    cell4.setCellValue(list.get(i).getCardNumber().doubleValue());
                }
                HSSFFont f2 = wb.createFont();
                f2.setUnderline(Font.U_SINGLE);
                f2.setFontHeightInPoints((short) 16);
                f2.setFontName("黑体");
                f2.setBold(true);
                HSSFCellStyle cs2 = wb.createCellStyle();
                cs2.setAlignment(HorizontalAlignment.CENTER);
                cs2.setVerticalAlignment(VerticalAlignment.CENTER);
                cs2.setFont(f2);
                Cell cell2 = row1.createCell(3);
                cell2.setCellValue("资产(设备)台卡");
                cell2.setCellStyle(cs2);
                Cell cell5 = row1.createCell(5);
                cell5.setCellValue("自编号");
                cell5.setCellStyle(cs);
                Cell cell6 = row1.createCell(6);
                cell6.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getOwnNumber())) {
                    cell6.setCellValue(list.get(i).getOwnNumber());
                }
                CellRangeAddress region1 = new CellRangeAddress(1, 1, 1, 2);
                sheet.addMergedRegion(region1);
                HSSFRow row2 = sheet.createRow(2);
                row2.setHeight((short) 400);
                Cell cell7 = row2.createCell(0);
                cell7.setCellValue("财务编号");
                cell7.setCellStyle(cs);
                Cell cell8 = row2.createCell(1);
                Cell cell69 = row2.createCell(2);
                cell69.setCellStyle(cs3);
                cell8.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getFinancialNumber())) {
                    cell8.setCellValue(list.get(i).getFinancialNumber());
                }
                Cell cell9 = row2.createCell(5);
                cell9.setCellValue("设备名称");
                cell9.setCellStyle(cs);
                Cell cell10 = row2.createCell(6);
                cell10.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getEquipmentName())) {
                    cell10.setCellValue(list.get(i).getEquipmentName());
                }
                CellRangeAddress region2 = new CellRangeAddress(2, 2, 1, 2);
                sheet.addMergedRegion(region2);
                HSSFRow row3 = sheet.createRow(3);
                row3.setHeight((short) 400);
                Cell cell11 = row3.createCell(0);
                cell11.setCellValue("使用部门");
                cell11.setCellStyle(cs);
                Cell cell12 = row3.createCell(1);
                Cell cell70 = row3.createCell(2);
                cell70.setCellStyle(cs3);
                cell12.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getDept())) {
                    cell12.setCellValue(list.get(i).getDept());
                }
                HSSFCellStyle cs5 = wb.createCellStyle();
                cs5.setBorderLeft(BorderStyle.THIN);
                cs5.setBorderRight(BorderStyle.THIN);
                Cell cell99 = row3.createCell(5);
                cell99.setCellStyle(cs5);
                Cell cell100 = row3.createCell(6);
                cell100.setCellStyle(cs5);
                CellRangeAddress region3 = new CellRangeAddress(3, 3, 1, 2);
                CellRangeAddress region4 = new CellRangeAddress(1, 3, 3, 4);
                CellRangeAddress region6 = new CellRangeAddress(2, 3, 5, 5);
                CellRangeAddress region7 = new CellRangeAddress(2, 3, 6, 6);
                sheet.addMergedRegion(region3);
                sheet.addMergedRegion(region4);
                sheet.addMergedRegion(region6);
                sheet.addMergedRegion(region7);
                HSSFRow row4 = sheet.createRow(4);
                row4.setHeight((short) 500);
                Cell cell13 = row4.createCell(0);
                cell13.setCellValue("制造单位");
                cell13.setCellStyle(cs);
                Cell cell14 = row4.createCell(1);
                Cell cell71 = row4.createCell(2);
                cell71.setCellStyle(cs3);
                cell14.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getManufacturer())) {
                    cell14.setCellValue(list.get(i).getManufacturer());
                }
                Cell cell15 = row4.createCell(3);
                cell15.setCellValue("出厂日期");
                cell15.setCellStyle(cs);
                Cell cell16 = row4.createCell(4);
                cell16.setCellStyle(cs);
                if (list.get(i).getManufacturerDate() != null) {
                    cell16.setCellValue(getStringDate(list.get(i).getManufacturerDate()));
                }
                Cell cell17 = row4.createCell(5);
                cell17.setCellValue("出厂编号");
                cell17.setCellStyle(cs);
                Cell cell18 = row4.createCell(6);
                cell18.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getManufacturerNumber())) {
                    cell18.setCellValue(list.get(i).getManufacturerNumber());
                }
                CellRangeAddress region8 = new CellRangeAddress(4, 4, 1, 2);
                sheet.addMergedRegion(region8);
                HSSFRow row5 = sheet.createRow(5);
                row5.setHeight((short) 400);
                Cell cell19 = row5.createCell(0);
                cell19.setCellValue("型号及规格");
                cell19.setCellStyle(cs);
                Cell cell20 = row5.createCell(1);
                cell20.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getTypeSpecification())) {
                    cell20.setCellValue(list.get(i).getTypeSpecification());
                }
                Cell cell21 = row5.createCell(5);
                cell21.setCellValue("总重(kg)");
                cell21.setCellStyle(cs);
                Cell cell22 = row5.createCell(6);
                cell22.setCellStyle(cs);
                if (list.get(i).getTotalWeight() != null) {
                    cell22.setCellValue(list.get(i).getTotalWeight());
                }
                HSSFRow row6 = sheet.createRow(6);
                row6.setHeight((short) 400);
                Cell cell106 = row6.createCell(1);
                cell106.setCellStyle(cs5);
                Cell cell23 = row6.createCell(5);
                cell23.setCellValue("原值(元)");
                cell23.setCellStyle(cs);
                Cell cell24 = row6.createCell(6);
                cell24.setCellStyle(cs);
                if (list.get(i).getOriginalValue() != null) {
                    cell24.setCellValue(list.get(i).getOriginalValue());
                }
                CellRangeAddress region10 = new CellRangeAddress(5, 6, 0, 0);
                CellRangeAddress region11 = new CellRangeAddress(5, 6, 1, 4);
                sheet.addMergedRegion(region10);
                sheet.addMergedRegion(region11);
                HSSFRow row7 = sheet.createRow(7);
                row7.setHeight((short) 400);
                Cell cell25 = row7.createCell(0);
                cell25.setCellValue("主要技术性能");
                cell25.setCellStyle(cs);
                Cell cell26 = row7.createCell(1);
                cell26.setCellValue("项目");
                cell26.setCellStyle(cs);
                Cell cell72 = row7.createCell(2);
                cell72.setCellStyle(cs3);
                Cell cell27 = row7.createCell(3);
                cell27.setCellValue("性能参数");
                cell27.setCellStyle(cs);
                Cell cell73 = row7.createCell(4);
                cell73.setCellStyle(cs3);
                Cell cell28 = row7.createCell(5);
                cell28.setCellValue("备注");
                cell28.setCellStyle(cs);
                Cell cell74 = row7.createCell(6);
                cell74.setCellStyle(cs3);
                CellRangeAddress region12 = new CellRangeAddress(7, 7, 1, 2);
                CellRangeAddress region13 = new CellRangeAddress(7, 7, 3, 4);
                CellRangeAddress region14 = new CellRangeAddress(7, 7, 5, 6);
                sheet.addMergedRegion(region12);
                sheet.addMergedRegion(region13);
                sheet.addMergedRegion(region14);
                HSSFRow row8 = sheet.createRow(8);
                row8.setHeight((short) 400);
                Cell cell29 = row8.createCell(1);
                cell29.setCellStyle(cs);
                Cell cell75 = row8.createCell(2);
                cell75.setCellStyle(cs3);
                if (StringUtil.isNotEmpty(list.get(i).getProject1())) {
                    cell29.setCellValue(list.get(i).getProject1());
                }
                Cell cell30 = row8.createCell(3);
                cell30.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getPerformanceParam1())) {
                    cell30.setCellValue(list.get(i).getPerformanceParam1());
                }
                Cell cell76 = row8.createCell(4);
                cell76.setCellStyle(cs3);
                Cell cell31 = row8.createCell(5);
                cell31.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getNote())) {
                    cell31.setCellValue(list.get(i).getNote());
                }
                HSSFCellStyle cs4 = wb.createCellStyle();
                cs4.setBorderRight(BorderStyle.THIN);
                Cell cell94 = row8.createCell(6);
                cell94.setCellStyle(cs4);
                CellRangeAddress region15 = new CellRangeAddress(8, 8, 1, 2);
                CellRangeAddress region16 = new CellRangeAddress(8, 8, 3, 4);
                //CellRangeAddress region17 = new CellRangeAddress(8,8,5,6);
                sheet.addMergedRegion(region15);
                sheet.addMergedRegion(region16);
                //sheet.addMergedRegion(region17);
                HSSFRow row9 = sheet.createRow(9);
                row9.setHeight((short) 400);
                Cell cell32 = row9.createCell(1);
                cell32.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getProject2())) {
                    cell32.setCellValue(list.get(i).getProject2());
                }
                Cell cell77 = row9.createCell(2);
                cell77.setCellStyle(cs3);
                Cell cell33 = row9.createCell(3);
                cell33.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getPerformanceParam2())) {
                    cell33.setCellValue(list.get(i).getPerformanceParam2());
                }
                Cell cell78 = row9.createCell(4);
                cell78.setCellStyle(cs3);
                Cell cell95 = row9.createCell(6);
                cell95.setCellStyle(cs4);
                CellRangeAddress region18 = new CellRangeAddress(9, 9, 1, 2);
                CellRangeAddress region19 = new CellRangeAddress(9, 9, 3, 4);
                sheet.addMergedRegion(region18);
                sheet.addMergedRegion(region19);
                HSSFRow row10 = sheet.createRow(10);
                row10.setHeight((short) 400);
                Cell cell34 = row10.createCell(1);
                cell34.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getProject3())) {
                    cell34.setCellValue(list.get(i).getProject3());
                }
                Cell cell79 = row10.createCell(2);
                cell79.setCellStyle(cs3);
                Cell cell35 = row10.createCell(3);
                cell35.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getPerformanceParam3())) {
                    cell35.setCellValue(list.get(i).getPerformanceParam3());
                }
                Cell cell80 = row10.createCell(4);
                cell80.setCellStyle(cs3);
                Cell cell96 = row10.createCell(6);
                cell96.setCellStyle(cs4);
                CellRangeAddress region21 = new CellRangeAddress(10, 10, 1, 2);
                CellRangeAddress region22 = new CellRangeAddress(10, 10, 3, 4);
                sheet.addMergedRegion(region21);
                sheet.addMergedRegion(region22);
                HSSFRow row11 = sheet.createRow(11);
                row11.setHeight((short) 400);
                Cell cell36 = row11.createCell(1);
                cell36.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getProject4())) {
                    cell36.setCellValue(list.get(i).getProject4());
                }
                Cell cell37 = row11.createCell(3);
                cell37.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getPerformanceParam4())) {
                    cell37.setCellValue(list.get(i).getPerformanceParam4());
                }
                Cell cell93 = row11.createCell(4);
                cell93.setCellStyle(cs3);
                Cell cell97 = row11.createCell(6);
                cell97.setCellStyle(cs4);
                CellRangeAddress region24 = new CellRangeAddress(11, 11, 1, 2);
                CellRangeAddress region25 = new CellRangeAddress(11, 11, 3, 4);
                sheet.addMergedRegion(region24);
                sheet.addMergedRegion(region25);
                HSSFRow row12 = sheet.createRow(12);
                row12.setHeight((short) 400);
                Cell cell38 = row12.createCell(1);
                cell38.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getProject5())) {
                    cell38.setCellValue(list.get(i).getProject5());
                }
                Cell cell81 = row12.createCell(2);
                cell81.setCellStyle(cs3);
                Cell cell39 = row12.createCell(3);
                cell39.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getPerformanceParam5())) {
                    cell39.setCellValue(list.get(i).getPerformanceParam5());
                }
                Cell cell82 = row12.createCell(4);
                cell82.setCellStyle(cs3);
                Cell cell98 = row12.createCell(6);
                cell98.setCellStyle(cs4);
                CellRangeAddress region27 = new CellRangeAddress(12, 12, 1, 2);
                CellRangeAddress region28 = new CellRangeAddress(12, 12, 3, 4);
                CellRangeAddress region30 = new CellRangeAddress(8, 12, 5, 6);
                CellRangeAddress region29 = new CellRangeAddress(7, 12, 0, 0);
                sheet.addMergedRegion(region27);
                sheet.addMergedRegion(region28);
                sheet.addMergedRegion(region30);
                sheet.addMergedRegion(region29);
                HSSFRow row13 = sheet.createRow(13);
                row13.setHeight((short) 540);
                Cell cell40 = row13.createCell(0);
                cell40.setCellValue("随机图纸及材料");
                cell40.setCellStyle(cs);
                Cell cell41 = row13.createCell(1);
                cell41.setCellValue("装箱单");
                cell41.setCellStyle(cs);
                Cell cell42 = row13.createCell(2);
                cell42.setCellStyle(cs);
                if (list.get(i).getPackingList() != null) {
                    cell42.setCellValue(list.get(i).getPackingList().doubleValue());
                }
                Cell cell43 = row13.createCell(3);
                cell43.setCellValue("合格证与等级");
                cell43.setCellStyle(cs);
                Cell cell44 = row13.createCell(4);
                cell44.setCellStyle(cs);
                if (list.get(i).getCertificateGrade() != null) {
                    cell44.setCellValue(list.get(i).getCertificateGrade().doubleValue());
                }
                Cell cell45 = row13.createCell(5);
                cell45.setCellValue("使用说明书");
                cell45.setCellStyle(cs);
                Cell cell46 = row13.createCell(6);
                cell46.setCellStyle(cs);
                if (list.get(i).getManualCopy() != null && list.get(i).getManualPage() != null) {
                    cell46.setCellValue(list.get(i).getManualCopy() + " 本共 " + list.get(i).getManualPage() + " 页");
                } else {
                    cell46.setCellValue(" 本共 页");
                }
                HSSFRow row14 = sheet.createRow(14);
                row14.setHeight((short) 540);
                Cell cell47 = row14.createCell(1);
                cell47.setCellValue("总装图");
                cell47.setCellStyle(cs);
                Cell cell48 = row14.createCell(2);
                cell48.setCellStyle(cs);
                if (list.get(i).getAssemblyDrawing() != null) {
                    cell48.setCellValue(list.get(i).getAssemblyDrawing().doubleValue());
                }
                Cell cell49 = row14.createCell(3);
                cell49.setCellValue("易碎件图");
                cell49.setCellStyle(cs);
                Cell cell50 = row14.createCell(4);
                cell50.setCellStyle(cs);
                if (list.get(i).getQuickWearPartDrawing() != null) {
                    cell50.setCellValue(list.get(i).getQuickWearPartDrawing().doubleValue());
                }
                Cell cell51 = row14.createCell(5);
                cell51.setCellValue("其他资料");
                cell51.setCellStyle(cs);
                Cell cell52 = row14.createCell(6);
                cell52.setCellStyle(cs);
                if (list.get(i).getOtherMaterials() != null) {
                    cell52.setCellValue(list.get(i).getOtherMaterials().doubleValue());
                }
                CellRangeAddress region31 = new CellRangeAddress(13, 14, 0, 0);
                sheet.addMergedRegion(region31);
                HSSFRow row15 = sheet.createRow(15);
                Cell cell53 = row15.createCell(0);
                cell53.setCellValue("安装单位");
                cell53.setCellStyle(cs);
                row15.setHeight((short) 400);
                Cell cell54 = row15.createCell(1);
                cell54.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getInstallUnit())) {
                    cell54.setCellValue(list.get(i).getInstallUnit());
                }
                HSSFCellStyle cs6 = wb.createCellStyle();
                cs6.setBorderBottom(BorderStyle.THIN);
                cs6.setBorderTop(BorderStyle.THIN);
                Cell cell101 = row15.createCell(2);
                cell101.setCellStyle(cs6);
                Cell cell83 = row15.createCell(3);
                cell83.setCellStyle(cs3);
                Cell cell55 = row15.createCell(4);
                cell55.setCellValue("设备责任人");
                cell55.setCellStyle(cs);
                Cell cell56 = row15.createCell(5);
                cell56.setCellStyle(cs);
                Cell cell84 = row15.createCell(6);
                cell84.setCellStyle(cs3);
                if (StringUtil.isNotEmpty(list.get(i).getResponsiblePerson())) {
                    cell56.setCellValue(list.get(i).getResponsiblePerson());
                }
                CellRangeAddress region32 = new CellRangeAddress(15, 15, 1, 3);
                CellRangeAddress region33 = new CellRangeAddress(15, 15, 5, 6);
                sheet.addMergedRegion(region32);
                sheet.addMergedRegion(region33);
                HSSFRow row16 = sheet.createRow(16);
                row16.setHeight((short) 400);
                Cell cell57 = row16.createCell(0);
                cell57.setCellValue("安装地点");
                cell57.setCellStyle(cs);
                Cell cell58 = row16.createCell(1);
                cell58.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getInstallPlace())) {
                    cell58.setCellValue(list.get(i).getInstallPlace());
                }
                Cell cell102 = row16.createCell(2);
                cell102.setCellStyle(cs6);
                Cell cell85 = row16.createCell(3);
                cell85.setCellStyle(cs3);
                Cell cell59 = row16.createCell(4);
                cell59.setCellValue("更新日期");
                cell59.setCellStyle(cs);
                Cell cell86 = row16.createCell(5);
                cell86.setCellStyle(cs);
                if (list.get(i).getUpdateDate() != null) {
                    cell86.setCellValue(getStringDate(list.get(i).getUpdateDate()));
                }
                Cell cell87 = row16.createCell(6);
                cell87.setCellStyle(cs3);
                CellRangeAddress region34 = new CellRangeAddress(16, 16, 1, 3);
                CellRangeAddress region35 = new CellRangeAddress(16, 16, 5, 6);
                sheet.addMergedRegion(region34);
                sheet.addMergedRegion(region35);
                HSSFRow row17 = sheet.createRow(17);
                row17.setHeight((short) 400);
                Cell cell60 = row17.createCell(0);
                cell60.setCellValue("安装日期");
                cell60.setCellStyle(cs);
                Cell cell61 = row17.createCell(1);
                cell61.setCellStyle(cs);
                if (list.get(i).getInstallDate() != null) {
                    cell61.setCellValue(getStringDate(list.get(i).getInstallDate()));
                }
                Cell cell103 = row17.createCell(2);
                cell103.setCellStyle(cs6);
                Cell cell92 = row17.createCell(3);
                cell92.setCellStyle(cs3);
                Cell cell62 = row17.createCell(4);
                cell62.setCellValue("启用日期");
                cell62.setCellStyle(cs);
                Cell cell63 = row17.createCell(5);
                cell63.setCellStyle(cs);
                if (list.get(i).getIntroductionDate() != null) {
                    cell63.setCellValue(getStringDate(list.get(i).getIntroductionDate()));
                }
                Cell cell88 = row17.createCell(6);
                cell88.setCellStyle(cs3);
                CellRangeAddress region36 = new CellRangeAddress(17, 17, 1, 3);
                CellRangeAddress region37 = new CellRangeAddress(17, 17, 5, 6);
                sheet.addMergedRegion(region36);
                sheet.addMergedRegion(region37);
                HSSFRow row18 = sheet.createRow(18);
                row18.setHeight((short) 3000);
                Cell cell64 = row18.createCell(0);
                cell64.setCellValue("来源与转移");
                cell64.setCellStyle(cs);
                Cell cell65 = row18.createCell(1);
                cell65.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getSourceTransferName())) {
                    cell65.setCellValue(list.get(i).getSourceTransferName());
                }
                Cell cell104 = row18.createCell(2);
                cell104.setCellStyle(cs6);
                Cell cell90 = row18.createCell(3);
                cell90.setCellStyle(cs3);
                Cell cell66 = row18.createCell(4);
                cell66.setCellValue("备注");
                cell66.setCellStyle(cs);
                Cell cell67 = row18.createCell(5);
                cell67.setCellStyle(cs);
                if (StringUtil.isNotEmpty(list.get(i).getRemarks())) {
                    cell67.setCellValue(list.get(i).getRemarks());
                }
                Cell cell89 = row18.createCell(6);
                cell89.setCellStyle(cs3);
                CellRangeAddress region38 = new CellRangeAddress(18, 18, 1, 3);
                CellRangeAddress region39 = new CellRangeAddress(18, 18, 5, 6);
                sheet.addMergedRegion(region38);
                sheet.addMergedRegion(region39);
                HSSFRow row19 = sheet.createRow(19);
                row19.setHeight((short) 400);
                HSSFCellStyle cs7 = wb.createCellStyle();
                cs7.setAlignment(HorizontalAlignment.RIGHT);
                cs7.setVerticalAlignment(VerticalAlignment.CENTER);
                cs7.setFont(font);
                Cell cell107 = row19.createCell(0);
                cell107.setCellStyle(cs7);
                if (list.get(i).getCreateCardDate() != null) {
                    StringBuffer sb = new StringBuffer("建卡日期:");
                    sb.append(getStringDate(list.get(i).getCreateCardDate()));
                    cell107.setCellValue(String.valueOf(sb));
                } else {
                    cell107.setCellValue("建卡日期:");
                }
                Cell cell108 = row19.createCell(4);
                cell108.setCellStyle(cs7);
                if (list.size() > 0) {
                    cell108.setCellValue("本部门台卡共 " + list.size() + " 页此台第 " + (i + 1) + " 页");
                }
                CellRangeAddress region40 = new CellRangeAddress(19, 19, 0, 1);
                CellRangeAddress region41 = new CellRangeAddress(19, 19, 2, 3);
                CellRangeAddress region42 = new CellRangeAddress(19, 19, 4, 6);
                sheet.addMergedRegion(region40);
                sheet.addMergedRegion(region41);
                sheet.addMergedRegion(region42);
                HSSFCellStyle cs8 = wb.createCellStyle();
                cs8.setBorderLeft(BorderStyle.THIN);
                Cell cell109 = row6.createCell(0);
                cell109.setCellStyle(cs8);
                Cell cell110 = row8.createCell(0);
                cell110.setCellStyle(cs8);
                Cell cell111 = row9.createCell(0);
                cell111.setCellStyle(cs8);
                Cell cell112 = row10.createCell(0);
                cell112.setCellStyle(cs8);
                Cell cell113 = row11.createCell(0);
                cell113.setCellStyle(cs8);
                Cell cell114 = row12.createCell(0);
                cell114.setCellStyle(cs8);
                Cell cell115 = row14.createCell(0);
                cell115.setCellStyle(cs8);

                sheet.setColumnWidth(0, 2500);
                sheet.setColumnWidth(1, 3500);
                sheet.setColumnWidth(2, 3200);
                sheet.setColumnWidth(3, 3500);
                sheet.setColumnWidth(4, 3200);
                sheet.setColumnWidth(5, 3000);
                sheet.setColumnWidth(6, 3500);
            }
            wb.write(out);
            //return ResultRespone.success(fileName);
        } catch (Exception e) {
            log.error("导出Excel异常{}", e.getMessage());
            //return ResultRespone.failed("导出Excel失败，请联系网站管理员");

        } finally {
            try {
                wb.close();
            } catch (IOException e1) {
                e1.printStackTrace();
            }
            if (out != null) {
                try {
                    out.close();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
        }
    }

    @Override
    public Page<Map> getHistory(Page page, DevopHistory devopHistory) {
        devopHistory.setDelFlag("0");
        devopHistory.setHistoryFlag("1");
        Page<Map> result = page.setRecords(devopsEquipmentMapper.getHistory(page,devopHistory));
        return result;
    }

    /**
     * 获取现在时间
     *
     * @return返回字符串格式 yyyy.MM.dd
     */
    private String getStringDate(Date date) {
        SimpleDateFormat formatter = new SimpleDateFormat("yyyy.MM.dd");
        String dateString = formatter.format(date);
        return dateString;
    }

}
