package com.tofly.dyrq.custom.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.util.Date8Util;
import com.tofly.common.core.util.FtpUtil;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.mapper.ToflyCodeCommonMapper;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrq.config.FilePathBean;
import com.tofly.dyrq.custom.entity.MessageSave;
import com.tofly.dyrq.custom.entity.ReceiptMessageReault;
import com.tofly.dyrq.custom.entity.SentMessageReault;
import com.tofly.dyrq.custom.mapper.CustomMessageMapper;
import com.tofly.dyrq.custom.service.CustomMessageService;
import com.tofly.dyrq.entity.Message;
import com.tofly.dyrq.entity.MessageFile;
import com.tofly.dyrq.entity.MessageRecipient;
import com.tofly.dyrq.entity.TfPdaResponseFile;
import com.tofly.dyrq.service.MessageFileService;
import com.tofly.dyrq.service.MessageRecipientService;
import com.tofly.dyrq.service.MessageService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.MultiValueMap;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.util.*;
import java.util.stream.Collectors;

@AllArgsConstructor
@Service
public class CustomMessageServiceImpl implements CustomMessageService {
    private final FilePathBean filePathBean;
    private final MessageFileService messageFileService;
    private final MessageService messageService;
    private final MessageRecipientService messageRecipientService;
    private final ToflyCodeCommonMapper toflyCodeCommonMapper;
    private final CustomMessageMapper customMessageMapper;

    @Override
    @Transactional
    public Boolean save(HttpServletRequest request, MessageSave messageSave) {
        //保存消息
        Message message = new Message();
        //默认类型为普通消息
        if (StringUtil.isEmpty(messageSave.getType())) {
            message.setType("0");
        } else {
            message.setType(messageSave.getType());
        }
        message.setMessage(messageSave.getMessage());
        //如果角色存在，则查询角色下的用户
        List<String> roles = new ArrayList<>();
        if (StringUtil.isNotEmpty(messageSave.getRoles())) {
            roles = Arrays.asList(messageSave.getRoles().split(","));
        }
        String receiptIds = messageSave.getReceipt();
        for (String role : roles) {
            String s = customMessageMapper.getRoleUsers(Long.valueOf(role));
            if (StringUtil.isNotEmpty(s)) {
                receiptIds = receiptIds + ',' + s;
            }
        }
        List<String> receipt = Arrays.asList(receiptIds.split(","));
        message.setReplyId(messageSave.getReplyId());
        messageService.save(message);
        Long messageId = message.getId();
        //保存消息的接收人
        receipt.forEach(receiptId -> {
            MessageRecipient messageRecipient = new MessageRecipient();
            messageRecipient.setMessageId(messageId);
            messageRecipient.setRecipient(Long.parseLong(receiptId));
            messageRecipientService.save(messageRecipient);
        });

        //遍历请求中key包含photo的所有文件，包括photo1，photo2中的多个文件
        MultiValueMap<String, MultipartFile> multiFileMap = ((MultipartHttpServletRequest) request).getMultiFileMap();
        multiFileMap.forEach((k, v) -> {
            //保存照片
            v.forEach(file -> {
                String fileName = file.getOriginalFilename();
                String filePath = filePathBean.getMessage() + IdWorker.getIdStr() + file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf("."));
                boolean b = FtpUtil.uploadFile2Ftp(file, filePath);
                if (b) {
                    MessageFile messageFile = new MessageFile();
                    messageFile.setFilePath(filePath);
                    messageFile.setFileName(fileName);
                    messageFile.setMessageId(messageId);
                    if (k.contains("photo")) {
                        messageFile.setType("0");
                    } else if (k.contains("audio")) {
                        messageFile.setType("1");
                    }
                    messageFileService.save(messageFile);
                }
            });
        });
        return true;
    }

    @Override
    public Boolean sendMessage(String userIds, String message, String type, String module) {
        MessageSave messageSave = new MessageSave();
        messageSave.setMessage(message);
        messageSave.setReceipt(userIds);
        messageSave.setType(type);
        messageSave.setModule(module);
        return this.saveNoFile(messageSave);
    }

    @Override
    public Boolean saveNoFile(MessageSave messageSave) {
        //保存消息
        Message message = new Message();
        //默认类型为普通消息
        if (StringUtil.isEmpty(messageSave.getType())) {
            message.setType("0");
        } else {
            message.setType(messageSave.getType());
        }
        message.setMessage(messageSave.getMessage());
        message.setModule(messageSave.getModule());
        //如果角色存在，则查询角色下的用户
        List<String> roles = new ArrayList<>();
        if (StringUtil.isNotEmpty(messageSave.getRoles())) {
            roles = Arrays.asList(messageSave.getRoles().split(","));
        }
        String receiptIds = messageSave.getReceipt();
        for (String role : roles) {
            String s = customMessageMapper.getRoleUsers(Long.valueOf(role));
            if (StringUtil.isNotEmpty(s)) {
                receiptIds = receiptIds + ',' + s;
            }
        }
        List<String> receipt = Arrays.asList(receiptIds.split(","));
        message.setReplyId(messageSave.getReplyId());
        messageService.save(message);
        Long messageId = message.getId();
        //保存消息的接收人
        receipt.forEach(receiptId -> {
            MessageRecipient messageRecipient = new MessageRecipient();
            messageRecipient.setMessageId(messageId);
            messageRecipient.setRecipient(Long.parseLong(receiptId));
            messageRecipientService.save(messageRecipient);
        });
        return true;
    }

    @Override
    @Transactional
    public Map getById(Long id) {
        //返回详情
        Map result = new HashMap();
        Message message = messageService.getById(id);
        if (message != null) {
            result.put("messageId", message.getId());
            result.put("createDate", message.getCreateDate());
            result.put("createUser", message.getCreateUser());
            result.put("createUserName", message.getCreateUserName());
            result.put("message", message.getMessage());
            result.put("module",message.getModule());
            List<MessageFile> photos = messageFileService.list(Wrappers.<MessageFile>lambdaQuery().eq(MessageFile::getMessageId, id).eq(MessageFile::getType, "0"));
            List<String> photosResult = photos.stream().map(MessageFile::getFilePath).collect(Collectors.toList());
            List<MessageFile> audios = messageFileService.list(Wrappers.<MessageFile>lambdaQuery().eq(MessageFile::getMessageId, id).eq(MessageFile::getType, "1"));
            List<String> audiosResult = audios.stream().map(MessageFile::getFilePath).collect(Collectors.toList());
            result.put("photos", photosResult);
            result.put("audios", audiosResult);
            //回复信息列表
            result.put("replyList", getReply(message.getId()));
        }

        //修改接收状态
        Boolean b = messageRecipientService.update(Wrappers.<MessageRecipient>lambdaUpdate()
                .eq(MessageRecipient::getMessageId, id)
                .eq(MessageRecipient::getRecipient, SecurityUtils.getUserId())
                .set(MessageRecipient::getFlag, "1")
                .set(MessageRecipient::getRecipientTime, Date8Util.getCurrentDate()));
        return result;
    }

    @Override
    @Transactional
    public Map getInfoById(Long id) {
        //返回详情
        Map result = new HashMap();
        Message message = messageService.getById(id);
        if (message != null) {
            result.put("messageId", message.getId());
            result.put("createDate", message.getCreateDate());
            result.put("createUser", message.getCreateUser());
            result.put("createUserName", message.getCreateUserName());
            result.put("message", message.getMessage());
            result.put("module",message.getModule());
            List<MessageFile> photos = messageFileService.list(Wrappers.<MessageFile>lambdaQuery().eq(MessageFile::getMessageId, id).eq(MessageFile::getType, "0"));
            List<String> photosResult = photos.stream().map(MessageFile::getFilePath).collect(Collectors.toList());
            List<MessageFile> audios = messageFileService.list(Wrappers.<MessageFile>lambdaQuery().eq(MessageFile::getMessageId, id).eq(MessageFile::getType, "1"));
            List<String> audiosResult = audios.stream().map(MessageFile::getFilePath).collect(Collectors.toList());
            result.put("photos", photosResult);
            result.put("audios", audiosResult);
            //回复信息列表
            result.put("replyList", getReply(message.getId()));
        }
        return result;
    }

//    @Override
//    public IPage receiptMessage(Page page, String searchText) {
//        Long userId = SecurityUtils.getUserId();
//        int total = 0;
//        //接收人
//        IPage result = messageRecipientService.pageMaps(page, Wrappers.<MessageRecipient>lambdaQuery()
//                .eq(MessageRecipient::getRecipient, userId)
//                .select(MessageRecipient::getMessageId, MessageRecipient::getFlag, MessageRecipient::getRecipientTime)
//                .orderByDesc(MessageRecipient::getId));
//        List<Map> records = result.getRecords();
//        Boolean isReply = false;
//        Boolean isExist = true;
//        for (int i = 0; i < records.size(); i++) {
//            Long messageId = Long.parseLong(records.get(i).get("messageId").toString());
//            Map<String, Object> messageMap = messageService.getMap(Wrappers.<Message>lambdaQuery()
//                    .eq(Message::getId, messageId)
//                    .select(Message::getCreateUser, Message::getCreateDate, Message::getMessage, Message::getReplyId, Message::getType)
//                    .like(StringUtil.isNotEmpty(searchText), Message::getMessage, searchText));
//            if (messageMap == null) {
//                records.remove(i);
//                isExist = false;
//            } else {
//                //不显示回复的消息
//                if (messageMap.get("replyId") != null) {
//                    List<Map<String, Object>> list = messageRecipientService.listMaps(Wrappers.<MessageRecipient>lambdaQuery()
//                            .eq(MessageRecipient::getMessageId, messageMap.get("replyId")));
//                    for (Map<String, Object> k : list) {
//                        if (Long.parseLong(k.get("recipient").toString()) == userId) {
//                            isReply = true;
//                            records.remove(i);
//                        }
//                    }
//                }
//                records.get(i).put("createUser", messageMap.get("createUser"));
//                records.get(i).put("createUserName", toflyCodeCommonMapper.getUserName(Long.parseLong(messageMap.get("createUser").toString())));
//                records.get(i).put("createUserAvatar", toflyCodeCommonMapper.getUserAvatar(Long.parseLong(messageMap.get("createUser").toString())));
//                records.get(i).put("receiptUserAvatar", toflyCodeCommonMapper.getUserAvatar(userId));
//                records.get(i).put("createDate", messageMap.get("createDate"));
//                records.get(i).put("message", messageMap.get("message"));
//                records.get(i).put("type", messageMap.get("type"));
//            }
//            if (!isExist || isReply) {
//                i--;
//                isExist = true;
//                isReply = false;
//                total++;
//            }
//        }
//        result.setRecords(records);
//        result.setTotal(result.getTotal() - total);
//        return result;
//    }

    @Override
    public Page<ReceiptMessageReault> receiptMessage(Page page, String searchText) {
        Long userId = SecurityUtils.getUserId();
        return page.setRecords(customMessageMapper.receiptMessage(page, searchText, userId));
    }

    @Override
    public Page<SentMessageReault> sent(Page page, String searchText) {
        Long userId = SecurityUtils.getUserId();
        return page.setRecords(customMessageMapper.sentMessage(page, searchText, userId));
    }

//    @Override
//    public IPage sent(Page page, String searchText) {
//        Long userId = SecurityUtils.getUserId();
//        IPage result = messageService.pageMaps(page, Wrappers.<Message>lambdaQuery()
//                .eq(Message::getCreateUser, userId)
//                .isNull(Message::getReplyId)
//                .select(Message::getCreateDate, Message::getCreateUser, Message::getId, Message::getMessage)
//                .like(StringUtil.isNotEmpty(searchText), Message::getMessage, searchText)
//                .orderByDesc(Message::getCreateDate));
//        List<Map> records = result.getRecords();
//        //接收人信息
//        for (Map i : records) {
//            Long messageId = Long.parseLong(i.get("id").toString());
//            List<Map<String, Object>> list = messageRecipientService.listMaps(Wrappers.<MessageRecipient>lambdaQuery()
//                    .eq(MessageRecipient::getMessageId, messageId)
//                    .select(MessageRecipient::getFlag, MessageRecipient::getRecipient, MessageRecipient::getRecipientTime));
//            list.forEach(k -> {
//                k.put("recipientName", toflyCodeCommonMapper.getUserName(Long.parseLong(k.get("recipient").toString())));
//            });
//            i.put("receipt", list);
//            i.put("createUserName", toflyCodeCommonMapper.getUserName(Long.parseLong(i.get("createUser").toString())));
//        }
//        result.setRecords(records);
//        return result;
//    }

    @Override
    @Transactional
    public Boolean deleteByIds(String ids) {
        Long userId = SecurityUtils.getUserId();
        List<String> idList = Arrays.asList(ids.split(","));
        idList.forEach(messageId -> {
            Boolean b = messageRecipientService.remove(Wrappers.<MessageRecipient>lambdaQuery()
                    .eq(MessageRecipient::getMessageId, messageId)
                    .eq(MessageRecipient::getRecipient, userId));
        });
        return true;
    }

    @Override
    @Transactional
    public Boolean readByIds(String ids) {
        Long userId = SecurityUtils.getUserId();
        List<String> idList = Arrays.asList(ids.split(","));
        idList.forEach(messageId -> {
            Boolean b = messageRecipientService.update(Wrappers.<MessageRecipient>lambdaUpdate()
                    .eq(MessageRecipient::getMessageId, messageId)
                    .eq(MessageRecipient::getRecipient, userId)
                    .set(MessageRecipient::getFlag, "1")
                    .set(MessageRecipient::getRecipientTime, Date8Util.getCurrentDate()));
        });
        return true;
    }

    //获取回复信息
    @Transactional
    public List<Map<String, Object>> getReply(Long messageId) {
        List<Map<String, Object>> replyList = messageService.listMaps(Wrappers.<Message>lambdaQuery()
                .eq(Message::getReplyId, messageId)
                .select(Message::getCreateDate, Message::getCreateUser, Message::getMessage, Message::getId)
                .orderByAsc(Message::getCreateDate));
        replyList.forEach(k -> {
            k.put("createUserName", toflyCodeCommonMapper.getUserName(Long.parseLong(k.get("createUser").toString())));
            List<MessageFile> photos = messageFileService.list(Wrappers.<MessageFile>lambdaQuery().eq(MessageFile::getMessageId, k.get("id")).eq(MessageFile::getType, "0"));
            List<String> photosResult = photos.stream().map(MessageFile::getFilePath).collect(Collectors.toList());
            List<MessageFile> audios = messageFileService.list(Wrappers.<MessageFile>lambdaQuery().eq(MessageFile::getMessageId, k.get("id")).eq(MessageFile::getType, "1"));
            List<String> audiosResult = audios.stream().map(MessageFile::getFilePath).collect(Collectors.toList());
            k.put("photos", photosResult);
            k.put("audios", audiosResult);
            //添加抄送人
            List<MessageRecipient> list = messageRecipientService.list(Wrappers.<MessageRecipient>lambdaQuery()
                    .eq(MessageRecipient::getMessageId, k.get("id")));
            String names = "";
            for (int i = 0; i < list.size(); i++) {
                if (i == 0) {
                    names = list.get(i).getRecipientName();
                } else {
                    names = names + "," + list.get(i).getRecipientName();
                }
            }
            k.put("recipients", names);
        });
        return replyList;
    }
}
