package com.tofly.dyrq.custom.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.util.BeanUtils;
import com.tofly.common.core.util.FtpUtil;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrq.config.FilePathBean;
import com.tofly.dyrq.custom.service.CustomPhotoService;
import com.tofly.dyrq.entity.SysUploadFile;
import com.tofly.dyrq.entity.UploadPhoto;
import com.tofly.dyrq.service.SysUploadFileService;
import com.tofly.dyrq.service.UploadPhotoService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.MultiValueMap;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author tuonbed
 * @date 2020/4/28  17:19
 */
@AllArgsConstructor
@Service
public class CustomPhotoServiceImpl implements CustomPhotoService {

    private final UploadPhotoService uploadPhotoService;
    private final SysUploadFileService sysUploadFileService;
    private final FilePathBean filePathBean;

    @Override
    @Transactional
    public boolean savePhoto(HttpServletRequest request, UploadPhoto uploadPhoto) {
        MultiValueMap<String, MultipartFile> multiFileMap = ((MultipartHttpServletRequest) request).getMultiFileMap();
        uploadPhoto.setCompanyid(SecurityUtils.getUserDeptId());
        uploadPhoto.setUploadUserid(SecurityUtils.getUserId());
        uploadPhotoService.save(uploadPhoto);
        setSysUploadFilePrt(uploadPhoto,multiFileMap);
        return true;
    }

    @Override
    public IPage getUploadPhotoPage(Page page, UploadPhoto uploadPhoto) {
        IPage iPage = uploadPhotoService.page(page, Wrappers.query(uploadPhoto));
        List<UploadPhoto> list=iPage.getRecords();
        if(list.size()==0){
            return iPage;
        }
//        list.sort(Comparator.comparing(m->(Date)m.get("createTime")));
        List<SysUploadFile> sysUploadFiles = sysUploadFileService.list(Wrappers.<SysUploadFile>lambdaQuery()
                .in(SysUploadFile::getTableid, list.stream().map(k ->k.getId()).collect(Collectors.toList())));
        Map<Long, List<SysUploadFile>> collect = sysUploadFiles.stream().
                collect(Collectors.toMap(SysUploadFile::getTableid, k -> {
                    List<SysUploadFile> list1=new ArrayList();
                    list1.add(k);
                    return list1;
                }, (List<SysUploadFile> k1,List<SysUploadFile> k2)->{
                    k1.addAll(k2);
                    return k1;
                }));
        List<Map> list1 = list.stream().map(upload -> {
            Map map = BeanUtils.beanToMap(upload);
            map.put("files",collect.get(upload.getId()));
//            SysUploadFile sysUploadFile = collect.get(upload.getId());
//            map.put("fileName", sysUploadFile.getFilename());
//            map.put("fileSize", sysUploadFile.getFilelength());
//            map.put("filePath", sysUploadFile.getPath());
//            map.put("thumbPath", sysUploadFile.getThumbPath());
//            map.put("fileType",sysUploadFile.getFiletype());
            return map;
        }).collect(Collectors.toList());
        iPage.setRecords(list1);
        return iPage;
    }

    @Override
    public Map getById(Long id) {
        UploadPhoto uploadPhoto = uploadPhotoService.getOne(Wrappers.<UploadPhoto>lambdaQuery()
                .eq(UploadPhoto::getId, id));
        SysUploadFile one = sysUploadFileService.getOne(Wrappers.<SysUploadFile>lambdaQuery().
                eq(SysUploadFile::getTableid, id));
        Map map = BeanUtils.beanToMap(uploadPhoto);
        map.put("fileName",one.getFilename());
        map.put("fileSize",one.getFilelength());
        map.put("filePath",one.getPath());
        map.put("fileType",one.getFiletype());
        //map.put("thumbPath",one.getThumbPath());
        return map;
    }

    @Override
    @Transactional
    public boolean updatePhoto(HttpServletRequest request, UploadPhoto uploadPhoto) {
        MultiValueMap<String, MultipartFile> multiFileMap = ((MultipartHttpServletRequest) request).getMultiFileMap();
        if(multiFileMap.size()>0){
            List<SysUploadFile> sysUploadFile=sysUploadFileService.list(Wrappers.<SysUploadFile>lambdaQuery().
                    eq(SysUploadFile::getTableid, uploadPhoto.getId()));
            for(SysUploadFile sysUploadFile1:sysUploadFile){
                FtpUtil.deleteFile(sysUploadFile1.getPath());
                FtpUtil.deleteFile(sysUploadFile1.getThumbPath());
            }
            setSysUploadFilePrt(uploadPhoto,multiFileMap);

        }
        uploadPhotoService.updateById(uploadPhoto);
        return true;
    }

    @Override
    @Transactional
    public boolean removeById(String ids) {
        List<String> list = Arrays.asList(ids.split(","));
        List<SysUploadFile> list1 = sysUploadFileService.list(Wrappers.<SysUploadFile>lambdaQuery()
                .in(SysUploadFile::getTableid, list));
        list1.forEach(k->{
            FtpUtil.deleteFile(k.getPath());
        });
        sysUploadFileService.remove(Wrappers.<SysUploadFile>lambdaQuery()
                .in(SysUploadFile::getTableid,list));
        uploadPhotoService.removeByIds(list);
        return true;
    }


    private void setSysUploadFilePrt(UploadPhoto uploadPhoto,MultiValueMap<String, MultipartFile> multiFileMap){
        for(Map.Entry<String, List<MultipartFile>> entry : multiFileMap.entrySet()) {
            SysUploadFile sysUploadFile=new SysUploadFile();
            sysUploadFile.setUploadUserid(SecurityUtils.getUserId());
            sysUploadFile.setCompanyid(SecurityUtils.getUserDeptId());
            sysUploadFile.setUploadNote(uploadPhoto.getDescription());
            sysUploadFile.setFtpState(BigDecimal.ONE);
            sysUploadFile.setCheckTime(uploadPhoto.getActualTime());
            sysUploadFile.setTablename("TF_UPLOAD_PHOTO");
            List<MultipartFile> v = entry.getValue();
            String fileType = v.get(0).getOriginalFilename().substring(v.get(0).getOriginalFilename().lastIndexOf("."));
            String fileFtpPath = filePathBean.getApp() + IdWorker.getIdStr() + fileType;
            String fileName = v.get(0).getOriginalFilename().substring(0, v.get(0).getOriginalFilename().lastIndexOf("."));
            boolean b = FtpUtil.uploadFile2Ftp(v.get(0), fileFtpPath);
            sysUploadFile.setFilelength(BigDecimal.valueOf(v.get(0).getSize()));
            sysUploadFile.setPath(fileFtpPath);
            sysUploadFile.setFiletype(fileType);
            sysUploadFile.setFilename(fileName);
            sysUploadFile.setTableid(uploadPhoto.getId());
            sysUploadFile.setUploadTime(uploadPhoto.getCreateTime());
            sysUploadFileService.save(sysUploadFile);
        }
    }

//    private void setSysUploadFilePrt(SysUploadFile sysUploadFile,MultiValueMap<String, MultipartFile> multiFileMap){
//        long size=0L;
//        Map m=new HashMap();
//        for(Map.Entry<String, List<MultipartFile>> entry : multiFileMap.entrySet()) {
//            List<MultipartFile> v = entry.getValue();
//            String fileType = v.get(0).getOriginalFilename().substring(v.get(0).getOriginalFilename().lastIndexOf("."));
//            String fileFtpPath = filePathBean.getApp() + IdWorker.getIdStr() + fileType;
//            String fileName = v.get(0).getOriginalFilename().substring(0, v.get(0).getOriginalFilename().lastIndexOf("."));
//            boolean b = FtpUtil.uploadFile2Ftp(v.get(0), fileFtpPath);
//            if (size == 0L && multiFileMap.size() > 1) {
//                m.put("fileType", fileType);
//                m.put("fileName", fileName);
//                m.put("filePath", fileFtpPath);
//                size = v.get(0).getSize();
//            } else {
//                if (v.get(0).getSize() > size) {
//                    sysUploadFile.setFilelength(BigDecimal.valueOf(v.get(0).getSize()));
//                    sysUploadFile.setPath(fileFtpPath);
//                    sysUploadFile.setFiletype(fileType);
//                    sysUploadFile.setFilename(fileName);
//                    sysUploadFile.setThumbPath((String) m.get("filePath"));
//                } else {
//                    sysUploadFile.setFilelength(BigDecimal.valueOf(size));
//                    sysUploadFile.setPath((String) m.get("filePath"));
//                    sysUploadFile.setFiletype((String) m.get("fileType"));
//                    sysUploadFile.setFilename((String) m.get("fileName"));
//                    sysUploadFile.setThumbPath(fileFtpPath);
//                }
//            }
//        }
//    }
}
