package com.tofly.dyrq.custom.service.impl;

import com.tofly.common.core.util.Date8Util;
import com.tofly.dyrq.custom.entity.PlanReportBasis;
import com.tofly.dyrq.custom.entity.PlanReportDetailInfo;
import com.tofly.dyrq.custom.entity.PlanReportResult;
import com.tofly.dyrq.custom.entity.PlanReportStatistics;
import com.tofly.dyrq.custom.mapper.CustomPlanReportMapper;
import com.tofly.dyrq.custom.service.CustomPlanReportService;
import lombok.AllArgsConstructor;
import org.springframework.stereotype.Service;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.*;

/**
 * @author pra
 * @date 2020/11/2 9:40
 * @description
 */
@Service
@AllArgsConstructor
public class CustomPlanReportServiceImpl implements CustomPlanReportService {
    private final CustomPlanReportMapper customPlanReportMapper;

    @Override
    public List<PlanReportResult> getPlanReport(String departmentId, String userId, String searchDate, String timeType) {
        Calendar cal = Calendar.getInstance();
        Date now = cal.getTime();
        Date sdate = null;
        Date edate = null;
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        try {
            sdate = sdf.parse(searchDate);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        cal.setTime(sdate);
        List<PlanReportResult> results = new ArrayList<>();
        List<PlanReportBasis> basis = new ArrayList<>();
        Map param = new HashMap();
        param.put("departmentId",departmentId);
        param.put("userId",userId);
        //判断日报或月报及查询时间
        if ("1".equals(timeType)){
            cal.set(Calendar.HOUR_OF_DAY, 23);
            cal.set(Calendar.MINUTE, 59);
            cal.set(Calendar.SECOND, 59);
            edate = cal.getTime();
            param.put("sdate",sdate);
            param.put("edate",edate);
        }
        else if ("2".equals(timeType)){
            cal.set(Calendar.DATE,1);
            sdate = cal.getTime();
            cal.add(Calendar.MONTH,1);
            cal.add(Calendar.MINUTE,-1);
            edate = cal.getTime();
            param.put("sdate",sdate);
            param.put("edate",edate);
        }
        basis = customPlanReportMapper.planReportBasis(param);
        for (PlanReportBasis prb : basis) {
            PlanReportResult reportResult = new PlanReportResult();
            param.put("userId",prb.getUserId());
            List<PlanReportDetailInfo> detailInfos = customPlanReportMapper.planReportDetailInfo(param);
            reportResult.setPlanReportBasis(prb);
            reportResult.setPlanReportDetailInfo(detailInfos);
            results.add(reportResult);
        }
        return results;
    }

    @Override
    public List<PlanReportResult> getPlanReportSpan(String departmentId, String userId, String beginPlanDate, String endPlanDate) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date sdate = null;
        Date edate = null;
        try {
            sdate = sdf.parse(beginPlanDate);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        try {
            edate = sdf.parse(endPlanDate);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        Map param = new HashMap();
        param.put("userId",userId);
        param.put("departmentId",departmentId);
        param.put("sdate",sdate);
        param.put("edate",edate);
        List<PlanReportResult> results = new ArrayList<>();
        List<PlanReportBasis> basis = new ArrayList<>();
        basis = customPlanReportMapper.planReportBasis(param);
        for (PlanReportBasis prb : basis) {
            PlanReportResult reportResult = new PlanReportResult();
            param.put("userId",prb.getUserId());
            List<PlanReportDetailInfo> detailInfos = customPlanReportMapper.planReportDetailInfo(param);
            reportResult.setPlanReportBasis(prb);
            reportResult.setPlanReportDetailInfo(detailInfos);
            results.add(reportResult);
        }
        return results;
    }

    @Override
    public List<PlanReportStatistics> getPlanReportStatistics(String departmentId, String beginPlanDate, String endPlanDate) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");
        Date sdate = null;
        Date edate = null;
        try {
            sdate = sdf.parse(beginPlanDate);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        try {
            edate = sdf.parse(endPlanDate);
        } catch (ParseException e) {
            e.printStackTrace();
        }
        Map param = new HashMap();
        param.put("departmentId",departmentId);
        param.put("sdate",sdate);
        param.put("edate",edate);
        return customPlanReportMapper.planReportStatistics(param);
    }

}
