package com.tofly.dyrq.custom.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.util.Date8Util;
import com.tofly.common.core.util.SpringContextHolder;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrq.custom.entity.*;
import com.tofly.dyrq.custom.mapper.CustomPlanMapper;
import com.tofly.dyrq.custom.mapper.OtherMapper;
import com.tofly.dyrq.custom.service.CustomPlanService;
import com.tofly.dyrq.custom.service.GisAboutService;
import com.tofly.dyrq.entity.*;
import com.tofly.dyrq.service.*;
import com.tofly.dyrq.util.PlanDetailEvent;
import lombok.AllArgsConstructor;
import lombok.RequiredArgsConstructor;
import lombok.SneakyThrows;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.lang.reflect.Array;
import java.math.BigDecimal;
import java.net.URL;
import java.net.URLConnection;
import java.util.*;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.stream.Collectors;


/**
 * @author tuonbed
 * @date 2020/4/13  13:42
 */
@Service
@RequiredArgsConstructor
public class CustomPlanServiceImpl implements CustomPlanService {
    private final CustomPlanMapper customPlanMapper;
    private final OtherMapper otherMapper;
    private final GpsRegionService gpsRegionService;
    private final PlanPointService planPointService;
    private final PlanService planService;
    private final PlanChildService planChildService;
    private final PlanPeriodTypeService planPeriodTypeService;
    private final PlanPointTypeService planPointTypeService;
    private final PlanPointTempService planPointTempService;
    private final TfPlanPathTempService planPathTempService;
    private final GisAboutService gisAboutService;
    private final PlanTransferService planTransferService;
    private final SysDeviceTypeService sysDeviceTypeService;
    private final MaintenanceDeviceService maintenanceDeviceService;
    private final MaintenanceDeviceConService maintenanceDeviceConService;
    private final MaintenanceDeviceFileService maintenanceDeviceFileService;
    private final MaintenanceService maintenanceService;
    private final MaintenanceContentService maintenanceContentService;
    private final MaintenanceFileService maintenanceFileService;

    private volatile long tf_plan_point_tempId;
    private volatile long tf_plan_path_tempId;
    private volatile long transfer_tf_plan_point_tempId;
    private volatile long transfer_tf_plan_path_tempId;
    private volatile long maintenanceDevice_id;
    private volatile long maintenanceDevice_con_id;
    private volatile long maintenanceDevice_file_id;
    private volatile long maintenance_id;
    private volatile long maintenance_con_id;
    private volatile long maintenance_file_id;

    @Override
    @Transactional
    @SneakyThrows
    public boolean updateGpsRegion(RegionValidEntity regionValidEntity) {
        if (StringUtil.isNotEmpty(regionValidEntity.getValidIds())) {
            List idList = Arrays.asList(regionValidEntity.getValidIds().split(","));
            checkUpdate(idList);
            planPointService.update(Wrappers.<PlanPoint>lambdaUpdate()
                    .set(PlanPoint::getValid, "1").in(PlanPoint::getId, idList));
        }
        if (StringUtil.isNotEmpty(regionValidEntity.getInvalidIds())) {
            List idList = Arrays.asList(regionValidEntity.getInvalidIds().split(","));
            checkUpdate(idList);
            planPointService.update(Wrappers.<PlanPoint>lambdaUpdate()
                    .set(PlanPoint::getValid, "0").in(PlanPoint::getId, idList));
        }
        return gpsRegionService.updateById(regionValidEntity);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    @SneakyThrows
    public boolean savePlan(CustomPlan customPlan) {

        //默认字段
        customPlan.setIsFindTrouble("0");
        customPlan.setIsread("0");
        customPlan.setIsvalid("0");
        Long userId = customPlan.getUserId();
        Long companyId = customPlan.getCompanyid();
        customPlan.setPlanState("1");
        if (customPlan.getGisInfo() != null) {
            customPlan.setPointInfo(customPlan.getGisInfo().toString());
        }
        String type = planPointTypeService.getOne(Wrappers.<PlanPointType>lambdaQuery().eq(PlanPointType::getId, customPlan.getDeviceTypeId())).getType();
        String gisUrl = customPlan.getGisUrl();
        Integer IntervalDays = new Integer(0);
        JSONArray geo = new JSONArray();
        //除了巡检点，巡检线的
        if (gisUrl != null) {
            //根据objectid请求arcgis服务
            String ids = customPlan.getObjectIds();
            String[] idsArr = ids.split(",");
            //objId个数超过1000后,分批次请求rest服务
            List<String[]> list = new ArrayList<>();
            for (int m = 0; m < ((double) idsArr.length / 1000); m++) {
                int start = m * 1000;
                int end = idsArr.length - start > 1000 ? (start + 1000) : (idsArr.length > 1000 ? idsArr.length : idsArr.length);
                list.add(Arrays.copyOfRange(idsArr, start, end));
            }
            for (String[] idArr : list) {
                String idStr = StringUtil.join(idArr, ',');
                ExcelParam excelParam = new ExcelParam();
                excelParam.setGeometry("1");
                excelParam.setObjectids(idStr);
                excelParam.setLayerurl(customPlan.getGisUrl());
                excelParam.setOutfields("SID,OBJECTID,PIPELENGTH,CODE");
                if (type.equals("0")) {
                    excelParam.setOutfields("SID,OBJECTID,CODE");
                }
                JSONArray geoTem = gisAboutService.exceloutList(excelParam);
                geo.addAll(geoTem);
            }
            customPlan.setPointInfo(geo.toJSONString());
        } else {
            if (customPlan.getGisInfo() == null) {
                throw new ToflyDeniedException("请添加巡检点的Json信息");
            }
            geo = customPlan.getGisInfo();
        }
        planService.save(customPlan);
        //存巡检点和线到TF_PLAN_POINT_TEMP，TF_PLAN_PATH_TEMP存的是巡检线的各个线上的点
        // 增加所有子计划
        int childPlanNums = 1;
        if (customPlan.getPeriodId() != 1) {
            PlanPeriodType planPeriodType = planPeriodTypeService.getOne(Wrappers.<PlanPeriodType>lambdaQuery().eq(PlanPeriodType::getId, customPlan.getPeriodId()));
            IntervalDays = planPeriodType.getIntervalDays().intValue();
            childPlanNums = (getTimeDistance(customPlan.getPlanBegindate(), customPlan.getPlanEnddate()) + 1) / IntervalDays;
        } else {
            IntervalDays = getTimeDistance(customPlan.getPlanBegindate(), customPlan.getPlanEnddate()) + 1;
        }
        Date beginDate = customPlan.getPlanBegindate();
        Calendar cal = Calendar.getInstance();
        Date now = cal.getTime();
        cal.setTime(beginDate);

        tf_plan_point_tempId = Math.max(tf_plan_point_tempId, customPlanMapper.getPlanTempMaxId("TF_PLAN_POINT_TEMP"));
        tf_plan_path_tempId = Math.max(tf_plan_path_tempId, customPlanMapper.getPlanTempMaxId("TF_PLAN_PATH_TEMP"));
        List<PlanPointTemp> pointTempList = new ArrayList<>();
        List<TfPlanPathTemp> pathTempList = new ArrayList<>();

        for (int i = 1; i <= childPlanNums; i++) {
            PlanChild planChild = new PlanChild();
            int pointNum = 0;
            cal.set(Calendar.HOUR_OF_DAY, 0);
            cal.set(Calendar.MINUTE, 0);
            cal.set(Calendar.SECOND, 0);
            planChild.setChildBeginDate(cal.getTime());
            cal.add(Calendar.DATE, IntervalDays - 1);
            cal.set(Calendar.HOUR_OF_DAY, 23);
            cal.set(Calendar.MINUTE, 59);
            cal.set(Calendar.SECOND, 59);
            planChild.setChildEndDate(cal.getTime());
            planChild.setChildPlanState("1");
            planChild.setMainPlanId(customPlan.getId());
            planChild.setChildNum(new BigDecimal(i));
            planChild.setPeriodId(customPlan.getPeriodId());
            planChild.setDeviceTypeId(customPlan.getDeviceTypeId());
            planChild.setLastTime(now);
            planChild.setRegionId(customPlan.getRegionId());
            planChild.setCompanyid(companyId);
            planChild.setUserId(userId);
            planChild.setPlanRate(customPlan.getPlanRate());


            //下个子计划日期加1
            cal.add(Calendar.DATE, 1);

            //存巡检点数据到临时记录表
            //巡检点
            if (type.equals("0")) {
                planChild.setPointNum(Long.valueOf(geo.size()));
                planChildService.save(planChild);
                
                for (int j = 0; j < geo.size(); j++) {
                    PlanPointTemp planPointTemp = new PlanPointTemp();
                    planPointTemp.setPlanId(customPlan.getId());
                    planPointTemp.setPeriodId(customPlan.getPeriodId());
                    planPointTemp.setUserId(userId);
                    planPointTemp.setCompanyid(companyId);
                    planPointTemp.setIsover("0");
                    planPointTemp.setChildPlanId(planChild.getId());
                    planPointTemp.setRegionId(customPlan.getRegionId());
                    planPointTemp.setDeviceTypeId(customPlan.getDeviceTypeId());
                    planPointTemp.setCreateDate(new Date());
                    if (gisUrl == null) {
                        planPointTemp.setPointId(geo.getJSONObject(j).getLong("sid"));
                        planPointTemp.setGeometry(geo.getJSONObject(j).getString("geometry"));
                        planPointTemp.setContent(geo.getJSONObject(j).getString("content"));
                        planPointTemp.setName(geo.getJSONObject(j).getString("name"));
                        planPointTemp.setAddress(geo.getJSONObject(j).getString("address"));
                        planPointTemp.setCode(geo.getJSONObject(j).getString("code"));
                        planPointTemp.setLat(geo.getJSONObject(j).getJSONObject("geometry").getBigDecimal("y"));
                        planPointTemp.setLng(geo.getJSONObject(j).getJSONObject("geometry").getBigDecimal("x"));
                        planPointTemp.setPipeLength("0");
                    } else {
                        planPointTemp.setPlanId(customPlan.getId());
                        planPointTemp.setPointId(geo.getJSONObject(j).getJSONObject("attributes").getLong("OBJECTID"));
                        planPointTemp.setGeometry(geo.getJSONObject(j).getString("geometry"));
                        planPointTemp.setLat(geo.getJSONObject(j).getJSONObject("geometry").getBigDecimal("y"));
                        planPointTemp.setLng(geo.getJSONObject(j).getJSONObject("geometry").getBigDecimal("x"));
                        planPointTemp.setPipeLength("0");
                    }
                    //planPointTempService.save(planPointTemp);
                    planPointTemp.setId(tf_plan_point_tempId);
                    pointTempList.add(planPointTemp);
                    tf_plan_point_tempId++;
                    //pointNum++;
                }
            }
            //巡检线
            else {
                planChildService.save(planChild);
                BigDecimal pipeLengthTotal = BigDecimal.ZERO;
                for (int j = 0; j < geo.size(); j++) {
                    JSONArray paths;
                    //巡检线
                    if (gisUrl == null) {
//                        paths = geo.getJSONObject(j).getJSONObject("geometry").getJSONArray("paths").getJSONArray(0);
                        JSONArray pipe;
                        pipe = geo.getJSONObject(j).getJSONArray("pipe");

                        for (int w = 0; w < pipe.size(); w++) {
                            PlanPointTemp planPointTemp = new PlanPointTemp();
                            planPointTemp.setPlanId(customPlan.getId());
                            planPointTemp.setPeriodId(customPlan.getPeriodId());
                            planPointTemp.setRegionId(customPlan.getRegionId());
                            planPointTemp.setDeviceTypeId(customPlan.getDeviceTypeId());
                            planPointTemp.setChildPlanId(planChild.getId());
                            planPointTemp.setUserId(userId);
                            planPointTemp.setCompanyid(companyId);
                            planPointTemp.setIsover("0");
                            planPointTemp.setCreateDate(new Date());

                            planPointTemp.setPointId(geo.getJSONObject(j).getLong("sid"));
                            planPointTemp.setGeometry(pipe.getJSONObject(w).getString("geometry"));
                            planPointTemp.setContent(geo.getJSONObject(j).getString("content"));
                            planPointTemp.setName(geo.getJSONObject(j).getString("name"));
                            planPointTemp.setAddress(geo.getJSONObject(j).getString("address"));
                            planPointTemp.setCode(geo.getJSONObject(j).getString("code"));

                            planPointTemp.setPipeLength(pipe.getJSONObject(w).getString("length"));
                            pipeLengthTotal = pipeLengthTotal.add(new BigDecimal(pipe.getJSONObject(w).getString("length")));
                            planPointTemp.setId(tf_plan_point_tempId);
                            pointTempList.add(planPointTemp);
                            tf_plan_point_tempId++;

                            //巡检线paths分段成两个点
                            paths = pipe.getJSONObject(w).getJSONObject("geometry").getJSONArray("paths").getJSONArray(0);
                            for (int e = 0; e < paths.size(); e++) {
                                TfPlanPathTemp tfPlanPathTemp = new TfPlanPathTemp();
                                tfPlanPathTemp.setTempId(planPointTemp.getId());
                                tfPlanPathTemp.setPointId(planPointTemp.getPointId());
                                tfPlanPathTemp.setLat(new BigDecimal(paths.getJSONArray(e).get(1).toString()));
                                tfPlanPathTemp.setLng(new BigDecimal(paths.getJSONArray(e).get(0).toString()));
                                tfPlanPathTemp.setIsover(new BigDecimal(0));
                                tfPlanPathTemp.setNum(new BigDecimal(e + 1));
                                tfPlanPathTemp.setChildPlanId(planChild.getId());
                                //planPathTempService.save(tfPlanPathTemp);
                                tfPlanPathTemp.setId(tf_plan_path_tempId);
                                pathTempList.add(tfPlanPathTemp);
                                tf_plan_path_tempId++;
                                pointNum++;
                            }
                        }
                    } else {
                        PlanPointTemp planPointTemp = new PlanPointTemp();
                        planPointTemp.setPlanId(customPlan.getId());
                        planPointTemp.setPeriodId(customPlan.getPeriodId());
                        planPointTemp.setRegionId(customPlan.getRegionId());
                        planPointTemp.setDeviceTypeId(customPlan.getDeviceTypeId());
                        planPointTemp.setChildPlanId(planChild.getId());
                        planPointTemp.setUserId(userId);
                        planPointTemp.setCompanyid(companyId);
                        planPointTemp.setIsover("0");
                        planPointTemp.setCreateDate(new Date());

                        planPointTemp.setPointId(geo.getJSONObject(j).getJSONObject("attributes").getLong("OBJECTID"));
                        planPointTemp.setGeometry(geo.getJSONObject(j).getString("geometry"));
                        planPointTemp.setPipeLength(geo.getJSONObject(j).getJSONObject("attributes").getString("PIPELENGTH"));
                        if (geo.getJSONObject(j).getJSONObject("attributes").getString("PIPELENGTH") != null) {
                            pipeLengthTotal = pipeLengthTotal.add(new BigDecimal(geo.getJSONObject(j).getJSONObject("attributes").getString("PIPELENGTH")));
                        }
                        paths = geo.getJSONObject(j).getJSONObject("geometry").getJSONArray("paths").getJSONArray(0);
                        planPointTemp.setId(tf_plan_point_tempId);
                        pointTempList.add(planPointTemp);
                        tf_plan_point_tempId++;

                        //巡检线信息存入
                        for (int k = 0; k < paths.size(); k++) {
                            TfPlanPathTemp tfPlanPathTemp = new TfPlanPathTemp();
                            tfPlanPathTemp.setTempId(planPointTemp.getId());
                            tfPlanPathTemp.setPointId(planPointTemp.getPointId());
                            tfPlanPathTemp.setLat(new BigDecimal(paths.getJSONArray(k).get(1).toString()));
                            tfPlanPathTemp.setLng(new BigDecimal(paths.getJSONArray(k).get(0).toString()));
                            tfPlanPathTemp.setIsover(new BigDecimal(0));
                            tfPlanPathTemp.setNum(new BigDecimal(k + 1));
                            tfPlanPathTemp.setChildPlanId(planChild.getId());
                            //planPathTempService.save(tfPlanPathTemp);
                            tfPlanPathTemp.setId(tf_plan_path_tempId);
                            pathTempList.add(tfPlanPathTemp);
                            tf_plan_path_tempId++;
                            pointNum++;
                        }
                    }
                    //planPointTempService.save(planPointTemp);
//                    planPointTemp.setId(tf_plan_point_tempId);
//                    pointTempList.add(planPointTemp);
//                    tf_plan_point_tempId++;
//                    //巡检线信息存入
//                    for (int k = 0; k < paths.size(); k++) {
//                        TfPlanPathTemp tfPlanPathTemp = new TfPlanPathTemp();
//                        tfPlanPathTemp.setTempId(planPointTemp.getId());
//                        tfPlanPathTemp.setPointId(planPointTemp.getPointId());
//                        if(gisUrl == null){
//                            tfPlanPathTemp.setLat(new BigDecimal(paths.getJSONArray(k).get(1).toString()));
//                            tfPlanPathTemp.setLng(new BigDecimal(paths.getJSONArray(k).get(0).toString()));
//                        }
//                        tfPlanPathTemp.setIsover(new BigDecimal(0));
//                        tfPlanPathTemp.setNum(new BigDecimal(k + 1));
//                        tfPlanPathTemp.setChildPlanId(planChild.getId());
//                        //planPathTempService.save(tfPlanPathTemp);
//                        tfPlanPathTemp.setId(tf_plan_path_tempId);
//                        pathTempList.add(tfPlanPathTemp);
//                        tf_plan_path_tempId++;
//                        pointNum++;
//                    }
                }
                //添加巡检点数
                planChildService.update(Wrappers.<PlanChild>lambdaUpdate()
                        .eq(PlanChild::getId, planChild.getId())
                        .set(PlanChild::getPointNum, pointNum)
                        .set(PlanChild::getTotal, pipeLengthTotal));
            }

        }
        //更新计划状态
        uploadPlanState();

        //处理详细信息，为了防止异步时ID冲突，在这里加入一条最大数据，后面处理的时候删除该数据
//        PlanPointTemp planPointTemp = new PlanPointTemp();
//        planPointTemp.setId(tf_plan_point_tempId);
//        planPointTempService.save(planPointTemp);
//        TfPlanPathTemp tfPlanPathTemp = new TfPlanPathTemp();
//        tfPlanPathTemp.setId(tf_plan_path_tempId);
//        planPathTempService.save(tfPlanPathTemp);
//        SpringContextHolder.publishEvent(new PlanDetailEvent(this,pointTempList,pathTempList));
        handler(pointTempList, pathTempList);
        return true;
    }

    public void handler(List<PlanPointTemp> pointTempList, List<TfPlanPathTemp> pathTempList) {
        if (pointTempList.size() > 0 || pathTempList.size() > 0) {
            ExecutorService executorService = Executors.newSingleThreadExecutor();
            executorService.execute(new Runnable() {
                @Override
                public void run() {
                    //为了防止SQL语句超出长度出错，分成几次插入
                    if (pointTempList.size() > 0) {
                        if (pointTempList.size() <= 2000) {
                            customPlanMapper.insertBacthPointTemp(pointTempList);
                        }
                        else{
                            int times = pointTempList.size()/2000;
                            for (int i = 0; i<=times; i++){
                                customPlanMapper.insertBacthPointTemp(pointTempList.subList(i*2000,Math.min((i+1)*2000,pointTempList.size())));
                            }
                        }
                    }
                    if (pathTempList.size() > 0) {
                        if (pathTempList.size() <= 2000) {
                            customPlanMapper.insertBacthPathTemp(pathTempList);
                        }
                        else{
                            int times = pathTempList.size()/2000;
                            for (int i = 0; i<=times; i++){
                                customPlanMapper.insertBacthPathTemp(pathTempList.subList(i*2000,Math.min((i+1)*2000,pathTempList.size())));
                            }
                        }
                    }
                }
            });
        }
    }

    private void handlerTransfer(List<PlanPointTemp> pointTempList, List<TfPlanPathTemp> pathTempList) {
        ExecutorService executorService = Executors.newSingleThreadExecutor();
        executorService.execute(new Runnable() {
            @Override
            public void run() {
                if (pointTempList.size() > 0) {
                    if (pointTempList.size() <= 2000) {
                        customPlanMapper.insertBacthPointTemp(pointTempList);
                    }
                    else{
                        int times = pointTempList.size()/2000;
                        for (int i = 0; i<=times; i++){
                            customPlanMapper.insertBacthPointTemp(pointTempList.subList(i*2000,Math.min((i+1)*2000,pointTempList.size())));
                        }
                    }
                }
                if (pathTempList.size() > 0) {
                    if (pathTempList.size() <= 2000) {
                        customPlanMapper.insertBacthPathTemp(pathTempList);
                    }
                    else{
                        int times = pathTempList.size()/2000;
                        for (int i = 0; i<=times; i++){
                            customPlanMapper.insertBacthPathTemp(pathTempList.subList(i*2000,Math.min((i+1)*2000,pathTempList.size())));
                        }
                    }
                }
            }
        });
    }

    @Transactional
    @Override
    public boolean removeByIds(String ids) {
        for (String id : Arrays.asList(ids.split(","))) {
            Plan plan = planService.getOne(Wrappers.<Plan>lambdaQuery().eq(Plan::getId, id).ge(Plan::getPlanState, "2"));
            if (plan != null) {
                throw new ToflyDeniedException("巡检计划" + id + "正在执行不能删除");
            }
        }
        planService.removeByIds(Arrays.asList(ids.split(",")));
        for (String id : Arrays.asList(ids.split(","))) {
            planChildService.remove(Wrappers.<PlanChild>lambdaQuery().eq(PlanChild::getMainPlanId, id));
            List<PlanChild> list = planChildService.list(Wrappers.<PlanChild>lambdaQuery().eq(PlanChild::getMainPlanId, id));
            for (PlanChild child : list) {
            	planPointTempService.remove(Wrappers.<PlanPointTemp>lambdaQuery().eq(PlanPointTemp::getChildPlanId, child.getId()));
                planPathTempService.remove(Wrappers.<TfPlanPathTemp>lambdaQuery().eq(TfPlanPathTemp::getChildPlanId, child.getId()));
            }
        }
        return true;
    }

    @Override
    public Page<Map> getPlanPage(Page page, Plan plan) {
        return page.setRecords(customPlanMapper.getPlanPage(page, plan));
    }

    @Override
    public Page<Map> getPlanChildPage(Page page, ChildPlanSearch childPlanSearch) {
        List<String> list = new ArrayList<>();
        if (childPlanSearch.getChildPlanState() != null) {
            list = Arrays.asList(childPlanSearch.getChildPlanState().split(","));
        }
        if (childPlanSearch.getIsToday() != null && childPlanSearch.getIsToday()) {
            childPlanSearch.setChildBeginDate(Date8Util.getStartTime());
            childPlanSearch.setChildEndDate(Date8Util.getEndTime());
        }
        return page.setRecords(customPlanMapper.getPlanChildPage(page, childPlanSearch, list));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean removeChildPlanByIds(String ids) {
        for (String id : Arrays.asList(ids.split(","))) {            
        	planPointTempService.remove(Wrappers.<PlanPointTemp>lambdaQuery().eq(PlanPointTemp::getChildPlanId, id));
            planPathTempService.remove(Wrappers.<TfPlanPathTemp>lambdaQuery().eq(TfPlanPathTemp::getChildPlanId, id));
//            if ("3".equals(planChild.getChildPlanState())) {
//                throw new ToflyDeniedException("巡检子计划" + planChild.getId().toString() + "已完成不允许删除");
//            } else if ("4".equals(planChild.getChildPlanState())) {
//                throw new ToflyDeniedException("巡检子计划" + planChild.getId().toString() + "已超期不允许删除");
//            } else if ("5".equals(planChild.getChildPlanState()) || "6".equals(planChild.getChildPlanState())) {
//                throw new ToflyDeniedException("巡检子计划" + planChild.getId().toString() + "已提交转移申请不允许删除");
//            } else if ("7".equals(planChild.getChildPlanState()) || "5".equals(planChild.getChildPlanState())) {
//                throw new ToflyDeniedException("巡检子计划" + planChild.getId().toString() + "已审核不允许删除");
//            } else {
//                throw new ToflyDeniedException("巡检子计划" + planChild.getId().toString() + "不允许删除");
//            }
            //            if (planChild != null) {
//                inPlan.add(planChild);
//                s = s + ',' + planChild.getId().toString();
//            }
        }
//        if (inPlan.size() > 0) {
//            throw new ToflyDeniedException("巡检子计划" + s + "正在执行不能删除");
//        }
        return planChildService.removeByIds(Arrays.asList(ids.split(",")));
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean transferPlan(String ids, String applyNote) {
        //子计划状态判断
        List<PlanChild> list = new ArrayList<>();
        String msg = new String();
        for (String id : Arrays.asList(ids.split(","))) {
            PlanChild child = planChildService.getOne(Wrappers.<PlanChild>lambdaQuery()
                    .eq(PlanChild::getId, id)
                    .ge(PlanChild::getChildPlanState, "3"));
            if (child != null) {
                list.add(child);
                msg = msg + child.getId() + ',';
            }
        }
        if (list.size() > 0) {
            throw new ToflyDeniedException("巡检子计划" + msg + "正在执行不能转移");
        }
        //更新子计划状态和添加申请转移记录
        for (String id : Arrays.asList(ids.split(","))) {
            String planState = planChildService.getById(Long.valueOf(id)).getChildPlanState();
            String newState = "5";
            if ("2".equals(planState)) {
                newState = "6";
            }
            planChildService.update(Wrappers.<PlanChild>lambdaUpdate()
                    .eq(PlanChild::getId, id)
                    .set(PlanChild::getChildPlanState, newState));
            PlanTransfer planTransfer = new PlanTransfer();
            planTransfer.setChildPlanId(Long.parseLong(id));
            planTransfer.setApplyUserid(SecurityUtils.getUserId());
            planTransfer.setApplyNote(applyNote);
            planTransfer.setCheckState((short) 2);
            planTransferService.save(planTransfer);
        }
        return true;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public boolean checkTransferPlan(CheckTransferPlan checkTransferPlan) {
        checkTransferPlan.setCheckUser(SecurityUtils.getUserId());
        for (String transId : Arrays.asList(checkTransferPlan.getIds().split(","))) {
            PlanTransfer pt = planTransferService.getById(transId);
            Long childPlanId = pt.getChildPlanId();
            if (pt.getCheckState() != 2) {
                throw new ToflyDeniedException("当前记录不为待审核状态");
            }
            PlanChild planChild = planChildService.getOne(Wrappers.<PlanChild>lambdaQuery().eq(PlanChild::getId, childPlanId));
            if (planChild == null) {
                throw new ToflyDeniedException("未查询到该计划");
            }
            List<String> ls = Arrays.asList("5", "6");
            if (!ls.contains((planChild.getChildPlanState()))) {
                throw new ToflyDeniedException("此计划不是申请转移状态，无法审核");
            }
            //更新申请记录
            pt.setCheckState(checkTransferPlan.getCheckState());
            pt.setCheckUser(checkTransferPlan.getCheckUser());
            Calendar time = Calendar.getInstance();
            Date now = time.getTime();
            pt.setCheckTime(now);
            int compareTo = now.compareTo(planChild.getChildEndDate());
            if (compareTo == 1) {
                throw new ToflyDeniedException("计划已过期，不能进行转移");
            }
            int compareTo2 = now.compareTo(planChild.getChildBeginDate());
            if (checkTransferPlan.getCheckNote() != null) {
                pt.setCheckNote(checkTransferPlan.getCheckNote());
            }
            //审核通过
            if (checkTransferPlan.getCheckState() == 1) {

                transfer_tf_plan_point_tempId = Math.max(transfer_tf_plan_point_tempId, customPlanMapper.getPlanTempMaxId("TF_PLAN_POINT_TEMP"));
                transfer_tf_plan_path_tempId = Math.max(transfer_tf_plan_path_tempId, customPlanMapper.getPlanTempMaxId("TF_PLAN_PATH_TEMP"));

                List<PlanPointTemp> pointTempList = new ArrayList<>();
                List<TfPlanPathTemp> pathTempList = new ArrayList<>();
                if (checkTransferPlan.getReplaceUserid() == null) {
                    throw new ToflyDeniedException("审核通过需添加被指派人ReplaceUserid");
                }
                pt.setReplaceUserid(checkTransferPlan.getReplaceUserid());
                Long dept = otherMapper.getUserDept(checkTransferPlan.getReplaceUserid());
                //添加新的主计划
                Plan plan = planService.getOne(Wrappers.<Plan>lambdaQuery().eq(Plan::getId, planChild.getMainPlanId()));
                plan.setCheckUser(checkTransferPlan.getCheckUser());
                plan.setIsFindTrouble("0");
                plan.setCreateTime(now);
                plan.setCompanyid(dept);
                plan.setIsvalid("0");
                plan.setIsread("0");
                plan.setUserId(checkTransferPlan.getReplaceUserid());
                plan.setCheckUser(checkTransferPlan.getCheckUser());
                plan.setPlanBegindate(planChild.getChildBeginDate());
                plan.setCreateTime(now);
                //当前计划状态情况
                if (compareTo <= 0 && compareTo2 >= 0) {
                    plan.setPlanState("2");
                } else {
                    plan.setPlanState("1");
                }
                planService.save(plan);
                //修改原计划状态
                planChildService.update(Wrappers.<PlanChild>lambdaUpdate()
                        .eq(PlanChild::getId, planChild.getId())
                        .set(PlanChild::getChildPlanState, "7"));
                //添加新的子计划
                planChild.setChildPlanState(plan.getPlanState());
                planChild.setMainPlanId(plan.getId());
                planChild.setCompanyid(plan.getCompanyid());
                planChild.setUserId(plan.getUserId());
                planChildService.save(planChild);
                //复制子计划对应的巡检点临时表信息
                //如果巡检类型为线，还需复制巡检线临时表
                String type = planPointTypeService.getOne(Wrappers.<PlanPointType>lambdaQuery()
                        .eq(PlanPointType::getId, plan.getDeviceTypeId()))
                        .getType();
                List<PlanPointTemp> pointTemps = planPointTempService.list(Wrappers.<PlanPointTemp>lambdaQuery().eq(PlanPointTemp::getChildPlanId, childPlanId));
                //计划检查类型
                SysDeviceType sysDeviceType = sysDeviceTypeService.getOne(Wrappers.<SysDeviceType>lambdaQuery()
                        .apply("point_type_id = (select c.device_type_id from TF_PLAN_CHILD c where id = {0} )", childPlanId));
                String check = "";
                if (sysDeviceType != null && sysDeviceType.getCheckContent() != null) {
                    check = sysDeviceType.getCheckContent();
                }
                //异步转移计划点的检查内容
                tranferMaintence(check, childPlanId, planChild.getId());

                for (PlanPointTemp t : pointTemps) {
                    Long pointId = t.getId();
                    t.setPlanId(planChild.getMainPlanId());
                    t.setChildPlanId(planChild.getId());
                    t.setUserId(checkTransferPlan.getReplaceUserid());
                    t.setCompanyid(plan.getCompanyid());
//                    planPointTempService.save(t);
                    t.setId(transfer_tf_plan_point_tempId);

                    //检查内容转移

                    transfer_tf_plan_point_tempId++;
                    pointTempList.add(t);
                    if ("1".equals(type)) {
                        List<TfPlanPathTemp> tfPlanPathTemps = planPathTempService.list(Wrappers.<TfPlanPathTemp>lambdaQuery()
                                .eq(TfPlanPathTemp::getChildPlanId, childPlanId).eq(TfPlanPathTemp::getTempId, pointId));
                        for (TfPlanPathTemp ppt : tfPlanPathTemps) {
                            //tempId为Piont_temp的ID
                            ppt.setTempId(t.getId());
                            ppt.setChildPlanId(planChild.getId());
//                            planPathTempService.save(ppt);
                            ppt.setId(transfer_tf_plan_path_tempId);
                            transfer_tf_plan_path_tempId++;
                            pathTempList.add(ppt);
                        }
                    }
                }
                handlerTransfer(pointTempList, pathTempList);
            } else if (checkTransferPlan.getCheckState() == 0) {
                if (compareTo <= 0 && compareTo2 >= 0) {
                    planChild.setChildPlanState("2");
                } else {
                    planChild.setChildPlanState("1");
                }
                planChildService.updateById(planChild);
            } else {
                throw new ToflyDeniedException("请选择正确的审核状态");
            }
            //更新审核记录
            planTransferService.updateById(pt);
        }
        return true;
    }

    /**
     * 异步转移计划点的检查内容
     *
     * @param check
     * @param childPlanId
     */
    private void tranferMaintence(String check, Long childPlanId, Long childPlanIdNew) {
        ExecutorService executorService = Executors.newSingleThreadExecutor();
        executorService.execute(new Runnable() {
            @Override
            public void run() {
                List<Long> mainIds = new ArrayList<>();
                Map<Long, Long> idUpdate = new HashMap<>();
                //设备检查项工单，内容，文件
                if ("0".equals(check)) {
                    maintenanceDevice_id = Math.max(maintenanceDevice_id, customPlanMapper.getPlanTempMaxId("TF_MAINTENANCE_DEVICE"));
                    maintenanceDevice_con_id = Math.max(maintenanceDevice_con_id, customPlanMapper.getPlanTempMaxId("TF_MAINTENANCE_DEVICE_CON"));
                    maintenanceDevice_file_id = Math.max(maintenanceDevice_file_id, customPlanMapper.getPlanTempMaxId("TF_MAINTENANCE_DEVICE_FILE"));

                    List<MaintenanceDevice> mds = maintenanceDeviceService.list(Wrappers.<MaintenanceDevice>lambdaQuery()
                            .eq(MaintenanceDevice::getChildPlanId, childPlanId));
                    for (MaintenanceDevice md : mds) {
                        Long oldId = md.getId();
                        mainIds.add(oldId);
                        md.setId(maintenanceDevice_id);
                        md.setChildPlanId(childPlanIdNew);
                        idUpdate.put(oldId, maintenanceDevice_id);
                        maintenanceDevice_id++;
                    }
                    List<MaintenanceDeviceCon> mdcs = maintenanceDeviceConService.list(Wrappers.<MaintenanceDeviceCon>lambdaQuery()
                            .in(mainIds.size() > 0, MaintenanceDeviceCon::getDeviceRecordId, mainIds));
                    for (MaintenanceDeviceCon mdc : mdcs) {
                        Long newId = idUpdate.get(mdc.getDeviceRecordId());
                        mdc.setDeviceRecordId(newId);
                        mdc.setId(maintenanceDevice_con_id);
                        maintenance_con_id++;
                    }
                    List<MaintenanceDeviceFile> mdfs = maintenanceDeviceFileService.list(Wrappers.<MaintenanceDeviceFile>lambdaQuery()
                            .in(mainIds.size() > 0, MaintenanceDeviceFile::getMaintenanceDeviceId, mainIds));
                    for (MaintenanceDeviceFile mdf : mdfs) {
                        Long newId = idUpdate.get(mdf.getMaintenanceDeviceId());
                        mdf.setMaintenanceDeviceId(newId);
                        mdf.setId(maintenanceDevice_file_id);
                        maintenanceDevice_file_id++;
                    }
                    maintenanceDeviceService.saveBatch(mds);
                    maintenanceDeviceConService.saveBatch(mdcs);
                    maintenanceDeviceFileService.saveBatch(mdfs);
                    //调压箱检查项工单，内容，文件
                } else if ("1".equals(check)) {
                    maintenance_id = Math.max(maintenance_id, customPlanMapper.getPlanTempMaxId("TF_MAINTENANCE"));
                    maintenance_con_id = Math.max(maintenance_con_id, customPlanMapper.getPlanTempMaxId("TF_MAINTENANCE_CONTENT"));
                    maintenance_file_id = Math.max(maintenance_file_id, customPlanMapper.getPlanTempMaxId("TF_MAINTENANCE_FILE"));

                    List<Maintenance> ms = maintenanceService.list(Wrappers.<Maintenance>lambdaQuery()
                            .eq(Maintenance::getChildPlanId, childPlanId));
                    for (Maintenance m : ms) {
                        Long oldId = m.getId();
                        mainIds.add(oldId);
                        m.setId(maintenance_id);
                        m.setChildPlanId(childPlanIdNew);
                        idUpdate.put(oldId, maintenance_id);
                        maintenance_id++;
                    }
                    List<MaintenanceContent> mcs = maintenanceContentService.list(Wrappers.<MaintenanceContent>lambdaQuery()
                            .in(mainIds.size() > 0, MaintenanceContent::getRecordId, mainIds));
                    for (MaintenanceContent mc : mcs) {
                        Long newId = idUpdate.get(mc.getRecordId());
                        mc.setRecordId(newId);
                        mc.setId(maintenance_con_id);
                        maintenance_con_id++;
                    }
                    List<MaintenanceFile> mfs = maintenanceFileService.list(Wrappers.<MaintenanceFile>lambdaQuery()
                            .in(mainIds.size() > 0, MaintenanceFile::getMaintenanceId, mainIds));
                    for (MaintenanceFile mf : mfs) {
                        Long newId = idUpdate.get(mf.getMaintenanceId());
                        mf.setMaintenanceId(newId);
                        mf.setId(maintenance_file_id);
                        maintenance_file_id++;
                    }
                    maintenanceService.saveBatch(ms);
                    maintenanceContentService.saveBatch(mcs);
                    maintenanceFileService.saveBatch(mfs);
                }

            }
        });
    }

    @Override
    public Page<Map> getTransferPage(Page page, TransferSearch transferSearch) {
        List<String> list = transferSearch.getStates() != null ? Arrays.asList(transferSearch.getStates().split(",")) : null;
        return page.setRecords(customPlanMapper.getTransferPage(page, transferSearch, list));
    }

    @Override
    public Page<Map> getPlanChildPageByMoblie(Page page, ChildPlanSearch childPlanSearch) {
        List<String> list = new ArrayList<>();
        if (childPlanSearch.getChildPlanState() != null) {
            list = Arrays.asList(childPlanSearch.getChildPlanState().split(","));
        }
        if (childPlanSearch.getIsToday() != null && childPlanSearch.getIsToday()) {
            childPlanSearch.setChildBeginDate(Date8Util.getStartTime());
            childPlanSearch.setChildEndDate(Date8Util.getEndTime());
        }
//        Page result =  page.setRecords(customPlanMapper.getPlanChildPageByMoblie(page, childPlanSearch, list));
//        List<CustomPlanChildByMoblie> records = (List<CustomPlanChildByMoblie>) result.getRecords();
//        for (CustomPlanChildByMoblie i:records){
//            JSONArray pointIofo =  JSONArray.parseArray(i.getPointInfo());
//            if (pointIofo != null){
//                i.setPointNum(pointIofo.size());
//            }
//            i.setPointInfo(null);
//        }
//        result.setRecords(records);
        return page.setRecords(customPlanMapper.getPlanChildPageByMoblie(page, childPlanSearch, list));
    }

    @Override
    public List<Map> getPlanPoint(PlanPointSearch planPointSearch) {
//        planPointSearch.setUserId(SecurityUtils.getUserId());
        if (planPointSearch.getSearchDate() == null) {
            planPointSearch.setSearchDate(Date8Util.getCurrentDate());
        }
        //planChildId批量查询
        List<String> list = new ArrayList<>();
        if (StringUtil.isNotEmpty(planPointSearch.getPlanChildIds())) {
            list = Arrays.asList(planPointSearch.getPlanChildIds().split(","));
        }
        return customPlanMapper.getPlanPoint(planPointSearch, list);
    }


    @Override
    public Page<Map> getPlanPointPage(Page page,PlanPointSearch planPointSearch) {
//        if (planPointSearch.getSearchDate() == null) {
//            planPointSearch.setSearchDate(Date8Util.getCurrentDate());
//        }
        //planChildId批量查询
        List<String> list = new ArrayList<>();
        if (StringUtil.isNotEmpty(planPointSearch.getPlanChildIds())) {
            list = Arrays.asList(planPointSearch.getPlanChildIds().split(","));
        }
        return page.setRecords(customPlanMapper.getPlanPointPage(page,planPointSearch, list));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void uploadPlanState() {
        Date now = Date8Util.getCurrentDate();
        planChildService.update(Wrappers.<PlanChild>lambdaUpdate()
                .le(PlanChild::getChildBeginDate, now)
                .ge(PlanChild::getChildEndDate, now)
                .eq(PlanChild::getChildPlanState, "1")//计划状态未开始
                .set(PlanChild::getChildPlanState, "2"));//变为执行中
        planService.update(Wrappers.<Plan>lambdaUpdate()
                .le(Plan::getPlanBegindate, now)
                .ge(Plan::getPlanEnddate, now)
                .eq(Plan::getPlanState, "1")//计划状态未开始
                .set(Plan::getPlanState, "2"));//变为执行中
    }

    @Override
    public List<Map> getPointNum(PointNumSearch pointNumSearch) {
        List<String> list = null;
        if (StringUtil.isNotEmpty(pointNumSearch.getPlanChildIds())) {
            list = Arrays.asList(pointNumSearch.getPlanChildIds().split(","));
        }
        pointNumSearch.setUserId(SecurityUtils.getUserId());
        return customPlanMapper.getPointNum(pointNumSearch, list);
    }

    @Override
    public List<Map> getChildPlanInfo(Long childPlanId) {
        return customPlanMapper.getChildPlanInfo(childPlanId);
    }


    private void checkUpdate(List idList) {
        Collection<PlanPoint> collection = planPointService.listByIds(idList);
        String msg = collection.stream().filter(gp -> "1".equals(gp.getIsPlan()))
                .map(PlanPoint::getPointName).collect(Collectors.joining(","));
        if (StringUtil.isNotEmpty(msg)) {
            throw new ToflyDeniedException("存在着计划不允许修改的数据" + msg);
        }
    }

    /**
     * 向指定 URL 发送POST方法的请求
     *
     * @param url    发送请求的 URL
     * @param string 请求参数，请求参数应该是 name1=value1&name2=value2 的形式。
     * @return 所代表远程资源的响应结果
     */
    private String sendPost(String url, String string) {
        PrintWriter out = null;
        BufferedReader in = null;
        String result = "";
        try {
            URL realUrl = new URL(url);
            // 打开和URL之间的连接
            URLConnection conn = realUrl.openConnection();
            // 设置通用的请求属性
            conn.setRequestProperty("accept", "*/*");
            conn.setRequestProperty("connection", "Keep-Alive");
            conn.setRequestProperty("user-agent",
                    "Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1;SV1)");
            // 发送POST请求必须设置如下两行
            conn.setDoOutput(true);
            conn.setDoInput(true);
            // 获取URLConnection对象对应的输出流
            out = new PrintWriter(conn.getOutputStream());
            // 发送请求参数
            out.print(string);
            // flush输出流的缓冲
            out.flush();
            // 定义BufferedReader输入流来读取URL的响应
            in = new BufferedReader(
                    new InputStreamReader(conn.getInputStream()));
            String line;
            while ((line = in.readLine()) != null) {
                result += line;
            }
        } catch (Exception e) {
            System.out.println("发送 POST 请求出现异常！" + e);
            e.printStackTrace();
        }
        //使用finally块来关闭输出流、输入流
        finally {
            try {
                if (out != null) {
                    out.close();
                }
                if (in != null) {
                    in.close();
                }
            } catch (IOException ex) {
                ex.printStackTrace();
            }
        }
        return result;
    }

    public static int getTimeDistance(Date beginDate, Date endDate) {
        Calendar beginCalendar = Calendar.getInstance();
        beginCalendar.setTime(beginDate);
        Calendar endCalendar = Calendar.getInstance();
        endCalendar.setTime(endDate);
        long beginTime = beginCalendar.getTime().getTime();
        long endTime = endCalendar.getTime().getTime();
        int betweenDays = (int) ((endTime - beginTime) / (1000 * 60 * 60 * 24));//先算出两时间的毫秒数之差大于一天的天数

        endCalendar.add(Calendar.DAY_OF_MONTH, -betweenDays);//使endCalendar减去这些天数，将问题转换为两时间的毫秒数之差不足一天的情况
        endCalendar.add(Calendar.DAY_OF_MONTH, -1);//再使endCalendar减去1天
        if (beginCalendar.get(Calendar.DAY_OF_MONTH) == endCalendar.get(Calendar.DAY_OF_MONTH)) {//比较两日期的DAY_OF_MONTH是否相等
            return betweenDays + 1;    //相等说明确实跨天了
        } else {
            return betweenDays + 0;//不相等说明确实未跨天
        }
    }

}
