package com.tofly.dyrq.custom.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.dyrq.custom.entity.CustomScada;
import com.tofly.dyrq.custom.entity.ScadaConfig;
import com.tofly.dyrq.custom.entity.ScadaStationResult;
import com.tofly.dyrq.custom.mapper.CustomScadaMapper;
import com.tofly.dyrq.custom.service.CustomScadaService;
import com.tofly.dyrq.entity.ScadaRealdata;
import com.tofly.dyrq.entity.ScadapntsDes;
import com.tofly.dyrq.service.ScadaRealdataService;
import com.tofly.dyrq.service.ScadapntsDesService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.*;

@Service
@AllArgsConstructor
@Slf4j
public class CustomScadaServiceImpl  extends ServiceImpl<CustomScadaMapper, CustomScada> implements CustomScadaService {
    private final ScadapntsDesService scadapntsDesService;
    private final ScadaRealdataService scadaRealdataService;
    private final CustomScadaMapper customScadaMapper;


    @Override
    public Page<Map> getStation(Page page, String filter) {
        return page.setRecords(baseMapper.getStation(page, filter));
    }

    @Override
    public CustomScada getStationById(String ptid) {
        return baseMapper.getStationById(ptid);
    }

    @Override
    public List<Map> getStationByList(List<String> list) {
        return baseMapper.getStationByList(list);
    }

    @Override
    public Page<Map> getAlarmConfigByList(Page page,List<String> list) {return page.setRecords(baseMapper.getAlarmConfigByList(page,list));}

    @Override
    public List<Map> getAlarmInfo() {return baseMapper.getAlarmInfo();}

    @Override
    public Boolean updateByName(List<ScadapntsDes> list) {
        for (ScadapntsDes scadapntsDes : list) {
            scadapntsDesService.update(Wrappers.<ScadapntsDes>lambdaUpdate()
                    .eq(ScadapntsDes::getName, scadapntsDes.getName())
                    .set(ScadapntsDes::getMaxval, scadapntsDes.getMaxval())
                    .set(ScadapntsDes::getMinval, scadapntsDes.getMinval())
            );
        }
        return true;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean updateRealData(List<ScadaRealdata> scadaRealdataList) {
        //清空表后更新值和数据

        List<ScadaRealdata> oldData = scadaRealdataService.list();
        Map oldTag = new HashMap();
        for (int i=0;i<oldData.size();i++){
            oldTag.put(oldData.get(i).getTag(),i);
        }
        //判断参数是否存在，更新or插入
        List<ScadaRealdata> insertData = new ArrayList<>();
        List<ScadaRealdata> updateData = new ArrayList<>();
        for (ScadaRealdata scadaRealdata:scadaRealdataList){
             String tag = scadaRealdata.getTag();
             if (oldTag.containsKey(tag)){
                 updateData.add(scadaRealdata);
             }
             else {
                 insertData.add(scadaRealdata);
             }
        }
        if (insertData.size() > 0){
            customScadaMapper.saveBatch(insertData);
        }
        if (updateData.size() > 0){
            customScadaMapper.updateBatch(updateData);
        }
        return true;
    }

    @Override
    public List<ScadaStationResult> getRealData(String searchText,String ptIds) {
        List<String> ptidList = new ArrayList<>();
        if (StringUtil.isNotEmpty(ptIds)){
            ptidList = Arrays.asList(ptIds.split(","));
        }
        List<ScadaStationResult> list = customScadaMapper.getRealData(searchText,ptidList);
//        List<ScadaStationResult> list = customScadaStationMapper.getRealData(searchText);
        return list;
    }

    @Override
    @Transactional
    public Boolean updateConfig(List<ScadaConfig> scadaConfigs) {
        customScadaMapper.updateConfig(scadaConfigs);
        return true;
    }


}
