package com.tofly.dyrq.custom.service.impl;

import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.tofly.common.core.util.FtpUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.dyrq.config.FilePathBean;
import com.tofly.dyrq.custom.entity.CustomTfPda;
import com.tofly.dyrq.custom.mapper.CustomTfPdaMapper;
import com.tofly.dyrq.custom.service.CustomTfPdaService;
import com.tofly.dyrq.service.TfPdaResponseFileService;
import com.tofly.dyrq.service.TfPdaTaskResponseService;
import com.tofly.dyrq.entity.TfPdaResponseFile;
import com.tofly.dyrq.entity.TfPdaTask;
import com.tofly.dyrq.entity.TfPdaTaskResponse;
import com.tofly.dyrq.service.TfPdaTaskService;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.MultiValueMap;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.Date;
import java.util.List;

import static org.springframework.util.ObjectUtils.isEmpty;

@Service("customTfPdaService")
@AllArgsConstructor
@Slf4j
public class CustomTfPdaServiceImpl extends ServiceImpl<CustomTfPdaMapper, CustomTfPda> implements CustomTfPdaService {

    private TfPdaTaskService tfPdaTaskService;
    private TfPdaTaskResponseService tfPdaTaskResponseService;
    private TfPdaResponseFileService tfPdaResponseFileService;
    private final FilePathBean filePathBean;


    @Override
    public Page<CustomTfPda> getTaskPage(Page page, CustomTfPda customTfPda) {
        return page.setRecords(baseMapper.getTaskPage(page, customTfPda));
    }

    @Override
    public CustomTfPda gettfPdaTaskDetail(Long id) {
        return baseMapper.gettfPdaTaskDetail(id);
    }

    @Override
    @Transactional
    @SneakyThrows
    public Boolean saveHandle(HttpServletRequest request, TfPdaTaskResponse tfPdaTaskResponse) {
        Long taskid = tfPdaTaskResponse.getTaskid();
        //更新时间
        LocalDateTime localDateTime = LocalDateTime.now();
        tfPdaTaskResponse.setFinishDate(Date.from(localDateTime.atZone(ZoneId.systemDefault()).toInstant()));

        List<TfPdaTask> list = tfPdaTaskService.list(Wrappers.<TfPdaTask>lambdaQuery()
                .select(TfPdaTask::getTaskStatus)
                .eq(TfPdaTask::getId, taskid)
                .eq(TfPdaTask::getTaskStatus, Long.valueOf(1)));
        if (list.size() > 0) {
            return false;
//            throw new ToflyDeniedException("工单已处理，不能提交反馈");
        }
        if (isEmpty(taskid)) {
            throw new ToflyDeniedException("taskid不能为空");
        }
        tfPdaTaskResponseService.save(tfPdaTaskResponse);
        Long resId = tfPdaTaskResponse.getId();
        //form-data一个key对应多个文件
//        MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
//        List<MultipartFile> files = multipartRequest.getFiles("file");
        //form-data 遍历所有文件
        MultiValueMap<String, MultipartFile> multiFileMap = ((MultipartHttpServletRequest) request).getMultiFileMap();
        multiFileMap.forEach((k, v) -> {
            String filename = filePathBean.getPdafile() + IdWorker.getIdStr() + v.get(0).getOriginalFilename().substring(v.get(0).getOriginalFilename().lastIndexOf("."));
            boolean b = FtpUtil.uploadFile2Ftp(v.get(0), filename);
            TfPdaResponseFile tfPdaResponseFile = new TfPdaResponseFile();
            tfPdaResponseFile.setFileid(filename);
            tfPdaResponseFile.setResponseid(resId);
            tfPdaResponseFileService.save(tfPdaResponseFile);
        });
        TfPdaTask tfPdaTask = new TfPdaTask();
        tfPdaTask.setTaskStatus(new Integer(1));
        tfPdaTask.setId(taskid);
        tfPdaTaskService.updateById(tfPdaTask);
        return true;
    }
}
