package com.tofly.dyrq.custom.service.impl;

import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrq.custom.entity.*;
import com.tofly.dyrq.custom.mapper.CustomTfPdaSignMapper;
import com.tofly.dyrq.custom.service.CustomTfPdaSignService;
import com.tofly.dyrq.custom.service.DyrqGpsService;
import com.tofly.dyrq.entity.*;
import com.tofly.dyrq.service.*;
import lombok.AllArgsConstructor;
import lombok.SneakyThrows;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;

import java.math.BigDecimal;
import java.time.DateTimeException;
import java.util.*;

@Service
@AllArgsConstructor
public class CustomTfPdaSignServiceImpl implements CustomTfPdaSignService {
    private final TfPdaSignService tfPdaSignService;
    private final DeviceUploadService deviceUploadService;
    private final PdaUserDeviceService pdaUserDeviceService;
    private final TfPdaUserService tfPdaUserService;
    private final PlanChildService planChildService;

    private final DyrqGpsService dyrqGpsService;
    private final CustomTfPdaSignMapper customTfPdaSignMapper;

    @Override
    @SneakyThrows
    @Transactional
    public boolean signSave(AddSign addSign) {
        Long userId = SecurityUtils.getUserId();
        //设备日志表验证
        EsPdaDeviceUpload esPdaDeviceUpload = addSign.getDevice();
        if (addSign.getDevice().getNetworkState() == null) {
            throw new ToflyDeniedException("networkState不能为空");
        }
        if (addSign.getDevice().getGpsStar() == null) {
            throw new ToflyDeniedException("gpsStar不能为空");
        }
        if (addSign.getDevice().getGpsState() == null) {
            throw new ToflyDeniedException("gpsState不能为空");
        }
        if (addSign.getDevice().getElectricity() == null) {
            throw new ToflyDeniedException("electricity不能为空");
        }
        if (addSign.getDevice().getCheckTime() == null) {
            throw new ToflyDeniedException("checkTime不能为空");
        }

        //签到表验证
        Calendar cal = Calendar.getInstance();
        Date now = cal.getTime();
        cal.set(Calendar.HOUR_OF_DAY, 0);
        cal.set(Calendar.MINUTE, 0);
        cal.set(Calendar.SECOND, 0);
        Date startDate = cal.getTime();
        cal.set(Calendar.HOUR_OF_DAY, 23);
        cal.set(Calendar.MINUTE, 59);
        cal.set(Calendar.SECOND, 59);
        Date endDate = cal.getTime();
        List<TfPdaSign> list = tfPdaSignService.list(Wrappers.<TfPdaSign>lambdaQuery()
                .eq(TfPdaSign::getUserid, userId)
                .between(TfPdaSign::getSignTime, startDate, endDate)
                .orderByDesc(TfPdaSign::getSignTime));
        if (list.size() > 0) {
            TfPdaSign lastest = list.get(0);
            if (addSign.getSignType().equals(lastest.getSignType()) && "1".equals(addSign.getSignType())) {
                throw new ToflyDeniedException("签到后不能再次签到");
            }
            if (addSign.getSignType().equals(lastest.getSignType()) && "2".equals(addSign.getSignType())) {
                throw new ToflyDeniedException("签退后不能再次签退");
            }
        }
        //签到表赋值赋值
        addSign.setUserid(userId);
        PdaUserDevice pdaUserDevice = pdaUserDeviceService.getOne(Wrappers.<PdaUserDevice>lambdaQuery()
                .eq(PdaUserDevice::getUserId, userId));
        if (pdaUserDevice == null) {
            throw new ToflyDeniedException("该人员未绑定设备");
        }
        addSign.setPadid(pdaUserDevice.getDeviceId());
        addSign.setCompanyid(SecurityUtils.getUserDeptId());
        addSign.setSignTime(now);
        //签退后用户应离线
//        if ("2".equals(addSign.getSignType())) {
//            TfPdaUser tfPdaUser = tfPdaUserService.getOne(Wrappers.<TfPdaUser>lambdaQuery().eq(TfPdaUser::getUserid, userId));
//            if (tfPdaUser != null) {
//                tfPdaUser.setOnlineState(new BigDecimal(0));
//                tfPdaUser.setPosttime(now);
//            } else {
//                throw new ToflyDeniedException("巡检系统用户表中未查询到该用户信息");
//            }
//        }

        //日志表赋值
        esPdaDeviceUpload.setUploadtime(now);
        esPdaDeviceUpload.setUserid(userId);
        esPdaDeviceUpload.setCheckTime(addSign.getDevice().getCheckTime());
        if ("1".equals(addSign.getSignType())) {
            //上班   工作状态 0：开机 1：上班  2：工作中  3：下班  4:异常
            esPdaDeviceUpload.setWorkState(new BigDecimal(1));
        } else if ("2".equals(addSign.getSignType())) {
            //下班
            esPdaDeviceUpload.setWorkState(new BigDecimal(3));
        } else {
            esPdaDeviceUpload.setWorkState(new BigDecimal(2));
        }
        tfPdaSignService.save(addSign);
        if(dyrqGpsService.save(esPdaDeviceUpload).getCode() != 1){
            throw new ToflyDeniedException("es存储设备上传信息失败");
        };
        return true;
    }

    @Override
    public TfPdaSignVO getSignVO(SignPlanSearch signPlanSearch) {
        List<PlanChild> getPlanChilds = planChildService.list(Wrappers.<PlanChild>lambdaQuery()
                .eq(PlanChild::getUserId, signPlanSearch.getUserId()));
                /*.gt(PlanChild::getChildBeginDate,endDate)
                .lt(PlanChild::getChildEndDate,beginDate))*/;
        List<Long> getPlanChildIds = new ArrayList<Long>();
        for (int i = 0; i < getPlanChilds.size(); i++) {
            if (getPlanChilds.get(i) != null && "1".equals(getPlanChilds.get(i).getDelFlag())) {
                if (!(signPlanSearch.getBeginDate().getTime() > getPlanChilds.get(i).getChildEndDate().getTime() ||
                        signPlanSearch.getEndDate().getTime() < getPlanChilds.get(i).getChildBeginDate().getTime())) {
                    getPlanChildIds.add(getPlanChilds.get(i).getId());
                }
            }
        }
        TfPdaSignVO tfPdaSignVO = new TfPdaSignVO();
        tfPdaSignVO.setPlanChildIds(getPlanChildIds);
        tfPdaSignVO.setPlanChildCounts(getPlanChildIds.size());
        //int sections = tfPdaSignService.count(Wrappers.lambdaQuery().eq(TfPdaSign::getUserid, tfPdaSign.getUserid()));
        List<TfPdaSign> getTfPdaSigns = tfPdaSignService.list(Wrappers.<TfPdaSign>lambdaQuery()
                .between(TfPdaSign::getSignTime, signPlanSearch.getBeginDate(), signPlanSearch.getEndDate())
                .eq(TfPdaSign::getUserid, signPlanSearch.getUserId())
                .orderByAsc(TfPdaSign::getSignTime));
        if (getTfPdaSigns.size() != 0) {
            if ("1".equals(getTfPdaSigns.get((getTfPdaSigns.size() - 1)).getSignType())) {
                /*if (getTfPdaSigns.size() == 1) {
                    tfPdaSignVO.setSections(0);
                } else {*/
                tfPdaSignVO.setSections((int) Math.floor(getTfPdaSigns.size() / 2.0) + 1);
                //}
            } else if ("2".equals(getTfPdaSigns.get((getTfPdaSigns.size() - 1)).getSignType())) {
                tfPdaSignVO.setSections((int) Math.floor(getTfPdaSigns.size() / 2.0));
            }
        } else {
            tfPdaSignVO.setSections(0);
        }
        List<Map<String, Date>> getSignDate = new ArrayList<>();
        List<TfPdaSign> getSignTime = tfPdaSignService.list(Wrappers.<TfPdaSign>lambdaQuery()
                .between(TfPdaSign::getSignTime, signPlanSearch.getBeginDate(), signPlanSearch.getEndDate())
                .eq(TfPdaSign::getUserid, signPlanSearch.getUserId())
                .eq(TfPdaSign::getSignType, 1)
                .orderByAsc(TfPdaSign::getSignTime));
        List<TfPdaSign> getOutTime = tfPdaSignService.list(Wrappers.<TfPdaSign>lambdaQuery()
                .between(TfPdaSign::getSignTime, signPlanSearch.getBeginDate(), signPlanSearch.getEndDate())
                .eq(TfPdaSign::getUserid, signPlanSearch.getUserId())
                .eq(TfPdaSign::getSignType, 2)
                .orderByAsc(TfPdaSign::getSignTime));
        if (getSignTime.size() > getOutTime.size()) {
            for (int i = 0; i < getOutTime.size(); i++) {
                Map<String, Date> map = new HashMap<>();
                map.put("signOut", getOutTime.get(i).getSignTime());
                map.put("signIn", getSignTime.get(i).getSignTime());
                getSignDate.add(map);
            }
            Map<String, Date> lastMap = new HashMap<>();
            lastMap.put("signOut", new Date());
            lastMap.put("signIn", getSignTime.get((getSignTime.size() - 1)).getSignTime());
            getSignDate.add(lastMap);
        } else if (getSignTime.size() == getOutTime.size()) {
            for (int i = 0; i < getSignTime.size(); i++) {
                Map<String, Date> map = new HashMap<>();
                map.put("signOut", getOutTime.get(i).getSignTime());
                map.put("signIn", getSignTime.get(i).getSignTime());
                getSignDate.add(map);
            }
        } else {
            throw new DateTimeException("签退日期多于签到日期,查询分段不正确");
        }
        tfPdaSignVO.setSignTime(getSignDate);
        return tfPdaSignVO;
    }

    @Override
    public List<Map> getMonthSign(MonthSignSearch monthSignSearch) {
        return customTfPdaSignMapper.getMonthSign(monthSignSearch);
    }
}
