package com.tofly.dyrq.custom.service.impl;

import com.alibaba.fastjson.JSONArray;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.util.Date8Util;
import com.tofly.common.core.util.FtpUtil;
import com.tofly.common.core.util.StringUtil;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrq.config.ComConstans;
import com.tofly.dyrq.config.FilePathBean;
import com.tofly.dyrq.custom.entity.*;
import com.tofly.dyrq.custom.mapper.CustomTroubleMapper;
import com.tofly.dyrq.custom.service.BaseInfoService;
import com.tofly.dyrq.custom.service.CustomMessageService;
import com.tofly.dyrq.custom.service.CustomTroubleService;
import com.tofly.dyrq.entity.*;
import com.tofly.dyrq.service.*;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.MultiValueMap;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.servlet.http.HttpServletRequest;
import java.math.BigDecimal;
import java.net.URLDecoder;
import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.*;
import java.util.stream.Collectors;

@Service
@AllArgsConstructor
@Slf4j
public class CustomTroubleServiceImpl implements CustomTroubleService {
    private final CustomTroubleMapper customTroubleMapper;
    private final TroubleService troubleService;
    private final FilePathBean filePathBean;
    private final TroubleFileService troubleFileService;
    private final CustomMessageService customMessageService;
    private final TroublePipeinfoService troublePipeinfoService;
    private final TroubleDeviceinfoService troubleDeviceinfoService;
    private final TroubleAuditSchemeService troubleAuditSchemeService;
    private final TroubleAuditEndService troubleAuditEndService;
    private final TroubleStateService troubleStateService;
    private final TroubleCheckService troubleCheckService;
    private final BaseInfoService baseInfoService;


    @Override
    public Page<Map> getList(Page page, TroubleSearch troubleSearch) {
        Set<Long> userIds = new HashSet<>();
        if (troubleSearch.getUserId() != null) {
            List<Long> onDutyUsers = new ArrayList<>();
            troubleService.list(Wrappers.<Trouble>lambdaQuery().eq(Trouble::getOnDutyUserId, troubleSearch.getUserId()))
                    .forEach(k -> onDutyUsers.add(k.getId()));
            List<Long> undertakerUsers = new ArrayList<>();
            troubleService.list(Wrappers.<Trouble>lambdaQuery().eq(Trouble::getUndertakerUserId, troubleSearch.getUserId()))
                    .forEach(k -> undertakerUsers.add(k.getId()));

            troubleService.list(Wrappers.<Trouble>lambdaQuery().select(Trouble::getId)
                    .eq(Trouble::getUndertakerHandleUserId, troubleSearch.getUserId()))
                    .forEach(t -> userIds.add(t.getId()));
            List<Long> dutyUsers = new ArrayList<>();
            troubleService.list(Wrappers.<Trouble>lambdaQuery().eq(Trouble::getDutyUserId, troubleSearch.getUserId()))
                    .forEach(k -> dutyUsers.add(k.getId()));
            List<Long> gisUsers = new ArrayList<>();
            troubleService.list(Wrappers.<Trouble>lambdaQuery().eq(Trouble::getGisUserId, troubleSearch.getUserId()))
                    .forEach(k -> gisUsers.add(k.getId()));
            List<Long> reportUsers = new ArrayList<>();
            troubleService.list(Wrappers.<Trouble>lambdaQuery().eq(Trouble::getReportUserId, troubleSearch.getUserId()))
                    .forEach(k -> reportUsers.add(k.getId()));
            List<Long> auditorSecretary = new ArrayList<>();
            troubleAuditSchemeService.list(Wrappers.<TroubleAuditScheme>lambdaQuery().eq(TroubleAuditScheme::getAuditorSecretary, troubleSearch.getUserId())
                    .eq(TroubleAuditScheme::getAuditmind, "0")).forEach(k -> auditorSecretary.add(k.getWorkOrderId()));
            List<Long> handleUsers = new ArrayList<>();
            troubleAuditSchemeService.list(Wrappers.<TroubleAuditScheme>lambdaQuery().eq(TroubleAuditScheme::getHandleUserId, troubleSearch.getUserId())
                    .eq(TroubleAuditScheme::getAuditmind, "0")).forEach(k -> handleUsers.add(k.getWorkOrderId()));
            List<Long> coprocessors = new ArrayList<>();
            List<TroubleAuditScheme> list = troubleAuditSchemeService.list(Wrappers.<TroubleAuditScheme>lambdaQuery().eq(TroubleAuditScheme::getAuditmind, "0"));
            for (TroubleAuditScheme t : list) {
                if (StringUtil.isNotEmpty(t.getCoprocessor())) {
                    String[] array = t.getCoprocessor().split(",");
                    for (String t2 : array) {
                        if (Long.parseLong(t2) == troubleSearch.getUserId()) {
                            coprocessors.add(t.getWorkOrderId());
                        }
                    }
                }
            }
            List<Long> auditorUndersecretary = new ArrayList<>();
            troubleAuditEndService.list(Wrappers.<TroubleAuditEnd>lambdaQuery().eq(TroubleAuditEnd::getAuditorUndersecretary, troubleSearch.getUserId())
                    .eq(TroubleAuditEnd::getAuditmind, "0")).forEach(k -> auditorUndersecretary.add(k.getWorkOrderId()));
            userIds.addAll(onDutyUsers);
            userIds.addAll(undertakerUsers);
            userIds.addAll(dutyUsers);
            userIds.addAll(gisUsers);
            userIds.addAll(reportUsers);
            userIds.addAll(auditorSecretary);
            userIds.addAll(handleUsers);
            userIds.addAll(coprocessors);
            userIds.addAll(auditorUndersecretary);
            if (userIds.isEmpty()) {
                userIds.add(Long.MAX_VALUE);
            }
        }
        if (StringUtil.isNotEmpty(troubleSearch.getProcessIds())) {

            troubleSearch.setProcessIdList(Arrays.stream(troubleSearch.getProcessIds().split(","))
                    .map(s -> Long.valueOf(s)).collect(Collectors.toList()));
        }
        List<Long> deptUsers = new ArrayList<>();
        if (troubleSearch.getDeptId() != null) {
            deptUsers = customTroubleMapper.getDeptUsers(troubleSearch.getDeptId());
        }
        troubleSearch.setDeptUsers(deptUsers);
        return page.setRecords(customTroubleMapper.pageByCondition(page, troubleSearch, userIds));
    }

    @Override
    @Transactional
    public Boolean save(HttpServletRequest request, TroubleSave troubleSave) {
        //录入隐患基本信息
        Trouble trouble = new Trouble();
        trouble.setDeptId(troubleSave.getDeptId());
        trouble.setTroubleName(troubleSave.getTroubleName());
        trouble.setUnitName(troubleSave.getUnitName());
//        trouble.setTroubleType(troubeSave.getTroubleType());
        trouble.setAddress(troubleSave.getAddress());
//        trouble.setLevelid(troubeSave.getLevelid());
        trouble.setTroubleInfo(troubleSave.getTroubleInfo());
//      trouble.setUploadType(troubleSave.getUploadType());
//      trouble.setOndutyUserid(troubeSave.getOndutyUserid());
        trouble.setFindUserId(troubleSave.getFindUserid());
        trouble.setFindTime(troubleSave.getFindTime());
        trouble.setNotes(troubleSave.getNotes());
        trouble.setPipeInfoDesc(troubleSave.getPipeInfoDesc());
        trouble.setDeviceInfoDesc(troubleSave.getDeviceInfoDesc());
        //trouble.setGisUserId(troubleSave.getUndersecretaryUserid());
        //trouble.setGisUserId(troubleSave.getSecretaryUserid());
        trouble.setLocationX(troubleSave.getLocationX());
        trouble.setLocationY(troubleSave.getLocationY());
        trouble.setTroubleTypeId(troubleSave.getTroubleTypeId());
        trouble.setLevelId(troubleSave.getLevelId());
        //上报人信息
        trouble.setReportUserId(SecurityUtils.getUserId());
        trouble.setReportTime(Date8Util.getCurrentDate());
        trouble.setUndertakerUserId(troubleSave.getUndertakerUserid());
        //生成隐患编号
        LocalDateTime localDateTime = LocalDateTime.now();
        DateTimeFormatter dtf = DateTimeFormatter.ofPattern("yyyyMMdd");
        int count = troubleService.count(Wrappers.<Trouble>lambdaQuery()
                .between(Trouble::getReportTime, Date8Util.getStartTime(), Date8Util.getEndTime())) + 1;
        String sid = ComConstans.TROUBLE_PREFIX + localDateTime.format(dtf) + String.format("%02d", count);
        trouble.setSid(sid);
        boolean b = true;

        /*//隐患流程
        //开始1：非工作时间
        if ("0".equals(troubleSave.getUploadType())) {
            trouble.setOnDutyUserId(SecurityUtils.getUserId());
            trouble.setProcessId((long) 1);
        }
        //开始2：工作时间现场录入
        else if ("1".equals(troubleSave.getUploadType())) {
            trouble.setProcessId((long) 4);
            trouble.setTroubleTypeId(troubleSave.getTroubleTypeId());
            trouble.setLevelId(troubleSave.getLevelId());
        }
        //开始3：工作时间其他转入
        else {
            trouble.setGisUserId(SecurityUtils.getUserId());
            trouble.setUndertakerUserId(troubleSave.getUndertakerUserid());
            trouble.setProcessId((long) 2);
        }*/
        trouble.setProcessId((long) 1);

        if (troubleSave.getId() == null) {
            if (trouble.getFindTime() == null) {
                trouble.setFindTime(trouble.getReportTime());
            }
            if (trouble.getFindUserId() == null) {
                trouble.setFindUserId(SecurityUtils.getUserId());
            }

            b = troubleService.save(trouble);
        } else if (troubleSave.getId() != null) {
            Long processId = troubleService.getById(troubleSave.getId()).getProcessId();
            if (processId != null && processId > 1) {
                throw new ToflyDeniedException("工单已派工,无法再次修改");
            }
            trouble.setId(troubleSave.getId());
            troubleService.updateById(trouble);
            List<TroublePipeinfo> pipeInfos = troublePipeinfoService.list(Wrappers.<TroublePipeinfo>lambdaQuery().eq(TroublePipeinfo::getTroubleId, troubleSave.getId()));
            List<TroubleDeviceinfo> deviceInfos = troubleDeviceinfoService.list(Wrappers.<TroubleDeviceinfo>lambdaQuery().eq(TroubleDeviceinfo::getTroubleId, troubleSave.getId()));
            List<Long> pipeIds = new ArrayList<>();
            List<Long> deviceIds = new ArrayList<>();
            if (!pipeInfos.isEmpty()) {
                pipeInfos.forEach(k -> {
                    pipeIds.add(k.getId());
                });
                troublePipeinfoService.removeByIds(pipeIds);
            }
            if (!deviceInfos.isEmpty()) {
                deviceInfos.forEach(k -> {
                    deviceIds.add(k.getId());
                });
                troubleDeviceinfoService.removeByIds(deviceIds);
            }
            List<String> getDeleteFilePaths = troubleSave.getDeleteFilePaths();
            if (getDeleteFilePaths != null) {
                List<TroubleFile> list = troubleFileService.list(Wrappers.<TroubleFile>lambdaQuery()
                        .eq(TroubleFile::getTroubleId, troubleSave.getId())
                        .in(TroubleFile::getFilePath, getDeleteFilePaths));

                List<Long> fileIds = new ArrayList<>();
                list.forEach(k -> {
                    fileIds.add(k.getId());
                });
                for (String path : getDeleteFilePaths) {
                    FtpUtil.deleteFile(path);
                }
                troubleFileService.removeByIds(fileIds);
            }
        }
        //管线基本信息表处理
        if (troubleSave.getPipeInfo() != null && !troubleSave.getPipeInfo().isEmpty()) {
            JSONArray pipe = JSONArray.parseArray(troubleSave.getPipeInfo());
            for (int i = 0; i < pipe.size(); i++) {
                TroublePipeinfo pipeInfo = new TroublePipeinfo();
                pipeInfo.setTroubleId(trouble.getId());
                if (pipe.getJSONObject(i).getString("sid") != null) {
                    pipeInfo.setSid("null".equals(pipe.getJSONObject(i).getString("sid").toLowerCase()) ? "" : pipe.getJSONObject(i).getString("sid"));
                }
                if (pipe.getJSONObject
                        (i).getString("name") != null) {
                    pipeInfo.setName("null".equals(pipe.getJSONObject(i).getString("name").toLowerCase()) ? "" : pipe.getJSONObject(i).getString("name"));
                }
                if (pipe.getJSONObject(i).getString("address") != null) {
                    pipeInfo.setAddress("null".equals(pipe.getJSONObject(i).getString("address").toLowerCase()) ? "" : pipe.getJSONObject(i).getString("address"));
                }
                if (pipe.getJSONObject(i).getString("material") != null) {
                    pipeInfo.setMaterial("null".equals(pipe.getJSONObject(i).getString("material").toLowerCase()) ? "" : pipe.getJSONObject(i).getString("material"));
                }
                if (pipe.getJSONObject(i).getString("length") != null) {
                    if ("".equals(pipe.getJSONObject(i).getString("length"))) {
                        pipeInfo.setLength(new BigDecimal(0));
                    } else {
                        pipeInfo.setLength("null".equals(pipe.getJSONObject(i).getString("length").toLowerCase()) ? new BigDecimal(0) : pipe.getJSONObject(i).getBigDecimal("length"));
                    }
                }
                if (pipe.getJSONObject(i).getString("pipeDiameter") != null) {
                    if ("".equals(pipe.getJSONObject(i).getString("pipeDiameter"))) {
                        pipeInfo.setPipeDiameter(new BigDecimal(0));
                    } else {
                        pipeInfo.setPipeDiameter("null".equals(pipe.getJSONObject(i).getString("pipeDiameter").toLowerCase()) ? new BigDecimal(0) : pipe.getJSONObject(i).getBigDecimal("pipeDiameter"));
                    }
                }
                pipeInfo.setStartX(pipe.getJSONObject(i).getBigDecimal("startX"));
                pipeInfo.setStartY(pipe.getJSONObject(i).getBigDecimal("startY"));
                pipeInfo.setEndX(pipe.getJSONObject(i).getBigDecimal("endX"));
                pipeInfo.setEndY(pipe.getJSONObject(i).getBigDecimal("endY"));
                if (pipe.getJSONObject(i).getString("startDepth") != null) {
                    if ("".equals(pipe.getJSONObject(i).getString("startDepth"))) {
                        pipeInfo.setStartDepth(new BigDecimal(0));
                    } else {
                        pipeInfo.setStartDepth("null".equals(pipe.getJSONObject(i).getString("startDepth").toLowerCase()) ? new BigDecimal(0) : pipe.getJSONObject(i).getBigDecimal("startDepth"));
                    }
                }
                if (pipe.getJSONObject(i).getString("endDepth") != null) {
                    if ("".equals(pipe.getJSONObject(i).getString("endDepth"))) {
                        pipeInfo.setEndDepth(new BigDecimal(0));
                    } else {
                        pipeInfo.setEndDepth("null".equals(pipe.getJSONObject(i).getString("endDepth").toLowerCase()) ? new BigDecimal(0) : pipe.getJSONObject(i).getBigDecimal("endDepth"));
                    }
                }
                if (pipe.getJSONObject(i).getString("startAltitude") != null) {
                    if ("".equals(pipe.getJSONObject(i).getString("startAltitude"))) {
                        pipeInfo.setStartAltitude(new BigDecimal(0));
                    } else {
                        pipeInfo.setStartAltitude("null".equals(pipe.getJSONObject(i).getString("startAltitude").toLowerCase()) ? new BigDecimal(0) : pipe.getJSONObject(i).getBigDecimal("startAltitude"));
                    }
                }
                if (pipe.getJSONObject(i).getString("endAltitude") != null) {
                    if ("".equals(pipe.getJSONObject(i).getString("endAltitude"))) {
                        pipeInfo.setEndAltitude(new BigDecimal(0));
                    } else {
                        pipeInfo.setEndAltitude("null".equals(pipe.getJSONObject(i).getString("endAltitude").toLowerCase()) ? new BigDecimal(0) : pipe.getJSONObject(i).getBigDecimal("endAltitude"));
                    }
                }
                if (pipe.getJSONObject(i).getString("geometry") != null) {
                    pipeInfo.setGeometry("null".equals(pipe.getJSONObject(i).getString("geometry").toLowerCase()) ? "" : pipe.getJSONObject(i).getString("geometry"));
                }
                if (pipe.getJSONObject(i).getString("layerId") != null) {
                    if ("".equals(pipe.getJSONObject(i).getString("layerId"))) {
                        pipeInfo.setLayerId(null);
                    } else {
                        pipeInfo.setLayerId("null".equals(pipe.getJSONObject(i).getString("layerId").toLowerCase()) ? null : pipe.getJSONObject(i).getLong("layerId"));
                    }
                }
                if (pipe.getJSONObject(i).getString("layerName") != null) {
                    pipeInfo.setLayerName("null".equals(pipe.getJSONObject(i).getString("layerName").toLowerCase()) ? "" : pipe.getJSONObject(i).getString("layerName"));
                }
                troublePipeinfoService.save(pipeInfo);
            }
        }

        //设备基本信息表处理
        if (troubleSave.getDeviceInfo() != null && !troubleSave.getDeviceInfo().isEmpty()) {
            JSONArray device = JSONArray.parseArray(troubleSave.getDeviceInfo());
            for (int i = 0; i < device.size(); i++) {
                TroubleDeviceinfo deviceInfo = new TroubleDeviceinfo();
                deviceInfo.setTroubleId(trouble.getId());
                if (device.getJSONObject(i).getString("sid") != null) {
                    deviceInfo.setSid("null".equals(device.getJSONObject(i).getString("sid").toLowerCase()) ? "" : device.getJSONObject(i).getString("sid"));
                }
                if (device.getJSONObject(i).getString("address") != null) {
                    deviceInfo.setAddress("null".equals(device.getJSONObject(i).getString("address").toLowerCase()) ? "" : device.getJSONObject(i).getString("address"));
                }
                if (device.getJSONObject(i).getString("name") != null) {
                    deviceInfo.setName("null".equals(device.getJSONObject(i).getString("name").toLowerCase()) ? "" : device.getJSONObject(i).getString("name"));
                }
                if (device.getJSONObject(i).getString("material") != null) {
                    deviceInfo.setMaterial("null".equals(device.getJSONObject(i).getString("material").toLowerCase()) ? "" : device.getJSONObject(i).getString("material"));
                }
                if (device.getJSONObject(i).getString("pipeDiameter") != null) {
                    if ("".equals(device.getJSONObject(i).getString("pipeDiameter"))) {
                        deviceInfo.setPipeDiameter(new BigDecimal(0));
                    } else {
                        deviceInfo.setPipeDiameter("null".equals(device.getJSONObject(i).getString("pipeDiameter").toLowerCase()) ? new BigDecimal(0) : device.getJSONObject(i).getBigDecimal("pipeDiameter"));
                    }
                }
                if (device.getJSONObject(i).getString("altitude") != null) {
                    if ("".equals(device.getJSONObject(i).getString("altitude"))) {
                        deviceInfo.setAltitude(new BigDecimal(0));
                    } else {
                        deviceInfo.setAltitude("null".equals(device.getJSONObject(i).getString("altitude").toLowerCase()) ? new BigDecimal(0) : device.getJSONObject(i).getBigDecimal("altitude"));
                    }
                }
                if (device.getJSONObject(i).getString("depth") != null) {
                    if ("".equals(device.getJSONObject(i).getString("depth"))) {
                        deviceInfo.setDepth(new BigDecimal(0));
                    } else {
                        deviceInfo.setDepth("null".equals(device.getJSONObject(i).getString("depth").toLowerCase()) ? new BigDecimal(0) : device.getJSONObject(i).getBigDecimal("depth"));
                    }
                }
                deviceInfo.setX(device.getJSONObject(i).getBigDecimal("x"));
                deviceInfo.setY(device.getJSONObject(i).getBigDecimal("y"));
                if (device.getJSONObject(i).getString("geometry") != null) {
                    deviceInfo.setGeometry("null".equals(device.getJSONObject(i).getString("geometry").toLowerCase()) ? "" : device.getJSONObject(i).getString("geometry"));
                }
                if (device.getJSONObject(i).getString("layerId") != null) {
                    if ("".equals(device.getJSONObject(i).getString("layerId"))) {
                        deviceInfo.setLayerId(null);
                    } else {
                        deviceInfo.setLayerId("null".equals(device.getJSONObject(i).getString("layerId").toLowerCase()) ? null : device.getJSONObject(i).getLong("layerId"));
                    }
                }
                if (device.getJSONObject(i).getString("layerName") != null) {
                    deviceInfo.setLayerName("null".equals(device.getJSONObject(i).getString("layerName").toLowerCase()) ? "" : device.getJSONObject(i).getString("layerName"));
                }
                troubleDeviceinfoService.save(deviceInfo);
            }
        }

        //troubleFile保存
        saveTroubleFile(request, trouble, null, null);

        String message = "您有一条新的隐患记录名称为" + troubleSave.getTroubleName() + "已上报";
       /* if (b && "0".equals(troubleSave.getUploadType())) {
            if (trouble.getOnDutyUserId() != null) {
                String onDutyUser = trouble.getOnDutyUserId().toString();
                customMessageService.sendMessage(onDutyUser, message, "4");
            }
        } else if (b && "1".equals(troubleSave.getUploadType())) {
            if (StringUtil.isNotEmpty(troubleSave.getSendUserIds())) {
                customMessageService.sendMessage(troubleSave.getSendUserIds(), message, "4");
            }
        } else if (b && "2".equals(troubleSave.getUploadType())) {
            if (troubleSave.getUndertakerUserid() != null) {
                String underTakerUser = troubleSave.getUndertakerUserid().toString();
                customMessageService.sendMessage(underTakerUser, message, "4");
            }
        }*/
        if (troubleSave.getUndertakerUserid() != null) {
            String underTakerUser = troubleSave.getUndertakerUserid().toString();
            customMessageService.sendMessage(underTakerUser, message, "4", "FBZSH");
        }
        saveCheck(trouble.getId(), trouble.getProcessId());

        return true;
    }

/*    @Override
    @Transactional
    public Boolean ondutyUserCheck(HttpServletRequest request, TroubleOndutyUpload troubleOndutyUpload) {
        Long troubleId = troubleOndutyUpload.getId();
        //判断隐患所处流程
        Trouble trouble = troubleService.getById(troubleId);
        if (trouble.getId() == null) {
            throw new ToflyDeniedException("未查询到该隐患ID");
        }
//        if (StringUtil.isNotEmpty(troubleOndutyUpload.getEventType())) {
//            throw new ToflyDeniedException("隐患类型不能为空");
//        }
        if (trouble.getProcessId() != 1) {
            throw new ToflyDeniedException("当前隐患不为值班人员现场勘查阶段");
        }
        if (!SecurityUtils.getUserId().equals(trouble.getOnDutyUserId())) {
            throw new ToflyDeniedException("当前用户不是该隐患责任人");
        }
        Trouble troubleUpdate = new Trouble();
        troubleUpdate.setId(troubleId);
        troubleUpdate.setProcessId((long) 4);
        troubleUpdate.setEventType(troubleOndutyUpload.getEventType());
        troubleUpdate.setTempHandleProject(troubleOndutyUpload.getTempHandleProject());
        troubleUpdate.setTempHandleResult(troubleOndutyUpload.getTempHandleResult());
        troubleUpdate.setTroubleTypeId(troubleOndutyUpload.getTroubleTypeId());
        troubleUpdate.setLevelId(troubleOndutyUpload.getLevelId());
       *//* troubleUpdate.setLayerId(troubleOndutyUpload.getLayerId());
        troubleUpdate.setLayerName(troubleOndutyUpload.getLayerName());*//*
        //非隐患结束工单
        if (troubleOndutyUpload.getEventType() != null) {
            TroubleEnentType byId = troubleEnentTypeService.getById(troubleOndutyUpload.getEventType());
//            if ("0".equals(troubleOndutyUpload.getEventType())) {
            if ("0".equals(byId.getIsTrouble())) {
                troubleUpdate.setState("0");
                troubleUpdate.setProcessId((long) 14);
                troubleUpdate.setHandleState("1");
            }
            boolean b = troubleService.updateById(troubleUpdate);
            if (b && "0".equals(byId.getIsTrouble())) {
                if (trouble.getReportUserId() != null) {
                    customMessageService.sendMessage(trouble.getReportUserId().toString(), "名称为:" + trouble.getTroubleName() + "的隐患上报勘察结果为非隐患", "4");
                }
            } else {
                if (StringUtil.isNotEmpty(troubleOndutyUpload.getSendUserIds())) {
                    customMessageService.sendMessage(troubleOndutyUpload.getSendUserIds(), "您有一条新的隐患记录名称为" + trouble.getTroubleName() + "待处理", "4");
                }
            }
        }


        //troubleFile保存
        saveTroubleFile(request, troubleUpdate, null, null);
        return true;
    }*/

    /*@Override
    public Boolean undertakerCheck(UndertakerCheck undertakerCheck) {
        Long troubleId = undertakerCheck.getId();
        //判断隐患所处流程
        Trouble trouble = troubleService.getById(troubleId);
        if (trouble.getId() == null) {
            throw new ToflyDeniedException("未查询到该隐患ID");
        }
        if (trouble.getProcessId() != 2) {
            throw new ToflyDeniedException("当前隐患不为承办班班长审核阶段");
        }
        if (!SecurityUtils.getUserId().equals(trouble.getUndertakerUserId())) {
            throw new ToflyDeniedException("当前用户不是该隐患承办班班长");
        }
        Trouble troubleUpdate = new Trouble();
        troubleUpdate.setId(troubleId);
        troubleUpdate.setProcessId((long) 3);
        troubleUpdate.setUndertakerCheckNotes(undertakerCheck.getUndertakerCheckNotes());
        troubleUpdate.setUndertakerHandleUserId(undertakerCheck.getUndertakerHandleUserid());
        troubleUpdate.setUndertakerCheckTime(Date8Util.getCurrentDate());
        boolean b = troubleService.updateById(troubleUpdate);
        if (b) {
            if (undertakerCheck.getUndertakerHandleUserid() != null) {
                customMessageService.sendMessage(undertakerCheck.getUndertakerHandleUserid().toString(), "您有一条新的隐患记录名称为:" + trouble.getTroubleName() + "等待现场核实", "4");
            }
        }
        return true;
    }*/

    /*@Override
    @Transactional
    public Boolean undertakerHandle(HttpServletRequest request, UndertakerHandle undertakerHandle) {
        Long troubleId = undertakerHandle.getId();
        //判断隐患所处流程
        Trouble trouble = troubleService.getById(troubleId);
        if (trouble.getId() == null) {
            throw new ToflyDeniedException("未查询到该隐患ID");
        }
        if (trouble.getProcessId() != 3) {
            throw new ToflyDeniedException("当前隐患不为巡检人员现场核实阶段");
        }
        *//*if (!SecurityUtils.getUserId().equals(trouble.getUndertakerHandleUserId())) {
            throw new ToflyDeniedException("当前用户不是该隐患指派的巡检人员");
        }*//*
        Trouble troubleUpdate = new Trouble();
        troubleUpdate.setId(troubleId);
        if ("0".equals(undertakerHandle.getState())) {
            troubleUpdate.setProcessId((long) 15);
            troubleUpdate.setState("0");
        } else if ("1".equals(undertakerHandle.getState())) {
            troubleUpdate.setProcessId((long) 4);
            troubleUpdate.setState("1");
        }
        troubleUpdate.setUndertakerHandleNotes(undertakerHandle.getUndertakerHandleNotes());
        troubleUpdate.setUndertakerHandleTime(Date8Util.getCurrentDate());
        troubleUpdate.setTroubleTypeId(undertakerHandle.getTroubleTypeid());
        troubleUpdate.setLevelId(undertakerHandle.getLevelid());
        //troubleUpdate.setLayerId(undertakerHandle.getLayerId());
        //troubleUpdate.setLayerName(undertakerHandle.getLayerName());
        boolean b = troubleService.updateById(troubleUpdate);

        if (b && "0".equals(undertakerHandle.getState())) {
            if (trouble.getReportUserId() != null) {
                customMessageService.sendMessage(trouble.getReportUserId().toString(), "名称为:" + trouble.getTroubleName() + "的隐患上报现场核实结果为非隐患", "4");
            }
        } else if (b && "1".equals(undertakerHandle.getState())) {
            if (StringUtil.isNotEmpty(undertakerHandle.getSendUserIds())) {
                customMessageService.sendMessage(undertakerHandle.getSendUserIds(), "您有一个新的隐患名称为" + trouble.getTroubleName() + "待处理", "4");
            }
        }

        //troubleFile保存
        saveTroubleFile(request, troubleUpdate, null, null);
        return true;
    }*/

    @Override
    @Transactional
    public Boolean undersecretaryCheck(HttpServletRequest request, UndersecretaryCheck undersecretaryCheck) {
        Long troubleId = undersecretaryCheck.getId();
        //判断隐患所处流程
        Trouble trouble = troubleService.getById(troubleId);
        if (trouble.getId() == null) {
            throw new ToflyDeniedException("未查询到该隐患ID");
        }
        if (trouble.getProcessId() != 1) {
            throw new ToflyDeniedException("当前隐患不为班组长审核阶段");
        }
        /*if (!SecurityUtils.getUserId().equals(trouble.getUndersecretaryUserId())) {
            throw new ToflyDeniedException("当前用户不是该隐患指派的副部长");
        }*/
        Trouble troubleUpdate = new Trouble();
        troubleUpdate.setId(troubleId);
        TroubleState troubleState = new TroubleState();
        troubleState.setTroubleId(troubleId);
        troubleState.setState(undersecretaryCheck.getState());

        if ("1".equals(undersecretaryCheck.getState())) {
            //转入其他班组长派工
            troubleUpdate.setProcessId((long) 3);
            troubleUpdate.setMaintainerUserId(undersecretaryCheck.getOtherDutyUserId());
            /*troubleUpdate.setState("3");
            //如果为抢维修，转移到抢维修申报去
            moveToQwx(troubleId, undersecretaryCheck.getMaintainerUserid());*/
            troubleUpdate.setDutyUserId(undersecretaryCheck.getOtherDutyUserId());
        }
        if ("0".equals(undersecretaryCheck.getState())) {
            //本班组长继续处理工单
            troubleUpdate.setProcessId((long) 2);
            troubleUpdate.setDutyUserId(SecurityUtils.getUserId());
            saveCheck(troubleId, troubleUpdate.getProcessId());
            //如果处置负责任不为空,则进入方案编制提交
            if (undersecretaryCheck.getHandleUserid() != null) {
                TroubleAuditScheme troubleAuditScheme = new TroubleAuditScheme();
                troubleAuditScheme.setWorkOrderId(undersecretaryCheck.getId());
                troubleAuditScheme.setHandleUserId(undersecretaryCheck.getHandleUserid());
                troubleAuditScheme.setDutyPlanNote(undersecretaryCheck.getDutyPlanNote());
                troubleAuditScheme.setDutyCheckTime(Date8Util.getCurrentDate());
                if (StringUtil.isNotEmpty(undersecretaryCheck.getCoprocessor())) {
                    String[] coprocessor = undersecretaryCheck.getCoprocessor().split(",");
                    List<Long> list = new ArrayList<>();
                    for (String c : coprocessor) {
                        list.add(Long.valueOf(c));
                    }
                    StringBuffer sb = new StringBuffer();
                    List<String> list1 = customTroubleMapper.getCoprocessorName(list);
                    if (list1 != null) {
                        for (int i = 0; i < list1.size(); i++) {
                            if (i < list1.size() - 1) {
                                sb.append(list1.get(i) + ",");
                            } else {
                                sb.append(list1.get(i));
                            }
                        }
                    }
                    troubleAuditScheme.setCoprocessor(undersecretaryCheck.getCoprocessor());
                    troubleAuditScheme.setCoprocessorName(sb.toString());
                }
                if ("0".equals(undersecretaryCheck.getFlag())) {
                    troubleUpdate.setProcessId((long) 6);
                } else if ("1".equals(undersecretaryCheck.getFlag()) || StringUtil.isEmpty(undersecretaryCheck.getFlag())) {
                    troubleUpdate.setProcessId((long) 8);
                }
                troubleAuditSchemeService.save(troubleAuditScheme);
                saveTroubleFile(request, troubleUpdate, troubleAuditScheme.getId(), null);
            }
        }
        if ("2".equals(undersecretaryCheck.getState())) {
            troubleUpdate.setProcessId((long) 4);
            //存入副部长ID
//            troubleUpdate.setUndertakerUserId(undersecretaryCheck.getUndersecretaryId());
        }
        if ("3".equals(undersecretaryCheck.getState())) {
            //wjmodify 20200910增加了存量隐患流程
            troubleUpdate.setProcessId(5L);
            troubleUpdate.setState("2");
        }
        troubleUpdate.setUndersecretaryCheckNotes(undersecretaryCheck.getUndersecretaryCheckNotes());
        troubleUpdate.setUndersecretaryCheckTime(Date8Util.getCurrentDate());

        List<TroubleState> lists = troubleStateService.list(Wrappers.<TroubleState>lambdaQuery()
                .eq(TroubleState::getTroubleId, troubleId));
        if (lists.size() > 0) {
            troubleState.setId(lists.get(0).getId());
            troubleStateService.updateById(troubleState);
        } else {
            troubleStateService.save(troubleState);
        }
        boolean b = troubleService.updateById(troubleUpdate);
        if (b) {
            if ("0".equals(undersecretaryCheck.getState()) && undersecretaryCheck.getHandleUserid() != null) {
                customMessageService.sendMessage(undersecretaryCheck.getHandleUserid().toString(), "您有一条隐患记录名称为" + trouble.getTroubleName() + "待处理", "4", "ZXHB");
            }
            else if("1".equals(undersecretaryCheck.getState()) && undersecretaryCheck.getOtherDutyUserId() != null) {
                customMessageService.sendMessage(undersecretaryCheck.getOtherDutyUserId().toString(), "由"+SecurityUtils.getUserName()+"转移的隐患记录名称为" + trouble.getTroubleName() + "待处理", "4", "BZZSH");
            }
            else if("2".equals(undersecretaryCheck.getState()) && undersecretaryCheck.getOtherDutyUserId() != null) {
                String userIds = baseInfoService.getUndersecretaryUsers().getResult().toString();
                customMessageService.sendMessage(userIds, "由"+SecurityUtils.getUserName()+"提交的隐患记录名称为" + trouble.getTroubleName() + "需副部长处理", "4", "FBZSH");
            }
        }
        //消息投资者修改成模块
//        else if (b && "2".equals(undersecretaryCheck.getState())) {
//            if (StringUtil.isNotEmpty(undersecretaryCheck.getSendUserIds())) {
//                customMessageService.sendMessage(undersecretaryCheck.getUndersecretaryId().toString(), "您有一条新的隐患名称为" + trouble.getTroubleName() + "待处理", "4", "FABZ");
//            }
//        }
        saveCheck(troubleId, troubleUpdate.getProcessId());

        return true;
    }

    /**
     * 进入抢维修环节
     *
     * @param
     *//*
    public void moveToQwx(Long troubleId, Long zrr) {
        Trouble trouble = troubleService.getById(troubleId);
        List<TroubleFile> list = troubleFileService.list(Wrappers.<TroubleFile>lambdaQuery()
                .eq(TroubleFile::getTroubleId, troubleId));
        RushRepair rushRepair = new RushRepair();
        String uploadType = trouble.getUploadType();
        if ("2".equals(uploadType)) {
            rushRepair.setCreatUser(trouble.getUndertakerHandleUserId());
            rushRepair.setContact(trouble.getUndertakerHandleUserName());
        } else {
            rushRepair.setCreatUser(trouble.getReportUserId());
            rushRepair.setContact(trouble.getReportUserName());
        }

        rushRepair.setDescription("隐患名称（" + trouble.getTroubleName() + "）转入；管线基本情况描述（" +
                trouble.getPipeInfoDesc() + "）；设备基本情况描述（" +
                trouble.getDeviceInfoDesc() + "）；隐患详情（" + trouble.getTroubleInfo() + "）。");
        rushRepair.setWorkOrderType("0");
        rushRepair.setWorkOrderId("隐患转入" + troubleId);
        rushRepair.setWorkOrderSource("隐患转入");
        rushRepair.setX(trouble.getLocationX());
        rushRepair.setY(trouble.getLocationY());
        rushRepair.setDispatchUserId(zrr);
        rushRepair.setAddress(trouble.getAddress());
        rushRepair.setCreatTime(new Date());
        rushRepair.setCreatUser(SecurityUtils.getUserId());
        rushRepairService.save(rushRepair);
        list.forEach(file -> {
            RushRepairFile rushRepairFile = new RushRepairFile();
            String fileType = file.getFileType();
            String fileName = file.getFileName();
            if ("0".equals(fileType) || "1".equals(fileType) || "2".equals(fileType)) {
                rushRepairFile.setFileType(fileType);
            } else {
                String theType = fileName.substring(fileName.lastIndexOf(".") + 1);
                if ("jpg".equals(theType) || "jpeg".equals(theType) || "png".equals(theType) || "gif".equals(theType)) {
                    rushRepairFile.setFileType("0");
                } else if ("amr".equals(theType)) {
                    rushRepairFile.setFileType("1");
                } else {
                    rushRepairFile.setFileType("2");
                }
            }
            rushRepairFile.setFileName(fileName);
            rushRepairFile.setPath(file.getFilePath());
            rushRepairFileService.save(rushRepairFile);
            RushRepairFiletorepair rushRepairFiletorepair = new RushRepairFiletorepair();
            rushRepairFiletorepair.setWorkOrderId(rushRepair.getId());
            rushRepairFiletorepair.setFileId(rushRepairFile.getId());
            rushRepairFiletorepairService.save(rushRepairFiletorepair);
        });
        if (zrr != null) {
            customMessageService.sendMessage(zrr.toString(), "您有一条隐患转入的抢维修工单信息", "1");
        }
    }*/
    @Override
    @Transactional
    public Boolean designSchema(HttpServletRequest request, DesignSchema designSchema) {
        Long troubleId = designSchema.getId();
        //判断隐患所处流程
        Trouble trouble = troubleService.getById(troubleId);
        if (trouble.getId() == null) {
            throw new ToflyDeniedException("未查询到该隐患ID");
        }
        if (trouble.getProcessId() != 2 && trouble.getProcessId() != 3 && trouble.getProcessId() != 4 && trouble.getProcessId() != 7) {
            throw new ToflyDeniedException("当前隐患不为编制方案阶段");
        }
        if (!SecurityUtils.getUserId().equals(trouble.getDutyUserId()) && trouble.getProcessId() != 4) {
            throw new ToflyDeniedException("当前用户不是该隐患责任班班长");
        }
        Trouble troubleUpdate = new Trouble();
        troubleUpdate.setId(troubleId);
        if (trouble.getProcessId() == 2 || trouble.getProcessId() == 3 || trouble.getProcessId() == 7) {
            if ("0".equals(designSchema.getFlag())) {
                troubleUpdate.setProcessId((long) 6);
            } else if ("1".equals(designSchema.getFlag()) || StringUtil.isEmpty(designSchema.getFlag())) {
                troubleUpdate.setProcessId((long) 8);
            }
        } else if (trouble.getProcessId() == 4) {
            //添加副部长ID
            troubleUpdate.setUndertakerUserId(SecurityUtils.getUserId());
            troubleUpdate.setDutyUserId(SecurityUtils.getUserId());
            troubleUpdate.setProcessId((long) 14);
        }
        TroubleAuditScheme troubleAuditScheme = new TroubleAuditScheme();
        troubleAuditScheme.setWorkOrderId(designSchema.getId());
        troubleAuditScheme.setHandleUserId(designSchema.getHandleUserid());
        troubleAuditScheme.setDutyPlanNote(designSchema.getDutyPlanNote());
        troubleAuditScheme.setDutyCheckTime(Date8Util.getCurrentDate());
        //添加处理人消息
        designSchema.setSendUserIds(designSchema.getHandleUserid().toString());
        if (StringUtil.isNotEmpty(designSchema.getCoprocessor())) {
            designSchema.setSendUserIds(designSchema.getSendUserIds()+","+designSchema.getCoprocessor());
            String[] coprocessor = designSchema.getCoprocessor().split(",");
            List<Long> list = new ArrayList<>();
            for (String c : coprocessor) {
                list.add(Long.valueOf(c));
            }
            StringBuffer sb = new StringBuffer();
            List<String> list1 = customTroubleMapper.getCoprocessorName(list);
            if (list1 != null) {
                for (int i = 0; i < list1.size(); i++) {
                    if (i < list1.size() - 1) {
                        sb.append(list1.get(i) + ",");
                    } else {
                        sb.append(list1.get(i));
                    }
                }
            }
            troubleAuditScheme.setCoprocessor(designSchema.getCoprocessor());
            troubleAuditScheme.setCoprocessorName(sb.toString());
        }

        //保存文件 file1:整改方案;file2:整改工程决算书;file3:隐患整改通知书;file4:管道保护通知书
        /*MultipartHttpServletRequest multipartRequest = (MultipartHttpServletRequest) request;
        MultipartFile file1 = multipartRequest.getFile("file1");
        if (file1 != null) {
            troubleUpdate.setRectifyFile(saveFiles(file1));
        }
        MultipartFile file2 = multipartRequest.getFile("file2");
        if (file2 != null) {
            troubleUpdate.setRectifyFile(saveFiles(file2));
        }
        MultipartFile file3 = multipartRequest.getFile("file3");
        if (file3 != null) {
            troubleUpdate.setRectifyFile(saveFiles(file3));
        }
        MultipartFile file4 = multipartRequest.getFile("file4");
        if (file4 != null) {
            troubleUpdate.setRectifyFile(saveFiles(file4));
        }*/
        troubleService.updateById(troubleUpdate);
        troubleAuditSchemeService.save(troubleAuditScheme);
        saveTroubleFile(request, troubleUpdate, troubleAuditScheme.getId(), null);
        if (StringUtil.isNotEmpty(designSchema.getSendUserIds()) && "0".equals(designSchema.getFlag())) {
            customMessageService.sendMessage(designSchema.getSendUserIds(), "您有一条:" + trouble.getTroubleName() + "的方案待审批", "4", "FBZSH");
        } else if (StringUtil.isNotEmpty(designSchema.getSendUserIds()) && "1".equals(designSchema.getFlag())) {
            customMessageService.sendMessage(designSchema.getSendUserIds(), "您有一条:" + trouble.getTroubleName() + "的隐患方案待处理", "4", "ZXHB");
        }
        saveCheck(troubleId, troubleUpdate.getProcessId());

        return true;
    }

    @Override
    @Transactional
    public Boolean secretaryCheck(HttpServletRequest request, SecretaryCheck secretaryCheck) {
        Long troubleId = secretaryCheck.getTroubleId();
        //判断隐患所处流程
        Trouble trouble = troubleService.getById(troubleId);
        if (trouble.getId() == null) {
            throw new ToflyDeniedException("未查询到该隐患ID");
        }
        if (trouble.getProcessId() != 6) {
            throw new ToflyDeniedException("当前隐患不为副部长审核阶段");
        }
      /*  if (!SecurityUtils.getUserId().equals(trouble.getSecretaryUserid())) {
            throw new ToflyDeniedException("当前用户不是该隐患部长");
        }*/
        Trouble troubleUpdate = new Trouble();
        troubleUpdate.setId(troubleId);
        List<TroubleAuditScheme> list = troubleAuditSchemeService.list(Wrappers.<TroubleAuditScheme>lambdaQuery()
                .isNull(TroubleAuditScheme::getAuditmind).eq(TroubleAuditScheme::getWorkOrderId, troubleId).orderByDesc(TroubleAuditScheme::getDutyCheckTime));
        if (list.isEmpty()) {
            throw new ToflyDeniedException("未查找到相关方案");
        }
        Long schemeId = list.get(0).getId();

        TroubleAuditScheme troubleAuditScheme = new TroubleAuditScheme();
        troubleAuditScheme.setId(schemeId);
        troubleAuditScheme.setAuditmind(secretaryCheck.getSecretaryCheckState());
        troubleAuditScheme.setAuditDescription(secretaryCheck.getSecretaryCheckNotes());
        troubleAuditScheme.setAuditorSecretary(SecurityUtils.getUserId());
        troubleAuditScheme.setAuditTime(new Date());
        boolean b = troubleAuditSchemeService.updateById(troubleAuditScheme);
        /*troubleUpdate.setSecretaryCheckNotes(secretaryCheck.getSecretaryCheckNotes());
        troubleUpdate.setSecretaryCheckState(secretaryCheck.getSecretaryCheckState());
        troubleUpdate.setSecretaryCheckTime(Date8Util.getCurrentDate());*/
        String message = "";
        if (b && "0".equals(secretaryCheck.getSecretaryCheckState())) {
            troubleUpdate.setProcessId((long) 8);
            message = trouble.getTroubleName() + "的隐患处理方案已通过审批";
            TroubleAuditScheme auditScheme = troubleAuditSchemeService.getById(schemeId);
            secretaryCheck.setSendUserIds(auditScheme.getHandleUserId().toString());
            if (StringUtil.isNotEmpty(secretaryCheck.getSendUserIds())) {
                customMessageService.sendMessage(secretaryCheck.getSendUserIds(), message, "4", "ZXHB");
            }
            //message = trouble.getTroubleName() + "隐患,编号:" + trouble.getSid() + "部长审批通过";
        } else if (b && "1".equals(secretaryCheck.getSecretaryCheckState())) {
            troubleUpdate.setProcessId((long) 7);
            //message = trouble.getTroubleName() + "隐患,编号:" + trouble.getSid() + "部长审批未通过";
            //boolean b1 = troubleService.save(trouble);
            //troubleUpdate.setRejectId(trouble.getId());
            secretaryCheck.setSendUserIds(trouble.getDutyUserId().toString());
            message = trouble.getTroubleName() + "的隐患处理方案未通过审批，需重新提交";
            if (StringUtil.isNotEmpty(secretaryCheck.getSendUserIds())) {
                customMessageService.sendMessage(secretaryCheck.getSendUserIds(), message, "4", "BZZSH");
            }
        }
        saveTroubleFile(request, troubleUpdate, troubleAuditScheme.getId(), null);
        //更新原记录
        troubleService.updateById(troubleUpdate);
        /*if (trouble.getDirector() != null) {
            //技术总监
            MessageSave messageSave = new MessageSave();
            messageSave.setMessage(message);
            messageSave.setReceipt(trouble.getDirector().toString());
            customMessageService.saveNoFile(messageSave);
        }*/
        saveCheck(troubleId, troubleUpdate.getProcessId());

        return true;
    }

    @Override
    @Transactional
    public Boolean handleSchema(HttpServletRequest request, HandleSchema handleSchema) {
        Long troubleId = handleSchema.getId();
        //判断隐患所处流程
        Trouble trouble = troubleService.getById(troubleId);
        if (trouble.getId() == null) {
            throw new ToflyDeniedException("未查询到该隐患ID");
        }
        if (trouble.getProcessId() != 8 && trouble.getProcessId() != 10 && trouble.getProcessId() != 12 && trouble.getProcessId() != 14) {
            throw new ToflyDeniedException("当前隐患不为工单负责人处理阶段");
        }
//        if(!SecurityUtils.getUserId().equals(trouble.getDutyUserId())){
//            throw new ToflyDeniedException("当前用户不是该工单负责人");
//        }
        /*if (!SecurityUtils.getUserId().equals(trouble.getHandleUserId())) {
            throw new ToflyDeniedException("当前用户不是该隐患工单处理人");
        }*/
        //troubleFile保存
        Trouble troubleUpdate = new Trouble();
        troubleUpdate.setId(troubleId);
        String module = "";
        if (trouble.getProcessId() == 8 || trouble.getProcessId() == 10) {
            troubleUpdate.setProcessId((long) 9);
            module = "BZZSH";
        } else if (trouble.getProcessId() == 14 || trouble.getProcessId() == 12) {
            troubleUpdate.setProcessId((long) 11);
            module = "FBZSH";
        }
        TroubleAuditEnd troubleAuditEnd = new TroubleAuditEnd();
        troubleAuditEnd.setWorkOrderId(troubleId);
        troubleAuditEnd.setHandleNote(handleSchema.getHandleNote());
        troubleAuditEnd.setHandleTime(Date8Util.getCurrentDate());
        troubleService.updateById(troubleUpdate);
        troubleAuditEndService.save(troubleAuditEnd);
        saveTroubleFile(request, troubleUpdate, null, troubleAuditEnd.getId());
        saveCheck(troubleId, troubleUpdate.getProcessId());
        customMessageService.sendMessage(trouble.getDutyUserId().toString(), "已处理隐患隐患:" + trouble.getTroubleName() + "待审核", "4", module);
        return true;
    }

    @Override
    @Transactional
    public Boolean handleCheck(HttpServletRequest request, HandleCheck handleCheck) {
        Long troubleId = handleCheck.getTroubleId();
        //判断隐患所处流程
        Trouble trouble = troubleService.getById(troubleId);
        if (trouble.getId() == null) {
            throw new ToflyDeniedException("未查询到该隐患ID");
        }
        if (trouble.getProcessId() != 9 && trouble.getProcessId() != 11) {
            throw new ToflyDeniedException("当前隐患不为副部长/班组长审核回执并归档阶段");
        }
        /*if (!SecurityUtils.getUserId().equals(trouble.getUndersecretaryUserId())) {
            throw new ToflyDeniedException("当前用户不是该隐患副部长");
        }*/
        Trouble troubleUpdate = new Trouble();
        troubleUpdate.setId(troubleId);
        // troubleUpdate.setHandleCheckTime(Date8Util.getCurrentDate());
        //troubleUpdate.setHandleNote(handleCheck.getHandleCheckNote());
        List<TroubleAuditEnd> list = troubleAuditEndService.list(Wrappers.<TroubleAuditEnd>lambdaQuery()
                .isNull(TroubleAuditEnd::getAuditmind).eq(TroubleAuditEnd::getWorkOrderId, troubleId).orderByDesc(TroubleAuditEnd::getHandleTime));
        if (list.isEmpty()) {
            throw new ToflyDeniedException("未查找到相关处理结果");
        }
        Long endId = list.get(0).getId();

        TroubleAuditEnd troubleAuditEnd = new TroubleAuditEnd();
        troubleAuditEnd.setId(endId);
        troubleAuditEnd.setAuditmind(handleCheck.getHandleCheck());
        troubleAuditEnd.setAuditDescription(handleCheck.getHandleCheckNote());
        troubleAuditEnd.setAuditorUndersecretary(SecurityUtils.getUserId());
        troubleAuditEnd.setAuditTime(new Date());
        boolean b = troubleAuditEndService.updateById(troubleAuditEnd);
        String message = "";
        if (b && "0".equals(handleCheck.getHandleCheck())) {
            handleCheck.setSendUserIds(trouble.getDutyUserId().toString());
            troubleUpdate.setProcessId((long) 13);
            message = trouble.getTroubleName() + "隐患已通过审核，完成归档";
            if (StringUtil.isNotEmpty(handleCheck.getSendUserIds())) {
                customMessageService.sendMessage(handleCheck.getSendUserIds(), message, "4", "");
            }
            //message = trouble.getTroubleName() + "隐患,编号:" + trouble.getSid() + "副部长审核通过，完成归档";
        } else if (b && "1".equals(handleCheck.getHandleCheck())) {
            if (trouble.getProcessId() == 9) {
                troubleUpdate.setProcessId((long) 10);
            } else if (trouble.getProcessId() == 11) {
                troubleUpdate.setProcessId((long) 12);
            }
            message = trouble.getTroubleName() + "隐患未通过审核，需继续处理";
            List<TroubleAuditScheme> lists = troubleAuditSchemeService.list(Wrappers.<TroubleAuditScheme>lambdaQuery()
                    .eq(TroubleAuditScheme::getWorkOrderId,troubleId).orderByDesc(TroubleAuditScheme::getId));
            handleCheck.setSendUserIds(lists.get(0).getHandleUserId().toString());
            if (StringUtil.isNotEmpty(handleCheck.getSendUserIds())) {
                customMessageService.sendMessage(handleCheck.getSendUserIds(), message, "4", "");
            }
            //boolean b1 = troubleService.save(trouble);
            //troubleUpdate.setRejectId(trouble.getId());
            //message = trouble.getTroubleName() + "隐患,编号:" + trouble.getSid() + "副部长审核未通过";
        }
        saveTroubleFile(request, troubleUpdate, null, troubleAuditEnd.getId());

        troubleService.updateById(troubleUpdate);
        //推送部长和技术总监
        /*if (trouble.getUndersecretaryUserId() != null && trouble.getDirector() != null) {
            String receipt = trouble.getUndersecretaryUserId().toString() + "," + trouble.getDirector();
            MessageSave messageSave = new MessageSave();
            messageSave.setMessage(message);
            messageSave.setReceipt(receipt);
            customMessageService.saveNoFile(messageSave);
        }*/
        saveCheck(troubleId, troubleUpdate.getProcessId());

        return true;
    }

    @Override
    public List<Map> lxtj(CustomRushRepairReportQuery customRushRepairReportQuery) {
        List<Map> lists = customTroubleMapper.lxtj(customRushRepairReportQuery);
        return HandlerList(lists, "type");
    }

    @Override
    public List<Map> qsfx(CustomRushRepairReportQuery customRushRepairReportQuery) {
        List<Map> lists = customTroubleMapper.qsfx(customRushRepairReportQuery);
        return HandlerList(lists, "troubleTypeName");
    }

    @Override
    @Transactional
    public boolean updatePipeDevice(CustomTroubleUpdatePipe customTroubleUpdatePipe) {
        Trouble trouble = new Trouble();
        Long id = customTroubleUpdatePipe.getId();
        trouble.setId(id);
        trouble.setPipeInfoDesc(customTroubleUpdatePipe.getPipeInfo());
        trouble.setDeviceInfoDesc(customTroubleUpdatePipe.getDeviceInfo());
        trouble.setAddress(customTroubleUpdatePipe.getAddress());
        trouble.setLocationX(customTroubleUpdatePipe.getLocationX());
        trouble.setLocationY(customTroubleUpdatePipe.getLocationY());

        troubleService.updateById(trouble);
        troublePipeinfoService.remove(Wrappers.<TroublePipeinfo>lambdaQuery().
                eq(TroublePipeinfo::getTroubleId, id));
        troubleDeviceinfoService.remove(Wrappers.<TroubleDeviceinfo>lambdaQuery().
                eq(TroubleDeviceinfo::getTroubleId, id));
        List<TroublePipeinfo> pipeList = customTroubleUpdatePipe.getPipeList();
        if (pipeList != null && pipeList.size() > 0) {
            pipeList.forEach(pipe -> {
                pipe.setTroubleId(id);
                troublePipeinfoService.save(pipe);
            });
        }
        List<TroubleDeviceinfo> deviceLis = customTroubleUpdatePipe.getDeviceList();
        if (deviceLis != null && deviceLis.size() > 0) {
            deviceLis.forEach(device -> {
                device.setTroubleId(id);
                troubleDeviceinfoService.save(device);
            });
        }
        return true;
    }

    @Override
    public Page<Map> getListNew(Page page, TroubleSearch troubleSearch) {
        Set<Long> userIds = new HashSet<>();
        if (troubleSearch.getUserId() != null) {
            List<Long> onDutyUsers = new ArrayList<>();
            troubleService.list(Wrappers.<Trouble>lambdaQuery().eq(Trouble::getOnDutyUserId, troubleSearch.getUserId())
                    .or().eq(Trouble::getUndertakerUserId, troubleSearch.getUserId())
                    .or().eq(Trouble::getDutyUserId, troubleSearch.getUserId())
                    .or().eq(Trouble::getMaintainerUserId, troubleSearch.getUserId())
                    .or().eq(Trouble::getFindUserId, troubleSearch.getUserId())
                    .or().eq(Trouble::getReportUserId, troubleSearch.getUserId()))
                    .forEach(k -> onDutyUsers.add(k.getId()));
//            List<Long> undertakerUsers = new ArrayList<>();
//            troubleService.list(Wrappers.<Trouble>lambdaQuery().eq(Trouble::getUndertakerUserId, troubleSearch.getUserId()))
//                    .forEach(k -> undertakerUsers.add(k.getId()));
//
//            troubleService.list(Wrappers.<Trouble>lambdaQuery().select(Trouble::getId)
//                    .eq(Trouble::getUndertakerHandleUserId, troubleSearch.getUserId()))
//                    .forEach(t -> userIds.add(t.getId()));
//            List<Long> dutyUsers = new ArrayList<>();
//            troubleService.list(Wrappers.<Trouble>lambdaQuery().eq(Trouble::getDutyUserId, troubleSearch.getUserId()).or())
//                    .forEach(k -> dutyUsers.add(k.getId()));
//            List<Long> gisUsers = new ArrayList<>();
//            troubleService.list(Wrappers.<Trouble>lambdaQuery().eq(Trouble::getGisUserId, troubleSearch.getUserId()))
//                    .forEach(k -> gisUsers.add(k.getId()));
//            List<Long> reportUsers = new ArrayList<>();
//            troubleService.list(Wrappers.<Trouble>lambdaQuery().eq(Trouble::getReportUserId, troubleSearch.getUserId()))
//                    .forEach(k -> reportUsers.add(k.getId()));
            List<Long> auditorSecretary = new ArrayList<>();
            troubleAuditSchemeService.list(Wrappers.<TroubleAuditScheme>lambdaQuery().eq(TroubleAuditScheme::getAuditorSecretary, troubleSearch.getUserId())
                    .eq(TroubleAuditScheme::getAuditmind, "0")).forEach(k -> auditorSecretary.add(k.getWorkOrderId()));
            List<Long> handleUsers = new ArrayList<>();
            troubleAuditSchemeService.list(Wrappers.<TroubleAuditScheme>lambdaQuery().eq(TroubleAuditScheme::getHandleUserId, troubleSearch.getUserId())
                    .eq(TroubleAuditScheme::getAuditmind, "0")).forEach(k -> handleUsers.add(k.getWorkOrderId()));
            List<Long> coprocessors = new ArrayList<>();
            List<TroubleAuditScheme> list = troubleAuditSchemeService.list(Wrappers.<TroubleAuditScheme>lambdaQuery().eq(TroubleAuditScheme::getAuditmind, "0"));
            for (TroubleAuditScheme t : list) {
                if (StringUtil.isNotEmpty(t.getCoprocessor())) {
                    String[] array = t.getCoprocessor().split(",");
                    for (String t2 : array) {
                        if (Long.parseLong(t2) == troubleSearch.getUserId()) {
                            coprocessors.add(t.getWorkOrderId());
                        }
                    }
                }
            }
            List<Long> auditorUndersecretary = new ArrayList<>();
            troubleAuditEndService.list(Wrappers.<TroubleAuditEnd>lambdaQuery().eq(TroubleAuditEnd::getAuditorUndersecretary, troubleSearch.getUserId())
                    .eq(TroubleAuditEnd::getAuditmind, "0")).forEach(k -> auditorUndersecretary.add(k.getWorkOrderId()));
            userIds.addAll(onDutyUsers);
//            userIds.addAll(undertakerUsers);
//            userIds.addAll(dutyUsers);
//            userIds.addAll(gisUsers);
//            userIds.addAll(reportUsers);
            userIds.addAll(auditorSecretary);
            userIds.addAll(handleUsers);
            userIds.addAll(coprocessors);
            userIds.addAll(auditorUndersecretary);
            if (userIds.isEmpty()) {
                userIds.add(Long.MAX_VALUE);
            }
        }
        if (StringUtil.isNotEmpty(troubleSearch.getProcessIds())) {

            troubleSearch.setProcessIdList(Arrays.stream(troubleSearch.getProcessIds().split(","))
                    .map(s -> Long.valueOf(s)).collect(Collectors.toList()));
        }
        List<Long> deptUsers = new ArrayList<>();
        if (troubleSearch.getDeptId() != null) {
            deptUsers = customTroubleMapper.getDeptUsers(troubleSearch.getDeptId());
        }
        troubleSearch.setDeptUsers(deptUsers);
        return page.setRecords(customTroubleMapper.pageByConditionNew(page, troubleSearch, userIds));
    }

    @Override
    public CustomTrouble getById(Long id) {
        return customTroubleMapper.getById(id);
    }


    private List<Map> HandlerList(List<Map> lists, String typeName) {
        if (lists.size() == 0) {
            return new ArrayList<>();
        }
        String type = (String) (lists.get(0).get("type"));
        List<Map> list = new ArrayList<>(), tempList = new ArrayList<>();
        Map map;
        for (Map m : lists) {
            if (!type.equals(m.get("type"))) {
                map = new HashMap();
                map.put(typeName, type);
                map.put("info", tempList);
                list.add(map);
                type = (String) m.get("type");
                tempList = new ArrayList<>();
                tempList.add(m);
            } else {
                tempList.add(m);
            }
        }
        ;
        map = new HashMap();
        map.put(typeName, type);
        map.put("info", tempList);
        list.add(map);
        return list;
    }

    private String saveFiles(MultipartFile file) {
        String filePath = filePathBean.getMessage() + IdWorker.getIdStr() + file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf("."));
        boolean b = FtpUtil.uploadFile2Ftp(file, filePath);
        return filePath;
    }

    private void saveTroubleFile(HttpServletRequest request, Trouble trouble, Long schemeId, Long endId) {
        //遍历请求中key包含photo的所有文件，包括photo1，photo2中的多个文件
        //遍历请求中key包含audio的所有文件，包括audio1，audio2中的多个文件
        MultiValueMap<String, MultipartFile> files = ((MultipartHttpServletRequest) request).getMultiFileMap();
        files.forEach((k, v) -> {
            //保存照片
            v.forEach(file -> {
                String fileName = file.getOriginalFilename();

                String filePath = filePathBean.getTrouble() + IdWorker.getIdStr() + file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf("."));
                boolean b = FtpUtil.uploadFile2Ftp(file, filePath);
                if (b) {
                    TroubleFile troubleFile = new TroubleFile();
                    troubleFile.setFilePath(filePath);
                    troubleFile.setFileName(fileName);
                    if (k.contains("photo")) {
                        troubleFile.setFileType("0");
                    } else if (k.contains("audio")) {
                        troubleFile.setFileType("1");
                    } else if (k.contains("file")) {
                        if (fileName.toLowerCase().endsWith(".bmp")
                                || fileName.toLowerCase().endsWith(".jpg")
                                || fileName.toLowerCase().endsWith(".jpeg")
                                || fileName.toLowerCase().endsWith(".gif")
                                || fileName.toLowerCase().endsWith(".png")) {
                            troubleFile.setFileType("0");
                        } else if (fileName.toLowerCase().endsWith(".amr")) {
                            troubleFile.setFileType("1");
                        } else {
                            troubleFile.setFileType("2");
                        }
                    } else if (k.contains("other")) {
                        troubleFile.setFileType("2");
                    } else if (k.contains("rectifyFinalFile")) {
                        troubleFile.setFileType("3");
                    } else if (k.contains("rectifyNoticeFile")) {
                        troubleFile.setFileType("4");
                    } else if (k.contains("protectNoticeFile")) {
                        troubleFile.setFileType("5");
                    } else if (k.contains("rectifyFile")) {
                        troubleFile.setFileType("6");
                    }
                    troubleFile.setTroubleId(trouble.getId());
                    troubleFile.setProcessId(trouble.getProcessId());
                    troubleFile.setAuditSchemeId(schemeId);
                    troubleFile.setAuditEndId(endId);
                    troubleFileService.save(troubleFile);
                }
            });
        });
    }

    private void saveCheck(Long troubleId, Long processId) {
        TroubleCheck troubleCheck = new TroubleCheck();
        troubleCheck.setTroubleId(troubleId);
        troubleCheck.setProcessId(processId);
        //添加1s,区别不同步骤时间
        Date date = new Date();
        if (processId == 6 || processId == 8){
            Calendar calendar = Calendar.getInstance();
            calendar.setTime(date);
            calendar.add(Calendar.SECOND, 1);
            date =  calendar.getTime();
        }
        troubleCheck.setNoteTime(date);
        troubleCheckService.save(troubleCheck);
    }

}
