package com.tofly.dyrq.custom.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.tofly.common.core.entity.ResultRespone;
import com.tofly.common.mapper.ToflyCodeCommonMapper;
import com.tofly.common.oauth.exception.ToflyDeniedException;
import com.tofly.common.oauth.util.SecurityUtils;
import com.tofly.dyrq.custom.entity.CustomQrLayer;
import com.tofly.dyrq.custom.entity.CustomSysDatabase;
import com.tofly.dyrq.custom.entity.PdaDeviceUser;
import com.tofly.dyrq.custom.entity.UserPda;
import com.tofly.dyrq.custom.mapper.OtherMapper;
import com.tofly.dyrq.custom.service.AuthService;
import com.tofly.dyrq.custom.service.OtherService;
import com.tofly.dyrq.entity.PdaDevice;
import com.tofly.dyrq.entity.PdaUserDevice;
import com.tofly.dyrq.service.PdaDeviceService;
import com.tofly.dyrq.service.PdaUserDeviceService;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author tuonbed
 * @date 2020/4/1  10:33
 */
@Service
@AllArgsConstructor
@Slf4j
public class OtherServiceImpl implements OtherService {
    private final OtherMapper otherMapper;
    private final PdaDeviceService pdaDeviceService;
    private final PdaUserDeviceService pdaUserDeviceService;
    private final AuthService authService;

    @Override
    public Page<UserPda> getUsersPda(Page page, UserPda user) {
        return page.setRecords(otherMapper.getUsersPda(page,user));
    }

    @Override
    public Page<Map> getUsersPdaByMobile(Page page, UserPda user) {
        return page.setRecords(otherMapper.getUsersPdaByMobile(page,user));
    }

    @Override
    @Transactional
    public boolean addPda(PdaDevice pdaDevice) {
        PdaUserDevice currentUserDevice = pdaUserDeviceService.getOne(Wrappers.<PdaUserDevice>lambdaQuery()
                .eq(PdaUserDevice::getUserId,SecurityUtils.getUserId()));
        if (currentUserDevice != null){
            throw new ToflyDeniedException("当前用户已绑定其他设备");
        }
        Long deptId = SecurityUtils.getUserDeptId();
        String deptName = otherMapper.getDeptName(deptId);
        //注册设备添加部门信息
        pdaDevice.setCompanyid(deptId);
        pdaDevice.setCompanyName(deptName);
        pdaDevice.setRegisterType(BigDecimal.ONE);
        //判断是否存在该设备,存在则更新设备部门与绑定信息
        PdaDevice one = pdaDeviceService.getOne(Wrappers.<PdaDevice>lambdaQuery().eq(PdaDevice::getImei,pdaDevice.getImei()));
        if (one != null){
            pdaDevice.setId(one.getId());
            pdaDeviceService.updateById(pdaDevice);
            PdaUserDevice pdaUserDevice = pdaUserDeviceService.getOne(Wrappers.<PdaUserDevice>lambdaQuery()
                    .eq(PdaUserDevice::getDeviceId,one.getId())
                    .ne(PdaUserDevice::getUserId,SecurityUtils.getUserId()));
            if (pdaUserDevice != null){
                throw new ToflyDeniedException("当前设备被其他账号绑定,请先解绑");
            }
        }
        //新设备
        else {
            pdaDeviceService.save(pdaDevice);
        }
        PdaUserDevice pdaUserDevice=new PdaUserDevice();
        pdaUserDevice.setUserId(SecurityUtils.getUserId());
        pdaUserDevice.setDeviceId(pdaDevice.getId());
        pdaUserDeviceService.save(pdaUserDevice);
        return true;
    }

    @Override
    public String getSignTypeByUser(Long id) {
        return otherMapper.getSignTypeByUser(id);
    }


    @Override
    @Transactional
    public String checkPda(String imei) {
        List<PdaDevice> deviceList = pdaDeviceService.list(Wrappers.<PdaDevice>lambdaQuery().eq(PdaDevice::getImei,imei));
        String restr= "0";
        //设备已注册
        if (deviceList != null) {
            for (PdaDevice device : deviceList) {
                List<PdaUserDevice> list = pdaUserDeviceService.list(Wrappers.<PdaUserDevice>lambdaQuery()
                        .eq(PdaUserDevice::getDeviceId, device.getId()));
                if (list.size() > 0) {
                    restr = "1";
                    if (!SecurityUtils.getUserId().equals(list.get(0).getUserId())) {
                        restr = "2";
                    }
                }
            }
        }
        //设备未注册
        else{
            PdaUserDevice currentUser = pdaUserDeviceService.getOne(Wrappers.<PdaUserDevice>lambdaQuery()
                    .eq(PdaUserDevice::getUserId,SecurityUtils.getUserId()));
            if (currentUser != null){
                restr= "3";
            }
        }
        return restr;
    }

    @Override
    @Transactional
    public Boolean userDeviceRemoveById(Long id) {
        PdaUserDevice pdaUserDevice = pdaUserDeviceService.getById(id);
        Long deviceId = pdaUserDevice.getDeviceId();
        Long userId = pdaUserDevice.getUserId();
        pdaUserDeviceService.removeById(id);
        //解除绑定后,设备的部门信息取消
        PdaDevice pdaDevice = new PdaDevice();
        pdaDevice.setId(deviceId);
        pdaDevice.setCompanyid(null);
        pdaDevice.setCompanyName(null);
        //使token失效
        ResultRespone resultRespone= authService.logout(userId);
        if (resultRespone.getCode() == 1 ||(resultRespone.getCode() == -1 && resultRespone.getMessage() == null)){
            return pdaDeviceService.updateById(pdaDevice);
        }
        else {
            throw new ToflyDeniedException("删除解绑用户token失败");
        }
    }

    @Override
    @Transactional
    public Boolean userDeviceSave(PdaUserDevice pdaUserDevice) {
        pdaUserDeviceService.save(pdaUserDevice);
        Long deviceId = pdaUserDevice.getDeviceId();
        //接触绑定后,跟新设备的部门信息取消
        PdaDevice pdaDevice = new PdaDevice();
        pdaDevice.setId(deviceId);
        Long deptId = SecurityUtils.getUserDeptId();
        String deptName = otherMapper.getDeptName(deptId);
        pdaDevice.setCompanyid(deptId);
        pdaDevice.setCompanyName(deptName);
        return pdaDeviceService.updateById(pdaDevice);
    }

    @Override
    public boolean insertQrLayer(CustomQrLayer customQrLayer) {
//        String sbids=customQrLayer.getSbid();
//        customQrLayer.setSbids(Arrays.asList(sbids.split(",")));
        otherMapper.insertQrLayer(customQrLayer);
        return true;
    }

    @Override
    public List<Map> getQrLayer(CustomQrLayer customQrLayer) {
        String sbids=customQrLayer.getSbid();
        customQrLayer.setSbids(Arrays.asList(sbids.split(",")));
        return otherMapper.getQrLayer(customQrLayer);
    }

    @Override
    public void handlerData(CustomQrLayer customQrLayer){
        List<Map> list=this.getQrLayer(customQrLayer);
        if(list.size()>0){
            Map<String, Object> collect = list.stream().collect(Collectors.toMap(k -> (String)k.get("sbid"), v -> v.get("layer")));
            List<String> collect1 = customQrLayer.getSbids().stream().filter(s -> collect.get(s) == null).collect(Collectors.toList());
            if(collect1!=null && collect1.size()>0){
                customQrLayer.setSbids(collect1);
                this.insertQrLayer(customQrLayer);
            }
        }else{
            this.insertQrLayer(customQrLayer);
        }
    }

    @Override
    public Page<Map> getSysDataSpace(Page page) {
        return page.setRecords(otherMapper.getSysDataSpace(page));
    }

    @Override
    public boolean updateSysDataBase(CustomSysDatabase customSysDatabase) {
        otherMapper.updateSysDataBase(customSysDatabase);
        return true;
    }

    @Override
    public Page<Map> getMaintenanceMenuPage(Page page, String name) {
        return page.setRecords(otherMapper.getMaintenanceMenuPage(page,name));
    }

}
