package com.tofly.dyrq.qrqd;

import com.google.zxing.BarcodeFormat;
import com.google.zxing.EncodeHintType;
import com.google.zxing.MultiFormatWriter;
import com.google.zxing.WriterException;
import com.google.zxing.common.BitMatrix;
import com.google.zxing.qrcode.decoder.ErrorCorrectionLevel;
import com.tofly.dyrq.entity.Qrcode;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

/**
 * 生成图片
 */
public class QRCodeMaker {

   /* private static final int PIC_WIDTH = PROPARAM.SIZE;
    private static final int PIC_HEIGHT = PROPARAM.SIZE;
    private static final int QR_SIZE = PROPARAM.SIZE;*/


    private static final int BLUE = 0xFF3ED547;
    private static final int WHITE = 0xFFFFFFFF;
    private static final int BLACK = 0xFF000000;
    private static final int RED = 0xFFFF0000;

    private static final String IMG_FORMATE = "jpg";


    private static final int LOGO_BORDER_WIDTH = 2;//LOGO边框宽度
    private static final Color LOGO_BORDER_COLOR = Color.WHITE;//LOGO边框颜色

    // TODO:logo file path


    /* *//**
     * 生成二维码图片
     *
     * @param info 二维码信息
     * @param out  目的图片地址
     *//*
    public static void MakeQRCode(String info, String out) throws WriterException, IOException {
        File outFile = new File(out);

        if (!outFile.getParentFile().exists()) {
            outFile.getParentFile().mkdirs();
        }

        BufferedImage image = toBufferedImg(getBitMatrix(info));

        ImageIO.write(image, IMG_FORMATE, outFile);

    }*/

    /**
     * 生成二维码（流）
     *
     * @param info         二维码信息
     * @param outputStream 图片输出流
     * @param qrcode       二维码实体类
     * @throws WriterException
     * @throws IOException
     */
    public static void MakeQRCode(String info, OutputStream outputStream, Qrcode qrcode) throws WriterException, IOException {
        BufferedImage image = toBufferedImg(getBitMatrix(info, qrcode.getQrSize()));

        ImageIO.write(image, IMG_FORMATE, outputStream);
    }

    /**
     * 生成二维码(数据库传参)
     *
     * @param info   二维码信息
     * @param out    目的图片地址
     * @param qrcode 二维码实体类
     * @throws WriterException
     * @throws IOException
     */
    public static void MakeQRCode(String info, String out, Qrcode qrcode) throws WriterException, IOException {
        File outFile = new File(out);

        if (!outFile.getParentFile().exists()) {
            outFile.getParentFile().mkdirs();
        }

        BufferedImage image = toBufferedImg(getBitMatrix(info, qrcode.getQrSize()));

        ImageIO.write(image, IMG_FORMATE, outFile);
    }

    /**
     * 生成带logo的二维码图片
     *
     * @param info     图片信息
     * @param out      输出图片地址
     * @param logoPath logo图片地址
     * @throws WriterException
     * @throws IOException
     *//*
    public static void MakeQrCodeWithLogo(String info, String out, String logoPath) throws WriterException, IOException {
        File outFile = new File(out);

        if (!outFile.getParentFile().exists()) {
            outFile.getParentFile().mkdirs();
        }


        BufferedImage image = toTopAndButtonImg(getBitMatrix(info), true, info, PROPARAM.TITLE);


        if (PROPARAM.LOGO_PATH != null) {
            addLogo2QrCode(image, new File(PROPARAM.LOGO_PATH));
        }


        ImageIO.write(image, IMG_FORMATE, outFile);
    }*/

    /**
     * 生成带logo的二维码图片（数据库传参）
     *
     * @param info     二维码信息
     * @param out      输出地址
     * @param logoPath logo图片地址
     * @param qrcode   二维码实体类
     * @throws WriterException
     * @throws IOException
     */
    public static void MakeQrCodeWithLogo(String info, String out, String logoPath, Qrcode qrcode, String number) throws WriterException, IOException {
        File outFile = new File(out);

        if (!outFile.getParentFile().exists()) {
            outFile.getParentFile().mkdirs();
        }

        BufferedImage image = toTopAndButtonImg(getBitMatrix(info, qrcode.getQrSize()), true, number, qrcode);

        addLogo2QrCode(image, new File(logoPath), qrcode.getPrefixMsg());


        ImageIO.write(image, IMG_FORMATE, outFile);
    }

    /**
     * 生成带logo的二维码图片（流）
     *
     * @param info         二维码信息
     * @param outputStream 输出流
     * @param logoPath     logo图片地址
     * @param qrcode       二维码实体类
     * @throws WriterException
     * @throws IOException
     */
    public static void MakeQrCodeWithLogo(String info, OutputStream outputStream, String logoPath, Qrcode qrcode, String number) throws WriterException, IOException {
        BufferedImage image = toTopAndButtonImg(getBitMatrix(info, qrcode.getQrSize()), true, number, qrcode);

        addLogo2QrCode(image, new File(logoPath), qrcode.getPrefixMsg());


        ImageIO.write(image, IMG_FORMATE, outputStream);
    }

    /**
     * get bitMatrix from content
     *
     * @param conten qr_code info
     * @return
     * @throws WriterException
     *//*
    private static BitMatrix getBitMatrix(String conten) throws WriterException {
        MultiFormatWriter writer = new MultiFormatWriter();

        Map hint = new HashMap();
        hint.put(EncodeHintType.CHARACTER_SET, "UTF-8");
        hint.put(EncodeHintType.ERROR_CORRECTION, ErrorCorrectionLevel.H);
        hint.put(EncodeHintType.MARGIN, 1);
        return writer.encode(conten, BarcodeFormat.QR_CODE, PIC_WIDTH, PIC_HEIGHT, hint);
    }*/

    /**
     * 重载get bitMatrix from content
     *
     * @param conten
     * @param qrSize 二维码尺寸
     * @return
     * @throws WriterException
     */
    private static BitMatrix getBitMatrix(String conten, int qrSize) throws WriterException {
        MultiFormatWriter writer = new MultiFormatWriter();
        int width = qrSize, height = qrSize;
        Map hint = new HashMap();
        hint.put(EncodeHintType.CHARACTER_SET, "UTF-8");
        hint.put(EncodeHintType.ERROR_CORRECTION, ErrorCorrectionLevel.H);
        hint.put(EncodeHintType.MARGIN, 1);
        return writer.encode(conten, BarcodeFormat.QR_CODE, width, height, hint);
    }

    private static BufferedImage toBufferedImg(BitMatrix bitMatrix) {
        int width = bitMatrix.getWidth();
        int height = bitMatrix.getHeight();

        BufferedImage image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);

        for (int x = 0; x < width; x++) {
            for (int y = 0; y < height; y++) {
                image.setRGB(x, y, bitMatrix.get(x, y) ? BLUE : WHITE);
            }
        }

        return image;
    }

    /*private static BufferedImage toTopAndButtonImg(BitMatrix bitMatrix, boolean isButtonDes, String info, String topMsg) {
        //info="编号："+info;
        int prefix_msg = 0;
        if (PROPARAM.PREFIX_MSG != null && PROPARAM.PREFIX_MSG != "") {
            prefix_msg = 30;
        }

        int width = bitMatrix.getWidth();
        int height = bitMatrix.getHeight();

        BufferedImage image = new BufferedImage(width, isButtonDes ? height + 60 + prefix_msg : height + prefix_msg, BufferedImage.TYPE_INT_RGB);

        for (int x = 0; x < width; x++) {
            for (int y = 0; y < height; y++) {
                image.setRGB(x, y + 30, bitMatrix.get(x, y) ? BLACK : WHITE);
            }
        }

        Graphics2D graph = image.createGraphics();


        //标题
        BufferedImage textImage2 = FontImage.getImage(topMsg, new Font("宋体", Font.BOLD, jssize(topMsg)), QR_SIZE, 30);
        int width3 = textImage2.getWidth(null);
        int height3 = textImage2.getHeight(null);
        Image src2 = textImage2;
        graph.drawImage(src2, 0, 0, width3, height3, null);
        //结尾
        BufferedImage textImage = FontImage.getImage(info, QR_SIZE, 30);
        int width2 = textImage.getWidth(null);
        int height2 = textImage.getHeight(null);
        Image src = textImage;
        graph.drawImage(src, 0, QR_SIZE + 30, width2, height2, null);

        if (PROPARAM.PREFIX_MSG != null && PROPARAM.PREFIX_MSG != "") {
            BufferedImage textImage3 = FontImage.getImage(PROPARAM.PREFIX_MSG, new Font("宋体", Font.BOLD, jssize(PROPARAM.PREFIX_MSG)), QR_SIZE, 30);
            int width4 = textImage3.getWidth(null);
            int height4 = textImage3.getHeight(null);
            Image src3 = textImage3;
            graph.drawImage(src3, 0, QR_SIZE + 60, width4, height4, null);
        }

        graph.dispose();
        return image;
    }*/

    /**
     * 重载toTopAndButtonImg（头部和底部插入信息)方法 从数据库导入配置信息
     *
     * @param bitMatrix
     * @param isButtonDes
     * @param number
     * @param qrcode      二维码实体类
     * @return
     */
    private static BufferedImage toTopAndButtonImg(BitMatrix bitMatrix, boolean isButtonDes, String number, Qrcode qrcode) {
        //info="编号："+info;
        int prefix_msg = 0;
        if (qrcode.getPrefixMsg() != null && qrcode.getPrefixMsg() != "") {
            prefix_msg = 30;
        }

        int width = bitMatrix.getWidth();
        int height = bitMatrix.getHeight();

        BufferedImage image = new BufferedImage(width, isButtonDes ? height + 60 + prefix_msg : height + prefix_msg, BufferedImage.TYPE_INT_RGB);

        for (int x = 0; x < width; x++) {
            for (int y = 0; y < height; y++) {
                image.setRGB(x, y + 30, bitMatrix.get(x, y) ? BLACK : WHITE);
            }
        }

        Graphics2D graph = image.createGraphics();


        //标题
        BufferedImage textImage2 = FontImage.getImage(qrcode.getTitle(), new Font("黑体", Font.BOLD, jssize(qrcode.getTitle(), qrcode.getQrSize())), qrcode.getQrSize(), 30);
        int width3 = textImage2.getWidth(null);
        int height3 = textImage2.getHeight(null);
        Image src2 = textImage2;
        graph.drawImage(src2, 0, 0, width3, height3, null);
        //结尾
        BufferedImage textImage = FontImage.getImage(number, new Font("黑体", Font.PLAIN, 25),qrcode.getQrSize(), 30);
        int width2 = textImage.getWidth(null);
        int height2 = textImage.getHeight(null);
        Image src = textImage;
        graph.drawImage(src, 0, qrcode.getQrSize() + 30, width2, height2, null);

        if (qrcode.getPrefixMsg() != null && qrcode.getPrefixMsg() != "") {
            BufferedImage textImage3 = FontImage.getImage(qrcode.getPrefixMsg(), new Font("黑体", Font.PLAIN, 23), qrcode.getQrSize(), 30);
            int width4 = textImage3.getWidth(null);
            int height4 = textImage3.getHeight(null);
            Image src3 = textImage3;
            graph.drawImage(src3, 0, qrcode.getQrSize() + 60, width4, height4, null);
        }

        graph.dispose();
        return image;
    }

    /*private static void addLogo2QrCode(BufferedImage qrImage, File logoFile) throws IOException {

        if (qrImage == null) {
            throw new RuntimeException("qrpic is null");
        }

        if (!logoFile.exists()) {
            throw new FileNotFoundException("can not found logo file at " + logoFile.getAbsolutePath());
        }

        BufferedImage logo = ImageIO.read(logoFile);

        int logoWidth = logo.getWidth(null);
        int logoHeight = logo.getHeight(null);

        int prefix_msg = 0;
        if (PROPARAM.PREFIX_MSG != null && PROPARAM.PREFIX_MSG != "") {
            prefix_msg = 30;
        }

        if (qrImage.getWidth() / 5 < logoWidth) {
            logoWidth = qrImage.getWidth() / 5;
            logoHeight = qrImage.getHeight() * logoWidth / qrImage.getWidth();
        }

        // get logo position
        int x = (qrImage.getWidth() - logoWidth) / 2;
        int y = (qrImage.getHeight() - prefix_msg - logoHeight) / 2;

        // draw logo on qr picture
        Graphics2D graphics2D = qrImage.createGraphics();

        graphics2D.drawImage(logo, x, y, logoWidth, logoHeight, null);
        graphics2D.drawRoundRect(x, y, logoWidth, logoHeight, 6, 6);

        graphics2D.setStroke(new BasicStroke(LOGO_BORDER_WIDTH));
        graphics2D.setColor(LOGO_BORDER_COLOR);
        graphics2D.drawRect(x, y, logoWidth, logoHeight);

        graphics2D.dispose();
    }
*/

    /**
     * 重载addLogo2QrCode（在二维码中插入图片）方法 从数据导入配置信息
     *
     * @param qrImage
     * @param logoFile
     * @param prefixMsg 末尾备注信息
     * @throws IOException
     */
    private static void addLogo2QrCode(BufferedImage qrImage, File logoFile, String prefixMsg) throws IOException {

        if (qrImage == null) {
            throw new RuntimeException("qrpic is null");
        }

        if (!logoFile.exists()) {
            throw new FileNotFoundException("can not found logo file at " + logoFile.getAbsolutePath());
        }

        BufferedImage logo = ImageIO.read(logoFile);

        int logoWidth = logo.getWidth(null);
        int logoHeight = logo.getHeight(null);

        int prefix_msg = 0;
        if (prefixMsg != null && prefixMsg != "") {
            prefix_msg = 30;
        }

        if (qrImage.getWidth() / 5 < logoWidth) {
            logoWidth = qrImage.getWidth() / 5;
            logoHeight = qrImage.getWidth() / 5;
            /*logoHeight = qrImage.getHeight() * logoWidth / qrImage.getWidth();*/
        }

        // get logo position
        int x = (qrImage.getWidth() - logoWidth) / 2;
        int y = (qrImage.getHeight() - prefix_msg - logoHeight) / 2;

        // draw logo on qr picture
        Graphics2D graphics2D = qrImage.createGraphics();

        graphics2D.drawImage(logo, x, y, logoWidth, logoHeight, null);
        graphics2D.drawRoundRect(x, y, logoWidth, logoHeight, 6, 6);

        graphics2D.setStroke(new BasicStroke(LOGO_BORDER_WIDTH));
        graphics2D.setColor(LOGO_BORDER_COLOR);
        graphics2D.drawRect(x, y, logoWidth, logoHeight);

        graphics2D.dispose();
    }

    /*private static int jssize(String context) {
        int length = context.length();
        return QR_SIZE / length - 1;
    }*/

    /**
     * 重载jssize方法 从数据库导入配置
     *
     * @param context
     * @param qrSize  二维码尺寸
     * @return
     */
    private static int jssize(String context, int qrSize) {
        int length = context.length();
        return Math.min(qrSize / length - 1, 30);
    }

}
