package com.tofly.dyrq.repository;


import com.tofly.dyrq.dto.InspectionReportCompanyStatisticDTO;
import org.springframework.data.jpa.repository.JpaRepository;
import org.springframework.data.jpa.repository.Query;
import org.springframework.stereotype.Repository;

import java.util.Date;
import java.util.List;
import java.util.Map;

@Repository
public interface InspectionReportCompanyStatisticRepository extends JpaRepository<InspectionReportCompanyStatisticDTO, Long> {

    @Query(value = "SELECT " +
            "a.id companyId, " +
            "a.companyName, " +
            "a.userId," +
            "a.userName," +
            "nvl(t1.total_time, 0) totalTime, " +
            "trunc(nvl(t1.total_distance, 0), 2) totalDistance, " +
            "nvl(t1.total_average, 0) totalAverage, " +
            "nvl(t1.total_valid_time, 0) totalValidTime, " +
            "trunc(nvl(t1.total_valid_distance, 0), 2) totalValidDistance, " +
            "nvl(t1.total_valid_average, 0) totalValidAverage, " +
            "nvl(t1.trouble_num, 0) troubleNum, " +
            "nvl(t1.buildingsitefinded, 0) buildingSitefinded, " +
            "nvl(t3.PointInPlan, 0) PointInPlan, " +
            "nvl(t4.PointPatroled, 0) PointPatroled, " +
            "trunc(nvl(t5.PathInPlan, 0), 2) PathInPlan, " +
            "trunc(nvl(t6.PathPatroled, 0), 2) PathPatroled, " +
            "nvl(t7.EquipmentsInPlan, 0) equipmentsInPlan, " +
            "nvl(t8.EquipmentsPatroled, 0) equipmentsPatroled " +
            "FROM " +
            "   (SELECT t.CODE,t.NAME companyName,t.ID id,x.REAL_NAME userName,x.ID userId FROM TF_SMPN_DEPARTMENT_X t  LEFT JOIN TF_SMPN_USER_X x ON t.ID = x.DEPARTMENT_ID WHERE t.ID = ?1 ORDER BY t.CODE) a " +
            "LEFT JOIN (SELECT s.department_id, " +
            "nvl(sum(t.plan_total_time), 0) total_time, " +
            "nvl(sum(t.plan_total_distance), 0) total_distance, " +
            "case " +
            "  when nvl(sum(t.plan_total_time), 0) = 0 then " +
            "   0 " +
            "  else " +
            "   trunc(nvl(sum(t.plan_total_distance), 0) / " +
            "         nvl(sum(t.plan_total_time), 0), " +
            "         4) " +
            "end total_average, " +
            "nvl(sum(t.valid_time), 0) total_valid_time, " +
            "nvl(sum(t.valid_distance), 0) total_valid_distance, " +
            "case " +
            "  when nvl(sum(t.valid_time), 0) = 0 then " +
            "   0 " +
            "  else " +
            "   trunc(nvl(sum(t.valid_distance), 0) / " +
            "         nvl(sum(t.valid_time), 0), " +
            "         4) " +
            "end total_valid_average, " +
            "nvl(sum(t.troublefinded), 0) trouble_num, " +
            "nvl(sum(t.buildingsitefinded), 0) buildingsitefinded " +
            "           FROM TF_GPS_DAY_CALCULATE t LEFT JOIN TF_SMPN_USER_X s ON t.USERID = s.ID " +
            "           WHERE t.PLAN_DATE >= ?2 AND t.PLAN_DATE <= ?3 GROUP BY s.DEPARTMENT_ID) t1 ON a.ID = t1.DEPARTMENT_ID " +
            "LEFT JOIN (SELECT " +
            "sum(t.total) PointInPlan, s.department_id " +
            "FROM (SELECT " +
            "max(total) total, userid " +
            "FROM TF_POINT_DAY_CALCULATE t " +
            "WHERE t.PLAN_DATE >= ?2 AND t.PLAN_DATE <= ?3 AND t.CHILD_END_DATE >= ?2 AND t.CHILD_END_DATE <= ?3 AND t.DEVICE_TYPE_ID IN('1') GROUP BY t.CHILD_PLAN_ID,t.USERID) t " +
            "LEFT JOIN TF_SMPN_USER_X s ON t.USERID = s.ID GROUP BY s.DEPARTMENT_ID)t3 ON a.ID = t3.DEPARTMENT_ID " +
            "LEFT JOIN (SELECT " +
            "sum(t.isovertototal) PointPatroled, s.department_id " +
            "FROM TF_POINT_DAY_CALCULATE t LEFT JOIN TF_SMPN_USER_X s ON t.USERID = s.ID " +
            "WHERE t.PLAN_DATE >= ?2 AND t.PLAN_DATE <= ?3 AND t.CHILD_END_DATE >= ?2 AND t.CHILD_END_DATE <= ?3 AND DEVICE_TYPE_ID IN('1') GROUP BY s.DEPARTMENT_ID) t4 ON a.ID = t4.DEPARTMENT_ID " +
            "LEFT JOIN (SELECT " +
            "sum(t.total) PathInPlan, s.department_id " +
            "FROM (SELECT " +
            "max(total) total, userid " +
            "FROM TF_POINT_DAY_CALCULATE t WHERE t.PLAN_DATE >= ?2 AND t.PLAN_DATE <= ?3 AND t.CHILD_END_DATE >= ?2 AND t.CHILD_END_DATE <= ?3 AND t.DEVICE_TYPE_ID IN('6') GROUP BY t.CHILD_PLAN_ID,t.USERID) t " +
            "LEFT JOIN TF_SMPN_USER_X s ON t.USERID = s.ID GROUP BY s.DEPARTMENT_ID) t5 ON a.ID = t5.DEPARTMENT_ID " +
            "LEFT JOIN (SELECT " +
            "sum(t.isovertototal) PathPatroled, s.department_id " +
            "FROM TF_POINT_DAY_CALCULATE t LEFT JOIN TF_SMPN_USER_X s ON t.USERID = s.ID " +
            "           WHERE t.PLAN_DATE >= ?2 AND t.PLAN_DATE <= ?3 AND t.CHILD_END_DATE >= ?2 AND t.CHILD_END_DATE <= ?3 AND t.DEVICE_TYPE_ID IN('6') GROUP BY s.DEPARTMENT_ID) t6 ON a.ID = t6.DEPARTMENT_ID " +
            "LEFT JOIN (SELECT " +
            "sum(t.total) EquipmentsInPlan, s.department_id " +
            "FROM (SELECT " +
            "max(total) total, userid " +
            "FROM TF_POINT_DAY_CALCULATE t WHERE t.PLAN_DATE >= ?2 AND t.PLAN_DATE <= ?3 AND t.CHILD_END_DATE >= ?2 AND t.CHILD_END_DATE <= ?3 AND t.DEVICE_TYPE_ID IN('2','3','4','7','8') GROUP BY t.CHILD_PLAN_ID,t.USERID) t " +
            "           LEFT JOIN TF_SMPN_USER_X s ON t.USERID = s.ID GROUP BY s.DEPARTMENT_ID) t7 ON  a.ID = t7.DEPARTMENT_ID " +
            "LEFT JOIN (SELECT " +
            "sum(t.isovertototal) EquipmentsPatroled, s.department_id " +
            "FROM TF_POINT_DAY_CALCULATE t LEFT JOIN TF_SMPN_USER_X s ON t.USERID = s.ID " +
            "WHERE t.PLAN_DATE >= ?2 AND t.PLAN_DATE <= ?3 AND t.CHILD_END_DATE >= ?2 AND t.CHILD_END_DATE <= ?3 AND t.DEVICE_TYPE_ID IN('2','3','4','7','8') GROUP BY s.DEPARTMENT_ID) t8 ON a.ID = t8.DEPARTMENT_ID ", nativeQuery = true)
    List<Map<String,Object>> getStaticPlanByCompany(String departmentId, Date sDate, Date eDate);

    @Query(value = "SELECT t.companyId,t.companyName,t.totalTime,t.totalDistance,t.totalAverage,t.totalValidTime,t.totalValidDistance, " +
                    "t.totalValidAverage,t.troubleNum,t.buildingSitefinded,t.pointInPlan,t.pointPatroled,t.pathInPlan,t.pathPatroled,t.equipmentsInPlan,t.equipmentsPatroled " +
                    "FROM ( " +
                    "SELECT " +
                    "a.id companyId, " +
                    "a.companyName, " +
                    "a.userId, " +
                    "a.userName," +
                    "nvl(t1.total_time, 0) totalTime, " +
                    "trunc(nvl(t1.total_distance, 0), 2) totalDistance, " +
                    "nvl(t1.total_average, 0) totalAverage, " +
                    "nvl(t1.total_valid_time, 0) totalValidTime, " +
                    "trunc(nvl(t1.total_valid_distance, 0), 2) totalValidDistance, " +
                    "nvl(t1.total_valid_average, 0) totalValidAverage, " +
                    "nvl(t1.trouble_num, 0) troubleNum, " +
                    "nvl(t1.buildingsitefinded, 0) buildingSitefinded, " +
                    "nvl(t3.PointInPlan, 0) pointInPlan, " +
                    "nvl(t4.PointPatroled, 0) pointPatroled, " +
                    "trunc(nvl(t5.PathInPlan, 0), 2) pathInPlan, " +
                    "trunc(nvl(t6.PathPatroled, 0), 2) pathPatroled, " +
                    "nvl(t7.EquipmentsInPlan, 0) equipmentsInPlan, " +
                    "nvl(t8.EquipmentsPatroled, 0) equipmentsPatroled " +
                    "FROM " +
                    "   (SELECT t.CODE,t.NAME companyName,t.ID id,x.REAL_NAME userName,x.ID userId FROM TF_SMPN_DEPARTMENT_X t LEFT JOIN TF_SMPN_USER_X x ON t.ID = x.DEPARTMENT_ID  ORDER BY t.CODE) a " +
                    "LEFT JOIN (SELECT s.department_id, " +
                    "nvl(sum(t.plan_total_time), 0) total_time, " +
                    "nvl(sum(t.plan_total_distance), 0) total_distance, " +
                    "case " +
                    "  when nvl(sum(t.plan_total_time), 0) = 0 then " +
                    "   0 " +
                    "  else " +
                    "   trunc(nvl(sum(t.plan_total_distance), 0) / " +
                    "         nvl(sum(t.plan_total_time), 0), " +
                    "         4) " +
                    "end total_average, " +
                    "nvl(sum(t.valid_time), 0) total_valid_time, " +
                    "nvl(sum(t.valid_distance), 0) total_valid_distance, " +
                    "case " +
                    "  when nvl(sum(t.valid_time), 0) = 0 then " +
                    "   0 " +
                    "  else " +
                    "   trunc(nvl(sum(t.valid_distance), 0) / " +
                    "         nvl(sum(t.valid_time), 0), " +
                    "         4) " +
                    "end total_valid_average, " +
                    "nvl(sum(t.troublefinded), 0) trouble_num, " +
                    "nvl(sum(t.buildingsitefinded), 0) buildingsitefinded " +
                    "           FROM TF_GPS_DAY_CALCULATE t LEFT JOIN TF_SMPN_USER_X s ON t.USERID = s.ID " +
                    "           WHERE t.PLAN_DATE >= ?1 AND t.PLAN_DATE <= ?2 GROUP BY s.DEPARTMENT_ID) t1 ON a.ID = t1.DEPARTMENT_ID " +
                    "LEFT JOIN (SELECT " +
                    "sum(t.total) PointInPlan, s.department_id " +
                    "FROM (SELECT " +
                    "max(total) total, userid " +
                    "FROM TF_POINT_DAY_CALCULATE t " +
                    "WHERE t.PLAN_DATE >= ?1 AND t.PLAN_DATE <= ?2 AND t.CHILD_END_DATE >= ?1 AND t.CHILD_END_DATE <= ?2 AND t.DEVICE_TYPE_ID IN('1') GROUP BY t.CHILD_PLAN_ID,t.USERID) t " +
                    "LEFT JOIN TF_SMPN_USER_X s ON t.USERID = s.ID GROUP BY s.DEPARTMENT_ID)t3 ON a.ID = t3.DEPARTMENT_ID " +
                    "LEFT JOIN (SELECT " +
                    "sum(t.isovertototal) PointPatroled, s.department_id " +
                    "FROM TF_POINT_DAY_CALCULATE t LEFT JOIN TF_SMPN_USER_X s ON t.USERID = s.ID " +
                    "WHERE t.PLAN_DATE >= ?1 AND t.PLAN_DATE <= ?2 AND t.CHILD_END_DATE >= ?1 AND t.CHILD_END_DATE <= ?2 AND DEVICE_TYPE_ID IN('1') GROUP BY s.DEPARTMENT_ID) t4 ON a.ID = t4.DEPARTMENT_ID " +
                    "LEFT JOIN (SELECT " +
                    "sum(t.total) PathInPlan, s.department_id " +
                    "FROM (SELECT " +
                    "max(total) total, userid " +
                    "FROM TF_POINT_DAY_CALCULATE t WHERE t.PLAN_DATE >= ?1 AND t.PLAN_DATE <= ?2 AND t.CHILD_END_DATE >= ?1 AND t.CHILD_END_DATE <= ?2 AND t.DEVICE_TYPE_ID IN('6') GROUP BY t.CHILD_PLAN_ID,t.USERID) t " +
                    "LEFT JOIN TF_SMPN_USER_X s ON t.USERID = s.ID GROUP BY s.DEPARTMENT_ID) t5 ON a.ID = t5.DEPARTMENT_ID " +
                    "LEFT JOIN (SELECT " +
                    "sum(t.isovertototal) PathPatroled, s.department_id " +
                    "FROM TF_POINT_DAY_CALCULATE t LEFT JOIN TF_SMPN_USER_X s ON t.USERID = s.ID " +
                    "           WHERE t.PLAN_DATE >= ?1 AND t.PLAN_DATE <= ?2 AND t.CHILD_END_DATE >= ?1 AND t.CHILD_END_DATE <= ?2 AND t.DEVICE_TYPE_ID IN('6') GROUP BY s.DEPARTMENT_ID) t6 ON a.ID = t6.DEPARTMENT_ID " +
                    "LEFT JOIN (SELECT " +
                    "sum(t.total) EquipmentsInPlan, s.department_id " +
                    "FROM (SELECT " +
                    "max(total) total, userid " +
                    "FROM TF_POINT_DAY_CALCULATE t WHERE t.PLAN_DATE >= ?1 AND t.PLAN_DATE <= ?2 AND t.CHILD_END_DATE >= ?1 AND t.CHILD_END_DATE <= ?2 AND t.DEVICE_TYPE_ID IN('2','3','4','7','8') GROUP BY t.CHILD_PLAN_ID,t.USERID) t " +
                    "           LEFT JOIN TF_SMPN_USER_X s ON t.USERID = s.ID GROUP BY s.DEPARTMENT_ID) t7 ON  a.ID = t7.DEPARTMENT_ID " +
                    "LEFT JOIN (SELECT " +
                    "sum(t.isovertototal) EquipmentsPatroled, s.department_id " +
                    "FROM TF_POINT_DAY_CALCULATE t LEFT JOIN TF_SMPN_USER_X s ON t.USERID = s.ID " +
                    "WHERE t.PLAN_DATE >= ?1 AND t.PLAN_DATE <= ?2 AND t.CHILD_END_DATE >= ?1 AND t.CHILD_END_DATE <= ?2 AND t.DEVICE_TYPE_ID IN('2','3','4','7','8') GROUP BY s.DEPARTMENT_ID) t8 ON a.ID = t8.DEPARTMENT_ID) t ", nativeQuery = true)
    List<Map<String,Object>> getStaticPlanAll(Date sDate, Date eDate);
}
