package com.tofly.dyrq.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.tofly.dyrq.dto.InspectionReportPlanDTO;
import com.tofly.dyrq.dto.InspectionReportPointDTO;
import com.tofly.dyrq.dto.InspectionReportUserDTO;
import com.tofly.dyrq.entity.GpsDayCalculate;
import com.tofly.dyrq.repository.*;
import com.tofly.dyrq.service.InspectionReportService;
import com.tofly.dyrq.util.DateTimeUtil;
import com.tofly.dyrq.vo.*;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @program: tofly-dyrq
 * @description: 巡检报表功能实现类
 * @author: Sumer
 * @create: 2020-09-08 09:33
 */
@Service
@Slf4j
public class InspectionReportServiceImpl implements InspectionReportService {

    @Autowired
    private InspectionReportUserRepository inspectionReportUserRepository;

    @Autowired
    private InspectionReportPlanRepository inspectionReportPlanRepository;

    @Autowired
    private InspectionReportPointRepository inspectionReportPointRepository;

    @Autowired
    private GpsDayCalculateRepository gpsDayCalculateRepository;

    @Autowired
    private InspectionReportCompanyStatisticRepository inspectionReportCompanyStatisticRepository;

    @Override
    public List<DailyPlanStaticsVO> disposeDailyPlanStatics(String departmentId, String userId, String searchDate, String timeType) {
        LocalDateTime assignTodayBeginTime = DateTimeUtil.fromLong2LocalDateTime(DateTimeUtil.localDateToDate(Objects.requireNonNull(DateTimeUtil.fromString2LocalDate(searchDate, DateTimeUtil.DAY_FORMAT))).getTime());
        LocalDateTime assignTodayEndTime = null;
        if ("1".equals(timeType)) {
            assignTodayEndTime = assignTodayBeginTime.plusDays(1).minusSeconds(1);
        } else if ("2".equals(timeType)) {
            assignTodayEndTime = assignTodayBeginTime.plusMonths(1).minusSeconds(1);
        }
        return dailyPlanStatics(departmentId, userId, assignTodayBeginTime, assignTodayEndTime,timeType);

    }

    /**
     * 巡检(日 OR 月)报处理
     *
     * @param departmentId         所属组织部门
     * @param userId               用户id
     * @param assignTodayBeginTime 巡检起始日期
     * @param assignTodayEndTime   巡检结束日期
     * @param timeType             日期类型
     */
    private List<DailyPlanStaticsVO> dailyPlanStatics(String departmentId, String userId, LocalDateTime assignTodayBeginTime, LocalDateTime assignTodayEndTime, String timeType) {

        Date sDate = DateTimeUtil.convertLDTToDate(assignTodayBeginTime);
        Date eDate = DateTimeUtil.convertLDTToDate(assignTodayEndTime);
        // 返回日期处理
        String planDate = DateTimeUtil.getLocalDateTime(assignTodayBeginTime,DateTimeUtil.DAY_FORMAT);
        if("2".equals(timeType)){
            planDate = DateTimeUtil.getLocalDateTime(DateTimeUtil.fromLong2LocalDateTime(sDate.getTime()),DateTimeUtil.MONTHS_FORMAT);
        }
        // 获取所有用户相关信息
        List<InspectionReportUserDTO> users = inspectionReportUserRepository.getUsers();
        // 获取当前时间范围对应巡检类型的计划相关信息
        List<InspectionReportPlanDTO> plans = inspectionReportPlanRepository.getPlans(sDate, eDate);
        if (StringUtils.isNotEmpty(userId)) {
            users = users.stream().filter(m -> m.getUserId().equals(Long.parseLong(userId))).collect(Collectors.toList());
        }
        if (StringUtils.isNotEmpty(departmentId)) {
            users = users.stream().filter(m -> m.getDepartmentId().equals(Long.parseLong(departmentId))).collect(Collectors.toList());
        }

        // 获取当前时间范围的点相关信息
        List<InspectionReportPointDTO> points = inspectionReportPointRepository.getPoints(sDate, eDate);

        List<DailyPlanStaticsVO> resultList = new ArrayList<>();
        for (InspectionReportUserDTO dto : users) {
            // 获取累计计算轨迹相关信息
            List<GpsDayCalculate> calculateAll = gpsDayCalculateRepository.getGpsDayCalculateList(dto.getUserId(), sDate, eDate);
            DailyPlanStaticsBasisVO vo = new DailyPlanStaticsBasisVO();
            Long userid = dto.getUserId();
            vo.setUserId(String.valueOf(userid));
            vo.setDepartmentId(String.valueOf(dto.getDepartmentId()));
            vo.setDepName(dto.getDepName());
            vo.setUserName(dto.getUserName());
            vo.setPatrolDate(planDate);
            if (null != calculateAll && calculateAll.size() > 0) {
                GpsDayCalculate dayCalculate = calculateAll.get(0);
                vo.setPatrolDateRange(dayCalculate.getSignRange() == null ? "" : dayCalculate.getSignRange());
                vo.setPlanTotalAverage(BigDecimal.valueOf(dayCalculate.getPlanTotalAverage() == null ? 0 : dayCalculate.getPlanTotalAverage()));
                vo.setPlanTotalDistance(BigDecimal.valueOf(dayCalculate.getPlanTotalDistance() == null ? 0 : dayCalculate.getPlanTotalDistance()));
                vo.setPlanTotalTime(BigDecimal.valueOf(dayCalculate.getPlanTotalTime() == null ? 0 : dayCalculate.getPlanTotalTime()));
                vo.setTotalValidAverage(BigDecimal.valueOf(dayCalculate.getValidAverage() == null ? 0 : dayCalculate.getValidAverage()));
                vo.setTotalValidDistance(BigDecimal.valueOf(dayCalculate.getValidDistance() == null ? 0 : dayCalculate.getValidDistance()));
                vo.setTotalValidTime(BigDecimal.valueOf(dayCalculate.getValidTime() == null ? 0 : dayCalculate.getValidTime()));
            } else {
                vo.setPatrolDateRange("");
                vo.setPlanTotalAverage(BigDecimal.ZERO);
                vo.setPlanTotalDistance(BigDecimal.ZERO);
                vo.setPlanTotalTime(BigDecimal.ZERO);
                vo.setTotalValidAverage(BigDecimal.ZERO);
                vo.setTotalValidDistance(BigDecimal.ZERO);
                vo.setTotalValidTime(BigDecimal.ZERO);
            }

            List<InspectionReportPlanDTO> planInfo = new ArrayList<>();
            if (null != plans && plans.size() > 0) {
                planInfo = plans.stream().filter(m -> m.getId().equals(userid)).sorted(Comparator.comparing(InspectionReportPlanDTO::getDeviceTypeId)).collect(Collectors.toList());
            }
            Optional<Double> reduce = planInfo.stream().map(InspectionReportPlanDTO::getPercent1).reduce(Double::sum);
            if (reduce.isPresent()) {
                vo.setTotalPercent(BigDecimal.valueOf(reduce.get()));
            } else {
                vo.setTotalPercent(BigDecimal.ZERO);
            }

            DailyPlanStaticsDetailInfoVO infoVO = new DailyPlanStaticsDetailInfoVO();
            planInfo.forEach(m ->
            {

                String stDate = DateTimeUtil.getLocalDateTime(DateTimeUtil.fromLong2LocalDateTime(m.getChildBeginDate().getTime()), DateTimeUtil.FULL_SLASH_FORMAT);
                String enDate = DateTimeUtil.getLocalDateTime(DateTimeUtil.fromLong2LocalDateTime(m.getChildEndDate().getTime()), DateTimeUtil.FULL_SLASH_FORMAT);
                infoVO.setPlanDate(stDate + "~" + enDate);
                infoVO.setPercent(BigDecimal.valueOf(m.getPercent1()).setScale(4, BigDecimal.ROUND_HALF_UP));
                infoVO.setPeriodName(m.getPeriodName());
                infoVO.setRegionName(m.getRegionName() + m.getDeviceTypeName() + ":" + m.getPeriodName());
                String deviceTypeName = m.getDeviceTypeName();
                long molecular = points.stream().filter(t -> t.getIsover().equals("1") && t.getState().equals("1") && t.getChildPlanId().equals(m.getId())).count();
                long denominator = points.stream().filter(t -> t.getChildPlanId().equals(m.getId())).count();
                infoVO.setMustCheck(m.getType().equals("0") ? deviceTypeName + "(" + molecular + "/" + denominator + ")" : deviceTypeName + "(" + molecular + "/" + denominator + ")");
                infoVO.setDeviceTypeName(deviceTypeName);
                infoVO.setDeviceTypeId(m.getType());
                List<DailyPlanStaticsPointVO> pointVOS = new ArrayList<>();

                points.forEach(t ->
                {
                    if (t.getChildPlanId().equals(m.getId())) {
                        DailyPlanStaticsPointVO pointVO = new DailyPlanStaticsPointVO();
                        pointVO.setPointName(t.getCode());
                        pointVO.setPointInfo(t.getIsover().equals("0") || t.getState().equals("0") ? "(未检)" : DateTimeUtil.getLocalDateTime(DateTimeUtil.fromLong2LocalDateTime(t.getOverDate().getTime()), DateTimeUtil.FULL_SLASH_FORMAT));
                        pointVO.setPointSpeed(t.getGpsSpeed() == null ? BigDecimal.ZERO : t.getGpsSpeed());
                        pointVOS.add(pointVO);
                    }

                });
                infoVO.setPointDetail(pointVOS);
            });
            DailyPlanStaticsVO detailInfo = new DailyPlanStaticsVO();
            detailInfo.setBasisVO(vo);
            detailInfo.setDetailInfoVO(infoVO);

            resultList.add(detailInfo);
        }

        return resultList;

    }


    @Override
    public List<StaticModelPlanResultVO> disposeStaticPlan(String departmentId, String beginPlanDate, String endPlanDate) {
        Date startDate = DateTimeUtil.localDateToDate(Objects.requireNonNull(DateTimeUtil.fromString2LocalDate(beginPlanDate, DateTimeUtil.DAY_FORMAT)));
        Date endDate = DateTimeUtil.localDateToDate(Objects.requireNonNull(DateTimeUtil.fromString2LocalDate(endPlanDate, DateTimeUtil.DAY_FORMAT)));
        List<Map<String, Object>> staticPlanMap = new ArrayList<>();

        if (StringUtils.isNotEmpty(departmentId)) {
            staticPlanMap = inspectionReportCompanyStatisticRepository.getStaticPlanByCompany(departmentId, startDate, endDate);
        } else {
            staticPlanMap = inspectionReportCompanyStatisticRepository.getStaticPlanAll(startDate, endDate);
        }


        // 映射
        List<StaticModelPlanResultVO> list = JSONObject.parseArray(JSONObject.toJSONString(staticPlanMap), StaticModelPlanResultVO.class);
        list.forEach(m ->
        {
            // 巡检管线完成率
            m.setPipelinePercent(m.getTotalDistance().equals(BigDecimal.ZERO) ? BigDecimal.ZERO : m.getTotalValidDistance().divide(m.getTotalDistance(), BigDecimal.ROUND_HALF_UP).setScale(4, BigDecimal.ROUND_HALF_UP));
            // 巡检设备个数完成率
            m.setEquipmentsPercent(m.getEquipmentsInPlan().equals(BigDecimal.ZERO) ? BigDecimal.ZERO : m.getEquipmentsPatroled().divide(m.getEquipmentsInPlan(), BigDecimal.ROUND_HALF_UP).setScale(4, BigDecimal.ROUND_HALF_UP));
            // 巡检线完成率
            m.setPathPercent(m.getPathInPlan().equals(BigDecimal.ZERO) ? BigDecimal.ZERO : m.getPathPatroled().divide(m.getPathInPlan(), BigDecimal.ROUND_HALF_UP).setScale(4, BigDecimal.ROUND_HALF_UP));
            // 巡检点完成率
            m.setPointPercent(Objects.equals(m.getPointInPlan(), BigDecimal.ZERO) ? BigDecimal.ZERO : m.getPointPatroled().divide(m.getPointInPlan(), BigDecimal.ROUND_HALF_UP).setScale(4, BigDecimal.ROUND_HALF_UP));
        });

        return list;

    }


}